@@ -2,11 +2,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource://calendar/modules/calUtils.jsm");
-Components.utils.import("resource://calendar/modules/calAlarmUtils.jsm");
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
-Components.utils.import("resource://gre/modules/Preferences.jsm");
+ChromeUtils.import("resource://calendar/modules/calUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Preferences.jsm");
+ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
+
+/* exported invokeEventDragSession, calendarViewDNDObserver,
+ *          calendarMailButtonDNDObserver, calendarCalendarButtonDNDObserver,
+ *          calendarTaskButtonDNDObserver
+ */
 
 var itemConversion = {
 
@@ -16,20 +21,20 @@ var itemConversion = {
      * @param aItem     The target calIItemBase.
      * @param aMessage  The nsIMsgHdr to convert from.
      */
-    calendarItemFromMessage: function iC_calendarItemFromMessage(aItem, aMsgHdr) {
+    calendarItemFromMessage: function(aItem, aMsgHdr) {
         let msgFolder = aMsgHdr.folder;
         let msgUri = msgFolder.getUriForMsg(aMsgHdr);
 
         aItem.calendar = getSelectedCalendar();
         aItem.title = aMsgHdr.mime2DecodedSubject;
 
-        cal.setDefaultStartEndHour(aItem);
+        cal.dtz.setDefaultStartEndHour(aItem);
         cal.alarms.setDefaultValues(aItem);
 
-        let messenger = Components.classes["@mozilla.org/messenger;1"]
-                                  .createInstance(Components.interfaces.nsIMessenger);
-        let streamListener = Components.classes["@mozilla.org/network/sync-stream-listener;1"]
-                                       .createInstance(Components.interfaces.nsISyncStreamListener);
+        let messenger = Cc["@mozilla.org/messenger;1"]
+                        .createInstance(Ci.nsIMessenger);
+        let streamListener = Cc["@mozilla.org/network/sync-stream-listener;1"]
+                             .createInstance(Ci.nsISyncStreamListener);
         messenger.messageServiceFromURI(msgUri).streamMessage(msgUri,
                                                               streamListener,
                                                               null,
@@ -57,22 +62,23 @@ var itemConversion = {
      * @param aItem     The item to copy from.
      * @param aTarget   the item to copy to.
      */
-    copyItemBase: function iC_copyItemBase(aItem, aTarget) {
-        const copyProps = ["SUMMARY", "LOCATION", "DESCRIPTION",
-                           "URL", "CLASS", "PRIORITY"];
+    copyItemBase: function(aItem, aTarget) {
+        const copyProps = [
+            "SUMMARY", "LOCATION", "DESCRIPTION", "URL", "CLASS", "PRIORITY"
+        ];
 
-        for each (var prop in copyProps) {
+        for (let prop of copyProps) {
             aTarget.setProperty(prop, aItem.getProperty(prop));
         }
 
         // Attendees
-        var attendees = aItem.getAttendees({});
-        for each (var attendee in attendees) {
+        let attendees = aItem.getAttendees({});
+        for (let attendee of attendees) {
             aTarget.addAttendee(attendee.clone());
         }
 
         // Categories
-        var categories = aItem.getCategories({});
+        let categories = aItem.getCategories({});
         aTarget.setCategories(categories.length, categories);
 
         // Organizer
@@ -95,7 +101,7 @@ var itemConversion = {
      * @param aEvent    The event to copy from.
      * @return          The resulting task.
      */
-    taskFromEvent: function iC_taskFromEvent(aEvent) {
+    taskFromEvent: function(aEvent) {
         let item = cal.createTodo();
 
         this.copyItemBase(aEvent, item);
@@ -107,19 +113,19 @@ var itemConversion = {
             item.dueDate = aEvent.endDate.clone();
 
             // Alarms
-            for each (let alarm in aEvent.getAlarms({})) {
+            for (let alarm of aEvent.getAlarms({})) {
                 item.addAlarm(alarm.clone());
             }
-            item.alarmLastAck = (aEvent.alarmLastAck ?
-                                 aEvent.alarmLastAck.clone() :
-                                 null);
+            item.alarmLastAck = aEvent.alarmLastAck
+                                ? aEvent.alarmLastAck.clone()
+                                : null;
         }
 
         // Map Status values
         let statusMap = {
-            "TENTATIVE": "NEEDS-ACTION",
-            "CONFIRMED": "IN-PROCESS",
-            "CANCELLED": "CANCELLED"
+            TENTATIVE: "NEEDS-ACTION",
+            CONFIRMED: "IN-PROCESS",
+            CANCELLED: "CANCELLED"
         };
         if (aEvent.getProperty("STATUS") in statusMap) {
             item.setProperty("STATUS", statusMap[aEvent.getProperty("STATUS")]);
@@ -135,7 +141,7 @@ var itemConversion = {
      * @param aTask     The task to copy from.
      * @return          The resulting event.
      */
-    eventFromTask: function iC_eventFromTask(aTask) {
+    eventFromTask: function(aTask) {
         let item = cal.createEvent();
 
         this.copyItemBase(aTask, item);
@@ -147,7 +153,7 @@ var itemConversion = {
                 item.startDate = aTask.dueDate.clone();
                 item.startDate.minute -= Preferences.get("calendar.event.defaultlength", 60);
             } else {
-                item.startDate = cal.getDefaultStartDate();
+                item.startDate = cal.dtz.getDefaultStartDate();
             }
         }
 
@@ -160,12 +166,12 @@ var itemConversion = {
         }
 
         // Alarms
-        for each (let alarm in aTask.getAlarms({})) {
+        for (let alarm of aTask.getAlarms({})) {
             item.addAlarm(alarm.clone());
         }
-        item.alarmLastAck = (aTask.alarmLastAck ?
-                             aTask.alarmLastAck.clone() :
-                             null);
+        item.alarmLastAck = aTask.alarmLastAck
+                            ? aTask.alarmLastAck.clone()
+                            : null;
 
         // Map Status values
         let statusMap = {
@@ -186,16 +192,16 @@ var itemConversion = {
  * @class calDNDBaseObserver
  */
 function calDNDBaseObserver() {
-    ASSERT(false, "Inheriting objects call calDNDBaseObserver!");
+    cal.ASSERT(false, "Inheriting objects call calDNDBaseObserver!");
 }
 
 calDNDBaseObserver.prototype = {
     // initialize this class's members
-    initBase: function calDNDInitBase() {
+    initBase: function() {
     },
 
-    getSupportedFlavours: function calDNDGetFlavors() {
-        var flavourSet = new FlavourSet();
+    getSupportedFlavours: function() {
+        let flavourSet = new FlavourSet();
         flavourSet.appendFlavour("text/calendar");
         flavourSet.appendFlavour("text/x-moz-url");
         flavourSet.appendFlavour("text/x-moz-message");
@@ -208,9 +214,9 @@ calDNDBaseObserver.prototype = {
      * Action to take when dropping the event.
      */
 
-    onDrop: function calDNDDrop(aEvent, aTransferData, aDragSession) {
-        var transferable = Components.classes["@mozilla.org/widget/transferable;1"]
-                           .createInstance(Components.interfaces.nsITransferable);
+    onDrop: function(aEvent, aTransferData, aDragSession) {
+        let transferable = Cc["@mozilla.org/widget/transferable;1"]
+                           .createInstance(Ci.nsITransferable);
         transferable.init(null);
         transferable.addDataFlavor("text/calendar");
         transferable.addDataFlavor("text/x-moz-url");
@@ -220,58 +226,62 @@ calDNDBaseObserver.prototype = {
 
         aDragSession.getData(transferable, 0);
 
-        var data = new Object();
-        var bestFlavor = new Object();
-        var length = new Object();
+        let data = new Object();
+        let bestFlavor = new Object();
+        let length = new Object();
         transferable.getAnyTransferData(bestFlavor, data, length);
 
         try {
-            data = data.value.QueryInterface(Components.interfaces.nsISupportsString);
+            data = data.value.QueryInterface(Ci.nsISupportsString);
         } catch (exc) {
             // we currently only supports strings:
             return;
         }
 
         // Treat unicode data with VEVENT in it as text/calendar
-        if (bestFlavor.value == "text/unicode" && data.toString().indexOf("VEVENT") != -1) {
+        if (bestFlavor.value == "text/unicode" &&
+            data.toString().includes("VEVENT")) {
             bestFlavor.value = "text/calendar";
         }
 
-        var destCal = getSelectedCalendar();
-        switch (bestFlavor.value) {
-            case "text/calendar":
-#ifdef XP_MACOSX
-                // Mac likes to convert all \r to \n, we need to reverse this.
-                data = data.data.replace(/\n\n/g, "\r\n");
-#endif
-                var parser = Components.classes["@mozilla.org/calendar/ics-parser;1"]
+         switch (bestFlavor.value) {
+            case "text/calendar": {
+                if (AppConstants.platform == "macosx") {
+                    // Mac likes to convert all \r to \n, we need to reverse this.
+                    data = data.data.replace(/\n\n/g, "\r\n");
+                }
+                let parser = Cc["@mozilla.org/calendar/ics-parser;1"]
                              .createInstance(Components.interfaces.calIIcsParser);
                 parser.parseString(data);
-                this.onDropItems(parser.getItems({}).concat(parser.getParentlessItems({})));
+                this.onDropItems(
+                    parser.getItems({}).concat(parser.getParentlessItems({}))
+                );
                 break;
-            case "text/unicode":
-                var droppedUrl = this.retrieveURLFromData(data, bestFlavor.value);
-                if (!droppedUrl)
+            }
+            case "text/unicode": {
+                let droppedUrl = this.retrieveURLFromData(data, bestFlavor.value);
+                if (!droppedUrl) {
                     return;
+                }
 
-                var url = makeURL(droppedUrl);
+                let url = Services.io.newURI(droppedUrl);
 
-                var localFileInstance = Components.classes["@mozilla.org/file/local;1"]
-                                        .createInstance(Components.interfaces.nsILocalFile);
+                let localFileInstance = Cc["@mozilla.org/file/local;1"]
+                                        .createInstance(Ci.nsIFile);
                 localFileInstance.initWithPath(url.path);
 
-                var inputStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
-                                  .createInstance(Components.interfaces.nsIFileInputStream);
+                let inputStream = Cc["@mozilla.org/network/file-input-stream;1"]
+                                  .createInstance(Ci.nsIFileInputStream);
                 inputStream.init(localFileInstance,
                                  MODE_RDONLY,
                                  parseInt("0444", 8),
                                  {});
 
                 try {
-                    //XXX support csv
-                    var importer = Components.classes["@mozilla.org/calendar/import;1?type=ics"]
-                                   .getService(Components.interfaces.calIImporter);
-                    var items = importer.importFromStream(inputStream, {});
+                    // XXX support csv
+                    let importer = Cc["@mozilla.org/calendar/import;1?type=ics"]
+                                   .getService(Ci.calIImporter);
+                    let items = importer.importFromStream(inputStream, {});
                     this.onDropItems(items);
                 }
                 finally {
@@ -279,26 +289,26 @@ calDNDBaseObserver.prototype = {
                 }
 
                 break;
+            }
             case "application/x-moz-file-promise":
-            case "text/x-moz-url":
-                var uri = Services.io.newURI(data.toString(), null, null);
-                var loader = Components.classes["@mozilla.org/network/unichar-stream-loader;1"]
-                             .createInstance(Components.interfaces.nsIUnicharStreamLoader);
-                var channel = Services.io.newChannelFromURI2(uri,
+            case "text/x-moz-url": {
+                let uri = Services.io.newURI(data.toString());
+                let loader = Cc["@mozilla.org/network/unichar-stream-loader;1"]
+                             .createInstance(Ci.nsIUnicharStreamLoader);
+                let channel = Services.io.newChannelFromURI2(uri,
                                                              null,
                                                              Services.scriptSecurityManager.getSystemPrincipal(),
                                                              null,
-                                                             Components.interfaces.nsILoadInfo.SEC_NORMAL,
-                                                             Components.interfaces.nsIContentPolicy.TYPE_OTHER);
-                channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE;
+                                                             Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+                                                             Ci.nsIContentPolicy.TYPE_OTHER);
+                channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
 
-                var self = this;
-
-                var listener = {
+                let self = this;
+                let listener = {
 
                     // nsIUnicharStreamLoaderObserver:
                     onDetermineCharset: function(loader, context, firstSegment, length) {
-                        var charset = null;
+                        let charset = null;
                         if (loader && loader.channel) {
                             charset = channel.contentCharset;
                         }
@@ -309,46 +319,48 @@ calDNDBaseObserver.prototype = {
                     },
 
                     onStreamComplete: function(loader, context, status, unicharString) {
-                        var parser = Components.classes["@mozilla.org/calendar/ics-parser;1"]
-                                     .createInstance(Components.interfaces.calIIcsParser);
+                        let parser = Cc["@mozilla.org/calendar/ics-parser;1"]
+                                     .createInstance(Ci.calIIcsParser);
                         parser.parseString(unicharString);
                         self.onDropItems(parser.getItems({}).concat(parser.getParentlessItems({})));
                     }
                 };
 
                 try {
-                    loader.init(listener, Components.interfaces.nsIUnicharStreamLoader.DEFAULT_SEGMENT_SIZE);
+                    loader.init(listener, Ci.nsIUnicharStreamLoader.DEFAULT_SEGMENT_SIZE);
                     channel.asyncOpen(loader, null);
                 } catch(e) {
-                    Components.utils.reportError(e)
+                    cal.ERROR(e);
                 }
                 break;
+            }
             case "text/x-moz-message":
                 this.onDropMessage(messenger.msgHdrFromURI(data));
                 break;
             default:
-                ASSERT(false, "unknown data flavour:" + bestFlavor.value+'\n');
+                cal.ASSERT(false, "unknown data flavour:" + bestFlavor.value+"\n");
                 break;
         }
     },
 
-    onDragStart: function calDNDStart(aEvent, aTransferData, aDragAction) {},
-    onDragOver: function calDNDOver(aEvent, aFlavor, aDragSession) {},
-    onDragExit: function calDNDExit(aEvent, aDragSession) {},
+    onDragStart: function(aEvent, aTransferData, aDragAction) {},
+    onDragOver: function(aEvent, aFlavor, aDragSession) {},
+    onDragExit: function(aEvent, aDragSession) {},
 
-    onDropItems: function calDNDDropItems(aItems) {},
-    onDropMessage: function calDNDDropMessage(aMessage) {},
+    onDropItems: function(aItems) {},
+    onDropMessage: function(aMessage) {},
 
 
-    retrieveURLFromData: function calDNDRetrieveURL(aData, aFlavor) {
-        var data;
+    retrieveURLFromData: function(aData, aFlavor) {
         switch (aFlavor) {
-            case "text/unicode":
-                data = aData.toString();
-                var separator = data.indexOf("\n");
-                if (separator != -1)
+            case "text/unicode": {
+                let data = aData.toString();
+                let separator = data.indexOf("\n");
+                if (separator != -1) {
                     data = data.substr(0, separator);
+                }
                 return data;
+            }
             case "application/x-moz-file":
                 return aData.URL;
             default:
@@ -379,14 +391,16 @@ calViewDNDObserver.prototype = {
      * try to add these items to the currently selected calendar.
      */
     onDropItems: function(aItems) {
-        var destCal = getSelectedCalendar();
+        let destCal = getSelectedCalendar();
         startBatchTransaction();
+        // we fall back explicitely to the popup to ask whether to send a
+        // notification to partticipants if required
+        let extResp = { responseMode: Ci.calIItipItem.USER };
         try {
-            for each (var item in aItems) {
-                doTransaction('add', item, destCal, null, null);
+            for (let item of aItems) {
+                doTransaction("add", item, destCal, null, null, extResp);
             }
-        }
-        finally {
+        } finally {
             endBatchTransaction();
         }
     }
@@ -417,9 +431,9 @@ calMailButtonDNDObserver.prototype = {
     onDropItems: function(aItems) {
         if (aItems && aItems.length > 0) {
             let item = aItems[0];
-            let recipients = cal.getRecipientList(item.getAttendees({}));
+            let recipients = cal.email.createRecipientList(item.getAttendees({}));
             let identity = item.calendar.getProperty("imip.identity");
-            sendMailTo(recipients, item.title, item.getProperty("DESCRIPTION"), identity);
+            cal.email.sendTo(recipients, item.title, item.getProperty("DESCRIPTION"), identity);
         }
     },
 
@@ -458,9 +472,9 @@ calCalendarButtonDNDObserver.prototype = {
      * @param aItems        An array of items to handle.
      */
     onDropItems: function(aItems) {
-        for each (var item in aItems) {
-            var newItem = item;
-            if (isToDo(item)) {
+        for (let item of aItems) {
+            let newItem = item;
+            if (cal.item.isToDo(item)) {
                 newItem = itemConversion.eventFromTask(item);
             }
             createEventWithDialog(null, null, null, null, newItem);
@@ -478,7 +492,7 @@ calCalendarButtonDNDObserver.prototype = {
      * @param aMessage     The message to handle.
      */
     onDropMessage: function(aMessage) {
-        var newItem = createEvent();
+        let newItem = cal.createEvent();
         itemConversion.calendarItemFromMessage(newItem, aMessage);
         createEventWithDialog(null, null, null, null, newItem);
     }
@@ -507,9 +521,9 @@ calTaskButtonDNDObserver.prototype = {
      * @param aItems        An array of items to handle.
      */
     onDropItems: function(aItems) {
-        for each (var item in aItems) {
-            var newItem = item;
-            if (isEvent(item)) {
+        for (let item of aItems) {
+            let newItem = item;
+            if (cal.item.isEvent(item)) {
                 newItem = itemConversion.taskFromEvent(item);
             }
             createTodoWithDialog(null, null, null, newItem);
@@ -525,7 +539,7 @@ calTaskButtonDNDObserver.prototype = {
      * @param aMessage     The message to handle.
      */
     onDropMessage: function(aMessage) {
-        var todo = createTodo();
+        let todo = cal.createTodo();
         itemConversion.calendarItemFromMessage(todo, aMessage);
         createTodoWithDialog(null, null, null, todo);
     }
@@ -539,13 +553,13 @@ calTaskButtonDNDObserver.prototype = {
  * @param aXULBox   The XUL box to invoke the drag session from.
  */
 function invokeEventDragSession(aItem, aXULBox) {
-    let transfer = Components.classes["@mozilla.org/widget/transferable;1"]
-                   .createInstance(Components.interfaces.nsITransferable);
+    let transfer = Cc["@mozilla.org/widget/transferable;1"]
+                   .createInstance(Ci.nsITransferable);
     transfer.init(null);
     transfer.addDataFlavor("text/calendar");
 
     let flavourProvider = {
-        QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIFlavorDataProvider]),
+        QueryInterface: XPCOMUtils.generateQI([Ci.nsIFlavorDataProvider]),
 
         item: aItem,
         getFlavorData: function(aInTransferable, aInFlavor, aOutData, aOutDataLen) {
@@ -554,37 +568,37 @@ function invokeEventDragSession(aItem, aXULBox) {
                 aOutData.value = aItem;
                 aOutDataLen.value = 1;
             } else {
-                ASSERT(false, "error:" + aInFlavor);
+                cal.ASSERT(false, "error:" + aInFlavor);
             }
         }
     };
 
-    if (isEvent(aItem)) {
-      transfer.addDataFlavor("application/vnd.x-moz-cal-event");
-      transfer.setTransferData("application/vnd.x-moz-cal-event", flavourProvider, 0);
-    } else if (isToDo(aItem)) {
-      transfer.addDataFlavor("application/vnd.x-moz-cal-task");
-      transfer.setTransferData("application/vnd.x-moz-cal-task", flavourProvider, 0);
+    if (cal.item.isEvent(aItem)) {
+        transfer.addDataFlavor("application/vnd.x-moz-cal-event");
+        transfer.setTransferData("application/vnd.x-moz-cal-event", flavourProvider, 0);
+    } else if (cal.item.isToDo(aItem)) {
+        transfer.addDataFlavor("application/vnd.x-moz-cal-task");
+        transfer.setTransferData("application/vnd.x-moz-cal-task", flavourProvider, 0);
     }
 
     // Also set some normal data-types, in case we drag into another app
-    let serializer = Components.classes["@mozilla.org/calendar/ics-serializer;1"]
-                               .createInstance(Components.interfaces.calIIcsSerializer);
+    let serializer = Cc["@mozilla.org/calendar/ics-serializer;1"]
+                     .createInstance(Ci.calIIcsSerializer);
     serializer.addItems([aItem], 1);
 
-    let supportsString = Components.classes["@mozilla.org/supports-string;1"]
-                         .createInstance(Components.interfaces.nsISupportsString);
+    let supportsString = Cc["@mozilla.org/supports-string;1"]
+                         .createInstance(Ci.nsISupportsString);
     supportsString.data = serializer.serializeToString();
     transfer.setTransferData("text/calendar", supportsString, supportsString.data.length * 2);
     transfer.setTransferData("text/unicode", supportsString, supportsString.data.length * 2);
 
-    let action = Components.interfaces.nsIDragService.DRAGDROP_ACTION_MOVE;
-    let supArray = Components.classes["@mozilla.org/supports-array;1"]
-                   .createInstance(Components.interfaces.nsISupportsArray);
-    supArray.AppendElement(transfer);
+    let action = Ci.nsIDragService.DRAGDROP_ACTION_MOVE;
+    let mutArray = Cc["@mozilla.org/array;1"]
+                   .createInstance(Ci.nsIMutableArray);
+    mutArray.appendElement(transfer);
     aXULBox.sourceObject = aItem;
     try {
-        cal.getDragService().invokeDragSession(aXULBox, supArray, null, action);
+        cal.getDragService().invokeDragSession(aXULBox, mutArray, null, action);
     } catch (error) {
         // Nothing done here because we only have to catch an exception that occurs when dragging
         // is cancelled with ESC. This is an odd behaviour of the nativeDragService which we have
