@@ -2,25 +2,24 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const {classes: Cc, interfaces: Ci, utils: Cu, results: Cr} = Components;
-Cu.import("resource:///modules/imXPCOMUtils.jsm");
-Cu.import("resource:///modules/imServices.jsm");
-Cu.import("resource:///modules/jsProtoHelper.jsm");
+ChromeUtils.import("resource:///modules/imXPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/imServices.jsm");
+ChromeUtils.import("resource:///modules/jsProtoHelper.jsm");
 
-const kPrefAutologinPending = "messenger.accounts.autoLoginPending";
-const kPrefMessengerAccounts = "messenger.accounts";
-const kPrefAccountPrefix = "messenger.account.";
-const kAccountKeyPrefix = "account";
-const kAccountOptionPrefPrefix = "options.";
-const kPrefAccountName = "name";
-const kPrefAccountPrpl = "prpl";
-const kPrefAccountAutoLogin = "autoLogin";
-const kPrefAccountAutoJoin = "autoJoin";
-const kPrefAccountAlias = "alias";
-const kPrefAccountFirstConnectionState = "firstConnectionState";
+var kPrefAutologinPending = "messenger.accounts.autoLoginPending";
+var kPrefMessengerAccounts = "messenger.accounts";
+var kPrefAccountPrefix = "messenger.account.";
+var kAccountKeyPrefix = "account";
+var kAccountOptionPrefPrefix = "options.";
+var kPrefAccountName = "name";
+var kPrefAccountPrpl = "prpl";
+var kPrefAccountAutoLogin = "autoLogin";
+var kPrefAccountAutoJoin = "autoJoin";
+var kPrefAccountAlias = "alias";
+var kPrefAccountFirstConnectionState = "firstConnectionState";
 
-const kPrefConvertOldPasswords = "messenger.accounts.convertOldPasswords";
-const kPrefAccountPassword = "password";
+var kPrefConvertOldPasswords = "messenger.accounts.convertOldPasswords";
+var kPrefAccountPassword = "password";
 
 XPCOMUtils.defineLazyGetter(this, "_", () =>
   l10nHelper("chrome://chat/locale/accounts.properties")
@@ -241,7 +240,7 @@ imAccount.prototype = {
         let autojoin = this.prefBranch.getComplexValue(
           kPrefAccountAutoJoin, Ci.nsISupportsString).data;
         if (autojoin) {
-          for each (let room in autojoin.trim().split(/,\s*/)) {
+          for (let room of autojoin.trim().split(/,\s*/)) {
             if (room)
               this.joinChat(this.getChatRoomDefaultFieldValues(room));
           }
@@ -382,7 +381,7 @@ imAccount.prototype = {
     }
 
     /* If the last successful connection is older than 10 seconds, reset the
-       number of reconnection attemps. */
+       number of reconnection attempts. */
     const kTimeBeforeSuccessfulConnection = 10;
     if (this.timeOfLastConnect &&
         this.timeOfLastConnect + kTimeBeforeSuccessfulConnection * 1000 < Date.now()) {
@@ -517,7 +516,7 @@ imAccount.prototype = {
       return "";
     }
     let normalizedName = this.normalizedName;
-    for each (let login in logins) {
+    for (let login of logins) {
       if (login.username == normalizedName) {
         this._password = login.password;
         if (this._connectionErrorReason == Ci.imIAccount.ERROR_MISSING_PASSWORD) {
@@ -555,7 +554,7 @@ imAccount.prototype = {
     try {
       let logins = Services.logins.findLogins({}, passwordURI, null, passwordURI);
       let saved = false;
-      for each (let login in logins) {
+      for (let login of logins) {
         if (newLogin.matches(login, true)) {
           if (aPassword)
             Services.logins.modifyLogin(login, newLogin);
@@ -580,7 +579,7 @@ imAccount.prototype = {
     this._sendUpdateNotification();
   },
   _handleMasterPasswordException: function(aException) {
-    if (aException.result != Components.results.NS_ERROR_ABORT)
+    if (aException.result != Cr.NS_ERROR_ABORT)
       throw aException;
 
     gUserCanceledMasterPasswordPrompt = true;
@@ -633,7 +632,7 @@ imAccount.prototype = {
     // protocol plugin is missing.
     login.init(passwordURI, null, passwordURI, this.normalizedName, "", "", "");
     let logins = Services.logins.findLogins({}, passwordURI, null, passwordURI);
-    for each (let l in logins) {
+    for (let l of logins) {
       if (login.matches(l, true)) {
         Services.logins.removeLogin(l);
         break;
@@ -843,7 +842,7 @@ AccountsService.prototype = {
     gConvertingOldPasswords =
       Services.prefs.getBoolPref(kPrefConvertOldPasswords);
     let accountList = this._accountList;
-    for each (let account in (accountList ? accountList.split(",") : [])) {
+    for (let account of (accountList ? accountList.split(",") : [])) {
       try {
         account.trim();
         if (!account)
@@ -861,7 +860,7 @@ AccountsService.prototype = {
       Services.prefs.setBoolPref(kPrefConvertOldPasswords, false);
 
     this._prefObserver = this.observe.bind(this);
-    Services.prefs.addObserver(kPrefMessengerAccounts, this._prefObserver, false);
+    Services.prefs.addObserver(kPrefMessengerAccounts, this._prefObserver);
   },
 
   _observingAccountListChange: true,
@@ -872,13 +871,13 @@ AccountsService.prototype = {
       return;
 
     this._accounts =
-      this._accountList.split(",").map(String.trim)
+      this._accountList.split(",").map(account => account.trim())
           .filter(k => k.startsWith(kAccountKeyPrefix))
           .map(k => parseInt(k.substr(kAccountKeyPrefix.length)))
           .map(this.getAccountByNumericId, this)
           .filter(a => a);
 
-    Services.obs.notifyObservers(this, "account-list-updated", null);
+    Services.obs.notifyObservers(this, "account-list-updated");
   },
 
   get _accountList() { return Services.prefs.getCharPref(kPrefMessengerAccounts); },
@@ -889,7 +888,7 @@ AccountsService.prototype = {
   },
 
   unInitAccounts: function() {
-    for each (let account in this._accounts)
+    for (let account of this._accounts)
       account.unInit();
     gAccountsService = null;
     delete this._accounts;
@@ -939,7 +938,7 @@ AccountsService.prototype = {
       let lastCrashTime = 0;
 
       /* Locate the LastCrash file */
-      let lastCrash = Services.dirsvc.get("UAppData", Ci.nsILocalFile);
+      let lastCrash = Services.dirsvc.get("UAppData", Ci.nsIFile);
       lastCrash.append("Crash Reports");
       lastCrash.append("LastCrash");
       if (lastCrash.exists()) {
@@ -988,7 +987,10 @@ AccountsService.prototype = {
   },
 
   processAutoLogin: function() {
-    for each (let account in this._accounts)
+    if (!this._accounts)  // if we're already shutting down
+      return;
+
+    for (let account of this._accounts)
       account.checkAutoLogin();
 
     // Make sure autologin is now enabled, so that we don't display a
@@ -998,7 +1000,7 @@ AccountsService.prototype = {
 
     // Notify observers so that any message stating that autologin is
     // disabled can be removed
-    Services.obs.notifyObservers(this, "autologin-processed", null);
+    Services.obs.notifyObservers(this, "autologin-processed");
   },
 
   _checkingIfPasswordStillMissing: false,
@@ -1008,7 +1010,7 @@ AccountsService.prototype = {
       return;
 
     this._checkingIfPasswordStillMissing = true;
-    for each (let account in this._accounts)
+    for (let account of this._accounts)
       account._checkIfPasswordStillMissing();
     delete this._checkingIfPasswordStillMissing;
   },
@@ -1060,7 +1062,7 @@ AccountsService.prototype = {
     let list = this._accountList;
     this._accountList = list ? list + "," + key : key;
 
-    Services.obs.notifyObservers(account, "account-added", null);
+    Services.obs.notifyObservers(account, "account-added");
     return account;
   },
 
@@ -1077,7 +1079,7 @@ AccountsService.prototype = {
     account.remove();
     this._accounts.splice(index, 1);
     delete this._accountsById[id];
-    Services.obs.notifyObservers(account, "account-removed", null);
+    Services.obs.notifyObservers(account, "account-removed");
 
     /* Update the account list pref. */
     let list = this._accountList;
@@ -1091,4 +1093,4 @@ AccountsService.prototype = {
   contractID: "@mozilla.org/chat/accounts-service;1"
 };
 
-const NSGetFactory = XPCOMUtils.generateNSGetFactory([AccountsService]);
+var NSGetFactory = XPCOMUtils.generateNSGetFactory([AccountsService]);
