@@ -2,9 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const {classes: Cc, interfaces: Ci, utils: Cu, results: Cr} = Components;
-Cu.import("resource:///modules/imXPCOMUtils.jsm");
-Cu.import("resource:///modules/imServices.jsm");
+ChromeUtils.import("resource:///modules/imXPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/imServices.jsm");
 
 XPCOMUtils.defineLazyGetter(this, "_", () =>
   l10nHelper("chrome://chat/locale/contacts.properties")
@@ -31,10 +30,14 @@ function getDBConnection()
   // Grow blist db in 512KB increments.
   try {
     conn.setGrowthIncrement(512 * 1024, "");
-  } catch (e if e.result == Cr.NS_ERROR_FILE_TOO_BIG) {
-    Services.console.logStringMessage("Not setting growth increment on " +
-                                      "blist.sqlite because the available " +
-                                      "disk space is limited");
+  } catch (e) {
+    if (e.result == Cr.NS_ERROR_FILE_TOO_BIG) {
+      Services.console.logStringMessage("Not setting growth increment on " +
+                                        "blist.sqlite because the available " +
+                                        "disk space is limited");
+    } else {
+      throw e;
+    }
   }
 
   // Create tables and indexes.
@@ -89,7 +92,7 @@ function getDBConnection()
 }
 
 // Wrap all the usage of DBConn inside a transaction that will be
-// commited automatically at the end of the event loop spin so that
+// committed automatically at the end of the event loop spin so that
 // we flush buddy list data to disk only once per event loop spin.
 var gDBConnWithPendingTransaction = null;
 this.__defineGetter__("DBConn", function() {
@@ -104,7 +107,7 @@ this.__defineGetter__("DBConn", function() {
         gDBConnection.asyncClose();
         gDBConnection = null;
       }
-    }, "profile-before-change", false);
+    }, "profile-before-change");
   }
   gDBConnWithPendingTransaction = gDBConnection;
   gDBConnection.beginTransaction();
@@ -222,14 +225,14 @@ Tag.prototype = {
   },
 
   addObserver: function(aObserver) {
-    if (this._observers.indexOf(aObserver) == -1)
+    if (!this._observers.includes(aObserver))
       this._observers.push(aObserver);
   },
   removeObserver: function(aObserver) {
     this._observers = this._observers.filter(o => o !== aObserver);
   },
   notifyObservers: function(aSubject, aTopic, aData) {
-    for each (let observer in this._observers)
+    for (let observer of this._observers)
       observer.observe(aSubject, aTopic, aData);
   },
 
@@ -251,17 +254,18 @@ var otherContactsTag = {
   _contactsInitialized: false,
   _saveHiddenTagsPref: function() {
     Services.prefs.setCharPref(this.hiddenTagsPref,
-                               [id for (id in this._hiddenTags)].join(","));
+                               Object.keys(this._hiddenTags).join(","));
   },
   showTag: function(aTag) {
     let id = aTag.id;
     delete this._hiddenTags[id];
-    for each (let contact in this._contacts)
+    let contacts = Object.keys(this._contacts).map(id => this._contacts[id]);
+    for (let contact of contacts)
       if (contact.getTags().some(t => t.id == id))
         this._removeContact(contact);
 
-    aTag.notifyObservers(aTag, "tag-shown", null);
-    Services.obs.notifyObservers(aTag, "tag-shown", null);
+    aTag.notifyObservers(aTag, "tag-shown");
+    Services.obs.notifyObservers(aTag, "tag-shown");
     this._saveHiddenTagsPref();
   },
   hideTag: function(aTag) {
@@ -272,12 +276,12 @@ var otherContactsTag = {
     if (this._contactsInitialized)
       this._hideTag(aTag);
 
-    aTag.notifyObservers(aTag, "tag-hidden", null);
-    Services.obs.notifyObservers(aTag, "tag-hidden", null);
+    aTag.notifyObservers(aTag, "tag-hidden");
+    Services.obs.notifyObservers(aTag, "tag-hidden");
     this._saveHiddenTagsPref();
   },
   _hideTag: function(aTag) {
-    for each (let contact in aTag.getContacts())
+    for (let contact of aTag.getContacts())
       if (!(contact.id in this._contacts) &&
           contact.getTags().every(t => t.id in this._hiddenTags))
         this._addContact(contact);
@@ -301,7 +305,7 @@ var otherContactsTag = {
     let pref = Services.prefs.getCharPref(this.hiddenTagsPref);
     if (!pref)
       return;
-    for each (let tagId in pref.split(","))
+    for (let tagId of pref.split(","))
       this._hiddenTags[tagId] = TagsById[tagId];
   },
   _initContacts: function() {
@@ -319,12 +323,14 @@ var otherContactsTag = {
     };
     this._contacts = {};
     this._contactsInitialized = true;
-    for each (let tag in this._hiddenTags)
+    for (let id in this._hiddenTags) {
+      let tag = this._hiddenTags[id];
       this._hideTag(tag);
-    Services.obs.addObserver(this, "contact-tag-added", false);
-    Services.obs.addObserver(this, "contact-tag-removed", false);
-    Services.obs.addObserver(this, "contact-added", false);
-    Services.obs.addObserver(this, "contact-removed", false);
+    }
+    Services.obs.addObserver(this, "contact-tag-added");
+    Services.obs.addObserver(this, "contact-tag-removed");
+    Services.obs.addObserver(this, "contact-added");
+    Services.obs.addObserver(this, "contact-removed");
   },
 
   // imITag implementation
@@ -332,7 +338,7 @@ var otherContactsTag = {
   get name() { return "__others__"; },
   set name(aNewName) { throw Cr.NS_ERROR_NOT_AVAILABLE; },
   getContacts: function(aContactCount) {
-    let contacts = [contact for each (contact in this._contacts)];
+    let contacts = Object.keys(this._contacts).map(id => this._contacts[id]);
     if (aContactCount)
       aContactCount.value = contacts.length;
     return contacts;
@@ -340,7 +346,7 @@ var otherContactsTag = {
   _addContact: function(aContact) {
     this._contacts[aContact.id] = aContact;
     this.notifyObservers(aContact, "contact-moved-in");
-    for each (let observer in ContactsById[aContact.id]._observers)
+    for (let observer of ContactsById[aContact.id]._observers)
       observer.observe(this, "contact-moved-in", null);
     aContact.addObserver(this._observer);
   },
@@ -348,19 +354,19 @@ var otherContactsTag = {
     delete this._contacts[aContact.id];
     aContact.removeObserver(this._observer);
     this.notifyObservers(aContact, "contact-moved-out");
-    for each (let observer in ContactsById[aContact.id]._observers)
+    for (let observer of ContactsById[aContact.id]._observers)
       observer.observe(this, "contact-moved-out", null);
   },
 
   addObserver: function(aObserver) {
-    if (this._observers.indexOf(aObserver) == -1)
+    if (!this._observers.includes(aObserver))
       this._observers.push(aObserver);
   },
   removeObserver: function(aObserver) {
     this._observers = this._observers.filter(o => o !== aObserver);
   },
   notifyObservers: function(aSubject, aTopic, aData) {
-    for each (let observer in this._observers)
+    for (let observer of this._observers)
       observer.observe(aSubject, aTopic, aData);
   },
 
@@ -403,9 +409,10 @@ Contact.prototype = {
     let oldDisplayName = this.displayName;
     this._alias = aNewAlias;
     this._notifyObservers("display-name-changed", oldDisplayName);
-    for each (let buddy in this._buddies)
-      for each (let accountBuddy in buddy._accounts)
+    for (let buddy of this._buddies) {
+      for (let accountBuddy of buddy._accounts)
         accountBuddy.serverAlias = aNewAlias;
+    }
     return aNewAlias;
   },
   _ensureNotDummy: function() {
@@ -455,7 +462,7 @@ Contact.prototype = {
     aTag._addContact(this);
 
     aTag.notifyObservers(this, "contact-moved-in");
-    for each (let observer in this._observers)
+    for (let observer of this._observers)
       observer.observe(aTag, "contact-moved-in", null);
     Services.obs.notifyObservers(this, "contact-tag-added", aTag.id);
   },
@@ -471,7 +478,7 @@ Contact.prototype = {
     aTag._removeContact(this);
 
     aTag.notifyObservers(this, "contact-moved-out");
-    for each (let observer in this._observers)
+    for (let observer of this._observers)
       observer.observe(aTag, "contact-moved-out", null);
     Services.obs.notifyObservers(this, "contact-tag-removed", aTag.id);
   },
@@ -529,10 +536,11 @@ Contact.prototype = {
     }
   },
   _isTagInherited: function(aTag) {
-    for each (let buddy in this._buddies)
-      for each (let accountBuddy in buddy._accounts)
+    for (let buddy of this._buddies) {
+      for (let accountBuddy of buddy._accounts)
         if (accountBuddy.tag.id == aTag.id)
           return true;
+    }
     return false;
   },
   _moved: function(aOldTag, aNewTag) {
@@ -566,17 +574,17 @@ Contact.prototype = {
     // Finally, notify of the changes.
     if (shouldRemove) {
       aOldTag.notifyObservers(this, "contact-moved-out");
-      for each (let observer in this._observers)
+      for (let observer of this._observers)
         observer.observe(aOldTag, "contact-moved-out", null);
       Services.obs.notifyObservers(this, "contact-tag-removed", aOldTag.id);
     }
     if (shouldAdd) {
       aNewTag.notifyObservers(this, "contact-moved-in");
-      for each (let observer in this._observers)
+      for (let observer of this._observers)
         observer.observe(aNewTag, "contact-moved-in", null);
       Services.obs.notifyObservers(this, "contact-tag-added", aNewTag.id);
     }
-    Services.obs.notifyObservers(this, "contact-moved", null);
+    Services.obs.notifyObservers(this, "contact-moved");
   },
 
   getBuddies: function(aBuddyCount) {
@@ -593,18 +601,18 @@ Contact.prototype = {
     // Avoid merging the contact with itself or merging into an
     // already removed contact.
     if (aContact.id == this.id || !(this.id in ContactsById))
-      throw Components.results.NS_ERROR_INVALID_ARG;
+      throw Cr.NS_ERROR_INVALID_ARG;
 
     this._ensureNotDummy();
     let contact = ContactsById[aContact.id]; // remove XPConnect wrapper
 
     // Copy all the contact-only tags first, otherwise they would be lost.
-    for each (let tag in contact.getTags())
+    for (let tag of contact.getTags())
       if (!contact._isTagInherited(tag))
         this.addTag(tag);
 
     // Adopt each buddy. Removing the last one will delete the contact.
-    for each (let buddy in contact.getBuddies())
+    for (let buddy of contact.getBuddies())
       buddy.contact = this;
     this._updatePreferredBuddy();
   },
@@ -632,7 +640,7 @@ Contact.prototype = {
   },
   adoptBuddy: function(aBuddy) {
     if (aBuddy.contact.id == this.id)
-      throw Components.results.NS_ERROR_INVALID_ARG;
+      throw Cr.NS_ERROR_INVALID_ARG;
 
     let buddy = BuddiesById[aBuddy.id]; // remove XPConnect wrapper
     buddy.contact = this;
@@ -650,7 +658,7 @@ Contact.prototype = {
       this._notifyObservers("removed");
       delete ContactsById[this._id];
 
-      for each (let tag in this._tags)
+      for (let tag of this._tags)
         tag._removeContact(this);
       let statement =
         DBConn.createStatement("DELETE FROM contact_tag WHERE contact_id = :id");
@@ -702,9 +710,9 @@ Contact.prototype = {
     // Should return a new contact with the same list of tags.
     let buddy = BuddiesById[aBuddy.id];
     if (buddy.contact.id != this.id)
-      throw Components.results.NS_ERROR_INVALID_ARG;
+      throw Cr.NS_ERROR_INVALID_ARG;
     if (buddy.contact._buddies.length == 1)
-      throw Components.results.NS_ERROR_UNEXPECTED;
+      throw Cr.NS_ERROR_UNEXPECTED;
 
     // Save the list of tags, it may be destoyed if the buddy was the last one.
     let tags = buddy.contact.getTags();
@@ -715,14 +723,14 @@ Contact.prototype = {
 
     // The first tag was inherited during the contact setter.
     // This will copy the remaining tags.
-    for each (let tag in tags)
+    for (let tag of tags)
       buddy.contact.addTag(tag);
 
     return buddy.contact;
   },
   remove: function() {
     this._massRemove = true;
-    for each (let buddy in this._buddies)
+    for (let buddy of this._buddies)
       buddy.remove();
   },
 
@@ -783,7 +791,7 @@ Contact.prototype = {
     let preferred;
     // |this._buddies| is ordered by user preference, so in case of
     // equal availability, keep the current value of |preferred|.
-    for each (let buddy in this._buddies) {
+    for (let buddy of this._buddies) {
       if (!preferred || preferred.statusType < buddy.statusType ||
           (preferred.statusType == buddy.statusType &&
            preferred.availabilityDetails < buddy.availabilityDetails))
@@ -842,7 +850,7 @@ Contact.prototype = {
   },
 
   addObserver: function(aObserver) {
-    if (this._observers.indexOf(aObserver) == -1)
+    if (!this._observers.includes(aObserver))
       this._observers.push(aObserver);
   },
   removeObserver: function(aObserver) {
@@ -853,10 +861,10 @@ Contact.prototype = {
   },
   // internal calls + calls from add-ons
   notifyObservers: function(aSubject, aTopic, aData) {
-    for each (let observer in this._observers)
+    for (let observer of this._observers)
       if ("observe" in observer) // avoid failing on destructed XBL bindings...
         observer.observe(aSubject, aTopic, aData);
-    for each (let tag in this._tags)
+    for (let tag of this._tags)
       tag.notifyObservers(aSubject, aTopic, aData);
     Services.obs.notifyObservers(aSubject, aTopic, aData);
   },
@@ -931,7 +939,7 @@ function Buddy(aId, aKey, aName, aSrvAlias, aContactId) {
 Buddy.prototype = {
   get id() { return this._id; },
   destroy: function() {
-    for each (let ab in this._accounts)
+    for (let ab of this._accounts)
       ab.unInit();
     delete this._accounts;
     delete this._observers;
@@ -945,7 +953,7 @@ Buddy.prototype = {
   get contact() { return this._contact; },
   set contact(aContact) /* not in imIBuddy */ {
     if (aContact.id == this._contact.id)
-      throw Components.results.NS_ERROR_INVALID_ARG;
+      throw Cr.NS_ERROR_INVALID_ARG;
 
     this._notifyObservers("moved-out-of-contact");
     this._contact._removeBuddy(this);
@@ -954,7 +962,7 @@ Buddy.prototype = {
     this._contact._buddies.push(this);
 
     // Ensure all the inherited tags are in the new contact.
-    for each (let accountBuddy in this._accounts)
+    for (let accountBuddy of this._accounts)
       this._contact.addTag(TagsById[accountBuddy.tag.id], true);
 
     let statement =
@@ -970,7 +978,7 @@ Buddy.prototype = {
     return aContact;
   },
   _hasAccountBuddy: function(aAccountId, aTagId) {
-    for each (let ab in this._accounts) {
+    for (let ab of this._accounts) {
       if (ab.account.numericId == aAccountId && ab.tag.id == aTagId)
         return true;
     }
@@ -985,7 +993,7 @@ Buddy.prototype = {
   _addAccount: function(aAccountBuddy, aTag) {
     this._accounts.push(aAccountBuddy);
     let contact = this._contact;
-    if (this._contact._tags.indexOf(aTag) == -1) {
+    if (!this._contact._tags.includes(aTag)) {
       this._contact._tags.push(aTag);
       aTag._addContact(contact);
     }
@@ -996,7 +1004,7 @@ Buddy.prototype = {
   get _empty() { return this._accounts.length == 0; },
 
   remove: function() {
-    for each (let account in this._accounts)
+    for (let account of this._accounts)
       account.remove();
   },
 
@@ -1054,7 +1062,7 @@ Buddy.prototype = {
 
     let preferred;
     //TODO take into account the order of the account-manager list.
-    for each (let account in this._accounts) {
+    for (let account of this._accounts) {
       if (!preferred || preferred.statusType < account.statusType ||
           (preferred.statusType == account.statusType &&
            preferred.availabilityDetails < account.availabilityDetails))
@@ -1117,7 +1125,7 @@ Buddy.prototype = {
   createConversation: function() { return this._preferredAccount.createConversation(); },
 
   addObserver: function(aObserver) {
-    if (this._observers.indexOf(aObserver) == -1)
+    if (!this._observers.includes(aObserver))
       this._observers.push(aObserver);
   },
   removeObserver: function(aObserver) {
@@ -1128,7 +1136,7 @@ Buddy.prototype = {
   // internal calls + calls from add-ons
   notifyObservers: function(aSubject, aTopic, aData) {
     try {
-      for each (let observer in this._observers)
+      for (let observer of this._observers)
         observer.observe(aSubject, aTopic, aData);
       this._contact._observe(aSubject, aTopic, aData);
     } catch (e) {
@@ -1311,8 +1319,10 @@ ContactsService.prototype = {
     TagsById = { };
     // Avoid shutdown leaks caused by references to native components
     // implementing prplIAccountBuddy.
-    for each (let buddy in BuddiesById)
+    for (let buddyId in BuddiesById) {
+      let buddy = BuddiesById[buddyId];
       buddy.destroy();
+    }
     BuddiesById = { };
     ContactsById = { };
   },
@@ -1320,7 +1330,9 @@ ContactsService.prototype = {
   getContactById: aId => ContactsById[aId],
   // Get an array of all existing contacts.
   getContacts: function(aContactCount) {
-    let contacts = [ContactsById[id] for (id in ContactsById) if (!ContactsById[id]._empty)];
+    let contacts = Object.keys(ContactsById)
+                         .filter(id => !ContactsById[id]._empty)
+                         .map(id => ContactsById[id]);
     if (aContactCount)
       aContactCount.value = contacts.length;
     return contacts;
@@ -1502,7 +1514,7 @@ ContactsService.prototype = {
       statement.finalize();
     }
 
-    // removing the account from the accounts table is not enought,
+    // removing the account from the accounts table is not enough,
     // we need to remove all the associated account_buddy entries too
     statement = DBConn.createStatement("DELETE FROM account_buddy " +
                                        "WHERE account_id = :accountId");
@@ -1520,5 +1532,5 @@ ContactsService.prototype = {
   contractID: "@mozilla.org/chat/contacts-service;1"
 };
 
-const NSGetFactory = XPCOMUtils.generateNSGetFactory([ContactsService,
+var NSGetFactory = XPCOMUtils.generateNSGetFactory([ContactsService,
                                                       TagsService]);
