@@ -2,28 +2,27 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const {classes: Cc, interfaces: Ci, utils: Cu, Constructor: CC} = Components;
+ChromeUtils.import("resource:///modules/hiddenWindow.jsm");
+ChromeUtils.import("resource:///modules/imServices.jsm");
+ChromeUtils.import("resource:///modules/imXPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/jsProtoHelper.jsm");
+ChromeUtils.import("resource:///modules/ToLocaleFormat.jsm");
 
-Cu.import("resource:///modules/hiddenWindow.jsm");
-Cu.import("resource:///modules/imServices.jsm");
-Cu.import("resource:///modules/imXPCOMUtils.jsm");
-Cu.import("resource:///modules/jsProtoHelper.jsm");
-
-Cu.import("resource://gre/modules/Task.jsm")
-XPCOMUtils.defineLazyModuleGetter(this, "OS", "resource://gre/modules/osfile.jsm");
+ChromeUtils.import("resource://gre/modules/Task.jsm")
+ChromeUtils.defineModuleGetter(this, "OS", "resource://gre/modules/osfile.jsm");
 
 XPCOMUtils.defineLazyGetter(this, "_", () =>
   l10nHelper("chrome://chat/locale/logger.properties")
 );
 
-const kLineBreak = "@mozilla.org/windows-registry-key;1" in Cc ? "\r\n" : "\n";
+var kLineBreak = "@mozilla.org/windows-registry-key;1" in Cc ? "\r\n" : "\n";
 
 /*
  * Maps file paths to promises returned by ongoing OS.File operations on them.
  * This is so that a file can be read after a pending write operation completes
  * and vice versa (opening a file multiple times concurrently may fail on Windows).
  */
-let gFilePromises = new Map();
+var gFilePromises = new Map();
 
 // Uses above map to queue operations on a file.
 function queueFileOperation(aPath, aOperation) {
@@ -127,7 +126,7 @@ function getLogFilePathForConversation(aConv, aFormat, aStartTime) {
 
 function getNewLogFileName(aFormat, aStartTime) {
   let date = aStartTime ? new Date(aStartTime) : new Date();
-  let dateTime = date.toLocaleFormat("%Y-%m-%d.%H%M%S");
+  let dateTime = ToLocaleFormat("%Y-%m-%d.%H%M%S", date);
   let offset = date.getTimezoneOffset();
   if (offset < 0) {
     dateTime += "+";
@@ -159,7 +158,7 @@ LogWriter.prototype = {
   // All log file paths used by this LogWriter.
   paths: [],
   // Path of the log file that is currently being written to.
-  get currentPath() this.paths[this.paths.length - 1],
+  get currentPath() { return this.paths[this.paths.length - 1]; },
   // Constructor sets this to a promise that will resolve when the log header
   // has been written.
   _initialized: null,
@@ -196,6 +195,8 @@ LogWriter.prototype = {
       header = JSON.stringify(header) + "\n";
     }
     else {
+      // - const dateTimeFormatter = Services.intl.createDateTimeFormat("en-US", {
+      // + onst dateTimeFormatter = new Services.intl.DateTimeFormat("en-US", {
       header = "Conversation with " + this._conv.name +
                " at " + (new Date(this._conv.startDate / 1000)).toLocaleString() +
                " on " + account.name +
@@ -215,8 +216,8 @@ LogWriter.prototype = {
     div.innerHTML = aString.replace(/\r?\n/g, "<br/>").replace(/<br>/gi, "<br/>");
     const type = "text/plain";
     let encoder =
-      Components.classes["@mozilla.org/layout/documentEncoder;1?type=" + type]
-                .createInstance(Components.interfaces.nsIDocumentEncoder);
+      Cc["@mozilla.org/layout/documentEncoder;1?type=" + type]
+        .createInstance(Ci.nsIDocumentEncoder);
     encoder.init(doc, type, 0);
     encoder.setContainerNode(div);
     encoder.setNodeFixup({fixupNode: function(aNode, aSerializeKids) {
@@ -301,14 +302,14 @@ LogWriter.prototype = {
   }
 };
 
-const dummyLogWriter = {
+var dummyLogWriter = {
   paths: null,
   currentPath: null,
   logMessage: function() {}
 };
 
 
-let gLogWritersById = new Map();
+var gLogWritersById = new Map();
 function getLogWriter(aConversation) {
   let id = aConversation.id;
   if (!gLogWritersById.has(id)) {
@@ -334,7 +335,7 @@ function SystemLogWriter(aAccount) {
   let header = "System log for account " + aAccount.name +
                " (" + aAccount.protocol.normalizedName +
                ") connected at " +
-               (new Date()).toLocaleFormat("%c") + kLineBreak;
+               ToLocaleFormat("%c", new Date()) + kLineBreak;
   this._initialized = appendToFile(this.path, this.encoder.encode(header), true);
   // Catch the error separately so that _initialized will stay rejected if
   // writing the header failed.
@@ -348,7 +349,7 @@ SystemLogWriter.prototype = {
   _initialized: null,
   path: null,
   logEvent: function sl_logEvent(aString) {
-    let date = (new Date()).toLocaleFormat("%x %X");
+    let date = ToLocaleFormat("%x %X", new Date());
     let lineToWrite =
       this.encoder.encode("---- " + aString + " @ " + date + " ----" + kLineBreak);
     this._initialized.then(() => {
@@ -358,13 +359,13 @@ SystemLogWriter.prototype = {
   }
 };
 
-const dummySystemLogWriter = {
+var dummySystemLogWriter = {
   path: null,
   logEvent: function() {}
 };
 
 
-let gSystemLogWritersById = new Map();
+var gSystemLogWritersById = new Map();
 function getSystemLogWriter(aAccount, aCreate) {
   let id = aAccount.id;
   if (aCreate) {
@@ -508,7 +509,7 @@ function Log(aEntries) {
   // Sort our list of entries for this day in increasing order.
   aEntries.sort((aLeft, aRight) => aLeft.time - aRight.time);
 
-  this._entryPaths = [entry.path for (entry of aEntries)];
+  this._entryPaths = aEntries.map(entry => entry.path);
   // Calculate the timestamp for the first entry down to the day.
   let timestamp = new Date(aEntries[0].time);
   timestamp.setHours(0);
@@ -810,6 +811,31 @@ Logger.prototype = {
     return this._getEnumerator(entries, aGroupByDay);
   }),
 
+  getLogFolderPathForAccount: function(aAccount) {
+    return getLogFolderPathForAccount(aAccount);
+  },
+
+  deleteLogFolderForAccount: function(aAccount) {
+    if (!aAccount.disconnecting && !aAccount.disconnected)
+      throw new Error("Account must be disconnected first before deleting logs.");
+
+    if (aAccount.disconnecting)
+      Cu.reportError("Account is still disconnecting while we attempt to remove logs.");
+
+    let logPath = this.getLogFolderPathForAccount(aAccount);
+    // Find all operations on files inside the log folder.
+    let pendingPromises = [];
+    function checkLogFiles(promiseOperation, filePath) {
+      if (filePath.startsWith(logPath))
+        pendingPromises.push(promiseOperation);
+    }
+    gFilePromises.forEach(checkLogFiles);
+    // After all operations finish, remove the whole log folder.
+    return Promise.all(pendingPromises)
+                  .then(values => { OS.File.removeDir(logPath, { ignoreAbsent: true }); })
+                  .catch(aError => Cu.reportError("Failed to remove log folders:\n" + aError));
+  },
+
   forEach: Task.async(function* (aCallback) {
     let getAllSubdirs = Task.async(function* (aPaths, aErrorMsg) {
       let entries = [];
@@ -857,14 +883,14 @@ Logger.prototype = {
   observe: function logger_observe(aSubject, aTopic, aData) {
     switch (aTopic) {
     case "profile-after-change":
-      Services.obs.addObserver(this, "final-ui-startup", false);
+      Services.obs.addObserver(this, "final-ui-startup");
       break;
     case "final-ui-startup":
       Services.obs.removeObserver(this, "final-ui-startup");
       ["new-text", "conversation-closed", "conversation-left-chat",
        "account-connected", "account-disconnected",
        "account-buddy-status-changed"].forEach(function(aEvent) {
-        Services.obs.addObserver(this, aEvent, false);
+        Services.obs.addObserver(this, aEvent);
       }, this);
       break;
     case "new-text":
@@ -917,4 +943,4 @@ Logger.prototype = {
   contractID: "@mozilla.org/chat/logger;1"
 };
 
-const NSGetFactory = XPCOMUtils.generateNSGetFactory([Logger]);
+var NSGetFactory = XPCOMUtils.generateNSGetFactory([Logger]);
