@@ -1,11 +1,11 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
-Components.utils.import("resource://gre/modules/Services.jsm");
-let irc = {};
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+var irc = {};
 Services.scriptloader.loadSubScript("resource:///components/irc.js", irc);
 
-const input = [
+var input = [
   undefined,
   "test",
   "\\test",
@@ -24,9 +24,7 @@ const input = [
   "\\\\atest"
 ];
 
-const expectedOutputCommand = "PRIVMSG";
-
-const expectedOutputParams = [
+var expectedOutputParams = [
   "ACTION",
   "ACTION test",
   "ACTION \\\\test",
@@ -45,20 +43,20 @@ const expectedOutputParams = [
   "ACTION \\\\\\\\atest"
 ];
 
-let outputParams = [];
+var outputParams = [];
 
 irc.ircAccount.prototype.sendMessage = function(aCommand, aParams) {
-  do_check_eq(expectedOutputCommand, aCommand);
+  equal("PRIVMSG", aCommand);
   outputParams.push(aParams[1]);
-}
+};
 
 function run_test() {
-  let output = input.map(aStr =>
+  input.map(aStr =>
     irc.ircAccount.prototype.sendCTCPMessage("", false, "ACTION", aStr));
 
   // Ensure both arrays have the same length.
-  do_check_eq(expectedOutputParams.length, outputParams.length);
+  equal(expectedOutputParams.length, outputParams.length);
   // Ensure the values in the arrays are equal.
   for (let i = 0; i < outputParams.length; ++i)
-    do_check_eq("\x01" + expectedOutputParams[i] + "\x01", outputParams[i]);
+    equal("\x01" + expectedOutputParams[i] + "\x01", outputParams[i]);
 }
