@@ -2,13 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var {classes: Cc, interfaces: Ci, results: Cr, utils: Cu} = Components;
-
-Cu.import("resource://gre/modules/Http.jsm");
-Cu.import("resource:///modules/imServices.jsm");
-Cu.import("resource:///modules/imXPCOMUtils.jsm");
-Cu.import("resource:///modules/jsProtoHelper.jsm");
-Cu.import("resource:///modules/twitter-text.jsm");
+ChromeUtils.import("resource://gre/modules/Http.jsm");
+ChromeUtils.import("resource:///modules/imServices.jsm");
+ChromeUtils.import("resource:///modules/imXPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/jsProtoHelper.jsm");
+ChromeUtils.import("resource:///modules/twitter-text.jsm");
 
 var NS_PREFBRANCH_PREFCHANGE_TOPIC_ID = "nsPref:changed";
 var kMaxMessageLength = 140;
@@ -154,7 +152,7 @@ var GenericTwitterConversation = {
     this.writeMessage("twitter.com", aMessage, flags);
   },
   onSentCallback: function(aMsg, aData) {
-    // The conversation may have been unitialized in the time it takes for
+    // The conversation may have been uninitialized in the time it takes for
     // the async callback to fire.  Use `_observers` as a proxy for uninit'd.
     if (!this._observers)
       return;
@@ -267,7 +265,7 @@ var GenericTwitterConversation = {
     let name = aUser.screen_name;
 
     let flags = name == this.nick ? {outgoing: true} : {incoming: true};
-    flags.time = Math.round(new Date(aTweet.created_at) / 1000);
+    flags.time = Math.floor(new Date(aTweet.created_at) / 1000);
     flags._iconURL = aUser.profile_image_url;
     if (aTweet.delayed)
       flags.delayed = true;
@@ -380,7 +378,7 @@ TimelineConversation.prototype = {
   sendTyping: function(aString) {
     if (aString.length == 0 && this.inReplyToStatusId) {
       delete this.inReplyToStatusId;
-      this.notifyObservers(null, "status-text-changed", "");
+      this.notifyObservers(null, "status-text-changed");
       return kMaxMessageLength;
     }
     return kMaxMessageLength - this.getTweetLength(aString);
@@ -794,7 +792,7 @@ Account.prototype = {
     this._streamingRequest.responseType = "moz-chunked-text";
     this._streamingRequest.onprogress = this.onDataAvailable.bind(this);
     this.resetStreamTimeout();
-    this.prefs.addObserver("track", this, false);
+    this.prefs.addObserver("track", this);
   },
   _streamTimeout: null,
   resetStreamTimeout: function() {
@@ -970,7 +968,7 @@ Account.prototype = {
       },
       QueryInterface: XPCOMUtils.generateQI([Ci.prplIRequestBrowser])
     };
-    Services.obs.notifyObservers(this._browserRequest, "browser-request", null);
+    Services.obs.notifyObservers(this._browserRequest, "browser-request");
   },
   finishAuthorizationRequest: function() {
     // Clean up the cookies, so that several twitter OAuth dialogs can work
@@ -1149,7 +1147,12 @@ Account.prototype = {
       },
       time_zone: null,
       protected: normalizeBool,
-      created_at: aDate => (new Date(aDate)).toLocaleDateString(),
+      created_at: function(aDate) {
+        const dateFormatter = new Services.intl.DateTimeFormat(undefined, {
+          dateStyle: "short"
+        });
+        return dateFormatter.format(new Date(aDate));
+      },
       statuses_count: null,
       friends_count: null,
       followers_count: null,
