@@ -3,28 +3,29 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource:///modules/editorUtilities.jsm");
+ChromeUtils.import("resource:///modules/editorUtilities.jsm");
+ChromeUtils.import("resource://services-common/async.js");
+ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
 
 /* Main Composer window UI control */
 
 var gComposerWindowControllerID = 0;
 var prefAuthorString = "";
 
-const kDisplayModeNormal = 0;
-const kDisplayModeAllTags = 1;
-const kDisplayModeSource = 2;
-const kDisplayModePreview = 3;
+var kDisplayModeNormal = 0;
+var kDisplayModeAllTags = 1;
+var kDisplayModeSource = 2;
+var kDisplayModePreview = 3;
+
 const kDisplayModeMenuIDs = ["viewNormalMode", "viewAllTagsMode", "viewSourceMode", "viewPreviewMode"];
 const kDisplayModeTabIDS = ["NormalModeButton", "TagModeButton", "SourceModeButton", "PreviewModeButton"];
 const kNormalStyleSheet = "chrome://editor/content/EditorContent.css";
 const kAllTagsStyleSheet = "chrome://editor/content/EditorAllTags.css";
 const kContentEditableStyleSheet = "resource://gre/res/contenteditable.css";
 
-const kTextMimeType = "text/plain";
-const kHTMLMimeType = "text/html";
-const kXHTMLMimeType = "application/xhtml+xml";
-
-const nsIWebNavigation = Components.interfaces.nsIWebNavigation;
+var kTextMimeType = "text/plain";
+var kHTMLMimeType = "text/html";
+var kXHTMLMimeType = "application/xhtml+xml";
 
 var gPreviousNonSourceDisplayMode = 1;
 var gEditorDisplayMode = -1;
@@ -55,11 +56,11 @@ var gLastFocusNodeWasSelected = false;
 // These must be kept in synch with the XUL <options> lists
 var gFontSizeNames = ["xx-small","x-small","small","medium","large","x-large","xx-large"];
 
-const nsIFilePicker = Components.interfaces.nsIFilePicker;
+var nsIFilePicker = Ci.nsIFilePicker;
 
-const kEditorToolbarPrefs = "editor.toolbars.showbutton.";
-const kUseCssPref         = "editor.use_css";
-const kCRInParagraphsPref = "editor.CR_creates_new_p";
+var kEditorToolbarPrefs = "editor.toolbars.showbutton.";
+var kUseCssPref         = "editor.use_css";
+var kCRInParagraphsPref = "editor.CR_creates_new_p";
 
 function ShowHideToolbarSeparators(toolbar) {
   // Make sure the toolbar actually exists.
@@ -110,7 +111,7 @@ nsPrefListener.prototype =
   {
     this.domain = prefName;
     try {
-      Services.prefs.addObserver(this.domain, this, false);
+      Services.prefs.addObserver(this.domain, this);
     } catch(ex) {
       dump("Failed to observe prefs: " + ex + "\n");
     }
@@ -186,8 +187,7 @@ const gSourceTextObserver =
   }
 };
 
-// This should be called by all editor users when they close their window
-//  or other similar "done with editor" actions, like recycling a Mail Composer window.
+// This should be called by all editor users when they close their window.
 function EditorCleanup()
 {
   SwitchInsertCharToAnotherEditorOrClose();
@@ -406,11 +406,11 @@ function EditorLoadUrl(url)
 {
   try {
     if (url)
-      GetCurrentEditorElement().webNavigation.loadURI(url, // uri string
-             nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE,     // load flags
-             null,                                         // referrer
-             null,                                         // post-data stream
-             null);
+      GetCurrentEditorElement().webNavigation.loadURI(url,                // uri string
+         Ci.nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE,                     // load flags
+         null,                                                            // referrer
+         null,                                                            // post-data stream
+         null);
   } catch (e) { dump(" EditorLoadUrl failed: "+e+"\n"); }
 }
 
@@ -446,7 +446,7 @@ function EditorSharedStartup()
     commandManager.addCommandObserver(gEditorDocumentObserver, "cmd_bold");
   } catch (e) { dump(e); }
 
-  var isMac = (GetOS() == gMac);
+  var isMac = AppConstants.platform == "macosx";
 
   // Set platform-specific hints for how to select cells
   // Mac uses "Cmd", all others use "Ctrl"
@@ -482,26 +482,6 @@ function EditorSharedStartup()
   gColorObj.LastHighlightColor = "";
 }
 
-// This method is only called by Message composer when recycling a compose window
-function EditorResetFontAndColorAttributes()
-{
-  try {  
-    var editor = GetCurrentEditor();
-    editor.rebuildDocumentFromSource("");
-    // Because the selection is now collapsed, the following line
-    // clears the typing state to discontinue all inline styles
-    editor.removeAllInlineProperties();
-    document.getElementById("cmd_fontFace").setAttribute("state", "");
-    ClearUsedFonts();
-    gColorObj.LastTextColor = "";
-    gColorObj.LastBackgroundColor = "";
-    gColorObj.LastHighlightColor = "";
-    document.getElementById("cmd_fontColor").setAttribute("state", "");
-    document.getElementById("cmd_backgroundColor").setAttribute("state", "");
-    UpdateDefaultColors();
-  } catch (e) {}
-}
-
 function SafeSetAttribute(nodeID, attributeName, attributeValue)
 {
     var theNode = document.getElementById(nodeID);
@@ -525,7 +505,7 @@ function DocumentHasBeenSaved()
   return true;
 }
 
-function CheckAndSaveDocument(command, allowDontSave)
+async function CheckAndSaveDocument(command, allowDontSave)
 {
   var document;
   try {
@@ -611,7 +591,7 @@ function CheckAndSaveDocument(command, allowDontSave)
     }
 
     // Save to local disk
-    return SaveDocument(false, false, editor.contentsMIMEType);
+    return await SaveDocument(false, false, editor.contentsMIMEType);
   }
 
   if (result == 2) // "Don't Save"
@@ -686,73 +666,173 @@ function onParagraphFormatChange(paraMenuList, commandID)
 function onFontFaceChange(fontFaceMenuList, commandID)
 {
   var commandNode = document.getElementById(commandID);
-  var state = commandNode.getAttribute("state");
+  var editorFont = commandNode.getAttribute("state");
 
-  switch (state) {
+  // Strip quotes in font names. Experiments have shown that we only
+  // ever get double quotes around the font name, never single quotes,
+  // even if they were in the HTML source. Also single or double
+  // quotes within the font name are never returned.
+  editorFont = editorFont.replace(/"/g, "");
+
+  switch (editorFont) {
   case "mixed":
-    //Selection is the "mixed" ( > 1 style) state
+    // Selection is the "mixed" ( > 1 style) state.
     fontFaceMenuList.selectedItem = null;
     fontFaceMenuList.setAttribute("label",GetString('Mixed'));
-    break;
+    return;
   case "":
   case "serif":
   case "sans-serif":
     // Generic variable width.
     fontFaceMenuList.selectedIndex = 0;
-    break;
+    return;
   case "tt":
   case "monospace":
     // Generic fixed width.
     fontFaceMenuList.selectedIndex = 1;
-    break;
+    return;
   default:
-    let menuPopup = fontFaceMenuList.menupopup;
-    let menuItems = menuPopup.childNodes;
+  }
 
-    // Bug 1139524: Normalise before we compare: Make it lower case
-    // and replace ", " with "," so that entries like
-    // "Helvetica, Arial, sans-serif" are always recognised correctly
-    let stateToLower = state.toLowerCase().replace(/, /g, ",");
-    let foundFont = false;
-    let notUsedFont = false;
-    let usedFontsSep = menuPopup.querySelector("menuseparator.fontFaceMenuAfterUsedFonts");
+  let menuPopup = fontFaceMenuList.menupopup;
+  let menuItems = menuPopup.childNodes;
 
-    for (let i = 0; i < menuItems.length; i++)
+  const genericFamilies = [ "serif", "sans-serif", "monospace", "fantasy", "cursive" ];
+  // Bug 1139524: Normalise before we compare: Make it lower case
+  // and replace ", " with "," so that entries like
+  // "Helvetica, Arial, sans-serif" are always recognised correctly
+  let editorFontToLower = editorFont.toLowerCase().replace(/, /g, ",");
+  let foundFont = null;
+  let exactMatch = false;
+  let usedFontsSep = menuPopup.querySelector("menuseparator.fontFaceMenuAfterUsedFonts");
+  let editorFontOptions = editorFontToLower.split(",");
+  let editorOptionsCount = editorFontOptions.length;
+  let matchedFontIndex = editorOptionsCount; // initialise to high invalid value
+
+  // The font menu has this structure:
+  // 0: Variable Width
+  // 1: Fixed Width
+  // 2: Separator
+  // 3: Helvetica, Arial (stored as Helvetica, Arial, sans-serif)
+  // 4: Times (stored as Times New Roman, Times, serif)
+  // 5: Courier (stored as Courier New, Courier, monospace)
+  // 6: Separator, "menuseparator.fontFaceMenuAfterDefaultFonts"
+  // from 7: Used Font Section (for quick selection)
+  // followed by separator, "menuseparator.fontFaceMenuAfterUsedFonts"
+  // followed by all other available fonts.
+  // The following variable keeps track of where we are when we loop over the menu.
+  let afterUsedFontSection = false;
+
+  // The menu items not only have "label" and "value", but also some other attributes:
+  // "value_parsed": Is the toLowerCase() and space-stripped value.
+  // "value_cache":  Is a concatenation of all editor fonts that were ever mapped
+  //                 onto this menu item. This is done for optimization.
+  // "used":         This item is in the used font section.
+
+  for (let i = 0; i < menuItems.length; i++)
+  {
+    let menuItem = menuItems.item(i);
+    if (menuItem.hasAttribute("label") && menuItem.hasAttribute("value_parsed"))
     {
-      let menuItem = menuItems.item(i);
-      if (menuItem.hasAttribute("label") && ("value" in menuItem)) {
-        // The element seems to represent a font <menuitem>.
-        if (menuItem.getAttribute("value_parsed") == stateToLower) {
-          // This menuitem contains the font we are looking for.
-          if (notUsedFont) {
-            // Copy it into the group of used fonts if it isn't there already.
-            let defaultFontsSep = menuPopup.querySelector("menuseparator.fontFaceMenuAfterDefaultFonts");
-            let copyItem = menuItem.cloneNode(true);
-            menuPopup.insertBefore(copyItem, defaultFontsSep.nextSibling);
-            usedFontsSep.hidden = false;
-            menuItem = copyItem;
-          }
-          fontFaceMenuList.selectedItem = menuItem;
-          foundFont = true;
-          break;
+      // The element seems to represent a font <menuitem>.
+      let fontMenuValue = menuItem.getAttribute("value_parsed");
+      if (fontMenuValue == editorFontToLower ||
+          (menuItem.hasAttribute("value_cache") &&
+           menuItem.getAttribute("value_cache").split("|").includes(editorFontToLower)))
+      {
+        // This menuitem contains the font we are looking for.
+        foundFont = menuItem;
+        exactMatch = true;
+        break;
+      }
+      else if (editorOptionsCount > 1 && afterUsedFontSection)
+      {
+        // Once we are in the list of all other available fonts,
+        // we will find the one that best matches one of the options.
+        let matchPos = editorFontOptions.indexOf(fontMenuValue);
+        if (matchPos >= 0 && matchPos < matchedFontIndex)
+        {
+          // This menu font comes earlier in the list of options,
+          // so prefer it.
+          matchedFontIndex = matchPos;
+          foundFont = menuItem;
+          // If we matched the first option, we don't need to look for
+          // a better match.
+          if (matchPos == 0)
+            break;
         }
-      } else {
-        // Some other element type.
-        if (menuItem == usedFontsSep)
-          notUsedFont = true;
       }
     }
-
-    if (!foundFont) {
-      // The text editor encountered a font that is not installed on this system,
-      // so add it to the font menu now.
-      let fontLabel = GetFormattedString("NotInstalled", state);
-      let menuItem = createFontFaceMenuitem(fontLabel, state, menuPopup);
-      usedFontsSep.hidden = false;
-      menuPopup.insertBefore(menuItem, usedFontsSep);
-      fontFaceMenuList.selectedItem = menuItem;
+    else
+    {
+      // Some other element type.
+      if (menuItem == usedFontsSep)
+      {
+        // We have now passed the section of used fonts and are now in the list of all.
+        afterUsedFontSection = true;
+      }
     }
   }
+
+  if (foundFont)
+  {
+    let defaultFontsSep = menuPopup.querySelector("menuseparator.fontFaceMenuAfterDefaultFonts");
+    if (exactMatch)
+    {
+      if (afterUsedFontSection)
+      {
+        // Copy the matched font into the section of used fonts.
+        // We insert after the separator following the default fonts,
+        // so right at the beginning of the used fonts section.
+        let copyItem = foundFont.cloneNode(true);
+        menuPopup.insertBefore(copyItem, defaultFontsSep.nextSibling);
+        usedFontsSep.hidden = false;
+        foundFont = copyItem;
+        foundFont.setAttribute("used", "true");
+      }
+    }
+    else
+    {
+      // Keep only the found font and generic families in the font string.
+      editorFont = editorFont.replace(/, /g, ",").split(",").filter(
+        font => ((font.toLowerCase() == foundFont.getAttribute("value_parsed")) ||
+                 genericFamilies.includes(font))).join(",");
+
+      // Check if such an item is already in the used font section.
+      if (afterUsedFontSection)
+        foundFont = menuPopup.querySelector('menuitem[used="true"][value_parsed="'+
+                    editorFont.toLowerCase()+'"]');
+      // If not, create a new entry which will be inserted into that section.
+      if (!foundFont)
+        foundFont = createFontFaceMenuitem(editorFont, editorFont, menuPopup);
+
+      // Add the editor font string into the 'cache' attribute in the element
+      // so we can later find it quickly without building the reduced string again.
+      let fontCache = "";
+      if (foundFont.hasAttribute("value_cache"))
+        fontCache = foundFont.getAttribute("value_cache");
+      foundFont.setAttribute("value_cache", fontCache + "|" + editorFontToLower);
+
+      // If we created a new item, set it up and insert.
+      if (!foundFont.hasAttribute("used")) {
+        foundFont.setAttribute("used", "true");
+        usedFontsSep.hidden = false;
+        menuPopup.insertBefore(foundFont, defaultFontsSep.nextSibling);
+      }
+    }
+  }
+  else
+  {
+    // The editor encountered a font that is not installed on this system.
+    // Add it to the font menu now, in the used-fonts section right at the
+    // bottom before the separator of the section.
+    let fontLabel = GetFormattedString("NotInstalled", editorFont);
+    foundFont = createFontFaceMenuitem(fontLabel, editorFont, menuPopup);
+    foundFont.setAttribute("used", "true");
+    usedFontsSep.hidden = false;
+    menuPopup.insertBefore(foundFont, usedFontsSep);
+  }
+  fontFaceMenuList.selectedItem = foundFont;
 }
 
 /**
@@ -859,47 +939,92 @@ function initFontFaceMenu(menuPopup)
     var children = menuPopup.childNodes;
     if (!children) return;
 
-    var firstHas = { value: false };
-    var anyHas = { value: false };
-    var allHas = { value: false };
+    var mixed = { value: false };
+    var editorFont = GetCurrentEditor().getFontFaceState(mixed);
 
-    // we need to set or clear the checkmark for each menu item since the selection
-    // may be in a new location from where it was when the menu was previously opened
+    // Strip quotes in font names. Experiments have shown that we only
+    // ever get double quotes around the font name, never single quotes,
+    // even if they were in the HTML source. Also single or double
+    // quotes within the font name are never returned.
+    editorFont = editorFont.replace(/"/g, "");
 
-    // Fixed width (second menu item) is special case: old TT ("teletype") attribute
-    EditorGetTextProperty("tt", "", "", firstHas, anyHas, allHas);
-    children[1].setAttribute("checked", allHas.value);
+    if (!mixed.value)
+    {
+      switch (editorFont)
+      {
+      case "":
+      case "serif":
+      case "sans-serif":
+        // Generic variable width.
+        editorFont = "";
+        break;
+      case "tt":
+      case "monospace":
+        // Generic fixed width.
+        editorFont = "tt";
+        break;
+      default:
+        editorFont = editorFont.toLowerCase().replace(/, /g, ","); // bug 1139524
+      }
+    }
 
-    if (!anyHas.value)
-      EditorGetTextProperty("font", "face", "", firstHas, anyHas, allHas);
-
-    children[0].setAttribute("checked", !anyHas.value);
-
-    // Skip over default, TT, and separator
-    for (var i = 3; i < children.length; i++)
+    var editorFontOptions = editorFont.split(',');
+    var matchedOption = editorFontOptions.length;  // initialise to high invalid value
+    for (var i = 0; i < children.length; i++)
     {
       var menuItem = children[i];
-      var faceType = menuItem.getAttribute("value");
-
-      if (faceType)
+      if (menuItem.localName == "menuitem")
       {
-        EditorGetTextProperty("font", "face", faceType, firstHas, anyHas, allHas);
-
-        // Check the menuitem only if all of selection has the face
-        if (allHas.value)
+        var matchFound = false;
+        if (!mixed.value)
         {
-          menuItem.setAttribute("checked", "true");
-          break;
+          var menuFont = menuItem.getAttribute("value").toLowerCase().replace(/, /g, ",");
+
+          // First compare the entire font string to match items that contain commas.
+          if (menuFont == editorFont)
+          {
+            menuItem.setAttribute("checked", "true");
+            break;
+          }
+
+          // Next compare the individual options.
+          else if (editorFontOptions.length > 1)
+          {
+            var matchPos = editorFontOptions.indexOf(menuFont);
+            if (matchPos >= 0 && matchPos < matchedOption) {
+              // This menu font comes earlier in the list of options,
+              // so prefer it.
+              menuItem.setAttribute("checked", "true");
+
+              // If we matched the first option, we don't need to look for
+              // a better match.
+              if (matchPos == 0)
+                break;
+
+              matchedOption = matchPos;
+              matchFound = true;
+            }
+          }
         }
 
-        // in case none match, make sure we've cleared the checkmark
-        menuItem.removeAttribute("checked");
+        // In case this item doesn't match, make sure we've cleared the checkmark.
+        if (!matchFound)
+          menuItem.removeAttribute("checked");
       }
     }
   }
 }
 
-const kFixedFontFaceMenuItems = 8; // number of fixed font face menuitems
+// Number of fixed font face menuitems, these are:
+// Variable Width
+// Fixed Width
+// ==separator
+// Helvetica, Arial
+// Times
+// Courier
+// ==separator
+// ==separator
+const kFixedFontFaceMenuItems = 8;
 
 function initLocalFontFaceMenu(menuPopup)
 {
@@ -908,8 +1033,8 @@ function initLocalFontFaceMenu(menuPopup)
     // Build list of all local fonts once per editor
     try 
     {
-      var enumerator = Components.classes["@mozilla.org/gfx/fontenumerator;1"]
-                                 .getService(Components.interfaces.nsIFontEnumerator);
+      var enumerator = Cc["@mozilla.org/gfx/fontenumerator;1"]
+                         .getService(Ci.nsIFontEnumerator);
       var localFontCount = { value: 0 }
       gLocalFonts = enumerator.EnumerateAllFonts(localFontCount);
     }
@@ -943,7 +1068,7 @@ function initLocalFontFaceMenu(menuPopup)
  * Creates a menuitem element for the font faces menulist. Returns the menuitem
  * but does not add it automatically to the menupopup.
  *
- * @param aFontLabel  Label to be displayed for the item
+ * @param aFontLabel  Label to be displayed for the item.
  * @param aFontName   The font face value to be used for the item.
  *                    Will be used in <font face="value"> in the edited document.
  * @param aMenuPopup  The menupopup for which this menuitem is created.
@@ -954,77 +1079,128 @@ function createFontFaceMenuitem(aFontLabel, aFontName, aMenuPopup)
   itemNode.setAttribute("label", aFontLabel);
   itemNode.setAttribute("value", aFontName);
   itemNode.setAttribute("value_parsed", aFontName.toLowerCase().replace(/, /g, ","));
+  itemNode.setAttribute("tooltiptext", aFontLabel);
   if (aMenuPopup.getAttribute("useRadios") == "true") {
     itemNode.setAttribute("type", "radio");
-    itemNode.setAttribute("name", "2");
     itemNode.setAttribute("observes", "cmd_renderedHTMLEnabler");
   }
   return itemNode;
 }
 
-function initFontSizeMenu(menuPopup)
+/**
+ * Helper function
+ */
+function getFontSizeIndex()
+{
+  var firstHas = { value: false };
+  var anyHas = { value: false };
+  var allHas = { value: false };
+
+  var fontSize = EditorGetTextProperty("font", "size", null, firstHas, anyHas, allHas);
+
+  // If the element has no size attribute and no size was found at all,
+  // we assume "medium" size. This is highly problematic since
+  // CSS sizes are not recognised and will show as "medium" as well.
+  // Currently we can't distinguish between "no attribute" which
+  // can imply "medium" and "CSS attribute present" which should not
+  // imply "medium".
+  if (!anyHas.value)
+    return 2;
+
+  // Mixed selection.
+  if (!allHas.value)
+    return -1;
+
+  switch (fontSize)
+  {
+    case "-3":
+    case "-2":
+    case "0":
+    case "1":
+      // x-small.
+      return 0;
+    case "-1":
+    case "2":
+      // small.
+      return 1;
+    case "3":
+      // medium.
+      return 2;
+    case "+1":
+    case "4":
+      // large.
+      return 3;
+    case "+2":
+    case "5":
+      // x-large.
+      return 4;
+    case "+3":
+    case "+4":
+    case "6":
+    case "7":
+      // xx-large.
+      return 5;
+  }
+
+  // We shouldn't get here. All the selection has a value we don't understand.
+  return -1;
+}
+
+function initFontSizeMenu(menuPopup, fullMenu)
 {
   if (menuPopup)
   {
     var children = menuPopup.childNodes;
-    if (!children) return;
+    if (!children)
+      return;
 
+    // Fixed size items start after menu separator depending on whether it is
+    // a full menu.
+    var menuIndex = fullMenu ? 3 : 0;
+
+    var setIndex = getFontSizeIndex();
+    if (setIndex >= 0)
+    {
+      children[menuIndex + setIndex].setAttribute("checked", true);
+    }
+    else
+    {
+      // In case of mixed, clear all items.
+      for (var i = menuIndex; i < children.length; i++) {
+        children[i].setAttribute("checked", false);
+      }
+    }
+
+    // Some configurations might not have the "small/big" indicator as
+    // last item. If there is no indicator, we are done.
+    if (!menuPopup.lastChild.id.includes("smallBigInfo"))
+      return;
+
+    // While it would be better to show the number of levels,
+    // at least this tells user if either of them are set.
     var firstHas = { value: false };
     var anyHas = { value: false };
     var allHas = { value: false };
 
-    var sizeWasFound = false;
+    // Show "small"/"big" indicator.
+    var htmlInfo = "";
+    EditorGetTextProperty("small", "", "", firstHas, anyHas, allHas);
+    if (anyHas.value)
+      htmlInfo = "<small>";
+    EditorGetTextProperty("big", "", "", firstHas, anyHas, allHas);
+    if (anyHas.value)
+      htmlInfo += "<big>";
 
-    // we need to set or clear the checkmark for each menu item since the selection
-    // may be in a new location from where it was when the menu was previously opened
-
-    // First 2 items add <small> and <big> tags
-    // While it would be better to show the number of levels,
-    //  at least this tells user if either of them are set
-    var menuItem = children[0];
-    if (menuItem)
+    if (htmlInfo)
     {
-      EditorGetTextProperty("small", "", "", firstHas, anyHas, allHas);
-      menuItem.setAttribute("checked", allHas.value);
-      sizeWasFound = anyHas.value;
+      menuPopup.lastChild.hidden = false;
+      menuPopup.lastChild.setAttribute("label", "HTML: " + htmlInfo);
+      menuPopup.lastChild.setAttribute("checked", true);
     }
-
-    menuItem = children[1];
-    if (menuItem)
+    else
     {
-      EditorGetTextProperty("big", "", "", firstHas, anyHas, allHas);
-      menuItem.setAttribute("checked", allHas.value);
-      sizeWasFound |= anyHas.value;
+      menuPopup.lastChild.hidden = true;
     }
-
-    // Fixed size items start after menu separator
-    var menuIndex = 3;
-    // Index of the medium (default) item
-    var mediumIndex = 5;
-
-    // Scan through all supported "font size" attribute values
-    for (var i = -2; i <= 3; i++)
-    {
-      menuItem = children[menuIndex];
-
-      // Skip over medium since it'll be set below.
-      // If font size=0 is actually set, we'll toggle it off below if
-      // we enter this loop in this case.
-      if (menuItem && (i != 0))
-      {
-        var sizeString = (i <= 0) ? String(i) : ("+" + String(i));
-        EditorGetTextProperty("font", "size", sizeString, firstHas, anyHas, allHas);
-        // Check the item only if all of selection has the size...
-        menuItem.setAttribute("checked", allHas.value);
-        // ...but remember if ANY of of selection had size set
-        sizeWasFound |= anyHas.value;
-      }
-      menuIndex++;
-    }
-
-    // if no size was found, then check default (medium)
-    // note that no item is checked in the case of "mixed" selection
-    children[mediumIndex].setAttribute("checked", !sizeWasFound);
   }
 }
 
@@ -1417,15 +1593,8 @@ function EditorDblClick(event)
     // Only bring up properties if clicked on an element or selected link
     var element;
     try {
-      if (gEditorDisplayMode == kDisplayModeAllTags)
-        element = event.explicitOriginalTarget.QueryInterface(
-                    Components.interfaces.nsIDOMElement);
-      else
-        element = event.rangeParent.childNodes[event.rangeOffset];
-
-      // Don't fire for <br>, it counts as double-clicking text.
-      if (element.nodeName.toLowerCase() == 'br')
-        element = null;
+      element = event.explicitOriginalTarget.QueryInterface(
+                    Ci.nsIDOMElement);
     } catch (e) {}
 
      //  We use "href" instead of "a" to not be fooled by named anchor
@@ -1434,7 +1603,12 @@ function EditorDblClick(event)
         element = GetCurrentEditor().getSelectedElement("href");
       } catch (e) {}
 
-    if (element)
+    // Don't fire for body/p and other block elements.
+    // It's common that people try to double-click
+    // to select a word, but the click hits an empty area.
+    if (element &&
+        !["body","p","h1","h2","h3","h4","h5","h6","blockquote","div","pre"]
+         .includes(element.nodeName.toLowerCase()))
     {
       goDoCommand("cmd_objectProperties");  
       event.preventDefault();
@@ -1454,10 +1628,9 @@ function EditorClick(event)
       // We check event.explicitOriginalTarget here because .target will never
       // be a textnode (bug 193689)
       var element = event.explicitOriginalTarget.QueryInterface(
-                        Components.interfaces.nsIDOMElement);
-      var name = element.localName.toLowerCase();
-      if (name != "body" && name != "table" &&
-          name != "td" && name != "th" && name != "caption" && name != "tr")
+                        Ci.nsIDOMElement);
+      var name = element.localName;
+      if (!["body", "caption", "table", "td", "th", "tr"].includes(name))
       {          
         GetCurrentEditor().selectElement(event.explicitOriginalTarget);
         event.preventDefault();
@@ -1727,7 +1900,7 @@ function SetDisplayMode(mode)
     try {
       var editor = GetCurrentEditor();
       editor.QueryInterface(nsIEditorStyleSheets);
-      editor instanceof Components.interfaces.nsIHTMLObjectResizer;
+      editor instanceof Ci.nsIHTMLObjectResizer;
 
       switch (mode)
       {
@@ -2142,14 +2315,8 @@ function EditorSetDefaultPrefsAndDoctype()
     // hitting other 8-bit char in other meta tags
     // grab charset pref and make it the default charset
     var element;
-    var prefCharsetString = 0;
-    try
-    {
-      prefCharsetString = Services.prefs.getComplexValue("intl.charset.default",
-                                                         Components.interfaces.nsIPrefLocalizedString).data;
-    }
-    catch (ex) {}
-    if ( prefCharsetString && prefCharsetString != 0)
+    var prefCharsetString = Services.prefs.getCharPref("intl.charset.fallback.override");
+    if (prefCharsetString)
       editor.documentCharacterSet = prefCharsetString;
 
     // let's start by assuming we have an author in case we don't have the pref
@@ -2159,7 +2326,7 @@ function EditorSetDefaultPrefsAndDoctype()
     try
     {
       prefAuthorString = Services.prefs.getComplexValue("editor.author",
-                                                        Components.interfaces.nsISupportsString).data;
+                                                        Ci.nsISupportsString).data;
     }
     catch (ex) {}
     if (prefAuthorString && prefAuthorString != 0 && !authorFound && headelement)
@@ -2326,14 +2493,14 @@ function getColorAndSetColorWell(ColorPickerID, ColorWellID)
 //-----------------------------------------------------------------------------------
 function IsSpellCheckerInstalled()
 {
-  return "@mozilla.org/spellchecker;1" in Components.classes;
+  return "@mozilla.org/spellchecker;1" in Cc;
 }
 
 //-----------------------------------------------------------------------------------
 function IsFindInstalled()
 {
-  return "@mozilla.org/embedcomp/rangefind;1" in Components.classes
-          && "@mozilla.org/find/find_service;1" in Components.classes;
+  return "@mozilla.org/embedcomp/rangefind;1" in Cc
+          && "@mozilla.org/find/find_service;1" in Cc;
 }
 
 //-----------------------------------------------------------------------------------
@@ -2391,7 +2558,7 @@ function RemoveItem(id)
 
 // Command Updating Strategy:
 //   Don't update on on selection change, only when menu is displayed,
-//   with this "oncreate" hander:
+//   with this "oncreate" handler:
 function EditorInitTableMenu()
 {
   try {
@@ -2729,7 +2896,7 @@ function SwitchInsertCharToThisWindow(windowWithDialog)
     // Switch the dialog's opener to current window's
     window.InsertCharWindow.opener = window;
 
-    // Bring dialog to the forground
+    // Bring dialog to the foreground
     window.InsertCharWindow.focus();
     return true;
   }
@@ -2746,8 +2913,8 @@ function FindEditorWithInsertCharDialog()
     {
       var tempWindow = enumerator.getNext();
 
-      if (tempWindow != window && "InsertCharWindow" in tempWindow &&
-          tempWindow.InsertCharWindow)
+      if (!tempWindow.closed && tempWindow != window &&
+          "InsertCharWindow" in tempWindow && tempWindow.InsertCharWindow)
       {
         return tempWindow;
       }
@@ -2796,7 +2963,8 @@ function SwitchInsertCharToAnotherEditorOrClose()
     while ( enumerator.hasMoreElements()  )
     {
       var  tempWindow = enumerator.getNext();
-      if (tempWindow != window && tempWindow != window.InsertCharWindow &&
+      if (!tempWindow.closed && tempWindow != window &&
+          tempWindow != window.InsertCharWindow &&
           "GetCurrentEditor" in tempWindow && tempWindow.GetCurrentEditor())
       {
         tempWindow.InsertCharWindow = window.InsertCharWindow;
@@ -2876,9 +3044,9 @@ function UpdateStructToolbar()
 
     toolbar.insertBefore(button, toolbar.firstChild);
 
-    button.addEventListener("command", newCommandListener(element), false);
+    button.addEventListener("command", newCommandListener(element));
 
-    button.addEventListener("contextmenu", newContextmenuListener(button, element), false);
+    button.addEventListener("contextmenu", newContextmenuListener(button, element));
 
     if (isFocusNode && oneElementSelected) {
       button.setAttribute("checked", "true");
@@ -3002,10 +3170,10 @@ function FillInHTMLTooltipEditor(tooltip)
     }
   } else {
     for (node = document.tooltipNode; node; node = node.parentNode) {
-      if (node instanceof Components.interfaces.nsIDOMHTMLImageElement ||
-          node instanceof Components.interfaces.nsIDOMHTMLInputElement)
+      if (node instanceof Ci.nsIDOMHTMLImageElement ||
+          node instanceof Ci.nsIDOMHTMLInputElement)
         tooltipText = node.getAttribute("src");
-      else if (node instanceof Components.interfaces.nsIDOMHTMLAnchorElement)
+      else if (node instanceof Ci.nsIDOMHTMLAnchorElement)
         tooltipText = node.getAttribute("href") || node.name;
       if (tooltipText) {
         tooltip.setAttribute("label", tooltipText);
