@@ -2,7 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
 
 /**** NAMESPACES ****/
 const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
@@ -13,27 +14,18 @@ const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 // Object to attach commonly-used widgets (all dialogs should use this)
 var gDialog = {};
 
-const kOutputEncodeBasicEntities = Components.interfaces.nsIDocumentEncoder.OutputEncodeBasicEntities;
-const kOutputEncodeHTMLEntities = Components.interfaces.nsIDocumentEncoder.OutputEncodeHTMLEntities;
-const kOutputEncodeLatin1Entities = Components.interfaces.nsIDocumentEncoder.OutputEncodeLatin1Entities;
-const kOutputEncodeW3CEntities = Components.interfaces.nsIDocumentEncoder.OutputEncodeW3CEntities;
-const kOutputFormatted = Components.interfaces.nsIDocumentEncoder.OutputFormatted;
-const kOutputLFLineBreak = Components.interfaces.nsIDocumentEncoder.OutputLFLineBreak;
-const kOutputSelectionOnly = Components.interfaces.nsIDocumentEncoder.OutputSelectionOnly;
-const kOutputWrap = Components.interfaces.nsIDocumentEncoder.OutputWrap;
+var kOutputEncodeBasicEntities = Ci.nsIDocumentEncoder.OutputEncodeBasicEntities;
+var kOutputEncodeHTMLEntities = Ci.nsIDocumentEncoder.OutputEncodeHTMLEntities;
+var kOutputEncodeLatin1Entities = Ci.nsIDocumentEncoder.OutputEncodeLatin1Entities;
+var kOutputEncodeW3CEntities = Ci.nsIDocumentEncoder.OutputEncodeW3CEntities;
+var kOutputFormatted = Ci.nsIDocumentEncoder.OutputFormatted;
+var kOutputLFLineBreak = Ci.nsIDocumentEncoder.OutputLFLineBreak;
+var kOutputSelectionOnly = Ci.nsIDocumentEncoder.OutputSelectionOnly;
+var kOutputWrap = Ci.nsIDocumentEncoder.OutputWrap;
 
 var gStringBundle;
 var gFilePickerDirectory;
 
-var gOS = "";
-const gWin = "Win";
-const gUNIX = "UNIX";
-const gMac = "Mac";
-
-const kWebComposerWindowID = "editorWindow";
-const kMailComposerWindowID = "msgcomposeWindow";
-
-var gIsHTMLEditor;
 /************* Message dialogs ***************/
 
 // Optional: Caller may supply text to substitue for "Ok" and/or "Cancel"
@@ -157,11 +149,11 @@ function GetSelectionAsText()
 
 
 /************* Get Current Editor and associated interfaces or info ***************/
-const nsIPlaintextEditor = Components.interfaces.nsIPlaintextEditor;
-const nsIHTMLEditor = Components.interfaces.nsIHTMLEditor;
-const nsITableEditor = Components.interfaces.nsITableEditor;
-const nsIEditorStyleSheets = Components.interfaces.nsIEditorStyleSheets;
-const nsIEditingSession = Components.interfaces.nsIEditingSession;
+const nsIPlaintextEditor = Ci.nsIPlaintextEditor;
+const nsIHTMLEditor = Ci.nsIHTMLEditor;
+const nsITableEditor = Ci.nsITableEditor;
+const nsIEditorStyleSheets = Ci.nsIEditorStyleSheets;
+const nsIEditingSession = Ci.nsIEditingSession;
 
 function GetCurrentEditor()
 {
@@ -177,8 +169,8 @@ function GetCurrentEditor()
 
     // Do QIs now so editor users won't have to figure out which interface to use
     // Using "instanceof" does the QI for us.
-    editor instanceof Components.interfaces.nsIPlaintextEditor;
-    editor instanceof Components.interfaces.nsIHTMLEditor;
+    editor instanceof Ci.nsIPlaintextEditor;
+    editor instanceof Ci.nsIHTMLEditor;
   } catch (e) { dump (e)+"\n"; }
 
   return editor;
@@ -310,7 +302,7 @@ function IsHTMLSourceChanged()
 function newCommandParams()
 {
   try {
-    return Components.classes["@mozilla.org/embedcomp/command-params;1"].createInstance(Components.interfaces.nsICommandParams);
+    return Cc["@mozilla.org/embedcomp/command-params;1"].createInstance(Ci.nsICommandParams);
   }
   catch(e) { dump("error thrown in newCommandParams: "+e+"\n"); }
   return null;
@@ -331,7 +323,7 @@ function SetDocumentTitle(title)
 {
 
   try {
-    GetCurrentEditor().setDocumentTitle(title);
+    GetCurrentEditorElement().contentDocument.title = title;
 
     // Update window title (doesn't work if called from a dialog)
     if ("UpdateWindowTitle" in window)
@@ -342,7 +334,7 @@ function SetDocumentTitle(title)
 var gAtomService;
 function GetAtomService()
 {
-  gAtomService = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService);
+  gAtomService = Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
 }
 
 function EditorGetTextProperty(property, attribute, value, firstHas, anyHas, allHas)
@@ -351,8 +343,8 @@ function EditorGetTextProperty(property, attribute, value, firstHas, anyHas, all
     if (!gAtomService) GetAtomService();
     var propAtom = gAtomService.getAtom(property);
 
-    GetCurrentEditor().getInlineProperty(propAtom, attribute, value,
-                                         firstHas, anyHas, allHas);
+    return GetCurrentEditor().getInlinePropertyWithAttrValue(propAtom,
+                                attribute, value, firstHas, anyHas, allHas);
   }
   catch(e) {}
 }
@@ -413,13 +405,13 @@ function SetElementEnabled(element, doEnable)
 function GetFileProtocolHandler()
 {
   let handler = Services.io.getProtocolHandler("file");
-  return handler.QueryInterface(Components.interfaces.nsIFileProtocolHandler);
+  return handler.QueryInterface(Ci.nsIFileProtocolHandler);
 }
 
 function GetStringPref(name)
 {
   try {
-    return Services.prefs.getComplexValue(name, Components.interfaces.nsISupportsString).data;
+    return Services.prefs.getComplexValue(name, Ci.nsISupportsString).data;
   } catch (e) {}
   return "";
 }
@@ -427,10 +419,10 @@ function GetStringPref(name)
 function SetStringPref(aPrefName, aPrefValue)
 {
   try {
-    let str = Components.classes["@mozilla.org/supports-string;1"]
-                        .createInstance(Components.interfaces.nsISupportsString);
+    let str = Cc["@mozilla.org/supports-string;1"]
+                .createInstance(Ci.nsISupportsString);
     str.data = aPrefValue;
-    Services.prefs.setComplexValue(aPrefName, Components.interfaces.nsISupportsString, str);
+    Services.prefs.setComplexValue(aPrefName, Ci.nsISupportsString, str);
   }
   catch(e) {}
 }
@@ -445,7 +437,7 @@ function SetFilePickerDirectory(filePicker, fileType)
       gFilePickerDirectory = filePicker.displayDirectory;
 
       let location = Services.prefs.getComplexValue("editor.lastFileLocation."+fileType,
-                                                    Components.interfaces.nsILocalFile);
+                                                    Ci.nsIFile);
       if (location)
         filePicker.displayDirectory = location;
     }
@@ -461,10 +453,10 @@ function SaveFilePickerDirectory(filePicker, fileType)
     try {
       var fileDir;
       if (filePicker.file.parent)
-        fileDir = filePicker.file.parent.QueryInterface(Components.interfaces.nsILocalFile);
+        fileDir = filePicker.file.parent.QueryInterface(Ci.nsIFile);
 
         Services.prefs.setComplexValue("editor.lastFileLocation." + fileType,
-                                       Components.interfaces.nsILocalFile, fileDir);
+                                       Ci.nsIFile, fileDir);
 
         Services.prefs.savePrefFile(null);
     } catch (e) {}
@@ -546,13 +538,14 @@ function MakeRelativeUrl(url)
 
   // Get just the file path part of the urls
   // XXX Should we use GetCurrentEditor().documentCharacterSet for 2nd param ?
-  let docPath = Services.io.newURI(docUrl, GetCurrentEditor().documentCharacterSet, null).path;
-  let urlPath = Services.io.newURI(inputUrl, GetCurrentEditor().documentCharacterSet, null).path;
+  let docPath = Services.io.newURI(docUrl, GetCurrentEditor().documentCharacterSet).path;
+  let urlPath = Services.io.newURI(inputUrl, GetCurrentEditor().documentCharacterSet).path;
+
+  // We only return "urlPath", so we can convert the entire docPath for
+  // case-insensitive comparisons.
+  var doCaseInsensitive = (docScheme == "file" &&
+                           AppConstants.platform == "win");
 
-  // We only return "urlPath", so we can convert
-  //  the entire docPath for case-insensitive comparisons
-  var os = GetOS();
-  var doCaseInsensitive = (docScheme == "file" && os == gWin);
   if (doCaseInsensitive)
     docPath = docPath.toLowerCase();
 
@@ -602,23 +595,23 @@ function MakeRelativeUrl(url)
 
       if (urlDir == docDir)
       {
-
         // Remove matching dir+"/" from each path
-        //  and continue to next dir
+        // and continue to next dir.
         docPath = docPath.slice(nextDocSlash+1);
         urlPath = urlPath.slice(nextUrlSlash+1);
       }
       else
       {
-        // No match, we're done
+        // No match, we're done.
         done = true;
 
         // Be sure we are on the same local drive or volume
         //   (the first "dir" in the path) because we can't
         //   relativize to different drives/volumes.
         // UNIX doesn't have volumes, so we must not do this else
-        //  the first directory will be misinterpreted as a volume name
-        if (firstDirTest && docScheme == "file" && os != gUNIX)
+        // the first directory will be misinterpreted as a volume name.
+        if (firstDirTest && docScheme == "file" &&
+            AppConstants.platform != "unix")
           return inputUrl;
       }
     }
@@ -659,7 +652,7 @@ function MakeAbsoluteUrl(url)
 
   // Make a URI object to use its "resolve" method
   let absoluteUrl = resultUrl;
-  let docUri = Services.io.newURI(docUrl, GetCurrentEditor().documentCharacterSet, null);
+  let docUri = Services.io.newURI(docUrl, GetCurrentEditor().documentCharacterSet);
 
   try {
     absoluteUrl = docUri.resolve(resultUrl);
@@ -694,7 +687,7 @@ function GetDocumentBaseUrl()
 function GetDocumentUrl()
 {
   try {
-    var aDOMHTMLDoc = GetCurrentEditor().document.QueryInterface(Components.interfaces.nsIDOMHTMLDocument);
+    var aDOMHTMLDoc = GetCurrentEditor().document.QueryInterface(Ci.nsIDOMHTMLDocument);
     return aDOMHTMLDoc.URL;
   }
   catch (e) {}
@@ -725,7 +718,7 @@ function GetHost(urlspec)
 
   var host = "";
   try {
-    host = Services.io.newURI(urlspec, null, null).host;
+    host = Services.io.newURI(urlspec).host;
    } catch (e) {}
 
   return host;
@@ -738,7 +731,7 @@ function GetUsername(urlspec)
 
   var username = "";
   try {
-    username = Services.io.newURI(urlspec, null, null).username;
+    username = Services.io.newURI(urlspec).username;
   } catch (e) {}
 
   return username;
@@ -752,10 +745,10 @@ function GetFilename(urlspec)
   var filename;
 
   try {
-    let uri = Services.io.newURI(urlspec, null, null);
+    let uri = Services.io.newURI(urlspec);
     if (uri)
     {
-      let url = uri.QueryInterface(Components.interfaces.nsIURL);
+      let url = uri.QueryInterface(Ci.nsIURL);
       if (url)
         filename = url.fileName;
     }
@@ -783,7 +776,7 @@ function StripUsernamePassword(urlspec, usernameObj, passwordObj)
   if (atIndex > 0)
   {
     try {
-      let uri = Services.io.newURI(urlspec, null, null);
+      let uri = Services.io.newURI(urlspec);
       let username = uri.username;
       let password = uri.password;
 
@@ -816,7 +809,7 @@ function StripPassword(urlspec, passwordObj)
   if (atIndex > 0)
   {
     try {
-      let password = Services.io.newURI(urlspec, null, null).password;
+      let password = Services.io.newURI(urlspec).password;
 
       if (passwordObj && password)
         passwordObj.value = password;
@@ -860,7 +853,7 @@ function InsertUsernameIntoUrl(urlspec, username)
     return urlspec;
 
   try {
-    let URI = Services.io.newURI(urlspec, GetCurrentEditor().documentCharacterSet, null);
+    let URI = Services.io.newURI(urlspec, GetCurrentEditor().documentCharacterSet);
     URI.username = username;
     return URI.spec;
   } catch (e) {}
@@ -868,26 +861,6 @@ function InsertUsernameIntoUrl(urlspec, username)
   return urlspec;
 }
 
-function GetOS()
-{
-  if (gOS)
-    return gOS;
-
-  var platform = navigator.platform.toLowerCase();
-
-  if (platform.includes("win"))
-    gOS = gWin;
-  else if (platform.includes("mac"))
-    gOS = gMac;
-  else if (platform.includes("unix") || platform.includes("linux") || platform.includes("sun"))
-    gOS = gUNIX;
-  else
-    gOS = "";
-  // Add other tests?
-
-  return gOS;
-}
-
 function ConvertRGBColorIntoHEXColor(color)
 {
   if ( /rgb\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)/.test(color) ) {
@@ -936,3 +909,81 @@ function Clone(obj)
   }
   return clone;
 }
+
+/**
+ * Utility functions to handle shortended data: URLs in EdColorProps.js and EdImageOverlay.js.
+ */
+
+/**
+ * Is the passed in image URI a shortened data URI?
+ * @return {bool}
+ */
+function isImageDataShortened(aImageData) {
+  return (/^data:/i.test(aImageData) && aImageData.includes("…"));
+}
+
+/**
+ * Event handler for Copy or Cut
+ * @param aEvent  the event
+ */
+function onCopyOrCutShortened(aEvent) {
+  // Put the original data URI onto the clipboard in case the value
+  // is a shortened data URI.
+  let field = aEvent.target;
+  let startPos = field.selectionStart;
+  if (startPos == undefined)
+    return;
+  let endPos = field.selectionEnd;
+  let selection = field.value.substring(startPos, endPos).trim();
+
+  // Test that a) the user selected the whole value,
+  //           b) the value is a data URI,
+  //           c) it contains the ellipsis we added. Otherwise it could be
+  //              a new value that the user pasted in.
+  if (selection == field.value.trim() && isImageDataShortened(selection)) {
+    aEvent.clipboardData.setData("text/plain", field.fullDataURI);
+    if (aEvent.type == "cut") {
+      // We have to cut the selection manually. Since we tested that
+      // everything was selected, we can just reset the field.
+      field.value = "";
+    }
+    aEvent.preventDefault();
+  }
+}
+
+/**
+ * Set up element showing an image URI with a shortened version.
+ * and add event handler for Copy or Cut.
+ *
+ * @param aImageData    the data: URL of the image to be shortened.
+ *                      Note: Original stored in 'aDialogField.fullDataURI'.
+ * @param aDialogField  The field of the dialog to contain the data.
+ * @return {bool}       URL was shortened?
+ */
+function shortenImageData(aImageData, aDialogField) {
+  let shortened = false;
+  aDialogField.value = aImageData.replace(/^(data:.+;base64,)(.*)/i,
+    function(match, nonDataPart, dataPart) {
+      if (dataPart.length <= 35)
+        return match;
+
+      shortened = true;
+      aDialogField.addEventListener("copy", onCopyOrCutShortened);
+      aDialogField.addEventListener("cut", onCopyOrCutShortened);
+      aDialogField.fullDataURI = aImageData;
+      aDialogField.removeAttribute("tooltiptext");
+      aDialogField.setAttribute("tooltip", "shortenedDataURI");
+      return nonDataPart + dataPart.substring(0, 5) + "…" +
+                           dataPart.substring(dataPart.length - 30);
+    });
+  return shortened;
+}
+
+/**
+ * Return full data URIs for a shortened element.
+ *
+ * @param aDialogField  The field of the dialog containing the data.
+ */
+function restoredImageData(aDialogField) {
+  return aDialogField.fullDataURI;
+}
