@@ -6,7 +6,7 @@
 
 #include "nsLDAPInternal.h"
 #include "nsIServiceManager.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsIComponentManager.h"
 #include "nsIDNSRecord.h"
 #include "nsLDAPConnection.h"
@@ -26,22 +26,21 @@
 #include "nsMemory.h"
 #include "nsLDAPUtils.h"
 #include "nsProxyRelease.h"
-#include "mozilla/Logging.h"
+#include "mozilla/Attributes.h"
 
 using namespace mozilla;
 
-const char kConsoleServiceContractId[] = "@mozilla.org/consoleservice;1";
 const char kDNSServiceContractId[] = "@mozilla.org/network/dns-service;1";
 
 // constructor
 //
 nsLDAPConnection::nsLDAPConnection()
-    : mConnectionHandle(0),
+    : mConnectionHandle(nullptr),
       mPendingOperationsMutex("nsLDAPConnection.mPendingOperationsMutex"),
       mPendingOperations(10),
       mSSL(false),
       mVersion(nsILDAPConnection::VERSION3),
-      mDNSRequest(0)
+      mDNSRequest(nullptr)
 {
 }
 
@@ -68,7 +67,7 @@ NS_INTERFACE_MAP_BEGIN(nsLDAPConnection)
   NS_INTERFACE_MAP_ENTRY(nsIObserver)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsILDAPConnection)
   NS_IMPL_QUERY_CLASSINFO(nsLDAPConnection)
-NS_INTERFACE_MAP_END_THREADSAFE
+NS_INTERFACE_MAP_END
 NS_IMPL_CI_INTERFACE_GETTER(nsLDAPConnection, nsILDAPConnection,
                              nsISupportsWeakReference, nsIDNSListener,
                              nsIObserver)
@@ -152,8 +151,9 @@ nsLDAPConnection::Init(nsILDAPURL *aUrl, const nsACString &aBindName,
   if (spacePos != kNotFound)
     mDNSHost.SetLength(spacePos);
 
-  rv = pDNSService->AsyncResolve(mDNSHost, 0, this, curThread,
-                                 getter_AddRefs(mDNSRequest));
+  mozilla::OriginAttributes attrs;
+  rv = pDNSService->AsyncResolveNative(mDNSHost, 0, this, curThread, attrs,
+                                       getter_AddRefs(mDNSRequest));
 
   if (NS_FAILED(rv)) {
     switch (rv) {
@@ -185,13 +185,11 @@ nsLDAPConnection::Close()
       // still be synchronous
       //
       rc = ldap_unbind(mConnectionHandle);
-#ifdef PR_LOGGING
       if (rc != LDAP_SUCCESS) {
           MOZ_LOG(gLDAPLogModule, mozilla::LogLevel::Warning,
                  ("nsLDAPConnection::Close(): %s\n",
                   ldap_err2string(rc)));
       }
-#endif
       mConnectionHandle = nullptr;
   }
 
@@ -205,9 +203,9 @@ nsLDAPConnection::Close()
   //
   if (mDNSRequest) {
       mDNSRequest->Cancel(NS_ERROR_ABORT);
-      mDNSRequest = 0;
+      mDNSRequest = nullptr;
   }
-  mInitListener = 0;
+  mInitListener = nullptr;
 
 }
 
@@ -381,7 +379,8 @@ class nsOnLDAPMessageRunnable : public Runnable
 {
 public:
   nsOnLDAPMessageRunnable(nsLDAPMessage *aMsg, bool aClear)
-    : m_msg(aMsg)
+    : Runnable("nsOnLDAPMessageRunnable")
+    , m_msg(aMsg)
     , m_clear(aClear)
   {}
   NS_DECL_NSIRUNNABLE
@@ -589,13 +588,13 @@ nsLDAPConnection::OnLookupComplete(nsICancelable *aRequest,
     // Drop the DNS request object, we no longer need it, and set the flag
     // indicating that DNS has finished.
     //
-    mDNSRequest = 0;
+    mDNSRequest = nullptr;
     mDNSHost.Truncate();
 
     // Call the listener, and then we can release our reference to it.
     //
     mInitListener->OnLDAPInit(this, rv);
-    mInitListener = 0;
+    mInitListener = nullptr;
 
     return rv;
 }
@@ -610,7 +609,7 @@ nsLDAPConnectionRunnable::nsLDAPConnectionRunnable(int32_t aOperationID,
 nsLDAPConnectionRunnable::~nsLDAPConnectionRunnable()
 {
   if (mConnection) {
-    NS_ReleaseOnMainThread(mConnection.forget());
+    NS_ReleaseOnMainThreadSystemGroup("nsLDAPConnectionRunnable::mConnection", mConnection.forget());
   }
 }
 
@@ -645,6 +644,7 @@ NS_IMETHODIMP nsLDAPConnectionRunnable::Run()
     case LDAP_RES_SEARCH_REFERENCE:
       // XXX what should we do with LDAP_RES_SEARCH_EXTENDED
       operationFinished = false;
+      MOZ_FALLTHROUGH;
     default:
     {
       msg = new nsLDAPMessage;
