@@ -18,7 +18,6 @@
 #include "nsIAuthModule.h"
 #include "nsArrayUtils.h"
 #include "nsMemory.h"
-#include "mozilla/Logging.h"
 
 // Helper function
 static nsresult TranslateLDAPErrorToNSError(const int ldapError)
@@ -76,7 +75,7 @@ NS_INTERFACE_MAP_BEGIN(nsLDAPOperation)
   NS_INTERFACE_MAP_ENTRY(nsILDAPOperation)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsILDAPOperation)
   NS_IMPL_QUERY_CLASSINFO(nsLDAPOperation)
-NS_INTERFACE_MAP_END_THREADSAFE
+NS_INTERFACE_MAP_END
 NS_IMPL_CI_INTERFACE_GETTER(nsLDAPOperation, nsILDAPOperation)
 
 /**
@@ -257,7 +256,7 @@ nsLDAPOperation::SaslStep(const char *token, uint32_t tokenLen)
 NS_IMETHODIMP
 nsLDAPOperation::SimpleBind(const nsACString& passwd)
 {
-    nsRefPtr<nsLDAPConnection> connection = mConnection;
+    RefPtr<nsLDAPConnection> connection = mConnection;
     // There is a possibilty that mConnection can be cleared by another
     // thread. Grabbing a local reference to mConnection may avoid this.
     // See https://bugzilla.mozilla.org/show_bug.cgi?id=557928#c1
@@ -274,7 +273,7 @@ nsLDAPOperation::SimpleBind(const nsACString& passwd)
     if (!passwd.IsEmpty())
       mSavePassword = passwd;
 
-    NS_PRECONDITION(mMessageListener != 0, "MessageListener not set");
+    NS_ASSERTION(mMessageListener, "MessageListener not set");
 
     rv = connection->GetBindName(bindName);
     if (NS_FAILED(rv))
@@ -286,7 +285,7 @@ nsLDAPOperation::SimpleBind(const nsACString& passwd)
 
     // this (nsLDAPOperation) may be released by RemovePendingOperation()
     // See https://bugzilla.mozilla.org/show_bug.cgi?id=1063829.
-    nsRefPtr<nsLDAPOperation> kungFuDeathGrip = this;
+    RefPtr<nsLDAPOperation> kungFuDeathGrip = this;
 
     // If this is a second try at binding, remove the operation from pending ops
     // because msg id has changed...
@@ -427,7 +426,7 @@ nsLDAPOperation::SearchExt(const nsACString& aBaseDn, int32_t aScope,
         if (NS_FAILED(rv)) {
             MOZ_LOG(gLDAPLogModule, mozilla::LogLevel::Error,
                    ("nsLDAPOperation::SearchExt(): error converting server "
-                    "control array: %x", rv));
+                    "control array: %" PRIx32, static_cast<uint32_t>(rv)));
             return rv;
         }
     }
@@ -438,7 +437,7 @@ nsLDAPOperation::SearchExt(const nsACString& aBaseDn, int32_t aScope,
         if (NS_FAILED(rv)) {
             MOZ_LOG(gLDAPLogModule, mozilla::LogLevel::Error,
                    ("nsLDAPOperation::SearchExt(): error converting client "
-                    "control array: %x", rv));
+                    "control array: %" PRIx32, static_cast<uint32_t>(rv)));
             ldap_controls_free(serverctls);
             return rv;
         }
@@ -520,7 +519,7 @@ nsLDAPOperation::AbandonExt()
     nsresult rv;
     nsresult retStatus = NS_OK;
 
-    if ( mMessageListener == 0 || mMsgID == 0 ) {
+    if (!mMessageListener || mMsgID == 0) {
         NS_ERROR("nsLDAPOperation::AbandonExt(): mMessageListener or "
                  "mMsgId not initialized");
         return NS_ERROR_NOT_INITIALIZED;
@@ -596,7 +595,7 @@ nsLDAPOperation::AddExt(const char *base,
                         LDAPControl **serverctrls,
                         LDAPControl **clientctrls)
 {
-  if (mMessageListener == 0) {
+  if (!mMessageListener) {
     NS_ERROR("nsLDAPOperation::AddExt(): mMessageListener not set");
     return NS_ERROR_NOT_INITIALIZED;
   }
@@ -697,8 +696,8 @@ nsLDAPOperation::AddExt(const nsACString& aBaseDn,
   if (NS_FAILED(rv)) {
     (void)ldap_abandon_ext(mConnectionHandle, mMsgID, 0, 0);
     MOZ_LOG(gLDAPLogModule, mozilla::LogLevel::Debug,
-           ("nsLDAPOperation::AddExt(): abandoned due to rv %x",
-            rv));
+           ("nsLDAPOperation::AddExt(): abandoned due to rv %" PRIx32,
+            static_cast<uint32_t>(rv)));
   }
   return rv;
 }
@@ -710,7 +709,7 @@ nsLDAPOperation::DeleteExt(const char *base,
                            LDAPControl **serverctrls,
                            LDAPControl **clientctrls)
 {
-  if (mMessageListener == 0) {
+  if (!mMessageListener) {
     NS_ERROR("nsLDAPOperation::DeleteExt(): mMessageListener not set");
     return NS_ERROR_NOT_INITIALIZED;
   }
@@ -747,8 +746,8 @@ nsLDAPOperation::DeleteExt(const nsACString& aBaseDn)
   if (NS_FAILED(rv)) {
     (void)ldap_abandon_ext(mConnectionHandle, mMsgID, 0, 0);
     MOZ_LOG(gLDAPLogModule, mozilla::LogLevel::Debug,
-           ("nsLDAPOperation::AddExt(): abandoned due to rv %x",
-            rv));
+           ("nsLDAPOperation::AddExt(): abandoned due to rv %" PRIx32,
+            static_cast<uint32_t>(rv)));
   }
   return rv;
 }
@@ -761,7 +760,7 @@ nsLDAPOperation::ModifyExt(const char *base,
                            LDAPControl **serverctrls,
                            LDAPControl **clientctrls)
 {
-  if (mMessageListener == 0) {
+  if (!mMessageListener) {
     NS_ERROR("nsLDAPOperation::ModifyExt(): mMessageListener not set");
     return NS_ERROR_NOT_INITIALIZED;
   }
@@ -860,8 +859,8 @@ nsLDAPOperation::ModifyExt(const nsACString& aBaseDn,
   if (NS_FAILED(rv)) {
     (void)ldap_abandon_ext(mConnectionHandle, mMsgID, 0, 0);
     MOZ_LOG(gLDAPLogModule, mozilla::LogLevel::Debug,
-           ("nsLDAPOperation::AddExt(): abandoned due to rv %x",
-            rv));
+           ("nsLDAPOperation::AddExt(): abandoned due to rv %" PRIx32,
+            static_cast<uint32_t>(rv)));
   }
   return rv;
 }
@@ -876,7 +875,7 @@ nsLDAPOperation::Rename(const char *base,
                         LDAPControl **serverctrls,
                         LDAPControl **clientctrls)
 {
-  if (mMessageListener == 0) {
+  if (!mMessageListener) {
     NS_ERROR("nsLDAPOperation::Rename(): mMessageListener not set");
     return NS_ERROR_NOT_INITIALIZED;
   }
@@ -923,8 +922,8 @@ nsLDAPOperation::Rename(const nsACString& aBaseDn,
   if (NS_FAILED(rv)) {
     (void)ldap_abandon_ext(mConnectionHandle, mMsgID, 0, 0);
     MOZ_LOG(gLDAPLogModule, mozilla::LogLevel::Debug,
-           ("nsLDAPOperation::AddExt(): abandoned due to rv %x",
-            rv));
+           ("nsLDAPOperation::AddExt(): abandoned due to rv %" PRIx32,
+            static_cast<uint32_t>(rv)));
   }
   return rv;
 }
