@@ -3,30 +3,31 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource://gre/modules/PluralForm.jsm");
-Components.utils.import("resource:///modules/iteratorUtils.jsm");
-Components.utils.import("resource:///modules/mailServices.js");
-Components.utils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/PluralForm.jsm");
+ChromeUtils.import("resource:///modules/iteratorUtils.jsm");
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
 
-let gFilterListMsgWindow = null;
-let gCurrentFilterList;
-let gCurrentFolder;
-let gSelectedFolder = null;
+var gFilterListMsgWindow = null;
+var gCurrentFilterList;
+var gCurrentFolder;
+var gSelectedFolder = null;
 
-let gFilterListbox = null;
-let gEditButton = null;
-let gDeleteButton = null;
-let gTopButton = null;
-let gUpButton = null;
-let gDownButton = null;
-let gBottomButton = null;
-let gSearchBox = null;
-let gRunFiltersFolder = null;
-let gRunFiltersButton = null;
+var gFilterListbox = null;
+var gEditButton = null;
+var gDeleteButton = null;
+var gCopyToNewButton = null;
+var gTopButton = null;
+var gUpButton = null;
+var gDownButton = null;
+var gBottomButton = null;
+var gSearchBox = null;
+var gRunFiltersFolder = null;
+var gRunFiltersButton = null;
 
-let gFilterBundle = null;
+var gFilterBundle = null;
 
-const msgMoveMotion = {
+var msgMoveMotion = {
   Up     : 0,
   Down   : 1,
   Top    : 2,
@@ -85,7 +86,7 @@ var filterEditorQuitObserver = {
     // Check whether or not we want to veto the quit request (unless another
     // observer already did.
     if (aTopic == "quit-application-requested" &&
-        (aSubject instanceof Components.interfaces.nsISupportsPRBool) &&
+        (aSubject instanceof Ci.nsISupportsPRBool) &&
         !aSubject.data)
       aSubject.data = !onFilterClose();
   }
@@ -93,15 +94,16 @@ var filterEditorQuitObserver = {
 
 function onLoad()
 {
-    gFilterListMsgWindow = Components.classes["@mozilla.org/messenger/msgwindow;1"]
-                                     .createInstance(Components.interfaces.nsIMsgWindow);
+    gFilterListMsgWindow = Cc["@mozilla.org/messenger/msgwindow;1"]
+                             .createInstance(Ci.nsIMsgWindow);
     gFilterListMsgWindow.domWindow = window;
-    gFilterListMsgWindow.rootDocShell.appType = Components.interfaces.nsIDocShell.APP_TYPE_MAIL;
+    gFilterListMsgWindow.rootDocShell.appType = Ci.nsIDocShell.APP_TYPE_MAIL;
     gFilterListMsgWindow.statusFeedback = gStatusFeedback;
 
     gFilterListbox    = document.getElementById("filterList");
     gEditButton       = document.getElementById("editButton");
     gDeleteButton     = document.getElementById("deleteButton");
+    gCopyToNewButton  = document.getElementById("copyToNewButton");
     gTopButton        = document.getElementById("reorderTopButton");
     gUpButton         = document.getElementById("reorderUpButton");
     gDownButton       = document.getElementById("reorderDownButton");
@@ -116,7 +118,7 @@ function onLoad()
     processWindowArguments(window.arguments[0]);
 
     Services.obs.addObserver(filterEditorQuitObserver,
-                             "quit-application-requested", false);
+                             "quit-application-requested");
 }
 
 /**
@@ -154,6 +156,10 @@ function processWindowArguments(aArguments) {
     // to show potential new filters if we were called for refresh.
     rebuildFilterList();
   }
+
+  // If a specific filter was requested, try to select it.
+  if ("filter" in aArguments)
+    selectFilter(aArguments.filter);
 }
 
 /**
@@ -266,6 +272,8 @@ function toggleFilter(aFilterItem)
 
   // Now update the checkbox
   aFilterItem.childNodes[1].setAttribute("enabled", filter.enabled);
+  // For accessibility set the checked state on listitem
+  aFilterItem.setAttribute("aria-checked", filter.enabled);
 }
 
 // update the server menulist
@@ -276,6 +284,31 @@ function selectFolder(aFolder)
   setFolder(aFolder);
 }
 
+/**
+ * Selects a specific filter in the filter list.
+ * The listbox view is scrolled to the corresponding item.
+ *
+ * @param aFilter  The nsIMsgFilter to select.
+ *
+ * @return  true/false indicating whether the filter was found and selected.
+ */
+function selectFilter(aFilter) {
+  if (currentFilter() == aFilter)
+    return true;
+
+  resetSearchBox(aFilter);
+
+  let filterCount = gCurrentFilterList.filterCount;
+  for (let i = 0; i < filterCount; i++) {
+    if (gCurrentFilterList.getFilterAt(i) == aFilter) {
+      gFilterListbox.ensureIndexIsVisible(i);
+      gFilterListbox.selectedIndex = i;
+      return true;
+    }
+  }
+  return false;
+}
+
 /**
  * Returns the currently selected filter. If multiple filters are selected,
  * returns the first one. If none are selected, returns null.
@@ -306,7 +339,43 @@ function onEditFilter()
   }
 }
 
-function onNewFilter(emailAddress)
+/**
+ * Handler function for the 'New...' buttons.
+ * Opens the filter dialog for creating a new filter.
+ */
+function onNewFilter()
+{
+  calculatePositionAndShowCreateFilterDialog({});
+}
+
+/**
+ * Handler function for the 'Copy...' button.
+ * Opens the filter dialog for copying the selected filter.
+ */
+function onCopyToNewFilter()
+{
+  if (gCopyToNewButton.disabled)
+    return;
+
+  let selectedFilter = currentFilter();
+  if (!selectedFilter)
+    return;
+
+  let args = {copiedFilter: selectedFilter};
+
+  calculatePositionAndShowCreateFilterDialog(args);
+}
+
+/**
+ * Calculates the position for inserting the new filter,
+ * and then displays the create dialog.
+ *
+ * @param args  The object containing the arguments for the dialog,
+ *              passed to the filterEditorOnLoad() function.
+ *              It will be augmented with the insertion position
+ *              and global filters list properties by this function.
+ */
+function calculatePositionAndShowCreateFilterDialog(args)
 {
   let selectedFilter = currentFilter();
   // If no filter is selected use the first position.
@@ -322,18 +391,22 @@ function onNewFilter(emailAddress)
       }
     }
   }
+  args.filterPosition = position;
 
-  let args = {filterList: gCurrentFilterList, filterPosition: position};
+  args.filterList = gCurrentFilterList;
 
-  window.openDialog("chrome://messenger/content/FilterEditor.xul", "FilterEditor", "chrome,modal,titlebar,resizable,centerscreen", args);
+  window.openDialog("chrome://messenger/content/FilterEditor.xul", "FilterEditor",
+    "chrome,modal,titlebar,resizable,centerscreen", args);
 
   if ("refresh" in args && args.refresh) {
     // On success: reset the search box if necessary!
-    resetSearchBox(gCurrentFilterList.getFilterAt(position));
+    resetSearchBox(args.newFilter);
     rebuildFilterList();
 
     // Select the new filter, it is at the position of previous selection.
     gFilterListbox.selectItem(gFilterListbox.getItemAtIndex(position));
+    if (currentFilter() != args.newFilter)
+      Cu.reportError("Filter created at an unexpected position!");
   }
 }
 
@@ -453,11 +526,11 @@ function moveFilter(motion) {
       return;
     case msgMoveMotion.Up:
       relativeStep = -1;
-      moveFilterNative = Components.interfaces.nsMsgFilterMotion.up;
+      moveFilterNative = Ci.nsMsgFilterMotion.up;
       break;
     case msgMoveMotion.Down:
       relativeStep = +1;
-      moveFilterNative = Components.interfaces.nsMsgFilterMotion.down;
+      moveFilterNative = Ci.nsMsgFilterMotion.down;
       break;
   }
 
@@ -539,16 +612,16 @@ function runSelectedFilters()
   var folder = gRunFiltersFolder._folder || gRunFiltersFolder.selectedItem._folder;
 
   let filterList = MailServices.filters.getTempFilterList(folder);
-  let folders = Components.classes["@mozilla.org/array;1"]
-                          .createInstance(Components.interfaces.nsIMutableArray);
-  folders.appendElement(folder, false);
+  let folders = Cc["@mozilla.org/array;1"]
+                  .createInstance(Ci.nsIMutableArray);
+  folders.appendElement(folder);
 
   // make sure the tmp filter list uses the real filter list log stream
   filterList.logStream = gCurrentFilterList.logStream;
   filterList.loggingEnabled = gCurrentFilterList.loggingEnabled;
 
   let index = 0;
-  for each (let item in gFilterListbox.selectedItems) {
+  for (let item of gFilterListbox.selectedItems) {
     filterList.insertFilterAt(index++, item._filter);
   }
 
@@ -635,6 +708,7 @@ function rebuildFilterList()
     {
       // If there are not enough listitems in the list, create a new one.
       listitem = document.createElement("listitem");
+      listitem.setAttribute("role", "checkbox");
       nameCell = document.createElement("listcell");
       enabledCell = document.createElement("listcell");
       enabledCell.setAttribute("class", "listcell-iconic");
@@ -647,12 +721,15 @@ function rebuildFilterList()
       listitem.addEventListener("click", onFilterClick, true);
       listitem.addEventListener("dblclick", onFilterDoubleClick, true);
     }
+    // For accessibility set the label on listitem.
+    listitem.setAttribute("label", filter.filterName);
     // Set the listitem values to represent the current filter.
     nameCell.setAttribute("label", filter.filterName);
     enabledCell.setAttribute("enabled", filter.enabled);
+    listitem.setAttribute("aria-checked", filter.enabled);
     listitem._filter = filter;
 
-    if (selectedNames.indexOf(filter.filterName) != -1)
+    if (selectedNames.includes(filter.filterName))
       gFilterListbox.addItemToSelection(listitem);
 
     listitemIndex++;
@@ -712,6 +789,9 @@ function updateButtons()
     let disabled = !oneFilterSelected || currentFilter().unparseable;
     gEditButton.disabled = disabled;
 
+    // "copy" is the same as "edit"
+    gCopyToNewButton.disabled = disabled;
+
     // "delete" only disabled when no filters are selected
     gDeleteButton.disabled = !numFiltersSelected;
 
@@ -769,8 +849,8 @@ function getServerThatCanHaveFilters()
     // If it cannot, check all accounts to find a server
     // that can have filters.
     let allServers = MailServices.accounts.allServers;
-    for (let currentServer in fixIterator(allServers,
-                                          Components.interfaces.nsIMsgIncomingServer))
+    for (let currentServer of fixIterator(allServers,
+                                          Ci.nsIMsgIncomingServer))
     {
       if (currentServer.canHaveFilters)
         return currentServer;
@@ -825,7 +905,7 @@ function onFilterListKeyPress(aEvent)
   else if (!aEvent.ctrlKey && !aEvent.altKey && !aEvent.metaKey) {
     switch (aEvent.charCode) {
       case KeyEvent.DOM_VK_SPACE:
-        for each (let item in gFilterListbox.selectedItems) {
+        for (let item of gFilterListbox.selectedItems) {
           toggleFilter(item);
         }
         break;
@@ -865,7 +945,7 @@ function getFirstFolder(msgFolder)
     {
       // Find Inbox for imap and pop; show Choose Folder if not found or
       // Local Folders or any other account type.
-      const nsMsgFolderFlags = Components.interfaces.nsMsgFolderFlags;
+      const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
       // If inbox does not exist then return null.
       return msgFolder.getFolderWithFlags(nsMsgFolderFlags.Inbox);
     }
