@@ -2,11 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource://gre/modules/PluralForm.jsm");
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource://gre/modules/PluralForm.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource:///modules/mailServices.js");
 
-const mailtolength = 7;
+var mailtolength = 7;
 
 /**
  * Function to change the highlighted row back to the row that is currently
@@ -25,6 +25,8 @@ function RestoreSelectionWithoutContentLoad(tree)
     view.selection = realSelection;
     // replay any calls to adjustSelection, this handles suppression.
     transientSelection.replayAdjustSelectionLog(realSelection);
+    // Avoid possible cycle leaks.
+    gRightMouseButtonSavedSelection.view = null;
     gRightMouseButtonSavedSelection = null;
 
     if (tree)
@@ -69,11 +71,22 @@ function fillMailContextMenu(event)
     }
   }
 
+  goUpdateCommand('cmd_killThread');
+  goUpdateCommand('cmd_killSubthread');
+  goUpdateCommand('cmd_watchThread');
+
   goUpdateCommand('cmd_printpreview');
   goUpdateCommand('cmd_print');
 
   updateCheckedStateForIgnoreAndWatchThreadCmds();
 
+  // Show "Edit Draft Message" menus only in a drafts folder; otherwise hide them.
+  showCommandInSpecialFolder("cmd_editDraftMsg",
+                             Ci.nsMsgFolderFlags.Drafts);
+  // Show "New Message from Template" menus only in a templates folder; otherwise hide them.
+  showCommandInSpecialFolder("cmd_newMsgFromTemplate",
+                             Ci.nsMsgFolderFlags.Templates);
+
   gContextMenu = new nsContextMenu(event.target, event.shiftKey);
   return gContextMenu.shouldDisplay;
 }
@@ -101,8 +114,8 @@ function FillMessageIdContextMenu(messageIdNode)
 
 function CopyMessageId(messageId)
 {
-   var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
-                             .getService(Components.interfaces.nsIClipboardHelper);
+   var clipboard = Cc["@mozilla.org/widget/clipboardhelper;1"]
+                     .getService(Ci.nsIClipboardHelper);
 
    clipboard.copyString(messageId);
 }
@@ -127,7 +140,7 @@ function GetMessageIdFromNode(messageIdNode, cleanMessageId)
 function OpenBrowserWithMessageId(messageId)
 {
   var browserURL = Services.prefs.getComplexValue("mailnews.messageid_browser.url",
-                                                  Components.interfaces.nsIPrefLocalizedString).data;
+                                                  Ci.nsIPrefLocalizedString).data;
   browserURL = browserURL.replace(/%mid/, messageId);
   try
   {
@@ -135,8 +148,8 @@ function OpenBrowserWithMessageId(messageId)
   }
   catch (ex)
   {
-    Components.utils.reportError("Failed to open message-id in browser; " +
-                                 "browserURL=" + browserURL);
+    Cu.reportError("Failed to open message-id in browser; " +
+                   "browserURL=" + browserURL);
   }
 }
 
@@ -166,7 +179,7 @@ function OpenMessageForMessageId(messageId)
     for (let i = 0; i < allServers.length && !messageHeader; i++)
     {
       let currentServer =
-        allServers.queryElementAt(i, Components.interfaces.nsIMsgIncomingServer);
+        allServers.queryElementAt(i, Ci.nsIMsgIncomingServer);
       if (currentServer && startServer != currentServer &&
           currentServer.canSearchMessages && !currentServer.isDeferredTo)
       {
@@ -255,7 +268,7 @@ function SearchForMessageIdInSubFolder(folder, messageId)
   {
     // search in current folder
     var currentFolder =
-      subFolders.getNext().QueryInterface(Components.interfaces.nsIMsgFolder);
+      subFolders.getNext().QueryInterface(Ci.nsIMsgFolder);
     messageHeader = CheckForMessageIdInFolder(currentFolder, messageId);
 
     // search in its subfolder
@@ -280,11 +293,11 @@ function CheckForMessageIdInFolder(folder, messageId)
   }
   catch (ex)
   {
-    Components.utils.reportError("Failed to find message-id in folder; " +
-                                 "messageId=" + messageId);
+    Cu.reportError("Failed to find message-id in folder; " +
+                   "messageId=" + messageId);
   }
 
-  const nsMsgFolderFlags = Components.interfaces.nsMsgFolderFlags;
+  const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
   if (!MailServices.mailSession.IsFolderOpenInWindow(folder) &&
       !(folder.flags & (nsMsgFolderFlags.Trash | nsMsgFolderFlags.Inbox)))
   {
@@ -330,7 +343,7 @@ function fillFolderPaneContextMenu(aEvent)
   var numSelected = folders.length;
 
   function checkIsVirtualFolder(folder) {
-    const kVirtualFlag = Components.interfaces.nsMsgFolderFlags.Virtual;
+    const kVirtualFlag = Ci.nsMsgFolderFlags.Virtual;
     return folder.flags & kVirtualFlag;
   }
   var haveAnyVirtualFolders = folders.some(checkIsVirtualFolder);
@@ -348,9 +361,18 @@ function fillFolderPaneContextMenu(aEvent)
   function checkCanSubscribeToFolder(folder) {
     if (checkIsVirtualFolder(folder))
       return false;
+
+    // All feed account folders, besides Trash, are subscribable.
+    if (folder.server.type == "rss" &&
+        !(folder.flags & nsMsgFolderFlags.Trash))
+      return true;
+
+    // We only want the subscribe item on the account nodes.
+    if (!folder.isServer)
+      return false;
+
     return folder.server.type == "nntp" ||
-           folder.server.type == "imap" ||
-           folder.server.type == "rss";
+           folder.server.type == "imap";
   }
   var haveOnlySubscribableFolders = folders.every(checkCanSubscribeToFolder);
 
@@ -366,7 +388,7 @@ function fillFolderPaneContextMenu(aEvent)
   var haveOnlyMailFolders = folders.every(checkIsMailFolder);
 
   function checkCanGetMessages(folder) {
-    const kTrashFlag = Components.interfaces.nsMsgFolderFlags.Trash;
+    const kTrashFlag = Ci.nsMsgFolderFlags.Trash;
     return (folder.isServer && (folder.server.type != "none")) ||
            checkIsNewsgroup(folder) ||
            ((folder.server.type == "rss") && !folder.isSpecialFolder(kTrashFlag, true) &&
@@ -400,6 +422,33 @@ function fillFolderPaneContextMenu(aEvent)
                 selectedServers.length == selectedFoldersThatCanGetMessages.length) ||
                selectedFoldersThatCanGetMessages.length == numSelected);
 
+  // --- Set up the pause all updates menu item.
+  // Show only if a feed server.
+  let showPausedAll = numSelected == 1 && folders[0].isServer &&
+                      FeedMessageHandler.isFeedFolder(folders[0]);
+  ShowMenuItem("folderPaneContext-pauseAllUpdates", showPausedAll);
+  // Adjust the checked state on the menu item.
+  if (showPausedAll) {
+    let menuitem = document.getElementById("folderPaneContext-pauseAllUpdates");
+    let optionsAcct = FeedUtils.getOptionsAcct(folders[0].server);
+    menuitem.setAttribute("checked", !optionsAcct.doBiff);
+  }
+
+  // --- Set up the pause single folder subscription updates menu item.
+  // Show only if a feed folder. A folder without feeds, even though it may
+  // have subfolders, is not eligible.
+  let showPaused = numSelected == 1 && !folders[0].isServer &&
+                   FeedUtils.getFeedUrlsInFolder(folders[0]);
+  ShowMenuItem("folderPaneContext-pauseUpdates", showPaused);
+  // Adjust the checked state on the menu item.
+  if (showPaused) {
+    let menuitem = document.getElementById("folderPaneContext-pauseUpdates");
+    let properties = FeedUtils.getFolderProperties(folders[0]);
+    let paused = properties.includes("isPaused");
+    menuitem.setAttribute("checked", paused);
+  }
+  ShowMenuItem("folderPaneContext-sepPause", showPausedAll || showPaused);
+
   // --- Set up new sub/folder menu item.
   if (numSelected == 1) {
     let showNewFolderItem =
@@ -453,7 +502,7 @@ function fillFolderPaneContextMenu(aEvent)
 
   // --- Set up the compact folder menu item.
   function checkCanCompactFolder(folder) {
-    const kVirtualFlag = Components.interfaces.nsMsgFolderFlags.Virtual;
+    const kVirtualFlag = Ci.nsMsgFolderFlags.Virtual;
     return folder.canCompact && !(folder.flags & kVirtualFlag) &&
            folder.isCommandEnabled("cmd_compactFolder");
   }
@@ -471,7 +520,7 @@ function fillFolderPaneContextMenu(aEvent)
                numSelected == 1 && !folders[0].isServer);
   if (numSelected == 1 && !folders[0].isServer)
   {
-    const kFavoriteFlag = Components.interfaces.nsMsgFolderFlags.Favorite;
+    const kFavoriteFlag = Ci.nsMsgFolderFlags.Favorite;
      // Adjust the checked state on the menu item.
     document.getElementById("folderPaneContext-favoriteFolder")
             .setAttribute("checked", folders[0].getFlag(kFavoriteFlag));
@@ -590,13 +639,13 @@ function addEmail()
 
 function composeEmailTo()
 {
-  let fields = Components.classes["@mozilla.org/messengercompose/composefields;1"]
-                         .createInstance(Components.interfaces.nsIMsgCompFields);
-  let params = Components.classes["@mozilla.org/messengercompose/composeparams;1"]
-                         .createInstance(Components.interfaces.nsIMsgComposeParams);
+  let fields = Cc["@mozilla.org/messengercompose/composefields;1"]
+                 .createInstance(Ci.nsIMsgCompFields);
+  let params = Cc["@mozilla.org/messengercompose/composeparams;1"]
+                 .createInstance(Ci.nsIMsgComposeParams);
   fields.to = getEmail(gContextMenu.linkURL);
-  params.type = Components.interfaces.nsIMsgCompType.New;
-  params.format = Components.interfaces.nsIMsgCompFormat.Default;
+  params.type = Ci.nsIMsgCompType.New;
+  params.format = Ci.nsIMsgCompFormat.Default;
   if (gFolderDisplay.displayedFolder) {
     params.identity = accountManager.getFirstIdentityForServer(
                         gFolderDisplay.displayedFolder.server);
@@ -618,8 +667,8 @@ function getEmail (url)
   // Let's try to unescape it using a character set
   try {
     var characterSet = gContextMenu.target.ownerDocument.characterSet;
-    const textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"]
-                                 .getService(Components.interfaces.nsITextToSubURI);
+    const textToSubURI = Cc["@mozilla.org/intl/texttosuburi;1"]
+                                 .getService(Ci.nsITextToSubURI);
     addresses = textToSubURI.unEscapeURIForUI(characterSet, addresses);
   }
   catch(ex) {
@@ -635,12 +684,12 @@ function CopyMessageUrl()
     var server = hdr.folder.server;
 
     // TODO let backend construct URL and return as attribute
-    var url = (server.socketType == Components.interfaces.nsMsgSocketType.SSL) ?
+    var url = (server.socketType == Ci.nsMsgSocketType.SSL) ?
               "snews://" : "news://";
     url += server.hostName + ":" + server.port + "/" + hdr.messageId;
 
-    var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
-                              .getService(Components.interfaces.nsIClipboardHelper);
+    var clipboard = Cc["@mozilla.org/widget/clipboardhelper;1"]
+                      .getService(Ci.nsIClipboardHelper);
     clipboard.copyString(url);
   }
   catch (ex) {
