@@ -1,15 +1,15 @@
-/** ***** BEGIN LICENSE BLOCK *****
+/**
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource:///modules/mailServices.js");
-Components.utils.import("resource:///modules/appIdleManager.js");
-Components.utils.import("resource:///modules/MailUtils.js");
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
-Components.utils.import("resource:///modules/gloda/log4moz.js");
-Components.utils.import("resource:///modules/gloda/public.js");
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource:///modules/appIdleManager.js");
+ChromeUtils.import("resource:///modules/MailUtils.js");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/gloda/log4moz.js");
+ChromeUtils.import("resource:///modules/gloda/public.js");
 
 //This file stores variables common to mail windows
 var messenger;
@@ -39,7 +39,7 @@ function OnMailWindowUnload()
   MailServices.mailSession.RemoveMsgWindow(msgWindow);
   // the tabs have the FolderDisplayWidget close their 'messenger' instances for us
 
-  window.QueryInterface(Components.interfaces.nsIDOMChromeWindow)
+  window.QueryInterface(Ci.nsIDOMChromeWindow)
         .browserDOMWindow = null;
 
   msgWindow.closeWindow();
@@ -48,45 +48,133 @@ function OnMailWindowUnload()
   msgWindow.notificationCallbacks = null;
   gDBView = null;
   window.MsgStatusFeedback.unload();
-  Components.classes["@mozilla.org/activity-manager;1"]
-            .getService(Components.interfaces.nsIActivityManager)
-            .removeListener(window.MsgStatusFeedback);
+  Cc["@mozilla.org/activity-manager;1"]
+    .getService(Ci.nsIActivityManager)
+    .removeListener(window.MsgStatusFeedback);
+}
+
+
+/**
+ * When copying/dragging, convert imap/mailbox URLs of images into data URLs so
+ * that the images can be accessed in a paste elsewhere.
+ */
+function onCopyOrDragStart(e) {
+  let sourceDoc = getBrowser().contentDocument;
+  if (e.target.ownerDocument != sourceDoc) {
+    return; // We're only interested if this is in the message content.
+  }
+
+  let imgMap = new Map(); // Mapping img.src -> dataURL.
+
+  // For copy, the data of what is to be copied is not accessible at this point.
+  // Figure out what images are a) part of the selection and b) visible in
+  // the current document. If their source isn't http or data already, convert
+  // them to data URLs.
+
+  let selection = sourceDoc.getSelection();
+  let draggedImg = selection.isCollapsed ? e.target : null;
+  for (let img of sourceDoc.images) {
+    if (/^(https?|data):/.test(img.src)) {
+      continue;
+    }
+
+    if (img.naturalWidth == 0) { // Broken/inaccessible image then...
+      continue;
+    }
+
+    if (!draggedImg && !selection.containsNode(img, true)) {
+      continue;
+    }
+
+    let style = window.getComputedStyle(img);
+    if (style.display == "none" || style.visibility == "hidden") {
+      continue;
+    }
+
+    // Do not convert if the image is specifically flagged to not snarf.
+    if (img.getAttribute("moz-do-not-send") == "true") {
+      continue;
+    }
+
+    // We don't need to wait for the image to load. If it isn't already loaded
+    // in the source document, we wouldn't want it anyway.
+    let canvas = sourceDoc.createElement("canvas");
+    canvas.width = img.width;
+    canvas.height = img.height;
+    canvas.getContext("2d").drawImage(img, 0, 0, img.width, img.height);
+
+    let type = /\.jpe?g$/i.test(img.src) ? "image/jpg" : "image/png";
+    imgMap.set(img.src, canvas.toDataURL(type));
+  }
+
+  if (imgMap.size == 0) {
+    // Nothing that needs converting!
+    return;
+  }
+
+  let clonedSelection = draggedImg ? draggedImg.cloneNode(false) :
+    selection.getRangeAt(0).cloneContents();
+  let div = sourceDoc.createElement("div");
+  div.appendChild(clonedSelection);
+
+  let images = div.querySelectorAll("img");
+  for (let img of images) {
+    if (!imgMap.has(img.src)) {
+      continue;
+    }
+    img.src = imgMap.get(img.src);
+  }
+
+  let html = div.innerHTML;
+  let parserUtils = Cc["@mozilla.org/parserutils;1"]
+                      .getService(Ci.nsIParserUtils);
+  let plain = parserUtils.convertToPlainText(html,
+    Ci.nsIDocumentEncoder.OutputForPlainTextClipboardCopy, 0);
+  if ("clipboardData" in e) { // copy
+    e.clipboardData.setData("text/html", html);
+    e.clipboardData.setData("text/plain", plain);
+    e.preventDefault();
+  }
+  else if ("dataTransfer" in e) { // drag
+    e.dataTransfer.setData("text/html", html);
+    e.dataTransfer.setData("text/plain", plain);
+  }
 }
 
 function CreateMailWindowGlobals()
 {
   // get the messenger instance
-  messenger = Components.classes["@mozilla.org/messenger;1"]
-                        .createInstance(Components.interfaces.nsIMessenger);
+  messenger = Cc["@mozilla.org/messenger;1"]
+                .createInstance(Ci.nsIMessenger);
 
-  window.addEventListener("blur", appIdleManager.onBlur, false);
-  window.addEventListener("focus", appIdleManager.onFocus, false);
+  window.addEventListener("blur", appIdleManager.onBlur);
+  window.addEventListener("focus", appIdleManager.onFocus);
 
   //Create windows status feedback
   // set the JS implementation of status feedback before creating the c++ one..
   window.MsgStatusFeedback = new nsMsgStatusFeedback();
   // double register the status feedback object as the xul browser window implementation
-  window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-        .getInterface(Components.interfaces.nsIWebNavigation)
-        .QueryInterface(Components.interfaces.nsIDocShellTreeItem).treeOwner
-        .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-        .getInterface(Components.interfaces.nsIXULWindow)
+  window.QueryInterface(Ci.nsIInterfaceRequestor)
+        .getInterface(Ci.nsIWebNavigation)
+        .QueryInterface(Ci.nsIDocShellTreeItem).treeOwner
+        .QueryInterface(Ci.nsIInterfaceRequestor)
+        .getInterface(Ci.nsIXULWindow)
         .XULBrowserWindow = window.MsgStatusFeedback;
 
-  window.QueryInterface(Components.interfaces.nsIDOMChromeWindow)
+  window.QueryInterface(Ci.nsIDOMChromeWindow)
         .browserDOMWindow = new nsBrowserAccess();
 
-  statusFeedback = Components.classes["@mozilla.org/messenger/statusfeedback;1"]
-                             .createInstance(Components.interfaces.nsIMsgStatusFeedback);
+  statusFeedback = Cc["@mozilla.org/messenger/statusfeedback;1"]
+                     .createInstance(Ci.nsIMsgStatusFeedback);
   statusFeedback.setWrappedStatusFeedback(window.MsgStatusFeedback);
 
-  Components.classes["@mozilla.org/activity-manager;1"]
-            .getService(Components.interfaces.nsIActivityManager)
-            .addListener(window.MsgStatusFeedback);
+  Cc["@mozilla.org/activity-manager;1"]
+    .getService(Ci.nsIActivityManager)
+    .addListener(window.MsgStatusFeedback);
 
   //Create message window object
-  msgWindow = Components.classes["@mozilla.org/messenger/msgwindow;1"]
-                        .createInstance(Components.interfaces.nsIMsgWindow);
+  msgWindow = Cc["@mozilla.org/messenger/msgwindow;1"]
+                .createInstance(Ci.nsIMsgWindow);
 
   accountManager = MailServices.accounts;
 
@@ -105,9 +193,12 @@ function InitMsgWindow()
   messagepane.docShell.allowAuth = false;
   messagepane.docShell.allowDNSPrefetch = false;
   msgWindow.rootDocShell.allowAuth = true;
-  msgWindow.rootDocShell.appType = Components.interfaces.nsIDocShell.APP_TYPE_MAIL;
+  msgWindow.rootDocShell.appType = Ci.nsIDocShell.APP_TYPE_MAIL;
   // Ensure we don't load xul error pages into the main window
   msgWindow.rootDocShell.useErrorPages = false;
+
+  document.addEventListener("copy", onCopyOrDragStart, true);
+  document.addEventListener("dragstart", onCopyOrDragStart, true);
 }
 
 // We're going to implement our status feedback for the mail window in JS now.
@@ -116,6 +207,7 @@ function InitMsgWindow()
 function nsMsgStatusFeedback()
 {
   this._statusText = document.getElementById("statusText");
+  this._statusPanel = document.getElementById("statusbar-display");
   this._progressBar = document.getElementById("statusbar-icon");
   this._progressBarContainer = document.getElementById("statusbar-progresspanel");
   this._throbber = document.getElementById("throbber-box");
@@ -129,6 +221,7 @@ nsMsgStatusFeedback.prototype =
 {
   // Document elements.
   _statusText: null,
+  _statusPanel: null,
   _progressBar: null,
   _progressBarContainer: null,
   _throbber: null,
@@ -159,7 +252,12 @@ nsMsgStatusFeedback.prototype =
   },
 
   setOverLink: function(link, context) {
-    this._statusText.label = link;
+    if (!document.getElementById("status-bar").hidden) {
+      this._statusText.label = link;
+    } else {
+      // Statusbar invisible: Show link in statuspanel instead.
+      this._statusPanel.label = link;
+    }
   },
 
   // Called before links are navigated to to allow us to retarget them if needed.
@@ -168,14 +266,14 @@ nsMsgStatusFeedback.prototype =
   },
 
   QueryInterface: function(iid) {
-    if (iid.equals(Components.interfaces.nsIMsgStatusFeedback) ||
-        iid.equals(Components.interfaces.nsIXULBrowserWindow) ||
-        iid.equals(Components.interfaces.nsIActivityMgrListener) ||
-        iid.equals(Components.interfaces.nsIActivityListener) ||
-        iid.equals(Components.interfaces.nsISupportsWeakReference) ||
-        iid.equals(Components.interfaces.nsISupports))
+    if (iid.equals(Ci.nsIMsgStatusFeedback) ||
+        iid.equals(Ci.nsIXULBrowserWindow) ||
+        iid.equals(Ci.nsIActivityMgrListener) ||
+        iid.equals(Ci.nsIActivityListener) ||
+        iid.equals(Ci.nsISupportsWeakReference) ||
+        iid.equals(Ci.nsISupports))
       return this;
-    throw Components.results.NS_NOINTERFACE;
+    throw Cr.NS_NOINTERFACE;
   },
 
   // nsIMsgStatusFeedback implementation.
@@ -281,7 +379,7 @@ nsMsgStatusFeedback.prototype =
 
       this._activeProcesses.forEach(function (element) {
           if (element.state ==
-              Components.interfaces.nsIActivityProcess.STATE_INPROGRESS &&
+              Ci.nsIActivityProcess.STATE_INPROGRESS &&
               element.percentComplete != -1) {
             currentProgress += element.percentComplete;
             ++progressCount;
@@ -330,10 +428,10 @@ nsMsgStatusFeedback.prototype =
     // ignore Gloda activity for status bar purposes
     if (aActivity.initiator == Gloda)
       return;
-    if (aActivity instanceof Components.interfaces.nsIActivityEvent) {
+    if (aActivity instanceof Ci.nsIActivityEvent) {
       this.showStatusString(aActivity.displayText);
     }
-    else if (aActivity instanceof Components.interfaces.nsIActivityProcess) {
+    else if (aActivity instanceof Ci.nsIActivityProcess) {
       this._activeProcesses.push(aActivity);
       aActivity.addListener(this);
       this.startMeteors();
@@ -364,7 +462,7 @@ nsMsgStatusFeedback.prototype =
     // only go as far as our process.
     for (var i = 0; i < index; ++i) {
       if (this._activeProcesses[i].status ==
-          Components.interfaces.nsIActivityProcess.STATE_INPROGRESS)
+          Ci.nsIActivityProcess.STATE_INPROGRESS)
         break;
     }
 
@@ -392,10 +490,10 @@ nsMsgWindowCommands.prototype =
 {
   QueryInterface : function(iid)
   {
-    if (iid.equals(Components.interfaces.nsIMsgWindowCommands) ||
-        iid.equals(Components.interfaces.nsISupports))
+    if (iid.equals(Ci.nsIMsgWindowCommands) ||
+        iid.equals(Ci.nsISupports))
       return this;
-    throw Components.results.NS_NOINTERFACE;
+    throw Cr.NS_NOINTERFACE;
   },
 
   selectFolder: function(folderUri)
@@ -425,8 +523,7 @@ nsMsgWindowCommands.prototype =
 function loadStartPage(aForce)
 {
   // If the preference isn't enabled, then don't load anything.
-  if (!aForce &&
-      !Application.prefs.getValue("mailnews.start_page.enabled", false))
+  if (!aForce && !Services.prefs.getBoolPref("mailnews.start_page.enabled"))
     return;
 
   gMessageNotificationBar.clearMsgNotifications();
@@ -438,7 +535,7 @@ function loadStartPage(aForce)
       GetMessagePaneFrame().location.href = uri.spec;
     }
     catch (e) {
-      Components.utils.reportError(e);
+      Cu.reportError(e);
     }
   }
   else
@@ -500,8 +597,8 @@ function OpenInboxForServer(server)
 
 /** Update state of zoom type (text vs. full) menu item. */
 function UpdateFullZoomMenu() {
-  var menuItem = document.getElementById("menu_fullZoomToggle");
-  menuItem.setAttribute("checked", !ZoomManager.useFullZoom);
+  let cmdItem = document.getElementById("cmd_fullZoomToggle");
+  cmdItem.setAttribute("checked", !ZoomManager.useFullZoom);
 }
 
 /**
@@ -527,10 +624,10 @@ BadCertHandler.prototype = {
 
   // nsISupports
   QueryInterface: function(iid) {
-    if (!iid.equals(Components.interfaces.nsIBadCertListener2) &&
-      !iid.equals(Components.interfaces.nsIInterfaceRequestor) &&
-      !iid.equals(Components.interfaces.nsISupports))
-      throw Components.results.NS_ERROR_NO_INTERFACE;
+    if (!iid.equals(Ci.nsIBadCertListener2) &&
+      !iid.equals(Ci.nsIInterfaceRequestor) &&
+      !iid.equals(Ci.nsISupports))
+      throw Cr.NS_ERROR_NO_INTERFACE;
     return this;
   }
 };
@@ -550,24 +647,23 @@ function InformUserOfCertError(socketInfo, status, targetSite)
 function nsBrowserAccess() { }
 
 nsBrowserAccess.prototype = {
-  QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIBrowserDOMWindow]),
-
-  openURI: function (aURI, aOpener, aWhere, aContext) {
-    const nsIBrowserDOMWindow = Components.interfaces.nsIBrowserDOMWindow;
-    let isExternal = aContext == nsIBrowserDOMWindow.OPEN_EXTERNAL;
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsIBrowserDOMWindow]),
 
+  openURI: function (aURI, aOpener, aWhere, aFlags, aTriggeringPrincipal = null) {
+    const nsIBrowserDOMWindow = Ci.nsIBrowserDOMWindow;
+    let isExternal = !!(aFlags & nsIBrowserDOMWindow.OPEN_EXTERNAL);
     if (isExternal && aURI && aURI.schemeIs("chrome")) {
-      Application.console.log("use -chrome command-line option to load external chrome urls\n");
+      Services.console.logStringMessage("use -chrome command-line option to load external chrome urls\n");
       return null;
     }
 
     let newWindow = null;
     let loadflags = isExternal ?
-      Components.interfaces.nsIWebNavigation.LOAD_FLAGS_FROM_EXTERNAL :
-      Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
+      Ci.nsIWebNavigation.LOAD_FLAGS_FROM_EXTERNAL :
+      Ci.nsIWebNavigation.LOAD_FLAGS_NONE;
 
     if (aWhere != nsIBrowserDOMWindow.OPEN_NEWTAB)
-      Application.console.log("Opening a URI in something other than a new tab is not supported, opening in new tab instead");
+      Services.console.logStringMessage("Opening a URI in something other than a new tab is not supported, opening in new tab instead");
 
     let win, needToFocusWin;
 
@@ -584,7 +680,7 @@ nsBrowserAccess.prototype = {
       throw("Couldn't get a suitable window for openURI");
 
     let loadInBackground =
-      Application.prefs.getValue("browser.tabs.loadDivertedInBackground", false);
+      Services.prefs.getBoolPref("browser.tabs.loadDivertedInBackground");
 
     let tabmail = win.document.getElementById("tabmail");
     let clickHandler = null;
@@ -597,17 +693,18 @@ nsBrowserAccess.prototype = {
                                                 clickHandler: clickHandler});
 
     newWindow = newTab.browser.docShell
-                      .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                      .getInterface(Components.interfaces.nsIDOMWindow);
+                      .QueryInterface(Ci.nsIInterfaceRequestor)
+                      .getInterface(Ci.nsIDOMWindow);
     try {
       if (aURI) {
+        let referrer = null;
         if (aOpener) {
           let location = aOpener.location;
-          let referrer = Services.io.newURI(location, null, null);
+          referrer = Services.io.newURI(location);
         }
-        newWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                 .getInterface(Components.interfaces.nsIWebNavigation)
-                 .loadURI(aURI.spec, loadflags, referrer, null, null);
+        newWindow.QueryInterface(Ci.nsIInterfaceRequestor)
+                 .getInterface(Ci.nsIWebNavigation)
+                 .loadURI(aURI.spec, loadflags, referrer, null, null, aTriggeringPrincipal);
       }
       if (needToFocusWin || (!loadInBackground && isExternal))
         newWindow.focus();
@@ -625,6 +722,9 @@ function MailSetCharacterSet(aEvent) {
   if (aEvent.target.hasAttribute("charset")) {
     msgWindow.mailCharacterSet = aEvent.target.getAttribute("charset");
     msgWindow.charsetOverride = true;
+    gMessageDisplay.keyForCharsetOverride =
+      ("messageKey" in gMessageDisplay.displayedMessage ?
+       gMessageDisplay.displayedMessage.messageKey : null);
   }
   messenger.setDocumentCharset(msgWindow.mailCharacterSet);
 }
