@@ -2,16 +2,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
-Components.utils.import("resource:///modules/gloda/gloda.js");
-Components.utils.import("resource:///modules/gloda/connotent.js");
-Components.utils.import("resource:///modules/gloda/mimemsg.js");
-Components.utils.import("resource:///modules/displayNameUtils.js");
-Components.utils.import("resource:///modules/mailServices.js");
-Components.utils.import("resource:///modules/templateUtils.js");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/gloda/gloda.js");
+ChromeUtils.import("resource:///modules/gloda/connotent.js");
+ChromeUtils.import("resource:///modules/gloda/mimemsg.js");
+ChromeUtils.import("resource:///modules/displayNameUtils.js");
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource:///modules/templateUtils.js");
 
-var gMessenger = Components.classes["@mozilla.org/messenger;1"]
-                           .createInstance(Components.interfaces.nsIMessenger);
+var gMessenger = Cc["@mozilla.org/messenger;1"]
+                   .createInstance(Ci.nsIMessenger);
 
 // Set up our string formatter for localizing strings.
 XPCOMUtils.defineLazyGetter(this, "formatString", function() {
@@ -60,8 +60,8 @@ LimitIterator.prototype = {
   },
 };
 
-const JS_HAS_SYMBOLS = typeof Symbol === "function";
-const ITERATOR_SYMBOL = JS_HAS_SYMBOLS ? Symbol.iterator : "@@iterator";
+var JS_HAS_SYMBOLS = typeof Symbol === "function";
+var ITERATOR_SYMBOL = JS_HAS_SYMBOLS ? Symbol.iterator : "@@iterator";
 
 /**
  * Iterate over the array until we hit the end or the maximum length,
@@ -193,7 +193,7 @@ MultiMessageSummary.prototype = {
    */
   makeSummaryItem: function(aMsgOrThread, aOptions) {
     let message, thread, numUnread, isStarred, tags;
-    if (aMsgOrThread instanceof Components.interfaces.nsIMsgDBHdr) {
+    if (aMsgOrThread instanceof Ci.nsIMsgDBHdr) {
       thread = null;
       message = aMsgOrThread;
 
@@ -246,7 +246,7 @@ MultiMessageSummary.prototype = {
                                 formatString("noSubject");
       subjectNode.addEventListener("click", function() {
         window.top.gFolderDisplay.selectMessages(thread);
-      }, false);
+      });
       itemHeaderNode.appendChild(subjectNode);
 
       if (thread && thread.length > 1) {
@@ -275,7 +275,7 @@ MultiMessageSummary.prototype = {
       authorNode.addEventListener("click", function() {
         window.top.gFolderDisplay.selectMessage(message);
         window.top.document.getElementById("messagepane").focus();
-      }, false);
+      });
       itemHeaderNode.appendChild(authorNode);
     }
 
@@ -298,10 +298,14 @@ MultiMessageSummary.prototype = {
         if (meta.author)
           authorNode.textContent = meta.author;
       }, false, {saneBodySize: true});
-    } catch (e if e.result == Components.results.NS_ERROR_FAILURE) {
-      // Offline messages generate exceptions, which is unfortunate.  When
-      // that's fixed, this code should adapt. XXX
-      snippetNode.textContent = "...";
+    } catch (e) {
+      if (e.result == Cr.NS_ERROR_FAILURE) {
+        // Offline messages generate exceptions, which is unfortunate.  When
+        // that's fixed, this code should adapt. XXX
+        snippetNode.textContent = "...";
+      } else {
+        throw e;
+      }
     }
 
     return row;
@@ -412,7 +416,7 @@ MultiMessageSummary.prototype = {
   _processItems: function(aItems) {
     let knownMessageNodes = new Map();
 
-    for (let [,glodaMsg] in Iterator(aItems)) {
+    for (let glodaMsg of aItems) {
       // Unread and starred will get set if any of the messages in a collapsed
       // thread qualify.  The trick here is that we may get multiple items
       // corresponding to the same thread (and hence DOM node), so we need to
@@ -505,13 +509,18 @@ ThreadSummarizer.prototype = {
     let messageList = document.getElementById("message_list");
 
     // Remove all ignored messages from summarization.
-    let summarizedMessages = [msg for (msg of aMessages) if (!msg.isKilled)];
+    let summarizedMessages = [];
+    for (let message of aMessages) {
+      if (!message.isKilled) {
+        summarizedMessages.push(message);
+      }
+    }
     let ignoredCount = aMessages.trueLength - summarizedMessages.length;
 
     // Summarize the selected messages.
     let subject = null;
     let maxCountExceeded = false;
-    for (let [i, msgHdr] in Iterator(summarizedMessages)) {
+    for (let [i, msgHdr] of summarizedMessages.entries()) {
       if (i > this.kMaxSummarizedMessages) {
         summarizedMessages.length = i;
         maxCountExceeded = true;
@@ -611,7 +620,7 @@ MultipleSelectionSummarizer.prototype = {
     // Summarize the selected messages by thread.
     let maxCountExceeded = false;
     let messageCount = 0;
-    for (let [i, msgs] in Iterator(threads)) {
+    for (let [i, msgs] of threads.entries()) {
       messageCount += msgs.length;
       if (messageCount > this.kMaxSummarizedMessages ||
           i > this.kMaxSummarizedThreads) {
