@@ -2,17 +2,20 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+"use strict";
+
 /* A note to the curious: a large portion of this code was copied over from
  * mozilla/browser/base/content/browser.js
  */
 
-Components.utils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
 
-#ifdef MOZ_CRASHREPORTER
-XPCOMUtils.defineLazyServiceGetter(this, "gCrashReporter",
-                                   "@mozilla.org/xre/app-info;1",
-                                   "nsICrashReporter");
-#endif
+if (AppConstants.MOZ_CRASHREPORTER) {
+  XPCOMUtils.defineLazyServiceGetter(this, "gCrashReporter",
+                                     "@mozilla.org/xre/app-info;1",
+                                     "nsICrashReporter");
+}
 
 function getPluginInfo(pluginElement)
 {
@@ -38,7 +41,7 @@ function getPluginInfo(pluginElement)
       }
     }
 
-    tagMimetype = pluginElement.QueryInterface(Components.interfaces.nsIObjectLoadingContent)
+    tagMimetype = pluginElement.QueryInterface(Ci.nsIObjectLoadingContent)
                  .actualType;
 
     if (tagMimetype == "") {
@@ -84,7 +87,7 @@ var gPluginHandler = {
 
   get CrashSubmit() {
     delete this.CrashSubmit;
-    Components.utils.import("resource://gre/modules/CrashSubmit.jsm", this);
+    ChromeUtils.import("resource://gre/modules/CrashSubmit.jsm", this);
     return this.CrashSubmit;
   },
 
@@ -158,10 +161,9 @@ var gPluginHandler = {
     return true;
   },
 
-  addLinkClickCallback: function ph_addLinkClickCallback(linkNode, callbackName /*callbackArgs...*/) {
+  addLinkClickCallback: function ph_addLinkClickCallback(linkNode, callbackName, ...callbackArgs) {
     // XXX just doing (callback)(arg) was giving a same-origin error. bug?
     let self = this;
-    let callbackArgs = Array.prototype.slice.call(arguments).slice(2);
     linkNode.addEventListener("click",
                               function(evt) {
                                 if (!evt.isTrusted)
@@ -190,8 +192,6 @@ var gPluginHandler = {
 
   // Helper to get the binding handler type from a plugin object
   _getBindingType : function(plugin) {
-    let Ci = Components.interfaces;
-
     if (!(plugin instanceof Ci.nsIObjectLoadingContent))
       return null;
 
@@ -224,7 +224,7 @@ var gPluginHandler = {
     let doc = plugin.ownerDocument;
 
     // We're expecting the target to be a plugin.
-    if (!(plugin instanceof Components.interfaces.nsIObjectLoadingContent))
+    if (!(plugin instanceof Ci.nsIObjectLoadingContent))
       return;
 
     let eventType = event.type;
@@ -257,9 +257,7 @@ var gPluginHandler = {
 
       case "PluginBlocklisted":
       case "PluginOutdated":
-#ifdef XP_MACOSX
       case "npapi-carbon-event-model-failure":
-#endif
         self.pluginUnavailable(plugin, eventType);
         break;
 
@@ -332,8 +330,6 @@ var gPluginHandler = {
 
   // Event listener for blocklisted/outdated/carbonFailure plugins.
   pluginUnavailable: function(plugin, eventType) {
-    let Cc = Components.classes;
-    let Ci = Components.interfaces;
     var tabmail = document.getElementById('tabmail');
     let browser = tabmail.getBrowserForDocument(plugin.ownerDocument
                                                 .defaultView).browser;
@@ -360,13 +356,12 @@ var gPluginHandler = {
       return true;
     }
 
-#ifdef XP_MACOSX
     function carbonFailurePluginsRestartBrowser()
     {
       // Notify all windows that an application quit has been requested.
       let cancelQuit = Cc["@mozilla.org/supports-PRBool;1"].
                          createInstance(Ci.nsISupportsPRBool);
-      Services.obs.notifyObservers(cancelQuit, "quit-application-requested", null);
+      Services.obs.notifyObservers(cancelQuit, "quit-application-requested");
 
       // Something aborted the quit process.
       if (cancelQuit.data)
@@ -376,14 +371,13 @@ var gPluginHandler = {
                             Ci.nsIAppStartup.eRestart |
                             Ci.nsIAppStartup.eAttemptQuit);
     }
-#endif
 
     let messengerBundle = document.getElementById("bundle_messenger");
 
     let notifications = {
       PluginBlocklisted : {
         barID: "blocked-plugins",
-        iconURL: "chrome://mozapps/skin/plugins/notifyPluginBlocked.png",
+        iconURL: "chrome://mozapps/skin/plugins/pluginGeneric-16.png",
         message: messengerBundle.getString("blockedpluginsMessage.title"),
         buttons: [{
           label: messengerBundle.getString("blockedpluginsMessage.infoButton.label"),
@@ -400,7 +394,7 @@ var gPluginHandler = {
       },
       PluginOutdated: {
         barID: "outdated-plugins",
-        iconURL: "chrome://mozapps/skin/plugins/notifyPluginOutdated.png",
+        iconURL: "chrome://mozapps/skin/plugins/pluginGeneric-16.png",
         message: messengerBundle.getString("outdatedpluginsMessage.title"),
         buttons: [{
           label: messengerBundle.getString("outdatedpluginsMessage.updateButton.label"),
@@ -408,11 +402,12 @@ var gPluginHandler = {
           popup: null,
           callback: showOutdatedPluginsInfo
         }],
-      },
-#ifdef XP_MACOSX
-      "npapi-carbon-event-model-failure": {
+      }
+    };
+    if (AppConstants.platform == "macosx") {
+      notifications["npapi-carbon-event-model-failure"] = {
         barID: "carbon-failure-plugins",
-        iconURL: "chrome://mozapps/skin/plugins/notifyPluginGeneric.png",
+        iconURL: "chrome://mozapps/skin/plugins/pluginGeneric-16.png",
         message: messengerBundle.getString("carbonFailurePluginsMessage.message"),
         buttons: [{
           label: messengerBundle.getString("carbonFailurePluginsMessage.restartButton.label"),
@@ -421,15 +416,14 @@ var gPluginHandler = {
           callback: carbonFailurePluginsRestartBrowser
         }],
       }
-#endif
-    };
+    }
 
     // If there is already an outdated plugin notification then do nothing
     if (outdatedNotification)
       return;
 
-#ifdef XP_MACOSX
-    if (eventType == "npapi-carbon-event-model-failure") {
+    if ((AppConstants.platform == "macosx") &&
+        (eventType == "npapi-carbon-event-model-failure")) {
       if (Services.prefs.getBoolPref("plugins.hide_infobar_for_carbon_failure_plugin"))
         return;
 
@@ -444,7 +438,6 @@ var gPluginHandler = {
       if (!macutils.isUniversalBinary)
         eventType = "PluginNotFound";
     }
-#endif
 
     if (eventType == "PluginBlocklisted") {
       if (blockedNotification)
@@ -472,23 +465,23 @@ var gPluginHandler = {
   // are dispatched to individual plugin instances.
   pluginCrashed : function(subject, topic, data) {
     let propertyBag = subject;
-    if (!(propertyBag instanceof Components.interfaces.nsIPropertyBag2) ||
-        !(propertyBag instanceof Components.interfaces.nsIWritablePropertyBag2))
+    if (!(propertyBag instanceof Ci.nsIPropertyBag2) ||
+        !(propertyBag instanceof Ci.nsIWritablePropertyBag2))
      return;
 
-#ifdef MOZ_CRASHREPORTER
-    let pluginDumpID = propertyBag.getPropertyAsAString("pluginDumpID");
-    let browserDumpID = propertyBag.getPropertyAsAString("browserDumpID");
-    let shouldSubmit = gCrashReporter.submitReports;
-    let doPrompt = true; // XXX followup to get via gCrashReporter
+    if (AppConstants.MOZ_CRASHREPORTER) {
+      let pluginDumpID = propertyBag.getPropertyAsAString("pluginDumpID");
+      let browserDumpID = propertyBag.getPropertyAsAString("browserDumpID");
+      let shouldSubmit = gCrashReporter.submitReports;
+      let doPrompt = true; // XXX followup to get via gCrashReporter
 
-    // Submit automatically when appropriate.
-    if (pluginDumpID && shouldSubmit && !doPrompt) {
-      this.submitReport(pluginDumpID, browserDumpID);
-      // Submission is async, so we can't easily show failure UI.
-      propertyBag.setPropertyAsBool("submittedCrashReport", true);
+      // Submit automatically when appropriate.
+      if (pluginDumpID && shouldSubmit && !doPrompt) {
+        this.submitReport(pluginDumpID, browserDumpID);
+        // Submission is async, so we can't easily show failure UI.
+        propertyBag.setPropertyAsBool("submittedCrashReport", true);
+      }
     }
-#endif
   },
 
   // Crashed-plugin event listener. Called for every instance of a
@@ -523,74 +516,75 @@ var gPluginHandler = {
     let doc = plugin.ownerDocument;
     let overlay = this.getPluginUI(plugin, "main");
     let statusDiv = this.getPluginUI(plugin, "submitStatus");
-#ifdef MOZ_CRASHREPORTER
-    let status;
 
-    // Determine which message to show regarding crash reports.
-    if (submittedReport) { // submitReports && !doPrompt, handled in observer
-      status = "submitted";
+    if (AppConstants.MOZ_CRASHREPORTER) {
+      let status;
+
+      // Determine which message to show regarding crash reports.
+      if (submittedReport) { // submitReports && !doPrompt, handled in observer
+        status = "submitted";
+      }
+      else if (!submitReports && !doPrompt) {
+        status = "noSubmit";
+      }
+      else { // doPrompt
+        status = "please";
+        // XXX can we make the link target actually be blank?
+        this.getPluginUI(plugin, "submitButton").addEventListener("click",
+          function (event) {
+            if (event.button != 0 || !event.isTrusted)
+              return;
+            this.submitReport(pluginDumpID, browserDumpID, plugin);
+            pref.setBoolPref("", optInCB.checked);
+          }.bind(this));
+        let optInCB = this.getPluginUI(plugin, "submitURLOptIn");
+        let pref = Services.prefs.getBranch("dom.ipc.plugins.reportCrashURL");
+        optInCB.checked = pref.getBoolPref("");
+      }
+
+      // If we don't have a minidumpID, we can't (or didn't) submit anything.
+      // This can happen if the plugin is killed from the task manager.
+      if (!pluginDumpID) {
+        status = "noReport";
+      }
+
+      statusDiv.setAttribute("status", status);
+
+      let helpIcon = this.getPluginUI(plugin, "helpIcon");
+      this.addLinkClickCallback(helpIcon, "openPluginCrashHelpPage");
+
+      // If we're showing the link to manually trigger report submission, we'll
+      // want to be able to update all the instances of the UI for this crash to
+      // show an updated message when a report is submitted.
+      if (doPrompt) {
+        let observer = {
+          QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver,
+                                                 Ci.nsISupportsWeakReference]),
+          observe : function(subject, topic, data) {
+            let propertyBag = subject;
+            if (!(propertyBag instanceof Ci.nsIPropertyBag2))
+              return;
+            // Ignore notifications for other crashes.
+            if (propertyBag.get("minidumpID") != pluginDumpID)
+              return;
+            statusDiv.setAttribute("status", data);
+          },
+
+          handleEvent : function(event) {
+              // Not expected to be called, just here for the closure.
+          }
+        };
+
+        // Use a weak reference, so we don't have to remove it...
+        Services.obs.addObserver(observer, "crash-report-status", true);
+        // ...alas, now we need something to hold a strong reference to prevent
+        // it from being GC. But I don't want to manually manage the reference's
+        // lifetime (which should be no greater than the page).
+        // Clever solution? Use a closue with an event listener on the document.
+        // When the doc goes away, so do the listener references and the closure.
+        doc.addEventListener("mozCleverClosureHack", observer);
+      }
     }
-    else if (!submitReports && !doPrompt) {
-      status = "noSubmit";
-    }
-    else { // doPrompt
-      status = "please";
-      // XXX can we make the link target actually be blank?
-      this.getPluginUI(plugin, "submitButton").addEventListener("click",
-        function (event) {
-	  if (event.button != 0 || !event.isTrusted)
-            return;
-          this.submitReport(pluginDumpID, browserDumpID, plugin);
-          pref.setBoolPref("", optInCB.checked);
-	}.bind(this));
-      let optInCB = this.getPluginUI(plugin, "submitURLOptIn");
-      let pref = Services.prefs.getBranch("dom.ipc.plugins.reportCrashURL");
-      optInCB.checked = pref.getBoolPref("");
-    }
-
-    // If we don't have a minidumpID, we can't (or didn't) submit anything.
-    // This can happen if the plugin is killed from the task manager.
-    if (!pluginDumpID) {
-      status = "noReport";
-    }
-
-    statusDiv.setAttribute("status", status);
-
-    let helpIcon = this.getPluginUI(plugin, "helpIcon");
-    this.addLinkClickCallback(helpIcon, "openPluginCrashHelpPage");
-
-    // If we're showing the link to manually trigger report submission, we'll
-    // want to be able to update all the instances of the UI for this crash to
-    // show an updated message when a report is submitted.
-    if (doPrompt) {
-      let observer = {
-        QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIObserver,
-                                               Components.interfaces.nsISupportsWeakReference]),
-        observe : function(subject, topic, data) {
-          let propertyBag = subject;
-          if (!(propertyBag instanceof Components.interfaces.nsIPropertyBag2))
-            return;
-          // Ignore notifications for other crashes.
-          if (propertyBag.get("minidumpID") != pluginDumpID)
-            return;
-          statusDiv.setAttribute("status", data);
-        },
-
-        handleEvent : function(event) {
-            // Not expected to be called, just here for the closure.
-        }
-      };
-
-      // Use a weak reference, so we don't have to remove it...
-      Services.obs.addObserver(observer, "crash-report-status", true);
-      // ...alas, now we need something to hold a strong reference to prevent
-      // it from being GC. But I don't want to manually manage the reference's
-      // lifetime (which should be no greater than the page).
-      // Clever solution? Use a closue with an event listener on the document.
-      // When the doc goes away, so do the listener references and the closure.
-      doc.addEventListener("mozCleverClosureHack", observer, false);
-    }
-#endif
 
     let crashText = this.getPluginUI(plugin, "crashedText");
     crashText.textContent = messageString;
@@ -640,7 +634,7 @@ var gPluginHandler = {
 
       // Configure the notification bar
       let priority = notificationBox.PRIORITY_WARNING_MEDIUM;
-      let iconURL = "chrome://mozapps/skin/plugins/notifyPluginCrashed.png";
+      let iconURL = "chrome://mozapps/skin/plugins/pluginGeneric-16.png";
       let reloadLabel = messengerBundle.getString("crashedpluginsMessage.reloadButton.label");
       let reloadKey   = messengerBundle.getString("crashedpluginsMessage.reloadButton.accesskey");
       let submitLabel = messengerBundle.getString("crashedpluginsMessage.submitButton.label");
@@ -652,19 +646,19 @@ var gPluginHandler = {
         popup: null,
         callback: function() { browser.reload(); },
       }];
-#ifdef MOZ_CRASHREPORTER
-      let submitButton = {
-        label: submitLabel,
-        accessKey: submitKey,
-        popup: null,
-          callback: function() { gPluginHandler.submitReport(pluginDumpID, browserDumpID); },
-      };
-      if (pluginDumpID)
-        buttons.push(submitButton);
-#endif
+      if (AppConstants.MOZ_CRASHREPORTER) {
+        let submitButton = {
+          label: submitLabel,
+          accessKey: submitKey,
+          popup: null,
+            callback: function() { gPluginHandler.submitReport(pluginDumpID, browserDumpID); },
+        };
+        if (pluginDumpID)
+          buttons.push(submitButton);
+      }
 
       notification = notificationBox.appendNotification(messageString, "plugin-crashed",
-                                                            iconURL, priority, buttons);
+                                                        iconURL, priority, buttons);
 
       // Add the "learn more" link.
       let XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
@@ -677,10 +671,10 @@ var gPluginHandler = {
       let description = notification.ownerDocument.getAnonymousElementByAttribute(notification, "anonid", "messageText");
       description.appendChild(link);
 
-      // Remove the notfication when the page is reloaded.
+      // Remove the notification when the page is reloaded.
       doc.defaultView.top.addEventListener("unload", function() {
         notificationBox.removeNotification(notification);
-      }, false);
+      });
     }
 
   }
