@@ -2,10 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource://gre/modules/AddonManager.jsm");
-Components.utils.import("resource:///modules/StringBundle.js");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/AddonManager.jsm");
+ChromeUtils.import("resource:///modules/StringBundle.js");
+ChromeUtils.import("resource://gre/modules/BrowserUtils.jsm");
 
 function tabProgressListener(aTab, aStartsBlank) {
   this.mTab = aTab;
@@ -65,7 +66,7 @@ tabProgressListener.prototype =
       // by a pushState or a replaceState. See bug 550565.
       if (aWebProgress.isLoadingDocument &&
           !(this.mBrowser.docShell.loadType &
-            Components.interfaces.nsIDocShell.LOAD_CMD_PUSHSTATE))
+            Ci.nsIDocShell.LOAD_CMD_PUSHSTATE))
         this.mBrowser.mIconURL = null;
 
       var location = aLocationURI ? aLocationURI.spec : "";
@@ -93,8 +94,8 @@ tabProgressListener.prototype =
 
     var oldBlank = this.mBlank;
 
-    const nsIWebProgressListener = Components.interfaces.nsIWebProgressListener;
-    const nsIChannel = Components.interfaces.nsIChannel;
+    const nsIWebProgressListener = Ci.nsIWebProgressListener;
+    const nsIChannel = Ci.nsIChannel;
     let tabmail = document.getElementById("tabmail");
 
     if (aStateFlags & nsIWebProgressListener.STATE_START) {
@@ -152,12 +153,12 @@ tabProgressListener.prototype =
       this.mProgressListener.onRefreshAttempted(aWebProgress, aURI, aDelay,
         aSameURI);
   },
-  QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIWebProgressListener,
-                                         Components.interfaces.nsIWebProgressListener2,
-                                         Components.interfaces.nsISupportsWeakReference])
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsIWebProgressListener,
+                                         Ci.nsIWebProgressListener2,
+                                         Ci.nsISupportsWeakReference])
 };
 
-const DOMLinkHandler = {
+var DOMLinkHandler = {
   handleEvent: function (event) {
     switch (event.type) {
     case "DOMLinkAdded":
@@ -171,7 +172,7 @@ const DOMLinkHandler = {
     if (!link || !link.ownerDocument || !rel || !link.href)
       return;
 
-    if (rel.split(/\s+/).indexOf("icon") != -1) {
+    if (rel.split(/\s+/).includes("icon")) {
       if (!Services.prefs.getBoolPref("browser.chrome.site_icons"))
         return;
 
@@ -196,17 +197,17 @@ const DOMLinkHandler = {
         // something we shouldn't. Firefox does this, so we're doing the same.
           try {
             Services.scriptSecurityManager.checkLoadURIWithPrincipal(targetDoc.nodePrincipal, uri,
-                                           Components.interfaces.nsIScriptSecurityManager.DISALLOW_SCRIPT);
+                                           Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
           }
           catch (ex) {
             return;
           }
       }
 
-      const nsIContentPolicy = Components.interfaces.nsIContentPolicy;
+      const nsIContentPolicy = Ci.nsIContentPolicy;
 
       try {
-        var contentPolicy = Components.classes["@mozilla.org/layout/content-policy;1"]
+        var contentPolicy = Cc["@mozilla.org/layout/content-policy;1"]
           .getService(nsIContentPolicy);
       }
       catch (e) {
@@ -238,12 +239,12 @@ const DOMLinkHandler = {
   }
 };
 
-const kTelemetryPrompted    = "toolkit.telemetry.prompted";
-const kTelemetryEnabled     = "toolkit.telemetry.enabled";
-const kTelemetryRejected    = "toolkit.telemetry.rejected";
-const kTelemetryServerOwner = "toolkit.telemetry.server_owner";
+var kTelemetryPrompted    = "toolkit.telemetry.prompted";
+var kTelemetryEnabled     = "toolkit.telemetry.enabled";
+var kTelemetryRejected    = "toolkit.telemetry.rejected";
+var kTelemetryServerOwner = "toolkit.telemetry.server_owner";
 // This is used to reprompt/renotify users when privacy message changes
-const kTelemetryPromptRev   = 2;
+var kTelemetryPromptRev   = 2;
 
 var contentTabBaseType = {
   inContentWhitelist: ['about:addons', 'about:preferences'],
@@ -276,7 +277,7 @@ var contentTabBaseType = {
                                      aTab.titleListener, true);
     aTab.browser.removeEventListener("DOMWindowClose",
                                      aTab.closeListener, true);
-    aTab.browser.removeEventListener("DOMLinkAdded", DOMLinkHandler, false);
+    aTab.browser.removeEventListener("DOMLinkAdded", DOMLinkHandler);
     gPluginHandler.removeEventListeners(aTab.browser);
     aTab.browser.webProgress.removeProgressListener(aTab.filter);
     aTab.filter.removeProgressListener(aTab.progressListener);
@@ -284,11 +285,13 @@ var contentTabBaseType = {
   },
 
   saveTabState: function onSaveTabState(aTab) {
-    aTab.browser.setAttribute("type", "content-targetable");
+    aTab.browser.setAttribute("type", "content");
+    aTab.browser.removeAttribute("primary");
   },
 
   showTab: function onShowTab(aTab) {
-    aTab.browser.setAttribute("type", "content-primary");
+    aTab.browser.setAttribute("type", "content");
+    aTab.browser.setAttribute("primary", "true");
   },
 
   getBrowser: function getBrowser(aTab) {
@@ -296,7 +299,7 @@ var contentTabBaseType = {
   },
 
   hideChromeForLocation: function hideChromeForLocation(aLocation) {
-    return this.inContentWhitelist.indexOf(aLocation) != -1;
+    return this.inContentWhitelist.includes(aLocation);
   },
 
   /* _setUpLoadListener attaches a load listener to the tab browser that
@@ -425,7 +428,8 @@ var contentTabBaseType = {
         PrintUtils.showPageSetup();
         break;
       case "cmd_print":
-        PrintUtils.print();
+        let browser = this.getBrowser(aTab);
+        PrintUtils.printWindow(browser.outerWindowID, browser);
         break;
       // XXX print preview not currently supported - bug 497994 to implement.
       //case "cmd_printpreview":
@@ -446,20 +450,118 @@ var specialTabs = {
   get _protocolSvc() {
     delete this._protocolSvc;
     return this._protocolSvc =
-      Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
-                .getService(Components.interfaces.nsIExternalProtocolService);
+      Cc["@mozilla.org/uriloader/external-protocol-service;1"]
+        .getService(Ci.nsIExternalProtocolService);
   },
 
   get mFaviconService() {
     delete this.mFaviconService;
     return this.mFaviconService =
-      Components.classes["@mozilla.org/browser/favicon-service;1"]
-                .getService(Components.interfaces.nsIFaviconService);
+      Cc["@mozilla.org/browser/favicon-service;1"]
+        .getService(Ci.nsIFaviconService);
+  },
+
+  /**
+   * We use an html image node to test the favicon, errors are well returned.
+   * Returning a url for nsITreeView.getImageSrc() will not indicate any
+   * error, and setAndFetchFaviconForPage() can't be used to detect
+   * failed icons due to Bug 740457. This also ensures 301 Moved or
+   * redirected urls will work (they won't otherwise in getImageSrc).
+   *
+   * @param  function successFunc - caller's success function.
+   * @param  function errorFunc   - caller's error function.
+   * @param  string iconUrl       - url to load.
+   * @return HTMLImageElement imageNode
+   */
+  loadFaviconImageNode: function(successFunc, errorFunc, iconUrl) {
+    let HTMLNS = "http://www.w3.org/1999/xhtml";
+    let imageNode = document.createElementNS(HTMLNS, "img");
+    imageNode.style.visibility = "collapse";
+    imageNode.addEventListener("load", event => successFunc(event, iconUrl),
+                               {capture: false, once: true});
+    imageNode.addEventListener("error", event => errorFunc(event, iconUrl),
+                               {capture: false, once: true});
+    imageNode.src = iconUrl;
+    return imageNode;
+  },
+
+  /**
+   * Favicon request timeout, 20 seconds.
+   */
+  REQUEST_TIMEOUT: 20 * 1000,
+
+  /**
+   * Get the favicon by parsing for <link rel=""> with "icon" from the page's
+   * dom <head>.
+   *
+   * @param  string aUrl          - a url from whose homepage to get a favicon.
+   * @param  function aCallback   - callback.
+   */
+  getFaviconFromPage: function(aUrl, aCallback) {
+    let url, uri;
+    try {
+      url = Services.io.newURI(aUrl).prePath;
+      uri = Services.io.newURI(url);
+    }
+    catch (ex) {
+      if (aCallback)
+        aCallback("");
+      return;
+    }
+
+    let onLoadSuccess = (aEvent => {
+      let iconUri = Services.io.newURI(aEvent.target.src);
+      specialTabs.mFaviconService.setAndFetchFaviconForPage(
+        uri, iconUri, false,
+        specialTabs.mFaviconService.FAVICON_LOAD_NON_PRIVATE,
+        null, Services.scriptSecurityManager.getSystemPrincipal());
+
+      if (aCallback)
+        aCallback(iconUri.spec);
+    });
+
+    let onDownloadError = (aEvent => {
+      if (aCallback)
+        aCallback("");
+    });
+
+    let onDownload = (aEvent => {
+      let request = aEvent.target;
+      let dom = request.response;
+      if (request.status != 200 || !(dom instanceof Ci.nsIDOMHTMLDocument)) {
+        onDownloadError(aEvent);
+        return;
+      }
+
+      let iconUri;
+      let linkNode = dom.head.querySelector('link[rel="shortcut icon"],' +
+                                            'link[rel="icon"]');
+      let href = linkNode ? linkNode.href : null;
+      try {
+        iconUri = Services.io.newURI(href);
+      }
+      catch (ex) {
+        onDownloadError(aEvent);
+        return;
+      }
+
+      specialTabs.loadFaviconImageNode(onLoadSuccess, onDownloadError,
+                                       iconUri.spec);
+    });
+
+    let request = new XMLHttpRequest();
+    request.open("GET", url, true);
+    request.responseType = "document";
+    request.onload = onDownload;
+    request.onerror = onDownloadError;
+    request.timeout = this.REQUEST_TIMEOUT;
+    request.ontimeout = onDownloadError;
+    request.send(null);
   },
 
   // This will open any special tabs if necessary on startup.
   openSpecialTabsOnStartup: function() {
-    window.addEventListener("unload", specialTabs.onunload, false);
+    window.addEventListener("unload", specialTabs.onunload);
 
     let browser = document.getElementById("dummycontentbrowser");
 
@@ -469,9 +571,9 @@ var specialTabs = {
     // Wire up session and global history before any possible
     // progress notifications for back/forward button updating
     browser.webNavigation.sessionHistory =
-      Components.classes["@mozilla.org/browser/shistory;1"]
-                .createInstance(Components.interfaces.nsISHistory);
-    Services.obs.addObserver(browser, "browser:purge-session-history", false);
+      Cc["@mozilla.org/browser/shistory;1"]
+        .createInstance(Ci.nsISHistory);
+    Services.obs.addObserver(browser, "browser:purge-session-history");
 
     // remove the disablehistory attribute so the browser cleans up, as
     // though it had done this work itself
@@ -481,10 +583,10 @@ var specialTabs = {
     try {
       browser.docShell.useGlobalHistory = true;
     } catch(ex) {
-      Components.utils.reportError("Places database may be locked: " + ex);
+      Cu.reportError("Places database may be locked: " + ex);
     }
 
-    Services.obs.addObserver(specialTabs, "mail-startup-done", false);
+    Services.obs.addObserver(specialTabs, "mail-startup-done");
 
     let tabmail = document.getElementById('tabmail');
 
@@ -576,8 +678,11 @@ var specialTabs = {
       // As we're opening this tab, showTab may not get called, so set
       // the type according to if we're opening in background or not.
       let background = ("background" in aArgs) && aArgs.background;
-      aTab.browser.setAttribute("type", background ? "content-targetable" :
-                                                     "content-primary");
+      aTab.browser.setAttribute("type", "content");
+      if (background)
+        aTab.browser.removeAttribute("primary");
+      else
+        aTab.browser.setAttribute("primary", "true");
 
       aTab.browser.setAttribute("id", "contentTabBrowser" + this.lastBrowserId);
 
@@ -590,7 +695,7 @@ var specialTabs = {
       // image attribute and just show the default tab icon.
       aTab.tabNode.setAttribute("onerror", "this.removeAttribute('image');");
 
-      aTab.browser.addEventListener("DOMLinkAdded", DOMLinkHandler, false);
+      aTab.browser.addEventListener("DOMLinkAdded", DOMLinkHandler);
       gPluginHandler.addEventListeners(aTab.browser);
 
       // Now initialise the find bar.
@@ -614,15 +719,15 @@ var specialTabs = {
       }
 
       // Create a filter and hook it up to our browser
-      let filter = Components.classes["@mozilla.org/appshell/component/browser-status-filter;1"]
-                             .createInstance(Components.interfaces.nsIWebProgress);
+      let filter = Cc["@mozilla.org/appshell/component/browser-status-filter;1"]
+                     .createInstance(Ci.nsIWebProgress);
       aTab.filter = filter;
-      aTab.browser.webProgress.addProgressListener(filter, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
+      aTab.browser.webProgress.addProgressListener(filter, Ci.nsIWebProgress.NOTIFY_ALL);
 
       // Wire up a progress listener to the filter for this browser
       aTab.progressListener = new tabProgressListener(aTab, false);
 
-      filter.addProgressListener(aTab.progressListener, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
+      filter.addProgressListener(aTab.progressListener, Ci.nsIWebProgress.NOTIFY_ALL);
 
       if ("onListener" in aArgs)
         aArgs.onListener(aTab.browser, aTab.progressListener);
@@ -697,7 +802,7 @@ var specialTabs = {
       savedAppVersion = Services.prefs.getCharPref(prefstring);
     } catch (ex) {}
 
-    let currentApplicationVersion = Application.version;
+    let currentApplicationVersion = Services.appinfo.version;
 
     if (savedAppVersion == "ignore")
       return [null, this.splitVersion(currentApplicationVersion)];
@@ -712,20 +817,20 @@ var specialTabs = {
    * Shows the what's new page in a content tab.
    */
   showWhatsNewPage: function onShowWhatsNewPage() {
-    let um = Components.classes["@mozilla.org/updates/update-manager;1"]
-               .getService(Components.interfaces.nsIUpdateManager);
+    let um = Cc["@mozilla.org/updates/update-manager;1"]
+               .getService(Ci.nsIUpdateManager);
 
     try {
       // If the updates.xml file is deleted then getUpdateAt will throw.
       var update = um.getUpdateAt(0)
-                     .QueryInterface(Components.interfaces.nsIPropertyBag);
+                     .QueryInterface(Ci.nsIPropertyBag);
     } catch (x) {
       Cu.reportError("Unable to find update: " + x);
       return;
     }
 
     let actions = update.getProperty("actions");
-    if (actions && actions.indexOf("silent") != -1)
+    if (actions && actions.includes("silent"))
       return;
 
     openWhatsNew();
@@ -813,7 +918,7 @@ var specialTabs = {
       notification.parentNode.removeNotification(notification, true);
       // Add a new notification to that tab, with no "Learn more" link
       notifyBox.appendNotification(telemetryText, "telemetry", null, notifyBox.PRIORITY_INFO_LOW, buttons);
-    }, false);
+    });
 
     let description = notification.ownerDocument.getAnonymousElementByAttribute(notification, "anonid", "messageText");
     description.appendChild(link);
@@ -1037,7 +1142,7 @@ var specialTabs = {
       // image attribute and just show the default tab icon.
       aTab.tabNode.setAttribute("onerror", "this.removeAttribute('image');");
 
-      aTab.browser.addEventListener("DOMLinkAdded", DOMLinkHandler, false);
+      aTab.browser.addEventListener("DOMLinkAdded", DOMLinkHandler);
 
 
       aTab.browser.setAttribute("id", "chromeTabBrowser" + this.lastBrowserId);
@@ -1072,7 +1177,7 @@ var specialTabs = {
                                        aTab.titleListener, true);
       aTab.browser.removeEventListener("DOMWindowClose",
                                        aTab.closeListener, true);
-      aTab.browser.removeEventListener("DOMLinkAdded", DOMLinkHandler, false);
+      aTab.browser.removeEventListener("DOMLinkAdded", DOMLinkHandler);
       aTab.browser.destroy();
     },
     saveTabState: function onSaveTabState(aTab) {
@@ -1148,7 +1253,8 @@ var specialTabs = {
           PrintUtils.showPageSetup();
           break;
         case "cmd_print":
-          PrintUtils.print();
+          let browser = this.getBrowser(aTab);
+          PrintUtils.printWindow(browser.outerWindowID, browser);
           break;
         // XXX print preview not currently supported - bug 497994 to implement.
         //case "cmd_printpreview":
@@ -1201,18 +1307,14 @@ var specialTabs = {
       return;
 
     Services.obs.removeObserver(specialTabs, "mail-startup-done");
-    Services.obs.addObserver(this.xpInstallObserver, "addon-install-disabled",
-                             false);
-    Services.obs.addObserver(this.xpInstallObserver, "addon-install-blocked",
-                             false);
-    Services.obs.addObserver(this.xpInstallObserver, "addon-install-failed",
-                             false);
-    Services.obs.addObserver(this.xpInstallObserver, "addon-install-complete",
-                             false);
+    Services.obs.addObserver(this.xpInstallObserver, "addon-install-disabled");
+    Services.obs.addObserver(this.xpInstallObserver, "addon-install-blocked");
+    Services.obs.addObserver(this.xpInstallObserver, "addon-install-failed");
+    Services.obs.addObserver(this.xpInstallObserver, "addon-install-complete");
   },
 
   onunload: function () {
-    window.removeEventListener("unload", specialTabs.onunload, false);
+    window.removeEventListener("unload", specialTabs.onunload);
 
     Services.obs.removeObserver(specialTabs.xpInstallObserver,
                                 "addon-install-disabled");
@@ -1226,11 +1328,10 @@ var specialTabs = {
 
   xpInstallObserver: {
     observe: function (aSubject, aTopic, aData) {
-      const Ci = Components.interfaces;
       let brandBundle = document.getElementById("bundle_brand");
       let messengerBundle = document.getElementById("bundle_messenger");
 
-      let installInfo = aSubject.QueryInterface(Ci.amIWebInstallInfo);
+      let installInfo = aSubject.wrappedJSObject;
       let browser = installInfo.browser;
       let notificationBox = getNotificationBox(browser.contentWindow);
       let notificationID = aTopic;
@@ -1289,7 +1390,7 @@ var specialTabs = {
         break;
       case "addon-install-failed":
         // XXX TODO This isn't terribly ideal for the multiple failure case
-        for (let [, install] in Iterator(installInfo.installs)) {
+        for (let install of installInfo.installs) {
           let host = (installInfo.originatingURI instanceof Ci.nsIStandardURL) &&
                       installInfo.originatingURI.host;
           if (!host)
@@ -1333,7 +1434,7 @@ var specialTabs = {
             accessKey: messengerBundle.getString("addonInstallRestartButton.accesskey"),
             popup: null,
             callback: function() {
-              Application.restart();
+              BrowserUtils.restartApplication();
             }
           }];
         }
@@ -1348,7 +1449,7 @@ var specialTabs = {
               // installs.
               let types = {};
               let bestType = null;
-              for (let [, install] in Iterator(installInfo.installs)) {
+              for (let install of installInfo.installs) {
                 if (install.type in types)
                   types[install.type]++;
                 else
@@ -1386,8 +1487,8 @@ var specialTabs = {
    */
   _shouldLoadFavIcon: function shouldLoadFavIcon(aURI) {
     return (aURI &&
-            Application.prefs.getValue("browser.chrome.site_icons", false) &&
-            Application.prefs.getValue("browser.chrome.favicons", false) &&
+            Services.prefs.getBoolPref("browser.chrome.site_icons") &&
+            Services.prefs.getBoolPref("browser.chrome.favicons") &&
             ("schemeIs" in aURI) &&
             (aURI.schemeIs("http") || aURI.schemeIs("https")));
   },
@@ -1438,7 +1539,8 @@ var specialTabs = {
     if (aIcon && this.mFaviconService)
       this.mFaviconService.setAndFetchFaviconForPage(
         aTab.browser.currentURI, makeURI(aIcon), false,
-        this.mFaviconService.FAVICON_LOAD_NON_PRIVATE);
+        this.mFaviconService.FAVICON_LOAD_NON_PRIVATE,
+        null, aTab.browser.contentPrincipal);
 
     // Save this off so we know about it later,
     aTab.browser.mIconURL = aIcon;
