@@ -9,8 +9,10 @@ var gThreadPaneCommandUpdater = null;
 function ThreadPaneOnClick(event)
 {
   // We only care about button 0 (left click) events.
-  if (event.button != 0)
+  if (event.button != 0) {
+    event.stopPropagation();
     return;
+  }
 
   // We already handle marking as read/flagged/junk cyclers in nsMsgDBView.cpp
   // so all we need to worry about here is doubleclicks and column header. We
@@ -75,52 +77,36 @@ function ThreadPaneOnClick(event)
 
 function HandleColumnClick(columnID)
 {
-  const columnMap = {dateCol: 'byDate',
-                     receivedCol: 'byReceived',
-                     senderCol: 'byAuthor',
-                     recipientCol: 'byRecipient',
-                     subjectCol: 'bySubject',
-                     locationCol: 'byLocation',
-                     accountCol: 'byAccount',
-                     unreadButtonColHeader: 'byUnread',
-                     statusCol: 'byStatus',
-                     sizeCol: 'bySize',
-                     priorityCol: 'byPriority',
-                     flaggedCol: 'byFlagged',
-                     threadCol: 'byThread',
-                     tagsCol: 'byTags',
-                     junkStatusCol: 'byJunkStatus',
-                     idCol: 'byId',
-                     attachmentCol: 'byAttachments',
-                     correspondentCol: 'byCorrespondent'};
+  if (gFolderDisplay.COLUMNS_MAP_NOSORT.has(columnID))
+    return;
 
-
-  var sortType;
-  if (columnID in columnMap) {
-    sortType = columnMap[columnID];
-  } else {
-    // If the column isn't in the map, check and see if it's a custom column
+  let sortType = gFolderDisplay.COLUMNS_MAP.get(columnID);
+  let curCustomColumn = gDBView.curCustomColumn;
+  if (!sortType) {
+    // If the column isn't in the map, check if it's a custom column.
     try {
-      // try to grab the columnHandler (an error is thrown if it does not exist)
+      // Test for the columnHandler (an error is thrown if it does not exist).
       gDBView.getColumnHandler(columnID);
 
-      // it exists - set it to be the current custom column
+      // Handler is registered - set column to be the current custom column.
       gDBView.curCustomColumn = columnID;
-
       sortType = "byCustom";
-    } catch(err) {
-        dump("unsupported sort column: " + columnID + " - no custom handler installed. (Error was: " + err + ")\n");
-        return; // bail out
+    }
+    catch (ex) {
+      dump("HandleColumnClick: No custom column handler registered for " +
+           "columnID: " + columnID + " - " + ex + "\n");
+      return;
     }
   }
 
   let viewWrapper = gFolderDisplay.view;
-  var simpleColumns = false;
+  let simpleColumns = false;
   try {
     simpleColumns = !Services.prefs.getBoolPref("mailnews.thread_pane_column_unthreads");
   }
   catch (ex) {
   }
+
   if (sortType == "byThread") {
     if (simpleColumns)
       MsgToggleThreaded();
@@ -128,30 +114,35 @@ function HandleColumnClick(columnID)
       MsgReverseSortThreadPane();
     else
       MsgSortByThread();
+
+    return;
+  }
+
+  if (!simpleColumns && viewWrapper.showThreaded) {
+    viewWrapper.showUnthreaded = true;
+    MsgSortThreadPane(sortType);
+    return;
+  }
+
+  if (viewWrapper.primarySortType == nsMsgViewSortType[sortType] &&
+      (viewWrapper.primarySortType != nsMsgViewSortType.byCustom ||
+       curCustomColumn == columnID)) {
+    MsgReverseSortThreadPane();
   }
   else {
-    if (!simpleColumns && viewWrapper.showThreaded) {
-      viewWrapper.showUnthreaded = true;
-      MsgSortThreadPane(sortType);
-    }
-    else if (viewWrapper.primarySortType == nsMsgViewSortType[sortType]) {
-      MsgReverseSortThreadPane();
-    }
-    else {
-      MsgSortThreadPane(sortType);
-    }
+    MsgSortThreadPane(sortType);
   }
 }
 
 function ThreadPaneDoubleClick()
 {
-  const nsMsgFolderFlags = Components.interfaces.nsMsgFolderFlags;
+  const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
   if (IsSpecialFolderSelected(nsMsgFolderFlags.Drafts, true)) {
     MsgComposeDraftMessage();
   }
   else if(IsSpecialFolderSelected(nsMsgFolderFlags.Templates, true)) {
-    ComposeMessage(Components.interfaces.nsIMsgCompType.Template,
-                   Components.interfaces.nsIMsgCompFormat.Default,
+    ComposeMessage(Ci.nsIMsgCompType.Template,
+                   Ci.nsIMsgCompFormat.Default,
                    gFolderDisplay.displayedFolder,
                    gFolderDisplay.selectedMessageUris);
   }
@@ -162,8 +153,22 @@ function ThreadPaneDoubleClick()
 
 function ThreadPaneKeyDown(event)
 {
-  if (event.keyCode == KeyEvent.DOM_VK_RETURN)
-    ThreadPaneDoubleClick();
+  if (event.keyCode != KeyEvent.DOM_VK_RETURN)
+    return;
+
+  // Grouped By Sort dummy header row <enter> toggles the thread's open/close
+  // state. Let tree.xml handle it.
+  if (gFolderDisplay.view.showGroupedBySort &&
+      gFolderDisplay.treeSelection && gFolderDisplay.treeSelection.count == 1 &&
+      gFolderDisplay.view.isGroupedByHeaderAtIndex(gFolderDisplay.treeSelection.currentIndex)) {
+    return;
+  }
+
+  // Prevent any thread that happens to be last selected (currentIndex) in a
+  // single or multi selection from toggling in tree.xml.
+  event.stopImmediatePropagation();
+
+  ThreadPaneDoubleClick();
 }
 
 function MsgSortByThread()
@@ -174,19 +179,79 @@ function MsgSortByThread()
 
 function MsgSortThreadPane(sortName)
 {
-  var sortType = nsMsgViewSortType[sortName];
+  let sortType = nsMsgViewSortType[sortName];
+  let grouped = gFolderDisplay.view.showGroupedBySort;
+  gFolderDisplay.view._threadExpandAll =
+    Boolean(gFolderDisplay.view._viewFlags & nsMsgViewFlagsType.kExpandAll);
+
+  if (!grouped) {
+    gFolderDisplay.view.sort(sortType, nsMsgViewSortOrder.ascending);
+    // Respect user's last expandAll/collapseAll choice, post sort direction change.
+    gFolderDisplay.restoreThreadState();
+    return;
+  }
+
   // legacy behavior dictates we un-group-by-sort if we were.  this probably
   //  deserves a UX call...
+
+  // For non virtual folders, do not ungroup (which sorts by the going away
+  // sort) and then sort, as it's a double sort.
+  // For virtual folders, which are rebuilt in the backend in a grouped
+  // change, create a new view upfront rather than applying viewFlags. There
+  // are oddities just applying viewFlags, for example changing out of a
+  // custom column grouped xfvf view with the threads collapsed works (doesn't)
+  // differently than other variations.
+  // So, first set the desired sortType and sortOrder, then set viewFlags in
+  // batch mode, then apply it all (open a new view) with endViewUpdate().
+  gFolderDisplay.view.beginViewUpdate();
+  gFolderDisplay.view._sort = [[sortType, nsMsgViewSortOrder.ascending]];
   gFolderDisplay.view.showGroupedBySort = false;
-  gFolderDisplay.view.sort(sortType, nsMsgViewSortOrder.ascending)
+  gFolderDisplay.view.endViewUpdate();
+
+  // Virtual folders don't persist viewFlags well in the back end,
+  // due to a virtual folder being either 'real' or synthetic, so make
+  // sure it's done here.
+  if (gFolderDisplay.view.isVirtual)
+    gFolderDisplay.view.dbView.viewFlags = gFolderDisplay.view.viewFlags;
 }
 
 function MsgReverseSortThreadPane()
 {
+  let grouped = gFolderDisplay.view.showGroupedBySort;
+  gFolderDisplay.view._threadExpandAll =
+    Boolean(gFolderDisplay.view._viewFlags & nsMsgViewFlagsType.kExpandAll);
+
+  // Grouped By view is special for column click sort direction changes.
+  if (grouped) {
+    if (gDBView.selection.count)
+      gFolderDisplay._saveSelection();
+
+    if (gFolderDisplay.view.isSingleFolder) {
+      if (gFolderDisplay.view.isVirtual)
+        gFolderDisplay.view.showGroupedBySort = false;
+      else
+       // Must ensure rows are collapsed and kExpandAll is unset.
+       gFolderDisplay.doCommand(nsMsgViewCommandType.collapseAll);
+    }
+  }
+
   if (gFolderDisplay.view.isSortedAscending)
     gFolderDisplay.view.sortDescending();
   else
     gFolderDisplay.view.sortAscending();
+
+  // Restore Grouped By state post sort direction change.
+  if (grouped) {
+    if (gFolderDisplay.view.isVirtual && gFolderDisplay.view.isSingleFolder)
+      MsgGroupBySort();
+
+    // Restore Grouped By selection post sort direction change.
+    gFolderDisplay._restoreSelection();
+  }
+
+  // Respect user's last expandAll/collapseAll choice, for both threaded and grouped
+  // views, post sort direction change.
+  gFolderDisplay.restoreThreadState();
 }
 
 function MsgToggleThreaded()
@@ -214,11 +279,25 @@ function MsgSortUnthreaded()
 
 function MsgSortAscending()
 {
+  if (gFolderDisplay.view.showGroupedBySort && gFolderDisplay.view.isSingleFolder) {
+    if (gFolderDisplay.view.isSortedDescending)
+       MsgReverseSortThreadPane();
+
+    return;
+  }
+
   gFolderDisplay.view.sortAscending();
 }
 
 function MsgSortDescending()
 {
+  if (gFolderDisplay.view.showGroupedBySort && gFolderDisplay.view.isSingleFolder) {
+    if (gFolderDisplay.view.isSortedAscending)
+       MsgReverseSortThreadPane();
+
+    return;
+  }
+
   gFolderDisplay.view.sortDescending();
 }
 
