@@ -2,28 +2,23 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const EXPORTED_SYMBOLS = ['DBViewWrapper', 'IDBViewWrapperListener'];
+this.EXPORTED_SYMBOLS = ['DBViewWrapper', 'IDBViewWrapperListener'];
 
-const Cc = Components.classes;
-const Ci = Components.interfaces;
-const Cr = Components.results;
-const Cu = Components.utils;
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource:///modules/mailViewManager.js");
+ChromeUtils.import("resource:///modules/searchSpec.js");
+ChromeUtils.import("resource:///modules/virtualFolderWrapper.js");
 
-Cu.import("resource:///modules/mailServices.js");
-Cu.import("resource:///modules/mailViewManager.js");
-Cu.import("resource:///modules/searchSpec.js");
-Cu.import("resource:///modules/virtualFolderWrapper.js");
+var nsMsgFolderFlags = Ci.nsMsgFolderFlags;
+var nsMsgViewType = Ci.nsMsgViewType;
+var nsMsgViewFlagsType = Ci.nsMsgViewFlagsType;
+var nsMsgViewSortType = Ci.nsMsgViewSortType;
+var nsMsgViewSortOrder = Ci.nsMsgViewSortOrder;
+var nsMsgMessageFlags = Ci.nsMsgMessageFlags;
 
-const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
-const nsMsgViewType = Ci.nsMsgViewType;
-const nsMsgViewFlagsType = Ci.nsMsgViewFlagsType;
-const nsMsgViewSortType = Ci.nsMsgViewSortType;
-const nsMsgViewSortOrder = Ci.nsMsgViewSortOrder;
-const nsMsgMessageFlags = Ci.nsMsgMessageFlags;
+var MSG_VIEW_FLAG_DUMMY = 0x20000000;
 
-const MSG_VIEW_FLAG_DUMMY = 0x20000000;
-
-const nsMsgViewIndex_None = 0xffffffff;
+var nsMsgViewIndex_None = 0xffffffff;
 
 /**
  * Helper singleton for DBViewWrapper that tells instances when something
@@ -111,9 +106,9 @@ var FolderNotificationHelper = {
   stalkFolders: function FolderNotificationHelper_stalkFolders(
       aFolders, aNotherFolder, aViewWrapper) {
     let folders = aFolders ? aFolders.concat() : [];
-    if (aNotherFolder && folders.indexOf(aNotherFolder) == -1)
+    if (aNotherFolder && !folders.includes(aNotherFolder))
       folders.push(aNotherFolder);
-    for each (let [, folder] in Iterator(folders)) {
+    for (let folder of folders) {
       let wrappers = this._interestedWrappers[folder.URI];
       if (wrappers == null)
         wrappers = this._interestedWrappers[folder.URI] = [];
@@ -158,7 +153,7 @@ var FolderNotificationHelper = {
       this._curiousWrappers.splice(this._curiousWrappers.indexOf(aViewWrapper), 1);
       return;
     }
-    for each (let [, folder] in Iterator(aFolders)) {
+    for (let folder of aFolders) {
       this._removeWrapperFromListener(
         this._interestedWrappers, folder, aViewWrapper);
       this._removeWrapperFromListener(
@@ -172,12 +167,12 @@ var FolderNotificationHelper = {
    *     person/code.
    */
   haveListeners: function FolderNotificationHelper_haveListeners() {
-    for each (let [folderURI, wrappers] in
-              Iterator(this._pendingFolderUriToViewWrapperLists)) {
+    for (let folderURI in this._pendingFolderUriToViewWrapperLists) {
+      let wrappers = this._pendingFolderUriToViewWrapperLists[folderURI];
       return true;
     }
-    for each (let [folderURI, wrappers] in
-              Iterator(this._interestedWrappers)) {
+    for (let folderURI in this._interestedWrappers) {
+      let wrappers = this._interestedWrappers[folderURI];
       return true;
     }
     return this._curiousWrappers.length != 0;
@@ -189,11 +184,11 @@ var FolderNotificationHelper = {
     let wrappers = this._interestedWrappers[aFolder.URI];
     if (wrappers) {
       // clone the list to avoid confusing mutation by listeners
-      for each (let [, wrapper] in Iterator(wrappers.concat())) {
+      for (let wrapper of wrappers.concat()) {
         wrapper[aHandlerName](aFolder);
       }
     }
-    for each (let wrapper in this._curiousWrappers)
+    for (let wrapper of this._curiousWrappers)
       wrapper[aHandlerName](aFolder);
   },
 
@@ -204,7 +199,7 @@ var FolderNotificationHelper = {
       let folderURI = aFolder.URI;
       let widgets = this._pendingFolderUriToViewWrapperLists[folderURI];
       if (widgets) {
-        for each (let [, widget] in Iterator(widgets)) {
+        for (let widget of widgets) {
           // we are friends, this is an explicit relationship.
           // (we don't use a generic callback mechanism because the 'this' stuff
           //  gets ugly and no one else should be hooking in at this level.)
@@ -234,6 +229,9 @@ var FolderNotificationHelper = {
     else if (eventType == "DeleteOrMoveMsgFailed") {
       this._notifyHelper(aFolder, "_deleteFailed");
     }
+    else if (eventType == "RenameCompleted") {
+      this._notifyHelper(aFolder, "_renameCompleted");
+    }
 
   },
 
@@ -262,7 +260,7 @@ var FolderNotificationHelper = {
     let wrappers = this._interestedWrappers[newURI];
     if (wrappers) {
       // clone the list to avoid confusing mutation by listeners
-      for each (let [, wrapper] in Iterator(wrappers.concat())) {
+      for (let wrapper of wrappers.concat()) {
         wrapper._folderMoved(aOldFolder, aNewFolder);
       }
     }
@@ -290,7 +288,7 @@ var FolderNotificationHelper = {
     let wrappers = this._interestedWrappers[aFolder.URI];
     if (wrappers) {
       // clone the list to avoid confusing mutation by listeners
-      for each (let [, wrapper] in Iterator(wrappers.concat())) {
+      for (let wrapper of wrappers.concat()) {
         wrapper._folderDeleted(aFolder);
       }
       // if the folder is deleted, it's not going to ever do anything again
@@ -584,11 +582,7 @@ DBViewWrapper.prototype = {
    *  against the displayedFolder attribute.
    */
   isUnderlyingFolder: function DBViewWrapper_isUnderlyingFolder(aFolder) {
-    for each (let [i,underlyingFolder] in Iterator(this._underlyingFolders)) {
-      if (aFolder == underlyingFolder)
-        return true;
-    }
-    return false;
+    return this._underlyingFolders.some(underlyingFolder => aFolder == underlyingFolder);
   },
 
   /**
@@ -666,7 +660,7 @@ DBViewWrapper.prototype = {
       doppel.dbView = this.dbView.cloneDBView(aListener.messenger,
                                               aListener.msgWindow,
                                               aListener.threadPaneCommandUpdater)
-                          .QueryInterface(Components.interfaces.nsITreeView);
+                          .QueryInterface(Ci.nsITreeView);
     // -- clone the search
     if (this.search)
       doppel.search = this.search.clone(doppel);
@@ -715,7 +709,7 @@ DBViewWrapper.prototype = {
                                                  this);
     if (this._underlyingFolders) {
       // (potentially) zero out the underlying msgDatabase references
-      for each (let [, folder] in Iterator(this._underlyingFolders))
+      for (let folder of this._underlyingFolders)
         this._releaseFolderDatabase(folder);
     }
 
@@ -951,8 +945,7 @@ DBViewWrapper.prototype = {
       let virtFolder = VirtualFolderHelper.wrapVirtualFolder(aFolder);
       // Filter out the server roots; they only exist for UI reasons.
       this._underlyingFolders =
-        [folder for each ([, folder] in Iterator(virtFolder.searchFolders))
-                if (!folder.isServer)];
+        virtFolder.searchFolders.filter(folder => !folder.isServer);
       this._underlyingData = (this._underlyingFolders.length > 1) ?
                              this.kUnderlyingMultipleFolder :
                              this.kUnderlyingRealFolder;
@@ -1160,7 +1153,7 @@ DBViewWrapper.prototype = {
       this.displayedFolder = aNewFolder;
 
     // indexOf doesn't work for this (reliably)
-    for each (let [i,underlyingFolder] in Iterator(this._underlyingFolders)) {
+    for (let [i, underlyingFolder] of this._underlyingFolders.entries()) {
       if (aOldFolder == underlyingFolder) {
         this._underlyingFolders[i] = aNewFolder;
         break;
@@ -1196,7 +1189,7 @@ DBViewWrapper.prototype = {
     }
 
     // indexOf doesn't work for this (reliably)
-    for each (let [i,underlyingFolder] in Iterator(this._underlyingFolders)) {
+    for (let [i, underlyingFolder] of this._underlyingFolders.entries()) {
       if (aFolder == underlyingFolder) {
         this._underlyingFolders.splice(i,1);
         break;
@@ -1266,6 +1259,16 @@ DBViewWrapper.prototype = {
     this.listener.onMessageRemovalFailed();
   },
 
+  _forceOpen: function DBViewWrapper__forceOpen(aFolder) {
+    this.displayedFolder = null;
+    this.open(aFolder);
+  },
+
+  _renameCompleted: function DBViewWrapper__renameCompleted(aFolder) {
+    if (aFolder == this.displayedFolder)
+      this._forceOpen(aFolder);
+  },
+
   /**
    * If the displayed folder had its total message count or total unread message
    *  count change, notify the listener.  (Note: only for the display folder;
@@ -1298,6 +1301,9 @@ DBViewWrapper.prototype = {
    *  the view gets (re)built.  If we have a view, depending on what's happening
    *  we may re-create the view or just set the bits.  The rules/reasons are:
    * - XFVF views can handle the flag changes, just set the flags.
+   * - XFVF threaded/unthreaded change must re-sort, the backend forgot.
+   * - Single-folder virtual folders (quicksearch) can handle viewFlag changes,
+   *    to/from grouped included, so set it.
    * - Single-folder threaded/unthreaded can handle a change to/from unthreaded/
    *    threaded, so set it.
    * - Single-folder can _not_ handle a change between grouped and not-grouped,
@@ -1305,26 +1311,50 @@ DBViewWrapper.prototype = {
    *    kUnreadOnly or kShowIgnored.
    */
   set _viewFlags(aViewFlags) {
-    if (this._viewUpdateDepth || !this.dbView)
+    if (this._viewUpdateDepth || !this.dbView) {
       this.__viewFlags = aViewFlags;
-    else {
-      let oldFlags = this.dbView.viewFlags;
-      let changedFlags = oldFlags ^ aViewFlags;
-      if ((this.isVirtual && this.isMultiFolder) ||
-          (this.isSingleFolder &&
-           !(changedFlags & (nsMsgViewFlagsType.kGroupBySort |
-                             nsMsgViewFlagsType.kUnreadOnly |
-                             nsMsgViewFlagsType.kShowIgnored)))) {
-        this.dbView.viewFlags = aViewFlags;
+      return;
+    }
+
+    // For viewFlag changes, do not make a random selection if there is not
+    // actually anything selected; some views do this (looking at xfvf).
+    if (this.dbView.selection && this.dbView.selection.count == 0)
+      this.dbView.selection.currentIndex = -1;
+
+    let setViewFlags = true;
+    let reSort = false;
+    let oldFlags = this.dbView.viewFlags;
+    let changedFlags = oldFlags ^ aViewFlags;
+
+    if (this.isVirtual) {
+      if (this.isMultiFolder &&
+          (changedFlags & nsMsgViewFlagsType.kThreadedDisplay &&
+           !(changedFlags & nsMsgViewFlagsType.kGroupBySort)))
+        reSort = true;
+      if (this.isSingleFolder)
         // ugh, and the single folder case needs us to re-apply his sort...
-        if (this.isSingleFolder)
-          this.dbView.sort(this.dbView.sortType, this.dbView.sortOrder);
-        this.listener.onSortChanged();
-      }
-      else {
-        this.__viewFlags = aViewFlags;
-        this._applyViewChanges();
+        reSort = true;
+    }
+    else {
+      // The regular single folder case.
+      if (changedFlags & (nsMsgViewFlagsType.kGroupBySort |
+                          nsMsgViewFlagsType.kUnreadOnly |
+                          nsMsgViewFlagsType.kShowIgnored)) {
+        setViewFlags = false;
       }
+      // ugh, and the single folder case needs us to re-apply his sort...
+      reSort = true;
+    }
+
+    if (setViewFlags) {
+      this.dbView.viewFlags = aViewFlags;
+      if (reSort)
+        this.dbView.sort(this.dbView.sortType, this.dbView.sortOrder);
+      this.listener.onSortChanged();
+    }
+    else {
+      this.__viewFlags = aViewFlags;
+      this._applyViewChanges();
     }
   },
 
@@ -1352,7 +1382,7 @@ DBViewWrapper.prototype = {
     this.dbView = this._createView();
     // if the synthetic view defines columns, add those for it
     if (this.isSynthetic) {
-      for (let [, customCol] in Iterator(this._syntheticView.customColumns)) {
+      for (let customCol of this._syntheticView.customColumns) {
         customCol.bindToView(this.dbView);
         this.dbView.addColumnHandler(customCol.id, customCol);
       }
@@ -1507,9 +1537,16 @@ DBViewWrapper.prototype = {
    *  on the underlying db view!  If you do not, make sure to fight us every
    *   step of the way, because we will keep clobbering your manually applied
    *   sort.
+   * For secondary and multiple custom column support, a byCustom aSortType and
+   *  aSecondaryType must be the column name string.
    */
   sort: function DBViewWrapper_sort(aSortType, aSortOrder,
                                     aSecondaryType, aSecondaryOrder) {
+    // For sort changes, do not make a random selection if there is not
+    // actually anything selected; some views do this (looking at xfvf).
+    if (this.dbView.selection && this.dbView.selection.count == 0)
+      this.dbView.selection.currentIndex = -1;
+
     this._sort = [[aSortType, aSortOrder]];
     if (aSecondaryType != null && aSecondaryOrder != null)
       this._sort.push([aSecondaryType, aSecondaryOrder]);
@@ -1519,7 +1556,9 @@ DBViewWrapper.prototype = {
     if ((this._viewUpdateDepth == 0) && this.dbView) {
       for (let iSort = this._sort.length - 1; iSort >=0; iSort--) {
         // apply them in the reverse order
-        let [sortType, sortOrder] = this._sort[iSort];
+        let [sortType, sortOrder, sortCustomCol] = this._getSortDetails(iSort);
+        if (sortCustomCol)
+          this.dbView.curCustomColumn = sortCustomCol;
         this.dbView.sort(sortType, sortOrder);
       }
       // (only generate the event since we're not in a update batch)
@@ -1553,19 +1592,32 @@ DBViewWrapper.prototype = {
    * Because we are lazy, we actually just poke the view's sort method and save
    *  the apparent secondary sort.  This also allows perfect compliance with the
    *  way this used to be implemented!
+   * For secondary and multiple custom column support, a byCustom aSortType must
+   *  be the column name string.
    */
   magicSort: function DBViewWrapper_magicSort(aSortType, aSortOrder) {
     if (this.dbView) {
+      // For sort changes, do not make a random selection if there is not
+      // actually anything selected; some views do this (looking at xfvf).
+      if (this.dbView.selection && this.dbView.selection.count == 0)
+        this.dbView.selection.currentIndex = -1;
+
       // so, the thing we just set obviously will be there
       this._sort = [[aSortType, aSortOrder]];
       // (make sure it is valid...)
       this._ensureValidSort();
+      // get sort details, handle custom column as string sortType
+      let [sortType, sortOrder, sortCustomCol] = this._getSortDetails(0);
+      if (sortCustomCol)
+        this.dbView.curCustomColumn = sortCustomCol;
       // apply the sort to see what happens secondary-wise
-      this.dbView.sort(this._sort[0][0], this._sort[0][1]);
+      this.dbView.sort(sortType, sortOrder);
       // there is only a secondary sort if it's not none and not the same.
       if (this.dbView.secondarySortType != nsMsgViewSortType.byNone &&
-          this.dbView.secondarySortType != this._sort[0][0])
-        this._sort.push([this.dbView.secondarySortType,
+          (this.dbView.secondarySortType != sortType ||
+           (this.dbView.secondarySortType == nsMsgViewSortType.byCustom &&
+            this.dbView.secondaryCustomColumn != sortCustomCol)))
+        this._sort.push([this.dbView.secondaryCustomColumn || this.dbView.secondarySortType,
                          this.dbView.secondarySortOrder]);
       // only tell our listener if we're not in a view update batch
       if (this._viewUpdateDepth == 0)
@@ -1586,7 +1638,7 @@ DBViewWrapper.prototype = {
         nsMsgViewFlagsType.kGroupBySort) {
       // We cannot be sorting by thread, id, none, or size.  If we are, switch
       //  to sorting by date.
-      for each (let [, sortPair] in Iterator(this._sort)) {
+      for (let sortPair of this._sort) {
         let sortType = sortPair[0];
         if (sortType == nsMsgViewSortType.byThread ||
             sortType == nsMsgViewSortType.byId ||
@@ -1616,10 +1668,12 @@ DBViewWrapper.prototype = {
   set showGroupedBySort(aShowGroupBySort) {
     if (this.showGroupedBySort != aShowGroupBySort) {
       if (aShowGroupBySort) {
-        // do not apply the flag change until we have made the sort safe
+        // For virtual single folders, the kExpandAll flag must be set.
+        // Do not apply the flag change until we have made the sort safe.
         let viewFlags = this._viewFlags |
                         nsMsgViewFlagsType.kGroupBySort |
-                         nsMsgViewFlagsType.kThreadedDisplay;
+                        nsMsgViewFlagsType.kExpandAll |
+                        nsMsgViewFlagsType.kThreadedDisplay;
         this._ensureValidSort(viewFlags);
         this._viewFlags = viewFlags;
       }
@@ -1989,7 +2043,7 @@ DBViewWrapper.prototype = {
       return this._syntheticView.getMsgHdrForMessageID(aMessageId);
     if (!this._underlyingFolders)
       return null;
-    for (let [, folder] in Iterator(this._underlyingFolders)) {
+    for (let folder of this._underlyingFolders) {
       let msgHdr = folder.msgDatabase.getMsgHdrForMessageID(aMessageId);
       if (msgHdr)
         return msgHdr;
