@@ -3,19 +3,19 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource:///modules/mailServices.js");
 
-const kNonVcardFields =
+var kNonVcardFields =
         ["NickNameContainer", "SecondaryEmailContainer", "ScreenNameContainer",
          "customFields", "preferDisplayName"];
 
-const kPhoneticFields =
+var kPhoneticFields =
         ["PhoneticLastName", "PhoneticLabel1", "PhoneticSpacer1",
          "PhoneticFirstName", "PhoneticLabel2", "PhoneticSpacer2"];
 
 // Item is |[dialogField, cardProperty]|.
-const kVcardFields =
+var kVcardFields =
         [ // Contact > Name
          ["FirstName", "FirstName"],
          ["LastName", "LastName"],
@@ -81,10 +81,10 @@ function OnLoadNewCard()
 
   gEditCard.card =
     (("arguments" in window) && (window.arguments.length > 0) &&
-     (window.arguments[0] instanceof Components.interfaces.nsIAbCard))
+     (window.arguments[0] instanceof Ci.nsIAbCard))
     ? window.arguments[0]
-    : Components.classes["@mozilla.org/addressbook/cardproperty;1"]
-                .createInstance(Components.interfaces.nsIAbCard);
+    : Cc["@mozilla.org/addressbook/cardproperty;1"]
+        .createInstance(Ci.nsIAbCard);
   gEditCard.titleProperty = "newContactTitle";
   gEditCard.selectedAB = "";
 
@@ -187,7 +187,7 @@ function EditCardOKButton()
   // create a list of mailing lists and the index where the card is at.
   for (let i = 0; i < listDirectoriesCount; i++) {
     let subdir = directory.addressLists
-                          .queryElementAt(i, Components.interfaces.nsIAbDirectory);
+                          .queryElementAt(i, Ci.nsIAbDirectory);
     if (subdir.isMailList) {
       // See if any card in this list is the one we edited.
       // Must compare card contents using .equals() instead of .indexOf()
@@ -195,7 +195,7 @@ function EditCardOKButton()
       let listCardsCount = subdir.addressLists.length;
       for (let index = 0; index < listCardsCount; index++) {
         let card = subdir.addressLists
-                         .queryElementAt(index, Components.interfaces.nsIAbCard);
+                         .queryElementAt(index, Ci.nsIAbCard);
         if (card.equals(gEditCard.card))
           foundDirectories.push({directory:subdir, cardIndex:index});
       }
@@ -220,7 +220,7 @@ function EditCardOKButton()
     let foundItem = foundDirectories.pop();
     foundItem.directory
              .addressLists
-             .replaceElementAt(gEditCard.card, foundItem.cardIndex, false);
+             .replaceElementAt(gEditCard.card, foundItem.cardIndex);
   }
 
   NotifySaveListeners(directory);
@@ -368,7 +368,7 @@ function InitPhoneticFields()
 {
   var showPhoneticFields =
     Services.prefs.getComplexValue("mail.addr_book.show_phonetic_fields",
-      Components.interfaces.nsIPrefLocalizedString).data;
+      Ci.nsIPrefLocalizedString).data;
 
   // show phonetic fields if indicated by the pref
   if (showPhoneticFields == "true")
@@ -392,7 +392,7 @@ function InitEditCard()
   try {
     var displayLastNameFirst =
       Services.prefs.getComplexValue("mail.addr_book.displayName.lastnamefirst",
-        Components.interfaces.nsIPrefLocalizedString).data;
+        Ci.nsIPrefLocalizedString).data;
     gEditCard.displayLastNameFirst = (displayLastNameFirst == "true");
     gEditCard.generateDisplayName =
       Services.prefs.getBoolPref("mail.addr_book.displayName.autoGeneration");
@@ -698,9 +698,10 @@ function calculateAge(aEvent, aElement) {
   // if the datepicker was updated, update the year element
   if (aElement == datepicker && !(datepicker.year == kDefaultYear && !yearElem.value))
     yearElem.value = datepicker.year;
+
   var year = yearElem.value;
   // if the year element's value is invalid set the year and age elements to null
-  if (isNaN(year) || year < 1 || year > 9999) {
+  if (isNaN(year) || year < kMinYear || year > kMaxYear) {
     yearElem.value = null;
     ageElem.value = null;
     datepicker.year = kDefaultYear;
@@ -728,7 +729,7 @@ function calculateAge(aEvent, aElement) {
 }
 
 /**
- * Calculates the year an event ocurred based on the number of years, months,
+ * Calculates the year an event occurred based on the number of years, months,
  * and days since the event and updates the relevant element.
  * @param aEvent   The event calling this method.
  * @param aElement Optional, but required if this function is not called from an
@@ -739,10 +740,10 @@ function calculateYear(aEvent, aElement) {
   if (aEvent)
     aElement = this;
   if (aElement.id == "Age") {
-    yearElem = document.getElementById("BirthYear");
     datepicker = document.getElementById("Birthday");
+    yearElem = document.getElementById("BirthYear");
   }
-  if (!yearElem || !datepicker)
+  if (!datepicker || !yearElem)
     return;
 
   // if the age is null, remove the year from the year element, and set the
@@ -766,7 +767,9 @@ function calculateYear(aEvent, aElement) {
     datepicker.year = kDefaultYear;
     // if there was an error (like invalid year) set the year and age to null
     yearElem.value = null;
-    ageElem.value = null;
+    let ageElem = document.getElementById("Age");
+    if (ageElem)
+      ageElem.value = null;
   }
 }
 
@@ -816,7 +819,7 @@ function modifyDatepicker(aDatepicker) {
       var dt = new Date(this.year, currentMonth, aValue);
       return dt.getMonth() != currentMonth ? 1 : aValue;
     }
-    var max = (aField == this.monthField) ? 11 : 9999;
+    var max = (aField == this.monthField) ? 11 : kMaxYear;
     // make sure the value isn't too high
     if (aValue > max)
       return aNoWrap ? max : min;
@@ -825,7 +828,7 @@ function modifyDatepicker(aDatepicker) {
   // sets the specified field to the given value, but allows blank fields
   // from: mozilla/toolkit/content/widgets/datetimepicker.xml#698
   aDatepicker._setFieldValue = function setValue(aField, aValue) {
-    if (aField == this.yearField && aValue > 0 && aValue < 10000) {
+    if (aField == this.yearField && aValue >= kMinYear && aValue <= kMaxYear) {
       var oldDate = this._dateValue;
       this._dateValue.setFullYear(aValue);
       if (oldDate != this._dateValue) {
@@ -869,7 +872,7 @@ function modifyDatepicker(aDatepicker) {
   }
 }
 
-const chatNameFieldIds =
+var chatNameFieldIds =
   ["Gtalk", "AIM", "Yahoo", "Skype", "QQ", "MSN", "ICQ", "XMPP", "IRC"];
 
 /**
@@ -880,7 +883,7 @@ function showChat()
 {
   document.getElementById('abTabPanels').parentNode.selectedTab =
     document.getElementById('chatTabButton');
-  for each (let id in chatNameFieldIds) {
+  for (let id of chatNameFieldIds) {
     let elt = document.getElementById(id);
     if (elt.value) {
       elt.focus();
@@ -897,7 +900,7 @@ function showChat()
 function updateChatName()
 {
   let value = "";
-  for each (let id in chatNameFieldIds) {
+  for (let id of chatNameFieldIds) {
     let val = document.getElementById(id).value;
     if (val) {
       value = val;
@@ -1005,22 +1008,23 @@ function removePhoto(aName) {
  * @return true if the OK button was clicked and a photo was chosen
  */
 function browsePhoto() {
-  var nsIFilePicker = Components.interfaces.nsIFilePicker;
-  var fp = Components.classes["@mozilla.org/filepicker;1"]
-                     .createInstance(nsIFilePicker);
+  var nsIFilePicker = Ci.nsIFilePicker;
+  var fp = Cc["@mozilla.org/filepicker;1"]
+             .createInstance(nsIFilePicker);
   fp.init(window, gAddressBookBundle.getString("browsePhoto"), nsIFilePicker.modeOpen);
 
   // Add All Files & Image Files filters and select the latter
   fp.appendFilters(nsIFilePicker.filterImages);
   fp.appendFilters(nsIFilePicker.filterAll);
 
-  if (fp.show() == nsIFilePicker.returnOK) {
+  fp.open(rv => {
+    if (rv != nsIFilePicker.returnOK) {
+      return;
+    }
     document.getElementById("PhotoFile").file = fp.file;
     let photoType = document.getElementById("FilePhotoType").value;
     onSwitchPhotoType(photoType);
-    return true;
-  }
-  return false;
+  });
 }
 
 /* A photo handler defines the behaviour of the contact editor
@@ -1081,8 +1085,8 @@ var filePhotoHandler = {
   onLoad: function(aCard, aDocument) {
     var photoURI = aCard.getProperty("PhotoURI", "");
     try {
-      var file = Services.io.newURI(photoURI, null, null)
-                            .QueryInterface(Components.interfaces.nsIFileURL)
+      var file = Services.io.newURI(photoURI)
+                            .QueryInterface(Ci.nsIFileURL)
                             .file;
     } catch (e) {}
 
