@@ -5,16 +5,12 @@
 
 //NOTE: gAddressBookBundle must be defined and set or this Overlay won't work
 
-Components.utils.import("resource://gre/modules/PlacesUtils.jsm");
-Components.utils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/PlacesUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 var gProfileDirURL;
-
-var gMapItURLFormat = Services.prefs.getComplexValue("mail.addr_book.mapit_url.format",
-  Components.interfaces.nsIPrefLocalizedString).data;
-
 var gFileHandler = Services.io.getProtocolHandler("file")
-  .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
+  .QueryInterface(Ci.nsIFileProtocolHandler);
 var gPhotoDisplayHandlers = {};
 
 var zListName;
@@ -172,9 +168,9 @@ function GetAddressesFromURI(uri)
   if (addressList) {
     var total = addressList.length;
     if (total > 0)
-      addresses = addressList.queryElementAt(0, Components.interfaces.nsIAbCard).primaryEmail;
+      addresses = addressList.queryElementAt(0, Ci.nsIAbCard).primaryEmail;
     for (var i = 1;  i < total; i++ ) {
-      addresses += ", " + addressList.queryElementAt(i, Components.interfaces.nsIAbCard).primaryEmail;
+      addresses += ", " + addressList.queryElementAt(i, Ci.nsIAbCard).primaryEmail;
     }
   }
   return addresses;
@@ -247,22 +243,12 @@ function DisplayCardViewPane(realCard)
                                card.getProperty("HomeZipCode")) || visible;
    visible = cvSetNode(data.cvHomeCountry, card.getProperty("HomeCountry")) ||
              visible;
-   if (visible) {
-     var homeMapItUrl = CreateMapItURL(card.getProperty("HomeAddress"),
-                                       card.getProperty("HomeAddress2"),
-                                       card.getProperty("HomeCity"),
-                                       card.getProperty("HomeState"),
-                                       card.getProperty("HomeZipCode"),
-                                       card.getProperty("HomeCountry"));
-    if (homeMapItUrl) {
-       cvSetVisible(data.cvbHomeMapItBox, true);
-       data.cvHomeMapIt.setAttribute('url', homeMapItUrl);
-    } else {
-       cvSetVisible(data.cvbHomeMapItBox, false);
-    }
-  } else {
-    cvSetVisible(data.cvbHomeMapItBox, false);
-  }
+
+  let mapURLList = data.cvHomeMapIt.firstChild;
+  if (visible)
+    mapURLList.initMapAddressFromCard(card, "Home");
+
+  cvSetVisible(data.cvbHomeMapItBox, !!mapURLList.mapURL);
 
   visible = HandleLink(data.cvHomeWebPage, "", card.getProperty("WebPage2"),
                        data.cvHomeWebPageBox, card.getProperty("WebPage2")) ||
@@ -292,25 +278,22 @@ function DisplayCardViewPane(realCard)
     var dateStr;
     if (day > 0 && day < 32 && month > 0 && month < 13) {
       var date;
-      // if the year exists, just use Date.toLocaleString
+      var formatter;
       if (year) {
         // use UTC-based calculations to avoid off-by-one day
         // due to time zone/dst discontinuity
         date = new Date(Date.UTC(year, month - 1, day));
         date.setUTCFullYear(year); // to handle two-digit years properly
-        dateStr = date.toLocaleDateString([], {timeZone: "UTC"});
+        formatter = new Services.intl.DateTimeFormat(undefined,
+                      { dateStyle: "long", timeZone: "UTC" });
       }
       // if the year doesn't exist, display Month DD (ex. January 01)
       else {
-        date = new Date(saneBirthYear(year), month - 1, day);
-        // toLocaleFormat() seems to have a different implementation than
-        // toLocaleDateString() and returns correct results even when not
-        // passing UTC times; besides, it would not support an options
-        // parameter for {timeZone: "UTC"} anyway.
-        dateStr = date.toLocaleFormat(
-          gAddressBookBundle.getString("dateFormatMonthDay")
-        );
+        date = new Date(Date.UTC(saneBirthYear(year), month - 1, day));
+        formatter = new Services.intl.DateTimeFormat(undefined,
+                      { month: "long", day: "numeric", timeZone: "UTC" });
       }
+      dateStr = formatter.format(date);
     }
     else if (year)
       dateStr = year;
@@ -391,25 +374,11 @@ function DisplayCardViewPane(realCard)
   addressVisible = cvSetNode(data.cvWorkCountry,
                              card.getProperty("WorkCountry")) || addressVisible;
 
-        if (addressVisible) {
-          var workMapItUrl = CreateMapItURL(card.getProperty("WorkAddress"),
-                                            card.getProperty("WorkAddress2"),
-                                            card.getProperty("WorkCity"),
-                                            card.getProperty("WorkState"),
-                                            card.getProperty("WorkZipCode"),
-                                            card.getProperty("WorkCountry"));
-          data.cvWorkMapIt.setAttribute('url', workMapItUrl);
-          if (workMapItUrl) {
-      cvSetVisible(data.cvbWorkMapItBox, true);
-            data.cvWorkMapIt.setAttribute('url', workMapItUrl);
-          }
-          else {
-      cvSetVisible(data.cvbWorkMapItBox, false);
-          }
-        }
-        else {
-    cvSetVisible(data.cvbWorkMapItBox, false);
-        }
+  mapURLList = data.cvWorkMapIt.firstChild;
+  if (addressVisible)
+    mapURLList.initMapAddressFromCard(card, "Work");
+
+  cvSetVisible(data.cvbWorkMapItBox, !!mapURLList.mapURL);
 
         visible = HandleLink(data.cvWorkWebPage, "",
                              card.getProperty("WebPage1"),
@@ -431,7 +400,7 @@ function setBuddyIcon(card, buddyIcon)
     if (myScreenName && card.primaryEmail) {
       if (!gProfileDirURL) {
         // lazily create these file urls, and keep them around
-        let profileDir = Services.dirsvc.get("ProfD", Components.interfaces.nsIFile);
+        let profileDir = Services.dirsvc.get("ProfD", Ci.nsIFile);
         gProfileDirURL = Services.io.newFileURI(profileDir);
       }
 
@@ -523,7 +492,7 @@ function cvAddAddressNodes(node, uri)
         }
         for (i = 0;  i < total; i++ ) {
           var descNode = document.createElement("description");
-          var card = addressList.queryElementAt(i, Components.interfaces.nsIAbCard);
+          var card = addressList.queryElementAt(i, Ci.nsIAbCard);
 
           descNode.setAttribute("class", "CardViewLink");
           node.appendChild(descNode);
@@ -566,38 +535,18 @@ function OpenURLWithHistory(url)
 {
   try {
     PlacesUtils.asyncHistory.updatePlaces({
-      uri: Services.io.newURI(url, null, null),
+      uri: Services.io.newURI(url),
       visits:  [{
         visitDate: Date.now() * 1000,
-        transitionType: Components.interfaces.nsINavHistoryService.TRANSITION_LINK
+        transitionType: Ci.nsINavHistoryService.TRANSITION_LINK
       }]
     });
-    var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();
-    messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
+    var messenger = Cc["@mozilla.org/messenger;1"].createInstance();
+    messenger = messenger.QueryInterface(Ci.nsIMessenger);
     messenger.launchExternalURL(url);
   } catch (ex) {}
 }
 
-function CreateMapItURL(address1, address2, city, state, zip, country)
-{
-  if (!gMapItURLFormat)
-    return null;
-
-  var urlFormat = gMapItURLFormat.replace("@A1", encodeURIComponent(address1));
-  urlFormat = urlFormat.replace("@A2", encodeURIComponent(address2));
-  urlFormat = urlFormat.replace("@CO", encodeURIComponent(country));
-  urlFormat = urlFormat.replace("@CI", encodeURIComponent(city));
-  urlFormat = urlFormat.replace("@ST", encodeURIComponent(state));
-  urlFormat = urlFormat.replace("@ZI", encodeURIComponent(zip));
-
-  return urlFormat;
-}
-
-function MapIt(id)
-{
-  OpenURLWithHistory(document.getElementById(id).getAttribute('url'));
-}
-
 function openLink(id)
 {
   OpenURLWithHistory(document.getElementById(id).getAttribute("href"));
@@ -606,6 +555,15 @@ function openLink(id)
   return false;
 }
 
+function openLinkWithUrl(aUrl)
+{
+  if (aUrl)
+    OpenURLWithHistory(aUrl);
+
+  // return false, so we don't load the href in the addressbook window
+  return false;
+}
+
 /* Display the contact photo from the nsIAbCard in the IMG element.
  * If the photo cannot be displayed, show the generic contact
  * photo.
