@@ -8,24 +8,26 @@
  * nsIMsgCloudFileProvider interface.
  */
 
-const {classes: Cc, interfaces: Ci, utils: Cu, results: Cr} = Components;
+var { results: Cr } = Components;
 
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-Cu.import("resource://gre/modules/Services.jsm");
-Cu.import("resource:///modules/gloda/log4moz.js");
-Cu.import("resource:///modules/cloudFileAccounts.js");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource:///modules/gloda/log4moz.js");
+ChromeUtils.import("resource:///modules/cloudFileAccounts.js");
+
+Cu.importGlobalProperties(["XMLHttpRequest"]);
 
 var gServerUrl = "https://dpi.yousendit.com"; // Production url
 // test url var gServerUrl = "https://test2-api.yousendit.com";
 
-const kApiKey = "7spvjdt7m4kycr7jyhywrdn2";
-const kAuthPath = "/dpi/v1/auth";
-const kUserInfoPath = "/dpi/v2/user";
-const kFolderPath = "/dpi/v1/folder/";
-const kFolderFilePath = "/dpi/v1/folder/file/";
-const kFolderInitUploadPath = kFolderPath + "file/initUpload";
-const kFolderCommitUploadPath = kFolderPath + "file/commitUpload";
-const kUrlTail = "s=4001583&cid=pm-4001583";
+var kApiKey = "7spvjdt7m4kycr7jyhywrdn2";
+var kAuthPath = "/dpi/v1/auth";
+var kUserInfoPath = "/dpi/v2/user";
+var kFolderPath = "/dpi/v1/folder/";
+var kFolderFilePath = "/dpi/v1/folder/file/";
+var kFolderInitUploadPath = kFolderPath + "file/initUpload";
+var kFolderCommitUploadPath = kFolderPath + "file/commitUpload";
+var kUrlTail = "s=4001583&cid=pm-4001583";
 
 function nsHightail() {
   this.log = Log4Moz.getConfiguredLogger("Hightail");
@@ -164,7 +166,7 @@ nsHightail.prototype = {
   /**
    * Attempt to upload a file to Hightail's servers.
    *
-   * @param aFile an nsILocalFile for uploading.
+   * @param aFile an nsIFile for uploading.
    * @param aCallback an nsIRequestObserver for monitoring the start and
    *                  stop states of the upload procedure.
    */
@@ -186,7 +188,6 @@ nsHightail.prototype = {
     }
 
     this._uploadingFile = aFile;
-    this._urlListener = aCallback;
 
     let finish = function() {
       this._finishUpload(aFile, aCallback);
@@ -197,8 +198,8 @@ nsHightail.prototype = {
     }.bind(this);
 
     let onAuthFailure = function() {
-      this._urlListener.onStopRequest(null, null,
-                                      Ci.nsIMsgCloudFileProvider.authErr);
+      aCallback.onStopRequest(null, null,
+                              Ci.nsIMsgCloudFileProvider.authErr);
     }.bind(this);
 
     this.log.info("Checking to see if we're logged in");
@@ -222,7 +223,7 @@ nsHightail.prototype = {
    * for a file. First, ensures that the file size is not too large, and that
    * we won't exceed our storage quota, and then kicks off the upload.
    *
-   * @param aFile the nsILocalFile to upload
+   * @param aFile the nsIFile to upload
    * @param aCallback the nsIRequestObserver for monitoring the start and stop
    *                  states of the upload procedure.
    */
@@ -272,7 +273,7 @@ nsHightail.prototype = {
   /**
    * Cancels an in-progress file upload.
    *
-   * @param aFile the nsILocalFile being uploaded.
+   * @param aFile the nsIFile being uploaded.
    */
   cancelFileUpload: function(aFile) {
     this.log.info("in cancel upload");
@@ -325,8 +326,7 @@ nsHightail.prototype = {
     this.log.info("getting user info");
     let args = "?email=" + this._userName + "&";
 
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
     req.open("GET", gServerUrl + kUserInfoPath + args + kUrlTail, true);
 
     req.onload = function() {
@@ -389,7 +389,7 @@ nsHightail.prototype = {
   /**
    * Returns the sharing URL for some uploaded file.
    *
-   * @param aFile the nsILocalFile to get the URL for.
+   * @param aFile the nsIFile to get the URL for.
    */
   urlForFile: function(aFile) {
     return this._urlsForFiles[aFile.path];
@@ -449,8 +449,7 @@ nsHightail.prototype = {
     let args = "?email=" + aEmailAddress + "&password=" + aPassword + "&firstname="
                + aFirstName + "&lastname=" + aLastName + "&";
 
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
 
     req.open("POST", gServerUrl + kUserInfoPath + args + kUrlTail, true);
 
@@ -530,8 +529,7 @@ nsHightail.prototype = {
 
     let args = "?includeFiles=false&includeFolders=true&";
 
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
 
     req.open("GET",
              gServerUrl + kFolderPath + aFolderId + args + kUrlTail,
@@ -580,8 +578,7 @@ nsHightail.prototype = {
 
     let args = "?name=" + aName + "&parentId=" + aParent + "&";
 
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
 
     req.open("POST", gServerUrl + kFolderPath.replace(/\/$/, '') + args + kUrlTail, true);
 
@@ -664,7 +661,7 @@ nsHightail.prototype = {
   /**
    * Attempts to delete an uploaded file.
    *
-   * @param aFile the nsILocalFile to delete.
+   * @param aFile the nsIFile to delete.
    * @param aCallback an nsIRequestObserver for monitoring the start and stop
    *                  states of the delete procedure.
    */
@@ -682,8 +679,7 @@ nsHightail.prototype = {
       throw Cr.NS_ERROR_FAILURE;
     }
 
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
 
     let args = kFolderFilePath + uploadInfo.fileId + "?";
 
@@ -759,7 +755,7 @@ nsHightail.prototype = {
 
     let passwordURI = gServerUrl;
     let logins = Services.logins.findLogins({}, passwordURI, null, passwordURI);
-    for each (let loginInfo in logins) {
+    for (let loginInfo of logins) {
       if (loginInfo.username == aUsername)
         return loginInfo.password;
     }
@@ -796,7 +792,7 @@ nsHightail.prototype = {
    */
   clearPassword: function() {
     let logins = Services.logins.findLogins({}, gServerUrl, null, gServerUrl);
-    for each (let loginInfo in logins)
+    for (let loginInfo of logins)
       if (loginInfo.username == this._userName)
         Services.logins.removeLogin(loginInfo);
   },
@@ -815,8 +811,7 @@ nsHightail.prototype = {
       this._password = this.getPassword(this._userName, !aWithUI);
     let args = "?email=" + this._userName + "&password=" + this._password + "&";
     this.log.info("Sending login information...");
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
     let curDate = Date.now().toString();
 
     req.open("POST", gServerUrl + kAuthPath + args + kUrlTail, true);
@@ -918,8 +913,7 @@ nsHightailFileUploader.prototype = {
    * @param failureCallback the callback fired if getting the URL fails
    */
   _prepareToSend: function(successCallback, failureCallback) {
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
 
     req.open("POST", gServerUrl + kFolderInitUploadPath + "?" + kUrlTail, true);
 
@@ -959,8 +953,7 @@ nsHightailFileUploader.prototype = {
    * the upload of the file to Hightail.
    */
   _uploadFile: function() {
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
 
     let curDate = Date.now().toString();
     this.log.info("upload url = " + this._urlInfo.uploadUrl[0]);
@@ -1009,7 +1002,7 @@ nsHightailFileUploader.prototype = {
     // Since js doesn't like binary data in strings, we're going to create
     // a temp file consisting of the message preamble, the file contents, and
     // the post script, and pass a stream based on that file to
-    // nsIXMLHttpRequest.send().
+    // XMLHttpRequest.send().
 
     try {
       this._tempFile = this.getTempFile(this.file.leafName);
@@ -1050,7 +1043,7 @@ nsHightailFileUploader.prototype = {
       this._bufStream = Cc["@mozilla.org/network/buffered-input-stream;1"]
                         .createInstance(Ci.nsIBufferedInputStream);
       this._bufStream.init(this._fstream, 4096);
-      // nsIXMLHttpRequest's nsIVariant handling requires that we QI
+      // XMLHttpRequest's nsIVariant handling requires that we QI
       // to nsIInputStream.
       req.send(this._bufStream.QueryInterface(Ci.nsIInputStream));
     } catch (ex) {
@@ -1083,8 +1076,7 @@ nsHightailFileUploader.prototype = {
    */
   _commitSend: function() {
     this.log.info("commit sending file " + this._urlInfo.fileId);
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
     let args = "?name=" + this.file.leafName +
                "&fileId=" + this._urlInfo.fileId +
                "&parentId=" + this.hightail._folderId + "&";
@@ -1161,8 +1153,7 @@ nsHightailFileUploader.prototype = {
    * @param aFailureCallback called if url is not found
    */
   _findDownloadUrl: function(aFileId, aSuccessCallback, aFailureCallback) {
-    let req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
-                .createInstance(Ci.nsIXMLHttpRequest);
+    let req = new XMLHttpRequest();
 
     req.open("GET", gServerUrl + kFolderFilePath + aFileId, true);
 
@@ -1198,7 +1189,7 @@ nsHightailFileUploader.prototype = {
   getTempFile: function(leafName) {
     let tempfile = Services.dirsvc.get("TmpD", Ci.nsIFile);
     tempfile.append(leafName)
-    tempfile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, parseInt("0666", 8));
+    tempfile.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, parseInt("0666", 8));
     // do whatever you need to the created file
     return tempfile.clone()
   },
@@ -1217,4 +1208,4 @@ nsHightailFileUploader.prototype = {
   },
 };
 
-const NSGetFactory = XPCOMUtils.generateNSGetFactory([nsHightail]);
+var NSGetFactory = XPCOMUtils.generateNSGetFactory([nsHightail]);
