@@ -2,39 +2,39 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const EXPORTED_SYMBOLS = [];
+this.EXPORTED_SYMBOLS = [];
 
-const {classes: Cc, interfaces: Ci, utils: Cu, Constructor: CC} = Components;
+var { Constructor: CC } = Components;
 
-Cu.import("resource:///modules/gloda/public.js");
-Cu.import("resource:///modules/gloda/datamodel.js");
-Cu.import("resource:///modules/gloda/indexer.js");
-Cu.import("resource:///modules/imServices.jsm");
-Cu.import("resource:///modules/imXPCOMUtils.jsm");
-Cu.import("resource:///modules/iteratorUtils.jsm");
-Cu.import("resource:///modules/mailServices.js");
-Cu.import("resource://gre/modules/FileUtils.jsm");
-Cu.import("resource://gre/modules/NetUtil.jsm");
+ChromeUtils.import("resource:///modules/gloda/public.js");
+ChromeUtils.import("resource:///modules/gloda/datamodel.js");
+ChromeUtils.import("resource:///modules/gloda/indexer.js");
+ChromeUtils.import("resource:///modules/imServices.jsm");
+ChromeUtils.import("resource:///modules/imXPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/iteratorUtils.jsm");
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource://gre/modules/FileUtils.jsm");
+ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
 
-XPCOMUtils.defineLazyModuleGetter(this, "OS", "resource://gre/modules/osfile.jsm");
-XPCOMUtils.defineLazyModuleGetter(this, "Task", "resource://gre/modules/Task.jsm");
-XPCOMUtils.defineLazyModuleGetter(this, "AsyncShutdown",
-                                  "resource://gre/modules/AsyncShutdown.jsm");
-XPCOMUtils.defineLazyModuleGetter(this, "GlodaDatastore",
-                                  "resource:///modules/gloda/datastore.js");
+ChromeUtils.defineModuleGetter(this, "OS", "resource://gre/modules/osfile.jsm");
+ChromeUtils.defineModuleGetter(this, "Task", "resource://gre/modules/Task.jsm");
+ChromeUtils.defineModuleGetter(this, "AsyncShutdown",
+                               "resource://gre/modules/AsyncShutdown.jsm");
+ChromeUtils.defineModuleGetter(this, "GlodaDatastore",
+                               "resource:///modules/gloda/datastore.js");
 
-const kCacheFileName = "indexedFiles.json";
+var kCacheFileName = "indexedFiles.json";
 
-const FileInputStream = CC("@mozilla.org/network/file-input-stream;1",
+var FileInputStream = CC("@mozilla.org/network/file-input-stream;1",
                            "nsIFileInputStream",
                            "init");
-const ScriptableInputStream = CC("@mozilla.org/scriptableinputstream;1",
+var ScriptableInputStream = CC("@mozilla.org/scriptableinputstream;1",
                                  "nsIScriptableInputStream",
                                  "init");
 
 // kIndexingDelay is how long we wait from the point of scheduling an indexing
 // job to actually carrying it out.
-const kIndexingDelay = 5000; // in milliseconds
+var kIndexingDelay = 5000; // in milliseconds
 
 XPCOMUtils.defineLazyGetter(this, "MailFolder", () =>
   Cc["@mozilla.org/rdf/resource-factory;1?name=mailbox"].createInstance(Ci.nsIMsgFolder)
@@ -69,7 +69,7 @@ GlodaIMConversation.prototype = {
 
     // Find the nsIIncomingServer for the current imIAccount.
     let mgr = MailServices.accounts;
-    for (let account in fixIterator(mgr.accounts, Ci.nsIMsgAccount)) {
+    for (let account of fixIterator(mgr.accounts, Ci.nsIMsgAccount)) {
       let incomingServer = account.incomingServer;
       if (!incomingServer || incomingServer.type != "im")
         continue;
@@ -115,7 +115,7 @@ GlodaIMConversation.prototype = {
 // FIXME
 var WidgetProvider = {
   providerName: "widget",
-  process: function () {
+  process: function*() {
     //XXX What is this supposed to do?
     yield Gloda.kWorkDone;
   }
@@ -155,7 +155,7 @@ IMConversationNoun.objFromRow = function(aRow) {
   return conv;
 };
 
-const EXT_NAME = "im";
+var EXT_NAME = "im";
 
 // --- special (on-row) attributes
 Gloda.defineAttribute({
@@ -238,9 +238,9 @@ var GlodaIMIndexer = {
   name: "index_im",
   cacheVersion: 1,
   enable: function() {
-    Services.obs.addObserver(this, "conversation-closed", false);
-    Services.obs.addObserver(this, "new-ui-conversation", false);
-    Services.obs.addObserver(this, "ui-conversation-closed", false);
+    Services.obs.addObserver(this, "conversation-closed");
+    Services.obs.addObserver(this, "new-ui-conversation");
+    Services.obs.addObserver(this, "ui-conversation-closed");
 
     // The shutdown blocker ensures pending saves happen even if the app
     // gets shut down before the timer fires.
@@ -448,7 +448,7 @@ var GlodaIMIndexer = {
         GlodaIndexer.indexJob(job);
       }
       conv.logFileCount = logFiles.length;
-    }.bind(this)).catch(Components.utils.reportError);
+    }.bind(this)).catch(Cu.reportError);
 
     // Now clear the job, so we can index in the future.
     this._knownConversations[convId].scheduledIndex = null;
@@ -521,7 +521,7 @@ var GlodaIMIndexer = {
   _getIdFromPath: function(aPath) {
     let selectStatement = GlodaDatastore._createAsyncStatement(
       "SELECT id FROM imConversations WHERE path = ?1");
-    selectStatement.bindStringParameter(0, aPath);
+    selectStatement.bindByIndex(0, aPath);
     let id;
     return new Promise((resolve, reject) => {
       selectStatement.executeAsync({
@@ -615,7 +615,7 @@ var GlodaIMIndexer = {
     return rv;
   }),
 
-  _worker_indexIMConversation: function(aJob, aCallbackHandle) {
+  _worker_indexIMConversation: function*(aJob, aCallbackHandle) {
     let glodaConv = {};
     let existingGlodaConv = aJob.conversation.glodaConv;
     if (existingGlodaConv &&
@@ -639,10 +639,12 @@ var GlodaIMIndexer = {
     yield Gloda.kWorkDone;
   },
 
-  _worker_logsFolderSweep: function(aJob) {
+  _worker_logsFolderSweep: function*(aJob) {
     let dir = FileUtils.getFile("ProfD", ["logs"]);
-    if (!dir.exists() || !dir.isDirectory())
-      return;
+    if (!dir.exists() || !dir.isDirectory()) {
+      // If the folder does not exist, then we are done.
+      yield GlodaIndexer.kWorkDone;
+    }
 
     // Sweep the logs directory for log files, adding any new entries to the
     // _knownFiles tree as we traverse.
@@ -679,9 +681,11 @@ var GlodaIMIndexer = {
         }
       }
     }
+
+    yield GlodaIndexer.kWorkDone;
   },
 
-  _worker_convFolderSweep: function(aJob, aCallbackHandle) {
+  _worker_convFolderSweep: function*(aJob, aCallbackHandle) {
     let folder = aJob.folder;
 
     let sessions = folder.directoryEntries;
@@ -745,8 +749,8 @@ var GlodaIMIndexer = {
           // so this should work.
           let pathComponents = OS.Path.split(row.getString(1)).components;
           if (pathComponents.length > 4) {
-            updateStatement.bindInt64Parameter(1, row.getInt64(0)); // id
-            updateStatement.bindStringParameter(0,
+            updateStatement.bindByIndex(1, row.getInt64(0)); // id
+            updateStatement.bindByIndex(0,
               pathComponents.slice(-4).join("/")); // Last 4 path components
             updateStatement.executeAsync({
               handleResult: () => {},
