@@ -320,7 +320,7 @@ nsSeamonkeyProfileMigrator::TransformPreferences(const nsAString& aSourcePrefFil
   nsCOMPtr<nsIFile> sourcePrefsFile;
   mSourceProfile->Clone(getter_AddRefs(sourcePrefsFile));
   sourcePrefsFile->Append(aSourcePrefFileName);
-  psvc->ReadUserPrefs(sourcePrefsFile);
+  psvc->ReadUserPrefsFromFile(sourcePrefsFile);
 
   nsCOMPtr<nsIPrefBranch> branch(do_QueryInterface(psvc));
   for (transform = gTransforms; transform < end; ++transform)
@@ -429,7 +429,8 @@ nsSeamonkeyProfileMigrator::CopySignatureFiles(PBStructArray &aIdentities,
       // turn the pref into a nsIFile
       nsCOMPtr<nsIFile> srcSigFile =
         do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
-      srcSigFile->SetPersistentDescriptor(nsDependentCString(pref->stringValue));
+      rv = srcSigFile->SetPersistentDescriptor(nsDependentCString(pref->stringValue));
+      NS_ENSURE_SUCCESS(rv, rv);
 
       nsCOMPtr<nsIFile> targetSigFile;
       rv = mTargetProfile->Clone(getter_AddRefs(targetSigFile));
@@ -447,7 +448,8 @@ nsSeamonkeyProfileMigrator::CopySignatureFiles(PBStructArray &aIdentities,
 
         // now write out the new descriptor
         nsAutoCString descriptorString;
-        targetSigFile->GetPersistentDescriptor(descriptorString);
+        rv = targetSigFile->GetPersistentDescriptor(descriptorString);
+        NS_ENSURE_SUCCESS(rv, rv);
         NS_Free(pref->stringValue);
         pref->stringValue = ToNewCString(descriptorString);
       }
@@ -465,8 +467,7 @@ nsSeamonkeyProfileMigrator::CopyMailFolders(PBStructArray &aMailServers,
   // (1) Fix up the directory path for the new profile
   // (2) copy the mail folder data from the source directory pref to the destination directory pref
 
-  nsresult rv = NS_OK;
-
+  nsresult rv;
   uint32_t count = aMailServers.Length();
   for (uint32_t i = 0; i < count; ++i)
   {
@@ -534,14 +535,16 @@ nsSeamonkeyProfileMigrator::CopyMailFolders(PBStructArray &aMailServers,
         // we should make sure the host name based directory we are going to migrate
         // the accounts into is unique. This protects against the case where the user
         // has multiple servers with the same host name.
-        targetMailFolder->CreateUnique(nsIFile::DIRECTORY_TYPE, 0777);
+        rv = targetMailFolder->CreateUnique(nsIFile::DIRECTORY_TYPE, 0777);
+        NS_ENSURE_SUCCESS(rv, rv);
 
-        rv = RecursiveCopy(sourceMailFolder, targetMailFolder);
+        (void) RecursiveCopy(sourceMailFolder, targetMailFolder);
         // now we want to make sure the actual directory pref that gets
         // transformed into the new profile's pref.js has the right file
         // location.
         nsAutoCString descriptorString;
-        targetMailFolder->GetPersistentDescriptor(descriptorString);
+        rv = targetMailFolder->GetPersistentDescriptor(descriptorString);
+        NS_ENSURE_SUCCESS(rv, rv);
         NS_Free(pref->stringValue);
         pref->stringValue = ToNewCString(descriptorString);
       }
@@ -557,7 +560,8 @@ nsSeamonkeyProfileMigrator::CopyMailFolders(PBStructArray &aMailServers,
 
       // turn the pref into a nsIFile
       nsCOMPtr<nsIFile> srcNewsRCFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
-      srcNewsRCFile->SetPersistentDescriptor(nsDependentCString(pref->stringValue));
+      rv = srcNewsRCFile->SetPersistentDescriptor(nsDependentCString(pref->stringValue));
+      NS_ENSURE_SUCCESS(rv, rv);
 
       // now make the copy
       bool exists;
@@ -571,7 +575,8 @@ nsSeamonkeyProfileMigrator::CopyMailFolders(PBStructArray &aMailServers,
 
         // now write out the new descriptor
         nsAutoCString descriptorString;
-        targetNewsRCFile->GetPersistentDescriptor(descriptorString);
+        rv = targetNewsRCFile->GetPersistentDescriptor(descriptorString);
+        NS_ENSURE_SUCCESS(rv, rv);
         NS_Free(pref->stringValue);
         pref->stringValue = ToNewCString(descriptorString);
       }
@@ -676,8 +681,6 @@ nsSeamonkeyProfileMigrator::WriteBranch(const char *branchName,
                                         nsIPrefService* aPrefService,
                                         PBStructArray &aPrefs)
 {
-  nsresult rv;
-
   // Enumerate the branch
   nsCOMPtr<nsIPrefBranch> branch;
   aPrefService->GetBranch(branchName, getter_AddRefs(branch));
@@ -687,15 +690,15 @@ nsSeamonkeyProfileMigrator::WriteBranch(const char *branchName,
     PrefBranchStruct* pref = aPrefs.ElementAt(i);
     switch (pref->type) {
     case nsIPrefBranch::PREF_STRING:
-      rv = branch->SetCharPref(pref->prefName, pref->stringValue);
+      (void) branch->SetCharPref(pref->prefName, pref->stringValue);
       NS_Free(pref->stringValue);
       pref->stringValue = nullptr;
       break;
     case nsIPrefBranch::PREF_BOOL:
-      rv = branch->SetBoolPref(pref->prefName, pref->boolValue);
+      (void) branch->SetBoolPref(pref->prefName, pref->boolValue);
       break;
     case nsIPrefBranch::PREF_INT:
-      rv = branch->SetIntPref(pref->prefName, pref->intValue);
+      (void) branch->SetIntPref(pref->prefName, pref->intValue);
       break;
     default:
       NS_WARNING("Invalid Pref Type in "
@@ -725,7 +728,7 @@ nsSeamonkeyProfileMigrator::CopyJunkTraining(bool aReplace)
 nsresult
 nsSeamonkeyProfileMigrator::CopyPasswords(bool aReplace)
 {
-  nsresult rv;
+  nsresult rv = NS_OK;
 
   nsCString signonsFileName;
   GetSignonFileName(aReplace, getter_Copies(signonsFileName));
