@@ -1,83 +1,64 @@
-/*
-# -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
- */
+/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //****************************************************************************//
 // Constants & Enumeration Values
 
-const PREF_DISABLED_PLUGIN_TYPES = "plugin.disable_full_page_plugin_for_types";
+var PREF_DISABLED_PLUGIN_TYPES = "plugin.disable_full_page_plugin_for_types";
 
 // Preferences that affect which entries to show in the list.
-const PREF_SHOW_PLUGINS_IN_LIST = "browser.download.show_plugins_in_list";
-const PREF_HIDE_PLUGINS_WITHOUT_EXTENSIONS =
+var PREF_SHOW_PLUGINS_IN_LIST = "browser.download.show_plugins_in_list";
+var PREF_HIDE_PLUGINS_WITHOUT_EXTENSIONS =
   "browser.download.hide_plugins_without_extensions";
 
 // The nsHandlerInfoAction enumeration values in nsIHandlerInfo identify
 // the actions the application can take with content of various types.
 // But since nsIHandlerInfo doesn't support plugins, there's no value
 // identifying the "use plugin" action, so we use this constant instead.
-const kActionUsePlugin = 5;
-
-/*
-#ifdef MOZ_WIDGET_GTK
-*/
-const ICON_URL_APP      = "moz-icon://dummy.exe?size=16";
-/*
-#else
-*/
-const ICON_URL_APP      = "chrome://messenger/skin/preferences/application.png";
-/*
-#endif
-*/
+var kActionUsePlugin = 5;
 
 // For CSS. Can be one of "ask", "save", "plugin" or "feed". If absent, the icon URL
 // was set by us to a custom handler icon and CSS should not try to override it.
-const APP_ICON_ATTR_NAME = "appHandlerIcon";
+var APP_ICON_ATTR_NAME = "appHandlerIcon";
 
 // CloudFile account tools used by gCloudFileTab.
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource:///modules/cloudFileAccounts.js");
-Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource:///modules/cloudFileAccounts.js");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 
 //****************************************************************************//
 // Utilities
 
 function getDisplayNameForFile(aFile) {
-/*
-#ifdef XP_WIN
-*/
-  if (aFile instanceof Components.interfaces.nsILocalFileWin) {
-    try {
-      return aFile.getVersionInfoField("FileDescription");
-    }
-    catch(ex) {
-      // fall through to the file name
+  if (AppConstants.platform == "win") {
+    if (aFile instanceof Ci.nsILocalFileWin) {
+      try {
+        return aFile.getVersionInfoField("FileDescription");
+      }
+      catch(ex) {
+        // fall through to the file name
+      }
     }
   }
-/*
-#endif
-#ifdef XP_MACOSX
-*/
-  if (aFile instanceof Components.interfaces.nsILocalFileMac) {
-    try {
-      return aFile.bundleDisplayName;
-    }
-    catch(ex) {
-      // fall through to the file name
+  else if (AppConstants.platform == "macosx") {
+    if (aFile instanceof Ci.nsILocalFileMac) {
+      try {
+        return aFile.bundleDisplayName;
+      }
+      catch(ex) {
+        // fall through to the file name
+      }
     }
   }
-/*
-#endif
-*/
+
   return aFile.leafName;
 }
 
 function getLocalHandlerApp(aFile) {
-  var localHandlerApp = Components.classes["@mozilla.org/uriloader/local-handler-app;1"]
-                                  .createInstance(Components.interfaces.nsILocalHandlerApp);
+  var localHandlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"]
+                          .createInstance(Ci.nsILocalHandlerApp);
   localHandlerApp.name = getDisplayNameForFile(aFile);
   localHandlerApp.executable = aFile;
 
@@ -140,11 +121,11 @@ HandlerInfoWrapper.prototype = {
   //**************************************************************************//
   // Convenience Utils
 
-  _handlerSvc: Components.classes["@mozilla.org/uriloader/handler-service;1"]
-                         .getService(Components.interfaces.nsIHandlerService),
+  _handlerSvc: Cc["@mozilla.org/uriloader/handler-service;1"]
+                 .getService(Ci.nsIHandlerService),
 
-  _categoryMgr: Components.classes["@mozilla.org/categorymanager;1"]
-                          .getService(Components.interfaces.nsICategoryManager),
+  _categoryMgr: Cc["@mozilla.org/categorymanager;1"]
+                  .getService(Ci.nsICategoryManager),
 
   //**************************************************************************//
   // nsIHandlerInfo
@@ -188,7 +169,7 @@ HandlerInfoWrapper.prototype = {
       if (this.possibleApplicationHandlers.indexOf(0, aNewHandler) != -1)
         return;
     } catch (e) { }
-    this.possibleApplicationHandlers.appendElement(aNewHandler, false);
+    this.possibleApplicationHandlers.appendElement(aNewHandler);
   },
 
   removePossibleApplicationHandler: function(aHandler) {
@@ -227,12 +208,12 @@ HandlerInfoWrapper.prototype = {
     // but the alwaysAskBeforeHandling getter will detect that situation
     // and always return true in that case to override this invalid value.
     if (this.wrappedHandlerInfo.preferredAction ==
-          Components.interfaces.nsIHandlerInfo.useHelperApp &&
+          Ci.nsIHandlerInfo.useHelperApp &&
         !gApplicationsPane.isValidHandlerApp(this.preferredApplicationHandler)) {
       if (this.wrappedHandlerInfo.hasDefaultHandler)
-        return Components.interfaces.nsIHandlerInfo.useSystemDefault;
+        return Ci.nsIHandlerInfo.useSystemDefault;
       else
-        return Components.interfaces.nsIHandlerInfo.saveToDisk;
+        return Ci.nsIHandlerInfo.saveToDisk;
     }
 
     return this.wrappedHandlerInfo.preferredAction;
@@ -263,8 +244,8 @@ HandlerInfoWrapper.prototype = {
     // app, but the preferredApplicationHandler is invalid, and there isn't
     // a default handler, so the preferredAction getter returns save to disk
     // instead.
-    if (!(this.wrappedHandlerInfo instanceof Components.interfaces.nsIMIMEInfo) &&
-        this.preferredAction == Components.interfaces.nsIHandlerInfo.saveToDisk)
+    if (!(this.wrappedHandlerInfo instanceof Ci.nsIMIMEInfo) &&
+        this.preferredAction == Ci.nsIHandlerInfo.saveToDisk)
       return true;
 
     return this.wrappedHandlerInfo.alwaysAskBeforeHandling;
@@ -285,7 +266,7 @@ HandlerInfoWrapper.prototype = {
   // those properties for an extension?
   get primaryExtension() {
     try {
-      if (this.wrappedHandlerInfo instanceof Components.interfaces.nsIMIMEInfo &&
+      if (this.wrappedHandlerInfo instanceof Ci.nsIMIMEInfo &&
           this.wrappedHandlerInfo.primaryExtension)
         return this.wrappedHandlerInfo.primaryExtension;
     } catch(ex) {}
@@ -319,7 +300,7 @@ HandlerInfoWrapper.prototype = {
   handledOnlyByPlugin: undefined,
 
   get isDisabledPluginType() {
-    return this._getDisabledPluginTypes().indexOf(this.type) != -1;
+    return this._getDisabledPluginTypes().includes(this.type);
   },
 
   _getDisabledPluginTypes: function() {
@@ -336,7 +317,7 @@ HandlerInfoWrapper.prototype = {
   disablePluginType: function() {
     var disabledPluginTypes = this._getDisabledPluginTypes();
 
-    if (disabledPluginTypes.indexOf(this.type) == -1)
+    if (!disabledPluginTypes.includes(this.type))
       disabledPluginTypes.push(this.type);
 
     Services.prefs.setCharPref(PREF_DISABLED_PLUGIN_TYPES,
@@ -394,7 +375,7 @@ HandlerInfoWrapper.prototype = {
     if (this.primaryExtension)
       return "moz-icon://goat." + this.primaryExtension + "?size=" + aSize;
 
-    if (this.wrappedHandlerInfo instanceof Components.interfaces.nsIMIMEInfo)
+    if (this.wrappedHandlerInfo instanceof Ci.nsIMIMEInfo)
       return "moz-icon://goat?size=" + aSize + "&contentType=" + this.type;
 
     // FIXME: consider returning some generic icon when we can't get a URL for
@@ -438,9 +419,6 @@ var gApplicationsTabController = {
     }
 
     let loadInContent = Services.prefs.getBoolPref("mail.preferences.inContent");
-    if (loadInContent) {
-      gSubDialog.init();
-    }
 
     this.mInitialized = true;
   },
@@ -550,7 +528,7 @@ var gCloudFileTab = {
     if (this._list.itemCount > 0)
       this._list.selectedIndex = 0;
 
-    window.addEventListener("unload", this, false);
+    window.addEventListener("unload", this, {capture: false, once: true});
     CommandUpdate_CloudFile();
 
     this.updateThreshold();
@@ -561,7 +539,6 @@ var gCloudFileTab = {
   destroy: function CFT_destroy() {
     // Remove any controllers or observers here.
     top.controllers.removeController(gCloudFileController);
-    window.removeEventListener("unload", this, false);
   },
 
   makeRichListItemForAccount: function CFT_makeRichListItemForAccount(aAccount) {
@@ -620,7 +597,7 @@ var gCloudFileTab = {
 
     accounts.sort(sortAccounts);
 
-    for (let [, account] in Iterator(accounts)) {
+    for (let account of accounts) {
       let rli = this.makeRichListItemForAccount(account);
       this._list.appendChild(rli);
       if (!(account.accountKey in this._accountCache))
@@ -629,7 +606,6 @@ var gCloudFileTab = {
   },
 
   requestUserInfoForItem: function CFT_requestUserInfoForItem(aItem, aWithUI) {
-    let Cr = Components.results;
     let accountKey = aItem.value;
     let account = cloudFileAccounts.getAccount(accountKey);
 
@@ -672,8 +648,6 @@ var gCloudFileTab = {
   },
 
   _mapResultToState: function CFT__mapResultToState(aItem, aResult) {
-    let Cr = Components.results;
-    let Ci = Components.interfaces;
     let itemState = "no-connection";
 
     if (aResult == Cr.NS_OK)
@@ -701,8 +675,6 @@ var gCloudFileTab = {
   },
 
   _showAccountInfo: function CFT__showAccountInfo(aAccountKey) {
-    let Ci = Components.interfaces;
-    let Cr = Components.results;
     let account = this._accountCache[aAccountKey].account;
     let result = this._accountCache[aAccountKey].result;
 
@@ -716,7 +688,7 @@ var gCloudFileTab = {
       this._settingsDeck.selectedPanel = this._authErrorPanel;
     }
     else {
-      Components.utils.reportError("Unexpected connection error.");
+      Cu.reportError("Unexpected connection error.");
     }
   },
 
@@ -737,32 +709,25 @@ var gCloudFileTab = {
     // When the iframe loads, populate it with the provider.
     this._settings.contentWindow.addEventListener("load",
       function loadProvider() {
-        iframe.contentWindow.removeEventListener("load",
-                                                 loadProvider,
-                                                 false);
         try {
           iframe.contentWindow
                 .wrappedJSObject
                 .onLoadProvider(aProvider);
         } catch(e) {
-          Components.utils.reportError(e);
+          Cu.reportError(e);
         }
-      }, false);
+      }, {capture: false, once: true});
 
     // When the iframe (or any subcontent) fires the DOMContentLoaded event,
     // attach the _onClickLink handler to any anchor elements that we can find.
     this._settings.contentWindow.addEventListener("DOMContentLoaded",
       function addClickListeners(e) {
-        iframe.contentWindow.removeEventListener("DOMContentLoaded",
-                                                 addClickListeners,
-                                                 false);
-
         let doc = e.originalTarget;
         let links = doc.getElementsByTagName("a");
 
-        for (let [, link] in Iterator(links))
+        for (let link of links)
           link.addEventListener("click", gCloudFileTab._onClickLink);
-      }, false);
+      }, {capture: false, once: true});
 
     CommandUpdate_CloudFile();
   },
@@ -838,10 +803,10 @@ var gCloudFileTab = {
     !document.getElementById("enableThreshold").checked;
   },
 
-  QueryInterface: XPCOMUtils.generateQI([Components.interfaces
-                                                   .nsIObserver,
-                                         Components.interfaces
-                                                   .nsISupportsWeakReference])
+  QueryInterface: XPCOMUtils.generateQI([Ci
+                                           .nsIObserver,
+                                         Ci
+                                           .nsISupportsWeakReference])
 }
 
 //****************************************************************************//
@@ -877,14 +842,14 @@ var gApplicationsPane = {
   _list           : null,
   _filter         : null,
 
-  _mimeSvc      : Components.classes["@mozilla.org/mime;1"]
-                            .getService(Components.interfaces.nsIMIMEService),
+  _mimeSvc      : Cc["@mozilla.org/mime;1"]
+                    .getService(Ci.nsIMIMEService),
 
-  _helperAppSvc : Components.classes["@mozilla.org/uriloader/external-helper-app-service;1"]
-                            .getService(Components.interfaces.nsIExternalHelperAppService),
+  _helperAppSvc : Cc["@mozilla.org/uriloader/external-helper-app-service;1"]
+                    .getService(Ci.nsIExternalHelperAppService),
 
-  _handlerSvc   : Components.classes["@mozilla.org/uriloader/handler-service;1"]
-                            .getService(Components.interfaces.nsIHandlerService),
+  _handlerSvc   : Cc["@mozilla.org/uriloader/handler-service;1"]
+                    .getService(Ci.nsIHandlerService),
 
   _loadInContent: Services.prefs.getBoolPref("mail.preferences.inContent"),
 
@@ -901,11 +866,11 @@ var gApplicationsPane = {
 
     // Observe preferences that influence what we display so we can rebuild
     // the view when they change.
-    Services.prefs.addObserver(PREF_SHOW_PLUGINS_IN_LIST, this, false);
-    Services.prefs.addObserver(PREF_HIDE_PLUGINS_WITHOUT_EXTENSIONS, this, false);
+    Services.prefs.addObserver(PREF_SHOW_PLUGINS_IN_LIST, this);
+    Services.prefs.addObserver(PREF_HIDE_PLUGINS_WITHOUT_EXTENSIONS, this);
 
     // Listen for window unload so we can remove our preference observers.
-    window.addEventListener("unload", this, false);
+    window.addEventListener("unload", this, {capture: false, once: true});
 
     // Figure out how we should be sorting the list.  We persist sort settings
     // across sessions, so we can't assume the default sort column/direction.
@@ -933,13 +898,12 @@ var gApplicationsPane = {
       self.rebuildView();
 
       // Notify observers that the UI is now ready
-      Services.obs.notifyObservers(window, "app-handler-pane-loaded", null);
+      Services.obs.notifyObservers(window, "app-handler-pane-loaded");
     }
     setTimeout(_delayedPaneLoad, 0, this);
   },
 
   destroy: function() {
-    window.removeEventListener("unload", this, false);
     Services.prefs.removeObserver(PREF_SHOW_PLUGINS_IN_LIST, this);
     Services.prefs.removeObserver(PREF_HIDE_PLUGINS_WITHOUT_EXTENSIONS, this);
   },
@@ -949,12 +913,12 @@ var gApplicationsPane = {
   // nsISupports
 
   QueryInterface: function(aIID) {
-    if (aIID.equals(Components.interfaces.nsIObserver) ||
-        aIID.equals(Components.interfaces.nsIDOMEventListener ||
-        aIID.equals(Components.interfaces.nsISupports)))
+    if (aIID.equals(Ci.nsIObserver) ||
+        aIID.equals(Ci.nsIDOMEventListener ||
+        aIID.equals(Ci.nsISupports)))
       return this;
 
-    throw Components.results.NS_ERROR_NO_INTERFACE;
+    throw Cr.NS_ERROR_NO_INTERFACE;
   },
 
 
@@ -1046,7 +1010,7 @@ var gApplicationsPane = {
     var wrappedHandlerInfos = this._handlerSvc.enumerate();
     while (wrappedHandlerInfos.hasMoreElements()) {
       let wrappedHandlerInfo =
-        wrappedHandlerInfos.getNext().QueryInterface(Components.interfaces.nsIHandlerInfo);
+        wrappedHandlerInfos.getNext().QueryInterface(Ci.nsIHandlerInfo);
       let type = wrappedHandlerInfo.type;
 
       let handlerInfoWrapper;
@@ -1086,7 +1050,7 @@ var gApplicationsPane = {
       // FIXME: should we also check the "suffixes" property of the plugin?
       // Filed as bug 395135.
       if (hidePluginsWithoutExtensions && handlerInfo.handledOnlyByPlugin &&
-          handlerInfo.wrappedHandlerInfo instanceof Components.interfaces.nsIMIMEInfo &&
+          handlerInfo.wrappedHandlerInfo instanceof Ci.nsIMIMEInfo &&
           !handlerInfo.primaryExtension)
         continue;
 
@@ -1114,7 +1078,7 @@ var gApplicationsPane = {
     if (this._filter.value)
       visibleTypes = visibleTypes.filter(this._matchesFilter, this);
 
-    for each (let visibleType in visibleTypes) {
+    for (let visibleType of visibleTypes) {
       let item = document.createElement("richlistitem");
       item.setAttribute("type", visibleType.type);
       item.setAttribute("typeDescription", this._describeType(visibleType));
@@ -1171,11 +1135,11 @@ var gApplicationsPane = {
    */
   _typeDetails: function(aHandlerInfo) {
     let exts = [];
-    if (aHandlerInfo.wrappedHandlerInfo instanceof Components.interfaces.nsIMIMEInfo) {
+    if (aHandlerInfo.wrappedHandlerInfo instanceof Ci.nsIMIMEInfo) {
       let extIter = aHandlerInfo.wrappedHandlerInfo.getFileExtensions();
       while(extIter.hasMore()) {
         let ext = "."+extIter.getNext();
-        if (exts.indexOf(ext) == -1)
+        if (!exts.includes(ext))
           exts.push(ext);
       }
     }
@@ -1214,19 +1178,19 @@ var gApplicationsPane = {
         return this._prefsBundle.getString("alwaysAsk");
 
     switch (aHandlerInfo.preferredAction) {
-      case Components.interfaces.nsIHandlerInfo.saveToDisk:
+      case Ci.nsIHandlerInfo.saveToDisk:
         return this._prefsBundle.getString("saveFile");
 
-      case Components.interfaces.nsIHandlerInfo.useHelperApp:
+      case Ci.nsIHandlerInfo.useHelperApp:
         var preferredApp = aHandlerInfo.preferredApplicationHandler;
         var name;
-        if (preferredApp instanceof Components.interfaces.nsILocalHandlerApp)
+        if (preferredApp instanceof Ci.nsILocalHandlerApp)
           name = getDisplayNameForFile(preferredApp.executable);
         else
           name = preferredApp.name;
         return this._prefsBundle.getFormattedString("useApp", [name]);
 
-      case Components.interfaces.nsIHandlerInfo.handleInternally:
+      case Ci.nsIHandlerInfo.handleInternally:
         // For other types, handleInternally looks like either useHelperApp
         // or useSystemDefault depending on whether or not there's a preferred
         // handler app.
@@ -1240,7 +1204,7 @@ var gApplicationsPane = {
         // then why would a preferredAction ever get set to this value
         // in the first place?
 
-      case Components.interfaces.nsIHandlerInfo.useSystemDefault:
+      case Ci.nsIHandlerInfo.useSystemDefault:
         return this._prefsBundle.getFormattedString("useDefault",
                                                     [aHandlerInfo.defaultDescription]);
 
@@ -1250,7 +1214,7 @@ var gApplicationsPane = {
                                                      this._brandShortName]);
       default:
         // Hopefully this never happens.
-        Components.utils.reportError("No description for action " + aHandlerInfo.preferredAction + " found!");
+        Cu.reportError("No description for action " + aHandlerInfo.preferredAction + " found!");
         return "";
     }
   },
@@ -1283,34 +1247,32 @@ var gApplicationsPane = {
     if (!aHandlerApp)
       return false;
 
-    if (aHandlerApp instanceof Components.interfaces.nsILocalHandlerApp)
+    if (aHandlerApp instanceof Ci.nsILocalHandlerApp)
       return this._isValidHandlerExecutable(aHandlerApp.executable);
 
-    if (aHandlerApp instanceof Components.interfaces.nsIWebHandlerApp)
+    if (aHandlerApp instanceof Ci.nsIWebHandlerApp)
       return aHandlerApp.uriTemplate;
 
-    if (aHandlerApp instanceof Components.interfaces.nsIWebContentHandlerInfo)
+    if (aHandlerApp instanceof Ci.nsIWebContentHandlerInfo)
       return aHandlerApp.uri;
 
     return false;
   },
 
   _isValidHandlerExecutable: function(aExecutable) {
-    return aExecutable &&
-           aExecutable.exists() &&
-           aExecutable.isExecutable() &&
+    let isExecutable = aExecutable &&
+                       aExecutable.exists() &&
+                       aExecutable.isExecutable();
 // XXXben - we need to compare this with the running instance executable
 //          just don't know how to do that via script...
 // XXXmano TBD: can probably add this to nsIShellService
-#ifdef XP_WIN
-#expand    aExecutable.leafName != "__MOZ_APP_NAME__.exe";
-#else
-#ifdef XP_MACOSX
-#expand    aExecutable.leafName != "__MOZ_MACBUNDLE_NAME__";
-#else
-#expand    aExecutable.leafName != "__MOZ_APP_NAME__-bin";
-#endif
-#endif
+    if (AppConstants.platform == "win")
+      return isExecutable && (aExecutable.leafName != (AppConstants.MOZ_APP_NAME + ".exe"));
+
+    if (AppConstants.platform == "macosx")
+      return isExecutable && (aExecutable.leafName != AppConstants.MOZ_MACBUNDLE_NAME);
+
+    return isExecutable && (aExecutable.leafName != (AppConstants.MOZ_APP_NAME + "-bin"));
   },
 
   /**
@@ -1345,9 +1307,9 @@ var gApplicationsPane = {
     // Create a menu item for saving to disk.
     // Note: this option isn't available to protocol types, since we don't know
     // what it means to save a URL having a certain scheme to disk.
-    if ((handlerInfo.wrappedHandlerInfo instanceof Components.interfaces.nsIMIMEInfo)) {
+    if ((handlerInfo.wrappedHandlerInfo instanceof Ci.nsIMIMEInfo)) {
       var saveMenuItem = document.createElement("menuitem");
-      saveMenuItem.setAttribute("action", Components.interfaces.nsIHandlerInfo.saveToDisk);
+      saveMenuItem.setAttribute("action", Ci.nsIHandlerInfo.saveToDisk);
       let label = this._prefsBundle.getString("saveFile");
       saveMenuItem.setAttribute("label", label);
       saveMenuItem.setAttribute("tooltiptext", label);
@@ -1363,7 +1325,7 @@ var gApplicationsPane = {
     // Create a menu item for the OS default application, if any.
     if (handlerInfo.hasDefaultHandler) {
       var defaultMenuItem = document.createElement("menuitem");
-      defaultMenuItem.setAttribute("action", Components.interfaces.nsIHandlerInfo.useSystemDefault);
+      defaultMenuItem.setAttribute("action", Ci.nsIHandlerInfo.useSystemDefault);
       let label = this._prefsBundle.getFormattedString("useDefault",
                                                        [handlerInfo.defaultDescription]);
       defaultMenuItem.setAttribute("label", label);
@@ -1383,9 +1345,9 @@ var gApplicationsPane = {
         continue;
 
       let menuItem = document.createElement("menuitem");
-      menuItem.setAttribute("action", Components.interfaces.nsIHandlerInfo.useHelperApp);
+      menuItem.setAttribute("action", Ci.nsIHandlerInfo.useHelperApp);
       let label;
-      if (possibleApp instanceof Components.interfaces.nsILocalHandlerApp)
+      if (possibleApp instanceof Ci.nsILocalHandlerApp)
         label = getDisplayNameForFile(possibleApp.executable);
       else
         label = possibleApp.name;
@@ -1416,14 +1378,18 @@ var gApplicationsPane = {
     }
 
     // Create a menu item for selecting a local application.
-#ifdef XP_WIN
-    // On Windows, selecting an application to open another application
-    // would be meaningless so we special case executables.
-    var executableType = Components.classes["@mozilla.org/mime;1"]
-                                   .getService(Components.interfaces.nsIMIMEService)
-                                   .getTypeFromExtension("exe");
-    if (handlerInfo.type != executableType)
-#endif
+    let createItem = true;
+    if (AppConstants.platform == "win") {
+      // On Windows, selecting an application to open another application
+      // would be meaningless so we special case executables.
+      var executableType = Cc["@mozilla.org/mime;1"]
+                             .getService(Ci.nsIMIMEService)
+                             .getTypeFromExtension("exe");
+      if (handlerInfo.type == executableType)
+        createItem = false;
+    }
+
+    if (createItem)
     {
       let menuItem = document.createElement("menuitem");
       menuItem.setAttribute("oncommand", "gApplicationsPane.chooseApp(event)");
@@ -1457,13 +1423,13 @@ var gApplicationsPane = {
     if (handlerInfo.alwaysAskBeforeHandling)
       menu.selectedItem = askMenuItem;
     else switch (handlerInfo.preferredAction) {
-      case Components.interfaces.nsIHandlerInfo.handleInternally:
+      case Ci.nsIHandlerInfo.handleInternally:
         menu.selectedItem = internalMenuItem;
         break;
-      case Components.interfaces.nsIHandlerInfo.useSystemDefault:
+      case Ci.nsIHandlerInfo.useSystemDefault:
         menu.selectedItem = defaultMenuItem;
         break;
-      case Components.interfaces.nsIHandlerInfo.useHelperApp:
+      case Ci.nsIHandlerInfo.useHelperApp:
         if (preferredApp)
           menu.selectedItem =
             possibleAppMenuItems.filter(v => v.handlerApp.equals(preferredApp))[0];
@@ -1471,7 +1437,7 @@ var gApplicationsPane = {
       case kActionUsePlugin:
         menu.selectedItem = pluginMenuItem;
         break;
-      case Components.interfaces.nsIHandlerInfo.saveToDisk:
+      case Ci.nsIHandlerInfo.saveToDisk:
         menu.selectedItem = saveMenuItem;
         break;
     }
@@ -1587,7 +1553,7 @@ var gApplicationsPane = {
       // legacy datastores that don't have the preferred app in the list
       // of possible apps still include the preferred app in the list of apps
       // the user can choose to handle the type.
-      if (action == Components.interfaces.nsIHandlerInfo.useHelperApp)
+      if (action == Ci.nsIHandlerInfo.useHelperApp)
         handlerInfo.preferredApplicationHandler = aActionItem.handlerApp;
 
       // Set the "always ask" flag.
@@ -1649,75 +1615,82 @@ var gApplicationsPane = {
     aEvent.stopPropagation();
 
     var handlerApp;
+    let onSelectionDone = function() {
+      // Rebuild the actions menu whether the user picked an app or canceled.
+      // If they picked an app, we want to add the app to the menu and select it.
+      // If they canceled, we want to go back to their previous selection.
+      this.rebuildActionsMenu();
 
-#ifdef XP_WIN
-    var params = {};
-    var handlerInfo = this._handledTypes[this._list.selectedItem.type];
-
-    params.mimeInfo = handlerInfo.wrappedHandlerInfo;
-
-    params.title         = this._prefsBundle.getString("fpTitleChooseApp");
-    params.description   = handlerInfo.description;
-    params.filename      = null;
-    params.handlerApp    = null;
-
-    if (this._loadInContent) {
-      gSubDialog.open("chrome://global/content/appPicker.xul",
-                      "resizable=no", params);
-    } else {
-      window.openDialog("chrome://global/content/appPicker.xul", null,
-                        "chrome,modal,centerscreen,titlebar,dialog=yes",
-                        params);
-    };
-
-    if (params.handlerApp &&
-        params.handlerApp.executable &&
-        params.handlerApp.executable.isFile()) {
-      handlerApp = params.handlerApp;
-
-      // Add the app to the type's list of possible handlers.
-      handlerInfo.addPossibleApplicationHandler(handlerApp);
-    }
-#else
-    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
-    var winTitle = this._prefsBundle.getString("fpTitleChooseApp");
-    fp.init(window, winTitle, Components.interfaces.nsIFilePicker.modeOpen);
-    fp.appendFilters(Components.interfaces.nsIFilePicker.filterApps);
-
-    // Prompt the user to pick an app.  If they pick one, and it's a valid
-    // selection, then add it to the list of possible handlers.
-    if (fp.show() == Components.interfaces.nsIFilePicker.returnOK && fp.file &&
-        this._isValidHandlerExecutable(fp.file)) {
-      handlerApp = Components.classes["@mozilla.org/uriloader/local-handler-app;1"]
-                             .createInstance(Components.interfaces.nsILocalHandlerApp);
-      handlerApp.name = getDisplayNameForFile(fp.file);
-      handlerApp.executable = fp.file;
-
-      // Add the app to the type's list of possible handlers.
-      let handlerInfo = this._handledTypes[this._list.selectedItem.type];
-      handlerInfo.addPossibleApplicationHandler(handlerApp);
-    }
-#endif
-
-    // Rebuild the actions menu whether the user picked an app or canceled.
-    // If they picked an app, we want to add the app to the menu and select it.
-    // If they canceled, we want to go back to their previous selection.
-    this.rebuildActionsMenu();
-
-    // If the user picked a new app from the menu, select it.
-    if (handlerApp) {
-      let typeItem = this._list.selectedItem;
-      let actionsMenu =
-        document.getAnonymousElementByAttribute(typeItem, "class", "actionsMenu");
-      let menuItems = actionsMenu.menupopup.childNodes;
-      for (let i = 0; i < menuItems.length; i++) {
-        let menuItem = menuItems[i];
-        if (menuItem.handlerApp && menuItem.handlerApp.equals(handlerApp)) {
-          actionsMenu.selectedIndex = i;
-          this.onSelectAction(menuItem);
-          break;
+      // If the user picked a new app from the menu, select it.
+      if (handlerApp) {
+        let typeItem = this._list.selectedItem;
+        let actionsMenu =
+          document.getAnonymousElementByAttribute(typeItem, "class", "actionsMenu");
+        let menuItems = actionsMenu.menupopup.childNodes;
+        for (let i = 0; i < menuItems.length; i++) {
+          let menuItem = menuItems[i];
+          if (menuItem.handlerApp && menuItem.handlerApp.equals(handlerApp)) {
+            actionsMenu.selectedIndex = i;
+            this.onSelectAction(menuItem);
+            break;
+          }
         }
       }
+    }.bind(this);
+
+    if (AppConstants.platform == "win") {
+      let params = {};
+      let handlerInfo = this._handledTypes[this._list.selectedItem.type];
+
+      params.mimeInfo = handlerInfo.wrappedHandlerInfo;
+
+      params.title         = this._prefsBundle.getString("fpTitleChooseApp");
+      params.description   = handlerInfo.description;
+      params.filename      = null;
+      params.handlerApp    = null;
+
+      if (this._loadInContent) {
+        gSubDialog.open("chrome://global/content/appPicker.xul",
+                        "resizable=no", params);
+      } else {
+        window.openDialog("chrome://global/content/appPicker.xul", null,
+                          "chrome,modal,centerscreen,titlebar,dialog=yes",
+                          params);
+      };
+
+      if (params.handlerApp &&
+          params.handlerApp.executable &&
+          params.handlerApp.executable.isFile()) {
+        handlerApp = params.handlerApp;
+
+        // Add the app to the type's list of possible handlers.
+        handlerInfo.addPossibleApplicationHandler(handlerApp);
+      }
+      onSelectionDone();
+    } else {
+      const nsIFilePicker = Ci.nsIFilePicker;
+      let fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
+      let winTitle = this._prefsBundle.getString("fpTitleChooseApp");
+      fp.init(window, winTitle, nsIFilePicker.modeOpen);
+      fp.appendFilters(nsIFilePicker.filterApps);
+
+      // Prompt the user to pick an app.  If they pick one, and it's a valid
+      // selection, then add it to the list of possible handlers.
+
+      fp.open(rv => {
+        if (rv == nsIFilePicker.returnOK && fp.file &&
+            this._isValidHandlerExecutable(fp.file)) {
+          handlerApp = Cc["@mozilla.org/uriloader/local-handler-app;1"]
+                         .createInstance(Ci.nsILocalHandlerApp);
+          handlerApp.name = getDisplayNameForFile(fp.file);
+          handlerApp.executable = fp.file;
+
+          // Add the app to the type's list of possible handlers.
+          let handlerInfo = this._handledTypes[this._list.selectedItem.type];
+          handlerInfo.addPossibleApplicationHandler(handlerApp);
+        }
+        onSelectionDone();
+      });
     }
   },
 
@@ -1773,11 +1746,11 @@ var gApplicationsPane = {
     }
 
     switch (aHandlerInfo.preferredAction) {
-      case Components.interfaces.nsIHandlerInfo.saveToDisk:
+      case Ci.nsIHandlerInfo.saveToDisk:
         aElement.setAttribute(APP_ICON_ATTR_NAME, "save");
         return true;
 
-      case Components.interfaces.nsIHandlerInfo.handleInternally:
+      case Ci.nsIHandlerInfo.handleInternally:
         break;
 
       case kActionUsePlugin:
@@ -1790,10 +1763,10 @@ var gApplicationsPane = {
 
   _getIconURLForPreferredAction: function(aHandlerInfo) {
     switch (aHandlerInfo.preferredAction) {
-      case Components.interfaces.nsIHandlerInfo.useSystemDefault:
+      case Ci.nsIHandlerInfo.useSystemDefault:
         return this._getIconURLForSystemDefault(aHandlerInfo);
 
-      case Components.interfaces.nsIHandlerInfo.useHelperApp:
+      case Ci.nsIHandlerInfo.useHelperApp:
         let preferredApp = aHandlerInfo.preferredApplicationHandler;
         if (this.isValidHandlerApp(preferredApp))
           return this._getIconURLForHandlerApp(preferredApp);
@@ -1804,13 +1777,13 @@ var gApplicationsPane = {
   },
 
   _getIconURLForHandlerApp: function(aHandlerApp) {
-    if (aHandlerApp instanceof Components.interfaces.nsILocalHandlerApp)
+    if (aHandlerApp instanceof Ci.nsILocalHandlerApp)
       return this._getIconURLForFile(aHandlerApp.executable);
 
-    if (aHandlerApp instanceof Components.interfaces.nsIWebHandlerApp)
+    if (aHandlerApp instanceof Ci.nsIWebHandlerApp)
       return this._getIconURLForWebApp(aHandlerApp.uriTemplate);
 
-    if (aHandlerApp instanceof Components.interfaces.nsIWebContentHandlerInfo)
+    if (aHandlerApp instanceof Ci.nsIWebContentHandlerInfo)
       return this._getIconURLForWebApp(aHandlerApp.uri)
 
     // We know nothing about other kinds of handler apps.
@@ -1819,14 +1792,14 @@ var gApplicationsPane = {
 
   _getIconURLForFile: function(aFile) {
     let urlSpec = Services.io.getProtocolHandler("file")
-      .QueryInterface(Components.interfaces.nsIFileProtocolHandler)
+      .QueryInterface(Ci.nsIFileProtocolHandler)
       .getURLSpecFromFile(aFile);
 
     return "moz-icon://" + urlSpec + "?size=16";
   },
 
   _getIconURLForWebApp: function(aWebAppURITemplate) {
-    var uri = Services.io.newURI(aWebAppURITemplate, null, null);
+    var uri = Services.io.newURI(aWebAppURITemplate);
 
     // Unfortunately we can't use the favicon service to get the favicon,
     // because the service looks in the annotations table for a record with
@@ -1848,8 +1821,8 @@ var gApplicationsPane = {
     if ("wrappedHandlerInfo" in aHandlerInfo) {
       let wrappedHandlerInfo = aHandlerInfo.wrappedHandlerInfo;
 
-      if (wrappedHandlerInfo instanceof Components.interfaces.nsIMIMEInfo &&
-          wrappedHandlerInfo instanceof Components.interfaces.nsIPropertyBag) {
+      if (wrappedHandlerInfo instanceof Ci.nsIMIMEInfo &&
+          wrappedHandlerInfo instanceof Ci.nsIPropertyBag) {
         try {
           let url = wrappedHandlerInfo.getProperty("defaultApplicationIconURL");
           if (url)
