@@ -7,31 +7,30 @@ MOZ_APP_BASENAME=Thunderbird
 MOZ_APP_NAME=thunderbird
 MOZ_UPDATER=1
 MOZ_THUNDERBIRD=1
-MOZ_APP_STATIC_INI=1
-MOZ_CHROME_FILE_FORMAT=omni
-MOZ_DISABLE_EXPORT_JS=1
-MOZ_NO_ACTIVEX_SUPPORT=1
-MOZ_ACTIVEX_SCRIPTING_SUPPORT=
 MOZ_LDAP_XPCOM=1
 MOZ_COMPOSER=1
+
+if test "$OS_ARCH" = "WINNT" -o \
+        "$OS_ARCH" = "Linux"; then
+  MOZ_BUNDLED_FONTS=1
+fi
+
 if test "$OS_ARCH" = "WINNT"; then
+  MOZ_MAINTENANCE_SERVICE=1
   if ! test "$HAVE_64BIT_BUILD"; then
     MOZ_VERIFY_MAR_SIGNATURE=1
-    MOZ_MAINTENANCE_SERVICE=1
   fi
 fi
 
-MOZ_SAFE_BROWSING=1
-MOZ_MEDIA_NAVIGATOR=1
 MOZ_MORK=1
 
 MOZ_APP_VERSION_TXT=${_topsrcdir}/$MOZ_BUILD_APP/config/version.txt
 MOZ_APP_VERSION=`cat $MOZ_APP_VERSION_TXT`
+MOZ_APP_VERSION_DISPLAY_TXT=${_topsrcdir}/$MOZ_BUILD_APP/config/version_display.txt
+MOZ_APP_VERSION_DISPLAY=`cat $MOZ_APP_VERSION_DISPLAY_TXT`
 THUNDERBIRD_VERSION=$MOZ_APP_VERSION
 
-MOZ_UA_BUILDID=20100101
-
-MOZ_BRANDING_DIRECTORY=mail/branding/aurora
+MOZ_BRANDING_DIRECTORY=mail/branding/nightly
 MOZ_OFFICIAL_BRANDING_DIRECTORY=other-licenses/branding/thunderbird
 MOZ_APP_ID={3550f703-e582-4d05-9a08-453d09bdfdc6}
 # This should usually be the same as the value MAR_CHANNEL_ID.
@@ -40,8 +39,13 @@ MOZ_APP_ID={3550f703-e582-4d05-9a08-453d09bdfdc6}
 ACCEPTED_MAR_CHANNEL_IDS=thunderbird-comm-release
 # The MAR_CHANNEL_ID must not contain the following 3 characters: ",\t "
 MAR_CHANNEL_ID=thunderbird-comm-release
-# Enable generational GC on desktop.
-JSGC_GENERATIONAL=1
 MOZ_PROFILE_MIGRATOR=1
-MOZ_JSDOWNLOADS=1
 MOZ_BINARY_EXTENSIONS=1
+MOZ_SEPARATE_MANIFEST_FOR_THEME_OVERRIDES=1
+
+# Enable building ./signmar and running libmar signature tests
+MOZ_ENABLE_SIGNMAR=1
+
+MOZ_DEVTOOLS=all
+
+NSS_EXTRA_SYMBOLS_FILE=${_topsrcdir}/../mailnews/nss-extra.symbols
