@@ -6,48 +6,40 @@
  * Tests the get an account (account provisioning) workflow.
  */
 
-var Cu = Components.utils;
-var Cc = Components.classes;
-var Ci = Components.interfaces;
+var MODULE_NAME = 'test-newmailaccount';
 
-const MODULE_NAME = 'test-newmailaccount';
+var RELATIVE_ROOT = '../shared-modules';
+var MODULE_REQUIRES = ['folder-display-helpers',
+                       'content-tab-helpers',
+                       'window-helpers',
+                       'newmailaccount-helpers',
+                       'keyboard-helpers',
+                       'dom-helpers'];
 
-const RELATIVE_ROOT = '../shared-modules';
-const MODULE_REQUIRES = ['folder-display-helpers',
-                         'content-tab-helpers',
-                         'window-helpers',
-                         'newmailaccount-helpers',
-                         'keyboard-helpers',
-                         'dom-helpers'];
-
-var controller = {};
-var mozmill = {};
 var elib = {};
-Cu.import('resource://mozmill/modules/controller.js', controller);
-Cu.import('resource://mozmill/modules/mozmill.js', mozmill);
-Cu.import('resource://mozmill/modules/elementslib.js', elib);
-Cu.import('resource://gre/modules/Services.jsm');
-Cu.import('resource:///modules/iteratorUtils.jsm');
-Cu.import("resource:///modules/mailServices.js");
-Cu.import('resource://mozmill/stdlib/httpd.js');
+ChromeUtils.import('resource://mozmill/modules/elementslib.js', elib);
+ChromeUtils.import('resource://gre/modules/Services.jsm');
+ChromeUtils.import('resource:///modules/iteratorUtils.jsm');
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import('resource://mozmill/stdlib/httpd.js');
 
 // RELATIVE_ROOT messes with the collector, so we have to bring the path back
 // so we get the right path for the resources.
 var url = collector.addHttpResource('../newmailaccount/html', '');
-const kProvisionerUrl = "chrome://messenger/content/newmailaccount/accountProvisioner.xhtml";
-const kProvisionerEnabledPref = "mail.provider.enabled";
-const kSuggestFromNamePref = "mail.provider.suggestFromName";
-const kProviderListPref = "mail.provider.providerList";
-const kAcceptedLanguage = "general.useragent.locale";
-const kDefaultServerPort = 4444;
-const kDefaultServerRoot = "http://localhost:" + kDefaultServerPort;
+var kProvisionerUrl = "chrome://messenger/content/newmailaccount/accountProvisioner.xhtml";
+var kProvisionerEnabledPref = "mail.provider.enabled";
+var kSuggestFromNamePref = "mail.provider.suggestFromName";
+var kProviderListPref = "mail.provider.providerList";
+var kAcceptedLanguage = "general.useragent.locale";
+var kDefaultServerPort = 4444;
+var kDefaultServerRoot = "http://localhost:" + kDefaultServerPort;
 
 Services.prefs.setCharPref(kProviderListPref, url + "providerList");
 Services.prefs.setCharPref(kSuggestFromNamePref, url + "suggestFromName");
 
 // Here's a name that we'll type in later on.  It's a global const because
 // we'll be using it in several distinct modal dialog event loops.
-const NAME = "Leonard Shelby";
+var NAME = "Leonard Shelby";
 
 // Record what the original value of the mail.provider.enabled pref is so
 // that we can put it back once the tests are done.
@@ -60,12 +52,9 @@ var gOldAcceptLangs = Services.prefs.getCharPref(kAcceptedLanguage);
 var gNumAccounts;
 
 function setupModule(module) {
-  collector.getModule('folder-display-helpers').installInto(module);
-  collector.getModule('content-tab-helpers').installInto(module);
-  collector.getModule('window-helpers').installInto(module);
-  collector.getModule('newmailaccount-helpers').installInto(module);
-  collector.getModule('keyboard-helpers').installInto(module);
-  collector.getModule('dom-helpers').installInto(module);
+  for (let lib of MODULE_REQUIRES) {
+    collector.getModule(lib).installInto(module);
+  }
 
   // Make sure we enable the Account Provisioner.
   Services.prefs.setBoolPref(kProvisionerEnabledPref, true);
@@ -88,7 +77,7 @@ function teardownModule(module) {
  * current profile.
  */
 function nAccounts() {
-  return [x for (x in fixIterator(MailServices.accounts.accounts))].length;
+  return MailServices.accounts.accounts.length;
 }
 
 /**
@@ -184,7 +173,7 @@ function subtest_get_an_account(w) {
   // Click on the first address. This reveals the button with the price.
   let address = w.window.document.querySelector(".address:first-child");
   w.click(new elib.Elem(address));
-  w.waitFor(() => w.window.document.querySelectorAll("button.create:not([hidden=true])").length > 0);
+  w.waitFor(() => w.window.document.querySelectorAll('button.create:not([disabled="true"])').length > 0);
 
   // Pick the email address green@example.com
   plan_for_content_tab_load();
@@ -904,12 +893,14 @@ function sub_get_to_order_form(aController, aAddress) {
   aController.click(aController.eid("searchSubmit"));
   wait_for_search_results(aController);
 
-  // Click on the first address. This reveals the button with the price.
-  let address = aController.window.document.querySelector(".address:first-child");
+  // Click on the requested address. This reveals the button with the price.
+  let addressElts = [...aController.window.document.querySelectorAll(".address")];
+  let address = addressElts.filter(a => a.textContent == aAddress).shift();
+  assert_true(!!address, "Couldn't find the requested address " + aAddress);
   aController.click(new elib.Elem(address));
-  aController.waitFor(() => aController.window.document.querySelectorAll("button.create:not([hidden=true])").length > 0);
+  aController.waitFor(() => aController.window.document.querySelectorAll('button.create:not([disabled="true"])').length > 0);
 
-  // Pick the email address green@example.com
+  // Pick the email address.
   plan_for_content_tab_load();
 
   // Clicking this button should close the modal dialog.
@@ -1093,7 +1084,7 @@ function subtest_disabled_fields_when_searching(aController) {
   wait_for_element_enabled(aController, aController.e("name"), false);
   let providerCheckboxes = doc.querySelectorAll(".providerCheckbox");
 
-  for (let [, checkbox] in Iterator(providerCheckboxes))
+  for (let checkbox of providerCheckboxes)
     wait_for_element_enabled(aController, checkbox, false);
 
   // Check to ensure that the buttons for switching to the wizard and closing
@@ -1107,7 +1098,7 @@ function subtest_disabled_fields_when_searching(aController) {
   wait_for_element_enabled(aController, aController.e("searchSubmit"), true);
   wait_for_element_enabled(aController, aController.e("name"), true);
 
-  for (let [, checkbox] in Iterator(providerCheckboxes))
+  for (let checkbox of providerCheckboxes)
     wait_for_element_enabled(aController, checkbox, true);
 
   // Ok, cleanup time. Put the old suggest URL back.
@@ -1212,8 +1203,8 @@ function test_search_button_disabled_if_no_query_on_init() {
   let url = "chrome://content/messenger/accountProvisionerStorage/accountProvisioner";
   let dsm = Services.domStorageManager;
 
-  let uri = Services.io.newURI(url, "", null);
-  let principal = Services.scriptSecurityManager.getNoAppCodebasePrincipal(uri);
+  let uri = Services.io.newURI(url);
+  let principal = Services.scriptSecurityManager.createCodebasePrincipal(uri, {});
   let storage = dsm.getLocalStorageForPrincipal(principal, url);
 
   // Ok, got it. Now let's blank out the name.
@@ -1306,10 +1297,10 @@ function subtest_per_address_prices(w) {
 
   // Click on the multi provider. This reveals the buttons with the prices.
   mc.click(new elib.Elem(multi));
-  mc.waitFor(() => w.window.document.querySelectorAll("button.create:not([hidden=true])").length > 0);
+  mc.waitFor(() => w.window.document.querySelectorAll('button.create:not([disabled="true"])').length > 0);
 
   // For each button, make sure it has the correct price.
-  let buttons = w.window.document.querySelectorAll("button.create:not([hidden=true])");
+  let buttons = w.window.document.querySelectorAll('button.create:not([disabled="true"])');
   let index = 0;
   for (let button of buttons) {
     // Emulate jquery's :visible selector
