@@ -104,7 +104,7 @@ class ThunderTestProfile(mozprofile.ThunderbirdProfile):
         # Do not allow check new mail to be set
         'mail.startup.enabledMailCheckOnce' :  True,
         # Disable compatibility checking
-        'extensions.checkCompatibility.43.0': False,
+        'extensions.checkCompatibility.56': False,
         # Stop any pings to AMO on add-on install
         'extensions.getAddons.cache.enabled': False,
         # In case a developer is working on a laptop without a network
@@ -127,6 +127,12 @@ class ThunderTestProfile(mozprofile.ThunderbirdProfile):
         'toolkit.telemetry.prompted': 999,
         }
 
+    menubar_preferences = {
+        # Many tests operate items in the main menu, so keep it shown
+        # until they are migrated to appmenu.
+        'mail.main_menu.collapse_by_default': False,
+    }
+
     # Dummied up local accounts to stop the account wizard
     account_preferences = {
         'mail.account.account1.server' :  "server1",
@@ -182,6 +188,12 @@ class ThunderTestProfile(mozprofile.ThunderbirdProfile):
         super(ThunderTestProfile, self).__init__(*args, **kwargs)
         self.set_preferences(self.preferences)
 
+        if (wrapper is not None and hasattr(wrapper, "DEFAULT_MENUBAR")
+            and wrapper.DEFAULT_MENUBAR):
+            pass
+        else:
+            self.set_preferences(self.menubar_preferences)
+
         if (wrapper is not None and hasattr(wrapper, "NO_ACCOUNTS")
             and wrapper.NO_ACCOUNTS):
             pass
@@ -212,7 +224,7 @@ class ThunderTestProfile(mozprofile.ThunderbirdProfile):
 
         if wrapper is not None and hasattr(wrapper, "on_profile_created"):
             # It's a little dangerous to allow on_profile_created access to the
-            # profile object, because it isn't fully initalized yet
+            # profile object, because it isn't fully initialized yet
             wrapper.on_profile_created(PROFILE_DIR)
 
         return PROFILE_DIR
@@ -307,7 +319,7 @@ class ThunderTestMozmill(mozmill.MozMill):
 
 def monkeypatched_15_run_tests(self, tests, sleeptime=0):
     frame = mozmill.jsbridge.JSObject(self.bridge,
-                "Components.utils.import('resource://mozmill/modules/frame.js')")
+                "ChromeUtils.import('resource://mozmill/modules/frame.js')")
     sleep(sleeptime)
 
     # transfer persisted data
@@ -510,8 +522,9 @@ def dumpRichResults():
         print '##### MOZMILL-RICH-FAILURES-END #####'
 
 def checkCrashesAtExit():
-    if mozcrash.check_for_crashes(os.path.join(PROFILE_DIR, 'minidumps'), SYMBOLS_PATH,
-                                  TEST_NAME):
+    if mozcrash.check_for_crashes(dump_directory=os.path.join(PROFILE_DIR, 'minidumps'),
+                                  symbols_path=SYMBOLS_PATH,
+                                  test_name=TEST_NAME):
         print >> sys.stderr, 'TinderboxPrint: ' + TEST_NAME + '<br/><em class="testfail">CRASH</em>'
         sys.exit(1)
 
