@@ -2,59 +2,59 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const MODULE_NAME = "window-helpers";
+var MODULE_NAME = "window-helpers";
 
-Cu.import('resource:///modules/iteratorUtils.jsm');
-Cu.import('resource://gre/modules/NetUtil.jsm');
-Cu.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import('resource:///modules/iteratorUtils.jsm');
+ChromeUtils.import('resource://gre/modules/NetUtil.jsm');
+ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 var mozmill = {};
-Cu.import('resource://mozmill/modules/mozmill.js', mozmill);
+ChromeUtils.import('resource://mozmill/modules/mozmill.js', mozmill);
 var controller = {};
-Cu.import('resource://mozmill/modules/controller.js', controller);
+ChromeUtils.import('resource://mozmill/modules/controller.js', controller);
 var elib = {};
-Cu.import('resource://mozmill/modules/elementslib.js', elib);
+ChromeUtils.import('resource://mozmill/modules/elementslib.js', elib);
 var frame = {};
-Cu.import('resource://mozmill/modules/frame.js', frame);
+ChromeUtils.import('resource://mozmill/modules/frame.js', frame);
 var utils = {};
-Cu.import('resource://mozmill/modules/utils.js', utils);
+ChromeUtils.import('resource://mozmill/modules/utils.js', utils);
 
 /**
  * Timeout to use when waiting for the first window ever to load.  This is
  *  long because we are basically waiting for the entire app startup process.
  */
-const FIRST_WINDOW_EVER_TIMEOUT_MS = 30000;
+var FIRST_WINDOW_EVER_TIMEOUT_MS = 30000;
 /**
  * Interval to check if the window has shown up for the first window ever to
  *  load.  The check interval is longer because it's less likely the window
  *  is going to show up quickly and there is a cost to the check.
  */
-const FIRST_WINDOW_CHECK_INTERVAL_MS = 300;
+var FIRST_WINDOW_CHECK_INTERVAL_MS = 300;
 
 /**
  * Timeout for opening a window.
  */
-const WINDOW_OPEN_TIMEOUT_MS = 10000;
+var WINDOW_OPEN_TIMEOUT_MS = 10000;
 /**
  * Check interval for opening a window.
  */
-const WINDOW_OPEN_CHECK_INTERVAL_MS = 100;
+var WINDOW_OPEN_CHECK_INTERVAL_MS = 100;
 
 /**
  * Timeout for closing a window.
  */
-const WINDOW_CLOSE_TIMEOUT_MS = 10000;
+var WINDOW_CLOSE_TIMEOUT_MS = 10000;
 /**
  * Check interval for closing a window.
  */
-const WINDOW_CLOSE_CHECK_INTERVAL_MS = 100;
+var WINDOW_CLOSE_CHECK_INTERVAL_MS = 100;
 
 /**
  * Timeout for focusing a window.  Only really an issue on linux.
  */
-const WINDOW_FOCUS_TIMEOUT_MS = 10000;
+var WINDOW_FOCUS_TIMEOUT_MS = 10000;
 
-const hiddenWindow = Services.appShell.hiddenDOMWindow;
+var hiddenWindow = Services.appShell.hiddenDOMWindow;
 
 // Have a dummy mark_action function in case test-folder-display-helpers does
 // not provide us with one.
@@ -92,6 +92,16 @@ function installInto(module) {
   module.augment_controller = augment_controller;
 }
 
+function getWindowTypeOrId(aWindowElem) {
+  let windowType = aWindowElem.getAttribute("windowtype");
+  // Ignore types that start with "prompt:". This prefix gets added in
+  // toolkit/components/prompts/src/CommonDialog.jsm since bug 1388238.
+  if (windowType && !windowType.startsWith("prompt:"))
+    return windowType;
+
+  return aWindowElem.getAttribute("id");
+}
+
 /**
  * Return the "windowtype" or "id" for the given xul window if it is available.
  * If not, return null.
@@ -103,7 +113,7 @@ function getWindowTypeForXulWindow(aXULWindow, aBusyOk) {
   if (aXULWindow.document &&
       aXULWindow.document.documentElement &&
       aXULWindow.document.documentElement.hasAttribute("windowtype"))
-    return aXULWindow.document.documentElement.getAttribute("windowtype");
+    return getWindowTypeOrId(aXULWindow.document.documentElement);
 
   let docshell = aXULWindow.docShell;
   // we need the docshell to exist...
@@ -126,8 +136,7 @@ function getWindowTypeForXulWindow(aXULWindow, aBusyOk) {
     return null;
 
   // finally, we can now have a windowtype!
-  let windowType = outerDoc.documentElement.getAttribute("windowtype") ||
-                   outerDoc.documentElement.getAttribute("id");
+  let windowType = getWindowTypeOrId(outerDoc.documentElement);
 
   if (windowType)
     return windowType;
@@ -223,7 +232,7 @@ var WindowWatcher = {
     //  window type yet.
     // because this iterates from old to new, this does the right thing in that
     //  side-effects of consider will pick the most recent window.
-    for (let xulWindow in fixIterator(
+    for (let xulWindow of fixIterator(
                                  mozmill.wm.getXULWindowEnumerator(null),
                                  Ci.nsIXULWindow)) {
       if (!this.consider(xulWindow))
@@ -244,13 +253,12 @@ var WindowWatcher = {
    */
   waitForWindowOpen: function WindowWatcher_waitForWindowOpen(aWindowType) {
     this.waitingForOpen = aWindowType;
-    utils.waitFor(function () this.monitorizeOpen(),
+    utils.waitFor(() => this.monitorizeOpen(),
                   "Timed out waiting for window open!",
                   this._firstWindowOpened ? WINDOW_OPEN_TIMEOUT_MS
                     : FIRST_WINDOW_EVER_TIMEOUT_MS,
                   this._firstWindowOpened ? WINDOW_OPEN_CHECK_INTERVAL_MS
-                    : FIRST_WINDOW_CHECK_INTERVAL_MS,
-                  this);
+                    : FIRST_WINDOW_CHECK_INTERVAL_MS);
 
     this.waitingForOpen = null;
     let xulWindow = this.waitingList.get(aWindowType);
@@ -350,10 +358,9 @@ var WindowWatcher = {
         startTest();
       } else {
         function onFocus(event) {
-          targetFocusedWindow.removeEventListener("focus", onFocus, true);
           targetFocusedWindow.setTimeout(startTest, 0);
         }
-        targetFocusedWindow.addEventListener("focus", onFocus, true);
+        targetFocusedWindow.addEventListener("focus", onFocus, {capture: true, once: true});
         targetFocusedWindow.focus();
       }
     }
@@ -384,9 +391,7 @@ var WindowWatcher = {
   },
 
   planForWindowClose: function WindowWatcher_planForWindowClose(aXULWindow) {
-    let windowType =
-      aXULWindow.document.documentElement.getAttribute("windowtype") ||
-      aXULWindow.document.documentElement.getAttribute("id");
+    let windowType = getWindowTypeOrId(aXULWindow.document.documentElement);
     this.waitingList.set(windowType, aXULWindow);
     this.waitingForClose = windowType;
   },
@@ -397,9 +402,10 @@ var WindowWatcher = {
    */
   waitingForClose: null,
   waitForWindowClose: function WindowWatcher_waitForWindowClose() {
-    utils.waitFor(function () this.monitorizeClose(),
+    utils.waitFor(() => this.monitorizeClose(),
                   "Timeout waiting for window to close!",
-      WINDOW_CLOSE_TIMEOUT_MS, WINDOW_CLOSE_CHECK_INTERVAL_MS, this);
+                  WINDOW_CLOSE_TIMEOUT_MS,
+                  WINDOW_CLOSE_CHECK_INTERVAL_MS);
     let didDisappear = (this.waitingList.get(this.waitingForClose) == null);
     let windowType = this.waitingForClose;
     this.waitingList.delete(windowType);
@@ -480,6 +486,7 @@ var WindowWatcher = {
                 [getWindowTypeForXulWindow(aXULWindow, true) +
                    " (" + getUniqueIdForXulWindow(aXULWindow) + ")",
                    "active?", Services.focus.focusedWindow == aXULWindow]);
+
     if (!this.consider(aXULWindow))
       this.monitorWindowLoad(aXULWindow);
   },
@@ -512,9 +519,7 @@ var WindowWatcher = {
   onCloseWindow: function WindowWatcher_onCloseWindow(aXULWindow) {
     let domWindow = aXULWindow.docShell.QueryInterface(Ci.nsIInterfaceRequestor)
                                        .getInterface(Ci.nsIDOMWindow);
-    let windowType =
-      domWindow.document.documentElement.getAttribute("windowtype") ||
-      domWindow.document.documentElement.getAttribute("id");
+    let windowType = getWindowTypeOrId(domWindow.document.documentElement);
     mark_action("winhelp", "onCloseWindow",
                 [getWindowTypeForXulWindow(aXULWindow, true) +
                    " (" + getUniqueIdForXulWindow(aXULWindow) + ")"]);
@@ -674,11 +679,7 @@ function wait_for_window_focused(aWindow) {
   if (focusedWindow == targetWindow) {
     focused = true;
   } else {
-    function onFocus(event) {
-      targetWindow.removeEventListener("focus", onFocus, true);
-      focused = true;
-    }
-    targetWindow.addEventListener("focus", onFocus, true);
+    targetWindow.addEventListener("focus", () => focused = true, {capture: true, once: true});
     targetWindow.focus();
   }
 
@@ -749,9 +750,13 @@ function _wait_for_generic_load(aDetails, aURLOrPredicate) {
 
   try {
     utils.waitFor(isLoadedChecker);
-  } catch (e if e instanceof utils.TimeoutError) {
-    mark_failure(["Timeout waiting for content page to load. Current URL is:",
-                  aDetails.currentURI.spec]);
+  } catch (e) {
+    if (e instanceof utils.TimeoutError) {
+      mark_failure(["Timeout waiting for content page to load. Current URL is:",
+                    aDetails.currentURI.spec]);
+    } else {
+      throw e;
+    }
   }
 
   // Lie to mozmill to convince it to not explode because these frames never
@@ -766,8 +771,8 @@ function _wait_for_generic_load(aDetails, aURLOrPredicate) {
 }
 
 
-let observationWaitFuncs = {};
-let observationSaw = {};
+var observationWaitFuncs = {};
+var observationSaw = {};
 /**
  * Plan for a notification to be sent via the observer service.
  *
@@ -782,7 +787,7 @@ function plan_for_observable_event(aTopic) {
       observationSaw[aTopic] = true;
     }
   };
-  Services.obs.addObserver(waiter, aTopic, false);
+  Services.obs.addObserver(waiter, aTopic);
 }
 
 /**
@@ -833,11 +838,9 @@ var AugmentEverybodyWith = {
       let elem = this.window.document.getElementById(aId);
       if (aQuery) {
         if (aQuery.tagName) {
-          let elems = Array.prototype.slice.call(
-                        elem.getElementsByTagName(aQuery.tagName));
+          let elems = Array.from(elem.getElementsByTagName(aQuery.tagName));
           if (aQuery.label)
-            elems = [elem for each (elem in elems)
-                          if (elem.label == aQuery.label)];
+            elems = elems.filter(elem => elem.label == aQuery.label);
           elem = elems[0];
         }
       }
@@ -908,7 +911,7 @@ var AugmentEverybodyWith = {
       }
       else {
         let msg = "Query constraint not implemented, query contained:";
-        for (let [key, val] in Iterator(aQuery)) {
+        for (let [key, val] of Object.entries(aQuery)) {
           msg += " '" + key + "': " + val;
         }
         throw new Error(msg);
@@ -945,9 +948,10 @@ var AugmentEverybodyWith = {
      * Dynamically-built/XBL-defined menus can be hard to work with, this makes it
      *  easier.
      *
-     * @param aRootPopup The base popup.  We will open it if it is not open or
-     *     wait for it to open if it is in the process.
-     * @param aActions A list of objects where each object has a single
+     * @param aRootPopup  The base popup. The caller is expected to activate it
+     *     (by clicking/rightclicking the right widget). We will only wait for it
+     *     to open if it is in the process.
+     * @param aActions  A list of objects where each object has a single
      *     attribute with a single value.  We pick the menu option whose DOM
      *     node has an attribute with that name and value.  We click whatever we
      *     find.  We throw if we don't find what you were asking for.
@@ -957,8 +961,6 @@ var AugmentEverybodyWith = {
      *          an empty array if aKeepOpen was set to false.
      */
     click_menus_in_sequence: function _click_menus(aRootPopup, aActions, aKeepOpen) {
-      if (aRootPopup.state == "closed")
-        aRootPopup.openPopup(null, "", 0, 0, true, true);
       if (aRootPopup.state != "open") { // handle "showing"
         utils.waitFor(() => aRootPopup.state == "open",
                       "Popup never opened! id=" + aRootPopup.id +
@@ -969,7 +971,7 @@ var AugmentEverybodyWith = {
       let closeStack = [aRootPopup];
 
       let curPopup = aRootPopup;
-      for each (let [iAction, actionObj] in Iterator(aActions)) {
+      for (let [iAction, actionObj] of aActions.entries()) {
         /**
          * Check if aNode attributes match all those given in actionObj.
          * Nodes that are obvious containers are skipped, and their children
@@ -988,7 +990,8 @@ var AugmentEverybodyWith = {
           }
 
           let matchedAll = true;
-          for each (let [name, value] in Iterator(actionObj)) {
+          for (let name in actionObj) {
+            let value = actionObj[name];
             if (!aNode.hasAttribute(name) ||
                 aNode.getAttribute(name) != value) {
               matchedAll = false;
@@ -1040,13 +1043,27 @@ var AugmentEverybodyWith = {
     close_popup_sequence: function _close_popup_sequence(aCloseStack) {
       while (aCloseStack.length) {
         let curPopup = aCloseStack.pop();
-        this.keypress(new elib.Elem(curPopup), "VK_ESCAPE", {});
+        if (curPopup.state == "open")
+          this.keypress(new elib.Elem(curPopup), "VK_ESCAPE", {});
         utils.waitFor(function() { return curPopup.state == "closed"; },
                       "Popup did not close! id=" + curPopup.id +
                       ", state=" +  curPopup.state, 5000, 50);
       }
     },
 
+    /**
+     * Get dropmarker arrow element from
+     *
+     * @param aNode  An element containing a dropmarker, e.g. menulist or menu-button
+     */
+    get_menu_dropmarker: function(aNode) {
+      let children = aNode.ownerDocument.getAnonymousNodes(aNode);
+      for (let node of children) {
+        if (node.tagName == "xul:dropmarker")
+          return node;
+      }
+    },
+
     /**
      * mark_action helper method that produces something that can be concat()ed
      *  onto a list being passed to mark_action in order to describe the focus
@@ -1093,12 +1110,12 @@ var AugmentEverybodyWith = {
  * border but are no longer so (bug 595652), so we need these wrappers to
  * perform the operations at the center when aLeft or aTop aren't passed in.
  */
-const MOUSE_OPS_TO_WRAP = [
+var MOUSE_OPS_TO_WRAP = [
   "click", "doubleClick", "mouseDown", "mouseOut", "mouseOver", "mouseUp",
   "middleClick", "rightClick",
 ];
 
-for (let [, mouseOp] in Iterator(MOUSE_OPS_TO_WRAP)) {
+for (let mouseOp of MOUSE_OPS_TO_WRAP) {
   let thisMouseOp = mouseOp;
   let wrapperFunc = function (aElem, aLeft, aTop) {
     let el = aElem.getNode();
@@ -1310,25 +1327,27 @@ var PerWindowTypeAugmentations = {
 
 function _augment_helper(aController, aAugmentDef) {
   if (aAugmentDef.elementsToExpose) {
-    for each (let [key, value] in Iterator(aAugmentDef.elementsToExpose)) {
+    for (let key in aAugmentDef.elementsToExpose) {
+      let value = aAugmentDef.elementsToExpose[key];
       aController[key] = aController.window.document.getElementById(value);
     }
   }
   if (aAugmentDef.elementsIDsToExpose) {
-    for each (let [key, value] in Iterator(aAugmentDef.elementIDsToExpose)) {
+    for (let key in aAugmentDef.elementIDsToExpose) {
+      let value = aAugmentDef.elementIDsToExpose[key];
       aController[key] = new elib.ID(
                            aController.window.document, value);
     }
   }
   if (aAugmentDef.globalsToExposeAtStartup) {
-    for each (let [key, value] in
-              Iterator(aAugmentDef.globalsToExposeAtStartup)) {
+    for (let key in aAugmentDef.globalsToExposeAtStartup) {
+      let value = aAugmentDef.globalsToExposeAtStartup[key];
       aController[key] = aController.window[value];
     }
   }
   if (aAugmentDef.globalsToExposeViaGetters) {
-    for each (let [key, value] in
-              Iterator(aAugmentDef.globalsToExposeViaGetters)) {
+    for (let key in aAugmentDef.globalsToExposeViaGetters) {
+      let value = aAugmentDef.globalsToExposeViaGetters[key];
       let globalName = value;
       aController.__defineGetter__(key, function() {
           return this.window[globalName];
@@ -1336,19 +1355,21 @@ function _augment_helper(aController, aAugmentDef) {
     }
   }
   if (aAugmentDef.getters) {
-    for each (let [key, value] in Iterator(aAugmentDef.getters)) {
+    for (let key in aAugmentDef.getters) {
+      let value = aAugmentDef.getters[key];
       aController.__defineGetter__(key, value);
     }
   }
   if (aAugmentDef.methods) {
-    for each (let [key, value] in Iterator(aAugmentDef.methods)) {
+    for (let key in aAugmentDef.methods) {
+      let value = aAugmentDef.methods[key];
       aController[key] = value;
     }
   }
 
   if (aAugmentDef.debugTrace) {
     let win = aController.window;
-    for each (let [, traceDef] in Iterator(aAugmentDef.debugTrace)) {
+    for (let traceDef of aAugmentDef.debugTrace) {
       let baseObj, useThis;
       // - Get the object that actually has the method to wrap
       if (traceDef.hasOwnProperty("onGlobal")) {
@@ -1388,7 +1409,7 @@ function _augment_helper(aController, aAugmentDef) {
       else {
         traceFunc = function() {
           mark_action("winhelp", reportAs,
-                      showArgs ? Array.prototype.slice.call(arguments) : []);
+                      showArgs ? Array.from(arguments) : []);
           try {
             return origFunc.apply(this, arguments);
           }
@@ -1531,8 +1552,7 @@ function getWindowDescribeyFromEvent(event) {
        .QueryInterface(Ci.nsIInterfaceRequestor)
        .getInterface(Ci.nsIDOMWindow);
   var docElem = owningWin.document.documentElement;
-  return (docElem.getAttribute("windowtype") ||
-          docElem.getAttribute("id") || "mysterious") +
+  return (getWindowTypeOrId(docElem) || "mysterious") +
          " (" + (UNIQUE_WINDOW_ID_ATTR in owningWin ?
                    owningWin[UNIQUE_WINDOW_ID_ATTR] :
                    "n/a") + ")";
@@ -1668,8 +1688,7 @@ var gNextOneUpUniqueID = 0;
  */
 function augment_controller(aController, aWindowType) {
   if (aWindowType === undefined)
-    aWindowType =
-      aController.window.document.documentElement.getAttribute("windowtype");
+    aWindowType = getWindowTypeOrId(aController.window.document.documentElement);
 
   _augment_helper(aController, AugmentEverybodyWith);
   if (PerWindowTypeAugmentations[aWindowType])
@@ -1702,12 +1721,12 @@ function augment_controller(aController, aWindowType) {
 
     // - bubbling listeners
     // (so we can see if the event got killed / eaten)
-    doc.addEventListener("mousedown", __bubbled_click_handler, false);
-    doc.addEventListener("click", __bubbled_click_handler, false);
-    doc.addEventListener("contextmenu", __bubbled_click_handler, false);
-    doc.addEventListener("mouseup", __bubbled_click_handler, false);
+    doc.addEventListener("mousedown", __bubbled_click_handler);
+    doc.addEventListener("click", __bubbled_click_handler);
+    doc.addEventListener("contextmenu", __bubbled_click_handler);
+    doc.addEventListener("mouseup", __bubbled_click_handler);
 
-    doc.addEventListener("keypress", __bubbled_keypress_handler, false);
+    doc.addEventListener("keypress", __bubbled_keypress_handler);
 
     // - also, add pop-up shown/hidden events....
     // We need to add these directly to the popups themselves in order to
@@ -1840,15 +1859,13 @@ function captureWindowStatesForErrorReporting(normalizeForJsonFunc) {
   while (enumerator.hasMoreElements()) {
     let win = enumerator.getNext().QueryInterface(Ci.nsIDOMWindow);
 
-    let winId = win.document.documentElement.getAttribute("windowtype") ||
-                win.document.documentElement.getAttribute("id") ||
+    let winId = getWindowTypeOrId(win.document.documentElement) ||
                 ("unnamed:" + iWin);
 
     let openPopups =
-      Array.prototype.slice.call(
-          win.document.documentElement.getElementsByTagName("menupopup"))
-        .filter(x => x.state != "closed")
-        .map(function (x) normalizeForJsonFunc(x));
+      Array.from(win.document.documentElement.getElementsByTagName("menupopup"))
+           .filter(x => x.state != "closed")
+           .map(x => normalizeForJsonFunc(x));
 
     let ignoredFocusedWindow = {};
     let winfo = {
