@@ -39,26 +39,26 @@ var EXPORTED_SYMBOLS = ['loadFile','register_function','Collector','Runner','eve
                         'jsbridge', 'runTestDirectory', 'runTestFile', 'log', 'getThread',
                         'timers', 'persisted'];
 
-Components.utils.import('resource://mozmill/stdlib/httpd.js');
+ChromeUtils.import('resource://mozmill/stdlib/httpd.js');
 
-var os = {};      Components.utils.import('resource://mozmill/stdlib/os.js', os);
-var strings = {}; Components.utils.import('resource://mozmill/stdlib/strings.js', strings);
-var arrays = {};  Components.utils.import('resource://mozmill/stdlib/arrays.js', arrays);
-var withs = {};   Components.utils.import('resource://mozmill/stdlib/withs.js', withs);
-var utils = {};   Components.utils.import('resource://mozmill/modules/utils.js', utils);
+var os = {};      ChromeUtils.import('resource://mozmill/stdlib/os.js', os);
+var strings = {}; ChromeUtils.import('resource://mozmill/stdlib/strings.js', strings);
+var arrays = {};  ChromeUtils.import('resource://mozmill/stdlib/arrays.js', arrays);
+var withs = {};   ChromeUtils.import('resource://mozmill/stdlib/withs.js', withs);
+var utils = {};   ChromeUtils.import('resource://mozmill/modules/utils.js', utils);
 var securableModule = {};
-  Components.utils.import('resource://mozmill/stdlib/securable-module.js', securableModule);
+  ChromeUtils.import('resource://mozmill/stdlib/securable-module.js', securableModule);
 
-var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].
-     getService(Components.interfaces.nsIConsoleService);
-var ios = Components.classes["@mozilla.org/network/io-service;1"]
-                    .getService(Components.interfaces.nsIIOService);
-var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
-                    .getService(Components.interfaces.mozIJSSubScriptLoader);
-var uuidgen = Components.classes["@mozilla.org/uuid-generator;1"]
-                    .getService(Components.interfaces.nsIUUIDGenerator);
+var aConsoleService = Cc["@mozilla.org/consoleservice;1"]
+                        .getService(Ci.nsIConsoleService);
+var ios = Cc["@mozilla.org/network/io-service;1"]
+            .getService(Ci.nsIIOService);
+var loader = Cc["@mozilla.org/moz/jssubscript-loader;1"]
+               .getService(Ci.mozIJSSubScriptLoader);
+var uuidgen = Cc["@mozilla.org/uuid-generator;1"]
+                .getService(Ci.nsIUUIDGenerator);
 
-Components.utils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
 var systemPrincipal = Services.scriptSecurityManager.getSystemPrincipal();
 
 var backstage = this;
@@ -78,40 +78,35 @@ mozmill = undefined; elementslib = undefined; modules = undefined;
 var loadTestResources = function () {
   if (mozmill == undefined) {
     mozmill = {};
-    Components.utils.import("resource://mozmill/modules/mozmill.js", mozmill);
+    ChromeUtils.import("resource://mozmill/modules/mozmill.js", mozmill);
   }
   if (elementslib == undefined) {
     elementslib = {};
-    Components.utils.import("resource://mozmill/modules/elementslib.js", elementslib);
+    ChromeUtils.import("resource://mozmill/modules/elementslib.js", elementslib);
   }
 }
 
 var loadFile = function(path, collector) {
-  var file = Components.classes["@mozilla.org/file/local;1"]
-                       .createInstance(Components.interfaces.nsILocalFile);
+  var file = Cc["@mozilla.org/file/local;1"]
+               .createInstance(Ci.nsIFile);
   file.initWithPath(path);
   var uri = ios.newFileURI(file).spec;
 
-  var module = new Components.utils.Sandbox(systemPrincipal);
+  var module = new Cu.Sandbox(systemPrincipal, { wantGlobalProperties: ["ChromeUtils"] });
   module.registeredFunctions = registeredFunctions;
   module.collector = collector
   loadTestResources();
   module.mozmill = mozmill;
   module.elementslib = elementslib;
   module.persisted = persisted;
-  module.Cc = Components.classes;
-  module.Ci = Components.interfaces;
-  module.Cu = Components.utils;
+  module.Cc = Cc;
+  module.Ci = Ci;
+  module.Cu = Cu;
   module.require = function (mod) {
     var loader = new securableModule.Loader({
       rootPaths: [ios.newFileURI(file.parent).spec],
       defaultPrincipal: "system",
-      globals : { mozmill: mozmill,
-                  elementslib: elementslib,
-                  persisted: persisted,
-                  Cc: Components.classes,
-                  Ci: Components.interfaces,
-                  Cu: Components.utils }
+      globals : { mozmill, elementslib, persistsed, Cc, Ci, Cu }
     });
     if (modules != undefined) {
         loader.modules = modules;
@@ -153,9 +148,9 @@ function registerFunction (name, func) {
   registeredFunctions[name] = func;
 }
 
-function stateChangeBase (possibilties, restrictions, target, cmeta, v) {
-  if (possibilties) {
-    if (!arrays.inArray(possibilties, v)) {
+function stateChangeBase (possibilities, restrictions, target, cmeta, v) {
+  if (possibilities) {
+    if (!arrays.inArray(possibilities, v)) {
       // TODO Error value not in this.poss
       return;
     }
@@ -235,7 +230,7 @@ events.pass = function (obj) {
   if (events.currentTest) {
     events.currentTest.__passes__.push(obj);
   }
-  for each(var timer in timers) {
+  for (var timer of timers) {
     timer.actions.push(
       {"currentTest":events.currentModule.__file__+"::"+events.currentTest.__name__, "obj":obj,
        "result":"pass"}
@@ -259,7 +254,7 @@ events.fail = function (obj) {
   if (events.currentTest) {
     events.currentTest.__fails__.push(obj);
   }
-  for each(var time in timers) {
+  for (var time of timers) {
     timer.actions.push(
       {"currentTest":events.currentModule.__file__+"::"+events.currentTest.__name__, "obj":obj,
        "result":"fail"}
@@ -270,7 +265,7 @@ events.fail = function (obj) {
 events.skip = function (reason) {
   events.currentTest.skipped = true;
   events.currentTest.skipped_reason = reason;
-  for each(var timer in timers) {
+  for (var timer of timers) {
     timer.actions.push(
       {"currentTest":events.currentModule.__file__+"::"+events.currentTest.__name__, "obj":reason,
        "result":"skip"}
@@ -287,7 +282,7 @@ events.fireEvent = function (name, obj) {
       this.listeners[name][i](obj);
     }
   }
-  for each(var listener in this.globalListeners) {
+  for (var listener of this.globalListeners) {
     listener(name, obj);
   }
 }
@@ -322,7 +317,7 @@ var log = function (obj) {
 }
 
 try {
-  var jsbridge = {}; Components.utils.import('resource://jsbridge/modules/events.js', jsbridge);
+  var jsbridge = {}; ChromeUtils.import('resource://jsbridge/modules/events.js', jsbridge);
 } catch(err) {
   var jsbridge = null;
 
@@ -342,7 +337,7 @@ function Collector () {
   this.testing = [];
   this.httpd_started = false;
   this.http_port = 43336;
-  // var logging = {}; Components.utils.import('resource://mozmill/stdlib/logging.js', logging);
+  // var logging = {}; ChromeUtils.import('resource://mozmill/stdlib/logging.js', logging);
   // this.logger = new logging.Logger('Collector');
 }
 
@@ -353,7 +348,7 @@ Collector.prototype.getModule = function (name) {
 Collector.prototype.getServer = function (port, basePath) {
   if (basePath) {
     var lp = Cc["@mozilla.org/file/local;1"]
-             .createInstance(Ci.nsILocalFile);
+             .createInstance(Ci.nsIFile);
     lp.initWithPath(basePath);
   }
 
@@ -400,8 +395,8 @@ Collector.prototype.addHttpResource = function (directory, ns) {
     ns = '/' + ns + '/';
   }
 
-  var lp = Components.classes["@mozilla.org/file/local;1"].
-           createInstance(Components.interfaces.nsILocalFile);
+  var lp = Cc["@mozilla.org/file/local;1"]
+             .createInstance(Ci.nsIFile);
   lp.initWithPath(os.abspath(directory, this.current_file));
   this.httpd.registerDirectory(ns, lp);
 
@@ -451,7 +446,7 @@ Collector.prototype.initTestModule = function (filename) {
   }
 
   if (test_module.MODULE_REQUIRES != undefined && test_module.RELATIVE_ROOT == undefined) {
-    for each(var t in test_module.__tests__) {
+    for (var t of test_module.__tests__) {
       t.__force_skip__ = "RELATIVE ROOT is not defined and test requires another module.";
     }
   }
@@ -495,13 +490,13 @@ AppQuitObserver.prototype = {
     events.appQuit = true;
   },
   register: function() {
-    var obsService = Components.classes["@mozilla.org/observer-service;1"]
-                     .getService(Components.interfaces.nsIObserverService);
-    obsService.addObserver(this, "quit-application", false);
+    var obsService = Cc["@mozilla.org/observer-service;1"]
+                     .getService(Ci.nsIObserverService);
+    obsService.addObserver(this, "quit-application");
   },
   unregister: function() {
-    var obsService = Components.classes["@mozilla.org/observer-service;1"]
-                     .getService(Components.interfaces.nsIObserverService);
+    var obsService = Cc["@mozilla.org/observer-service;1"]
+                     .getService(Ci.nsIObserverService);
     obsService.removeObserver(this, "quit-application");
   }
 }
@@ -513,7 +508,7 @@ function Runner (collector, invokedFromIDE) {
   events.fireEvent('startRunner', true);
   // var logging = {}; Components.utils.import('resource://mozmill/stdlib/logging.js', logging);
   // this.logger = new logging.Logger('Runner');
-  var m = {}; Components.utils.import('resource://mozmill/modules/mozmill.js', m);
+  var m = {}; ChromeUtils.import('resource://mozmill/modules/mozmill.js', m);
   this.platform = m.platform;
 }
 Runner.prototype.runTestDirectory = function (directory) {
@@ -561,9 +556,9 @@ Runner.prototype.getDependencies = function (module) {
 }
 
 Runner.prototype.wrapper = function (func, arg) {
-  thread = Components.classes["@mozilla.org/thread-manager;1"]
-                     .getService(Components.interfaces.nsIThreadManager)
-                     .currentThread;
+  thread = Cc["@mozilla.org/thread-manager;1"]
+             .getService(Ci.nsIThreadManager)
+             .currentThread;
 
   if (func.EXCLUDED_PLATFORMS != undefined) {
     if (arrays.inArray(func.EXCLUDED_PLATFORMS, this.platform)) {
@@ -602,14 +597,14 @@ Runner.prototype.wrapper = function (func, arg) {
     // Allow the exception if a user shutdown was expected
     if (!events.isUserShutdown()) {
       events.fail({'exception': e, 'test':func})
-      Components.utils.reportError(e);
+      Cu.reportError(e);
     }
   }
 }
 
 Runner.prototype._runTestModule = function (module) {
   if (module.__requirements__ != undefined && module.__force_skip__ == undefined) {
-    for each(var req in module.__requirements__) {
+    for (var req of module.__requirements__) {
       module[req] = this.collector.getModule(req);
     }
   }
@@ -665,7 +660,7 @@ Runner.prototype._runTestModule = function (module) {
       events.endTest(test)
     }
   } else {
-    for each(var test in module.__tests__) {
+    for (var test of module.__tests__) {
       events.setTest(test);
       events.skip("setupModule failed.");
       events.endTest(test);
@@ -721,10 +716,10 @@ var getThread = function () {
 
 function registerModule(name, path) {
   let protocolHandler = Services.io.getProtocolHandler("resource")
-                                .QueryInterface(Components.interfaces.nsIResProtocolHandler);
+                                .QueryInterface(Ci.nsIResProtocolHandler);
 
-  let modulesFile = Components.classes["@mozilla.org/file/local;1"]
-                              .createInstance(Components.interfaces.nsILocalFile);
+  let modulesFile = Cc["@mozilla.org/file/local;1"]
+                      .createInstance(Ci.nsIFile);
   modulesFile.initWithPath(path);
   protocolHandler.setSubstitution(name, ios.newFileURI(modulesFile));
 }
