@@ -21,6 +21,7 @@
 #include "mozITXTToHTMLConv.h"
 #include "nsIAbManager.h"
 
+#include "nsVariant.h"
 #include "nsIProperty.h"
 #include "nsCOMArray.h"
 #include "nsArrayEnumerator.h"
@@ -36,7 +37,8 @@ const char sAddrbookProperties[] = "chrome://messenger/locale/addressbook/addres
 enum EAppendType {
   eAppendLine,
   eAppendLabel,
-  eAppendCityStateZip
+  eAppendCityStateZip,
+  eAppendUndefined
 };
 
 struct AppendItem {
@@ -92,6 +94,8 @@ static const AppendItem CUSTOM_ATTRS_ARRAY[] = {
   {kNotesProperty, "", eAppendLine}
 };
 
+#ifdef MOZ_THUNDERBIRD
+
 static const AppendItem CHAT_ATTRS_ARRAY[] = {
   {kGtalkProperty, "propertyGtalk", eAppendLabel},
   {kAIMProperty, "propertyAIM", eAppendLabel},
@@ -103,6 +107,7 @@ static const AppendItem CHAT_ATTRS_ARRAY[] = {
   {kXMPPProperty, "propertyXMPP", eAppendLabel},
   {kIRCProperty, "propertyIRC", eAppendLabel}
 };
+#endif
 
 nsAbCardProperty::nsAbCardProperty()
   : m_IsMailList(false)
@@ -231,18 +236,13 @@ nsAbSimpleProperty::GetValue(nsIVariant* *aValue)
     return NS_OK;
 }
 
-static PLDHashOperator
-PropertyHashToArrayFunc (const nsACString &aKey, nsIVariant* aData, void *userArg)
-{
-  nsCOMArray<nsIProperty>* propertyArray = static_cast<nsCOMArray<nsIProperty> *>(userArg);
-  propertyArray->AppendObject(new nsAbSimpleProperty(aKey, aData));
-  return PL_DHASH_NEXT;
-}
-
 NS_IMETHODIMP nsAbCardProperty::GetProperties(nsISimpleEnumerator **props)
 {
   nsCOMArray<nsIProperty> propertyArray(m_properties.Count());
-  m_properties.EnumerateRead(PropertyHashToArrayFunc, &propertyArray);
+  for (auto iter = m_properties.Iter(); !iter.Done(); iter.Next()) {
+    propertyArray.AppendObject(new nsAbSimpleProperty(iter.Key(),
+                                                      iter.UserData()));
+  }
   return NS_NewArrayEnumerator(props, propertyArray);
 }
 
@@ -301,7 +301,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsAString(const char *name, const nsA
 {
   NS_ENSURE_ARG_POINTER(name);
 
-  nsCOMPtr<nsIWritableVariant> variant = do_CreateInstance(NS_VARIANT_CONTRACTID);
+  nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsAString(value);
   m_properties.Put(nsDependentCString(name), variant);
   return NS_OK;
@@ -311,7 +311,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsAUTF8String(const char *name, const
 {
   NS_ENSURE_ARG_POINTER(name);
 
-  nsCOMPtr<nsIWritableVariant> variant = do_CreateInstance(NS_VARIANT_CONTRACTID);
+  nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsAUTF8String(value);
   m_properties.Put(nsDependentCString(name), variant);
   return NS_OK;
@@ -321,7 +321,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsUint32(const char *name, uint32_t v
 {
   NS_ENSURE_ARG_POINTER(name);
 
-  nsCOMPtr<nsIWritableVariant> variant = do_CreateInstance(NS_VARIANT_CONTRACTID);
+  nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsUint32(value);
   m_properties.Put(nsDependentCString(name), variant);
   return NS_OK;
@@ -331,7 +331,7 @@ NS_IMETHODIMP nsAbCardProperty::SetPropertyAsBool(const char *name, bool value)
 {
   NS_ENSURE_ARG_POINTER(name);
 
-  nsCOMPtr<nsIWritableVariant> variant = do_CreateInstance(NS_VARIANT_CONTRACTID);
+  nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
   variant->SetAsBool(value);
   m_properties.Put(nsDependentCString(name), variant);
   return NS_OK;
@@ -528,7 +528,10 @@ nsresult nsAbCardProperty::ConvertToEscapedVCard(nsACString &aResult)
 
     (void)GetDisplayName(str);
     if (!str.IsEmpty()) {
-        myAddPropValue(vObj, VCFullNameProp, str.get(), &vCardHasData);
+      // Full name is a field with the same name.
+      t = isAPropertyOf(vObj, VCFullNameProp);
+      if (!t) t = addProp(vObj, VCFullNameProp);
+      myAddPropValue(t, VCFullNameProp, str.get(), &vCardHasData);
     }
 
     (void)GetLastName(str);
@@ -685,10 +688,10 @@ nsresult nsAbCardProperty::ConvertToEscapedVCard(nsACString &aResult)
     uint32_t format;
     rv = GetPropertyAsUint32(kPreferMailFormatProperty, &format);
     if (NS_SUCCEEDED(rv) && format == nsIAbPreferMailFormat::html) {
-        myAddPropValue(vObj, VCUseHTML, MOZ_UTF16("TRUE"), &vCardHasData);
+        myAddPropValue(vObj, VCUseHTML, u"TRUE", &vCardHasData);
     }
     else if (NS_SUCCEEDED(rv) && format == nsIAbPreferMailFormat::plaintext) {
-        myAddPropValue(vObj, VCUseHTML, MOZ_UTF16("FALSE"), &vCardHasData);
+        myAddPropValue(vObj, VCUseHTML, u"FALSE", &vCardHasData);
     }
 
     rv = GetPropertyAsAString(kWorkWebPageProperty, str);
@@ -697,7 +700,7 @@ nsresult nsAbCardProperty::ConvertToEscapedVCard(nsACString &aResult)
         myAddPropValue(vObj, VCURLProp, str.get(), &vCardHasData);
     }
 
-    myAddPropValue(vObj, VCVersionProp, MOZ_UTF16("2.1"), nullptr);
+    myAddPropValue(vObj, VCVersionProp, u"2.1", nullptr);
 
     if (!vCardHasData) {
         aResult.Truncate();
@@ -733,7 +736,7 @@ nsresult nsAbCardProperty::ConvertToBase64EncodedXML(nsACString &result)
     rv = stringBundleService->CreateBundle(sAddrbookProperties, getter_AddRefs(bundle));
     if (NS_SUCCEEDED(rv)) {
       nsString addrBook;
-      rv = bundle->GetStringFromName(MOZ_UTF16("addressBook"), getter_Copies(addrBook));
+      rv = bundle->GetStringFromName("addressBook", addrBook);
       if (NS_SUCCEEDED(rv)) {
         xmlStr.AppendLiteral("<title xmlns=\"http://www.w3.org/1999/xhtml\">");
         xmlStr.Append(addrBook);
@@ -825,7 +828,7 @@ nsresult nsAbCardProperty::ConvertToXMLPrintData(nsAString &aXMLSubstr)
     xmlStr.AppendLiteral("<section><sectiontitle>");
 
     nsString headingAddresses;
-    rv = bundle->GetStringFromName(MOZ_UTF16("headingAddresses"), getter_Copies(headingAddresses));
+    rv = bundle->GetStringFromName("headingAddresses", headingAddresses);
     NS_ENSURE_SUCCESS(rv, rv);
 
     xmlStr.Append(headingAddresses);
@@ -913,7 +916,7 @@ nsresult nsAbCardProperty::AppendSection(const AppendItem *aArray, int16_t aCoun
 
   if (!sectionIsEmpty && !aHeading.IsEmpty()) {
     nsString heading;
-    rv = aBundle->GetStringFromName(aHeading.get(), getter_Copies(heading));
+    rv = aBundle->GetStringFromName(NS_ConvertUTF16toUTF8(aHeading).get(), heading);
     NS_ENSURE_SUCCESS(rv, rv);
 
     aResult.AppendLiteral("<sectiontitle>");
@@ -991,7 +994,7 @@ nsresult nsAbCardProperty::AppendLabel(const AppendItem &aItem,
   if (NS_FAILED(rv) || attrValue.IsEmpty())
     return NS_OK;
 
-  rv = aBundle->GetStringFromName(NS_ConvertUTF8toUTF16(aItem.mLabel).get(), getter_Copies(label));
+  rv = aBundle->GetStringFromName(aItem.mLabel, label);
   NS_ENSURE_SUCCESS(rv, rv);
 
   aResult.AppendLiteral("<labelrow><label>");
@@ -1034,6 +1037,7 @@ nsresult nsAbCardProperty::AppendCityStateZip(const AppendItem &aItem,
 
   item.mColumn = statePropName;
   item.mLabel = "";
+  item.mAppendType = eAppendUndefined;
 
   rv = AppendLine(item, aConv, stateResult);
   NS_ENSURE_SUCCESS(rv,rv);
@@ -1047,18 +1051,18 @@ nsresult nsAbCardProperty::AppendCityStateZip(const AppendItem &aItem,
 
   if (!cityResult.IsEmpty() && !stateResult.IsEmpty() && !zipResult.IsEmpty()) {
     const char16_t *formatStrings[] = { cityResult.get(), stateResult.get(), zipResult.get() };
-    rv = aBundle->FormatStringFromName(MOZ_UTF16("cityAndStateAndZip"), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
+    rv = aBundle->FormatStringFromName("cityAndStateAndZip", formatStrings, ArrayLength(formatStrings), formattedString);
     NS_ENSURE_SUCCESS(rv,rv);
   }
   else if (!cityResult.IsEmpty() && !stateResult.IsEmpty() && zipResult.IsEmpty()) {
     const char16_t *formatStrings[] = { cityResult.get(), stateResult.get() };
-    rv = aBundle->FormatStringFromName(MOZ_UTF16("cityAndStateNoZip"), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
+    rv = aBundle->FormatStringFromName("cityAndStateNoZip", formatStrings, ArrayLength(formatStrings), formattedString);
     NS_ENSURE_SUCCESS(rv,rv);
   }
   else if ((!cityResult.IsEmpty() && stateResult.IsEmpty() && !zipResult.IsEmpty()) ||
           (cityResult.IsEmpty() && !stateResult.IsEmpty() && !zipResult.IsEmpty())) {
     const char16_t *formatStrings[] = { cityResult.IsEmpty() ? stateResult.get() : cityResult.get(), zipResult.get() };
-    rv = aBundle->FormatStringFromName(MOZ_UTF16("cityOrStateAndZip"), formatStrings, ArrayLength(formatStrings), getter_Copies(formattedString));
+    rv = aBundle->FormatStringFromName("cityOrStateAndZip", formatStrings, ArrayLength(formatStrings), formattedString);
     NS_ENSURE_SUCCESS(rv,rv);
   }
   else {
@@ -1101,7 +1105,7 @@ NS_IMETHODIMP nsAbCardProperty::GenerateName(int32_t aGenerateFormat,
       nsCOMPtr<nsIStringBundleService> stringBundleService =
         mozilla::services::GetStringBundleService();
       NS_ENSURE_TRUE(stringBundleService, NS_ERROR_UNEXPECTED);
-        
+
       rv = stringBundleService->CreateBundle(sAddrbookProperties,
                                              getter_AddRefs(bundle));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -1112,16 +1116,16 @@ NS_IMETHODIMP nsAbCardProperty::GenerateName(int32_t aGenerateFormat,
     if (aGenerateFormat == GENERATE_LAST_FIRST_ORDER) {
       const char16_t *stringParams[2] = {lastName.get(), firstName.get()};
 
-      rv = bundle->FormatStringFromName(MOZ_UTF16("lastFirstFormat"),
-                                        stringParams, 2, getter_Copies(result));
+      rv = bundle->FormatStringFromName("lastFirstFormat",
+                                        stringParams, 2, result);
     }
     else {
       const char16_t *stringParams[2] = {firstName.get(), lastName.get()};
 
-      rv = bundle->FormatStringFromName(MOZ_UTF16("firstLastFormat"),
-                                        stringParams, 2, getter_Copies(result));
+      rv = bundle->FormatStringFromName("firstLastFormat",
+                                        stringParams, 2, result);
     }
-    NS_ENSURE_SUCCESS(rv, rv); 
+    NS_ENSURE_SUCCESS(rv, rv);
 
     aResult.Assign(result);
   }
