@@ -17,7 +17,6 @@
 #include "prmem.h"
 #include "nsIServiceManager.h"
 #include "mozIDOMWindow.h"
-#include "nsIFilePicker.h"
 #include "plbase64.h"
 #include "nsIWindowWatcher.h"
 #include "nsDirectoryServiceUtils.h"
@@ -51,7 +50,7 @@ struct ExportAttributesTableStruct
 // see bugs bug #116692 and #118454
 #define MOZ_AB_OBJECTCLASS "mozillaAbPersonAlpha"
 
-// for now, the oder of the attributes with true for includeForPlainText
+// for now, the order of the attributes with true for includeForPlainText
 // should be in the same order as they are in the import code
 // see importMsgProperties and nsImportStringBundle.
 //
@@ -524,6 +523,16 @@ enum ADDRESSBOOK_EXPORT_FILE_TYPE
  LDIF_EXPORT_TYPE     = 5,
 };
 
+NS_IMPL_ISUPPORTS(nsAbManager::nsFilePickerShownCallback,
+                  nsIFilePickerShownCallback)
+nsAbManager::nsFilePickerShownCallback::nsFilePickerShownCallback(
+  nsAbManager* aAbManager, nsIFilePicker* aFilePicker, nsIAbDirectory *aDirectory)
+  : mFilePicker(aFilePicker)
+  , mAbManager(aAbManager)
+  , mDirectory(aDirectory)
+{
+}
+
 NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsIAbDirectory *aDirectory)
 {
   NS_ENSURE_ARG_POINTER(aParentWin);
@@ -544,8 +553,8 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsI
   const char16_t *formatStrings[] = { dirName.get() };
 
   nsString title;
-  rv = bundle->FormatStringFromName(u"ExportAddressBookNameTitle", formatStrings,
-                                    ArrayLength(formatStrings), getter_Copies(title));
+  rv = bundle->FormatStringFromName("ExportAddressBookNameTitle", formatStrings,
+                                    ArrayLength(formatStrings), title);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = filePicker->Init(aParentWin, title, nsIFilePicker::modeSave);
@@ -556,46 +565,52 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsI
   nsString filterString;
 
   // CSV: System charset and UTF-8.
-  rv = bundle->GetStringFromName(u"CSVFilesSysCharset", getter_Copies(filterString));
+  rv = bundle->GetStringFromName("CSVFilesSysCharset", filterString);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.csv"));
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = bundle->GetStringFromName(u"CSVFilesUTF8", getter_Copies(filterString));
+  rv = bundle->GetStringFromName("CSVFilesUTF8", filterString);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.csv"));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Tab separated: System charset and UTF-8.
-  rv = bundle->GetStringFromName(u"TABFilesSysCharset", getter_Copies(filterString));
+  rv = bundle->GetStringFromName("TABFilesSysCharset", filterString);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.tab; *.txt"));
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = bundle->GetStringFromName(u"TABFilesUTF8", getter_Copies(filterString));
+  rv = bundle->GetStringFromName("TABFilesUTF8", filterString);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.tab; *.txt"));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = bundle->GetStringFromName(u"VCFFiles", getter_Copies(filterString));
+  rv = bundle->GetStringFromName("VCFFiles", filterString);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.vcf"));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = bundle->GetStringFromName(u"LDIFFiles", getter_Copies(filterString));
+  rv = bundle->GetStringFromName("LDIFFiles", filterString);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.ldi; *.ldif"));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  int16_t dialogResult;
-  filePicker->Show(&dialogResult);
+  nsCOMPtr<nsIFilePickerShownCallback> callback =
+    new nsAbManager::nsFilePickerShownCallback(this, filePicker, aDirectory);
+  return filePicker->Open(callback);
+}
 
-  if (dialogResult == nsIFilePicker::returnCancel)
-    return rv;
+NS_IMETHODIMP
+nsAbManager::nsFilePickerShownCallback::Done(int16_t aResult)
+{
+  nsresult rv;
+  if (aResult == nsIFilePicker::returnCancel)
+    return NS_OK;
 
   nsCOMPtr<nsIFile> localFile;
-  rv = filePicker->GetFile(getter_AddRefs(localFile));
+  rv = mFilePicker->GetFile(getter_AddRefs(localFile));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (dialogResult == nsIFilePicker::returnReplace) {
+  if (aResult == nsIFilePicker::returnReplace) {
     // be extra safe and only delete when the file is really a file
     bool isFile;
     rv = localFile->IsFile(&isFile);
@@ -608,7 +623,7 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsI
   // The type of export is determined by the drop-down in
   // the file picker dialog.
   int32_t exportType;
-  rv = filePicker->GetFilterIndex(&exportType);
+  rv = mFilePicker->GetFilterIndex(&exportType);
   NS_ENSURE_SUCCESS(rv,rv);
 
   nsAutoString fileName;
@@ -627,7 +642,7 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsI
         fileName.AppendLiteral(LDIF_FILE_EXTENSION2);
         localFile->SetLeafName(fileName);
       }
-      rv = ExportDirectoryToLDIF(aDirectory, localFile);
+      rv = mAbManager->ExportDirectoryToLDIF(mDirectory, localFile);
       break;
 
     case CSV_EXPORT_TYPE: // csv
@@ -639,7 +654,8 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsI
         fileName.AppendLiteral(CSV_FILE_EXTENSION);
         localFile->SetLeafName(fileName);
       }
-      rv = ExportDirectoryToDelimitedText(aDirectory, CSV_DELIM, CSV_DELIM_LEN, localFile,
+      rv = mAbManager->ExportDirectoryToDelimitedText(
+                                          mDirectory, CSV_DELIM, CSV_DELIM_LEN, localFile,
                                           exportType==CSV_EXPORT_TYPE_UTF8);
       break;
 
@@ -653,7 +669,8 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsI
         fileName.AppendLiteral(TXT_FILE_EXTENSION);
         localFile->SetLeafName(fileName);
       }
-      rv = ExportDirectoryToDelimitedText(aDirectory, TAB_DELIM, TAB_DELIM_LEN, localFile,
+      rv = mAbManager->ExportDirectoryToDelimitedText(
+                                          mDirectory, TAB_DELIM, TAB_DELIM_LEN, localFile,
                                           exportType==TAB_EXPORT_TYPE_UTF8);
       break;
 
@@ -665,7 +682,7 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsI
         fileName.AppendLiteral(VCF_FILE_EXTENSION);
         localFile->SetLeafName(fileName);
       }
-      rv = ExportDirectoryToVCard(aDirectory, localFile);
+      rv = mAbManager->ExportDirectoryToVCard(mDirectory, localFile);
       break;
   };
 
@@ -714,7 +731,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory,
 
       // We don't need to truncate the string here as getter_Copies will
       // do that for us.
-      if (NS_FAILED(bundle->GetStringFromID(EXPORT_ATTRIBUTES_TABLE[i].plainTextStringID, getter_Copies(columnName))))
+      if (NS_FAILED(bundle->GetStringFromID(EXPORT_ATTRIBUTES_TABLE[i].plainTextStringID, columnName)))
         columnName.AppendInt(EXPORT_ATTRIBUTES_TABLE[i].plainTextStringID);
 
       rv = nsMsgI18NConvertFromUnicode(useUTF8 ? NS_LITERAL_CSTRING("UTF-8") :
