@@ -5,7 +5,7 @@
 
 #ifndef __nsAbManager_h
 #define __nsAbManager_h
- 
+
 #include "nsIAbManager.h"
 #include "nsTObserverArray.h"
 #include "nsCOMPtr.h"
@@ -14,6 +14,7 @@
 #include "nsInterfaceHashtable.h"
 #include "nsIAbDirFactoryService.h"
 #include "nsIAbDirectory.h"
+#include "nsIFilePicker.h"
 
 class nsIAbLDAPAttributeMap;
 
@@ -21,7 +22,7 @@ class nsAbManager : public nsIAbManager,
                     public nsICommandLineHandler,
                     public nsIObserver
 {
-  
+
 public:
 	nsAbManager();
 
@@ -63,6 +64,26 @@ private:
     }
   };
 
+  class nsFilePickerShownCallback
+    : public nsIFilePickerShownCallback
+  {
+    virtual ~nsFilePickerShownCallback()
+    { }
+
+  public:
+    nsFilePickerShownCallback(nsAbManager* aInput,
+                              nsIFilePicker* aFilePicker,
+                              nsIAbDirectory *aDirectory);
+    NS_DECL_ISUPPORTS
+
+    NS_IMETHOD Done(int16_t aResult) override;
+
+  private:
+    nsCOMPtr<nsIFilePicker> mFilePicker;
+    RefPtr<nsAbManager> mAbManager;
+    RefPtr<nsIAbDirectory> mDirectory;
+  };
+
   nsTObserverArray<abListener> mListeners;
   nsCOMPtr<nsIAbDirectory> mCacheTopLevelAb;
   nsInterfaceHashtable<nsCStringHashKey, nsIAbDirectory> mAbStore;
