@@ -13,7 +13,7 @@
 #include "nsIMsgFilterHitNotify.h"
 #include "nsMsgUtils.h"
 #include "nsMsgSearchTerm.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgFilterService.h"
 #include "nsMsgSearchScopeTerm.h"
@@ -33,7 +33,7 @@ static const char16_t unicodeFormatter[] = {
 };
 
 // Marker for EOF or failure during read
-#define EOF_CHAR (char) 0xFF
+#define EOF_CHAR -1
 
 nsMsgFilterList::nsMsgFilterList() :
     m_fileVersion(0)
@@ -287,9 +287,8 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
   nsresult rv = GetFilterCount(&filterCount);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsMsgSearchScopeTerm* scope = new nsMsgSearchScopeTerm(nullptr, nsMsgSearchScope::offlineMail, folder);
-  scope->AddRef();
-  if (!scope) return NS_ERROR_OUT_OF_MEMORY;
+  RefPtr<nsMsgSearchScopeTerm> scope =
+    new nsMsgSearchScopeTerm(nullptr, nsMsgSearchScope::offlineMail, folder);
 
   for (uint32_t filterIndex = 0; filterIndex < filterCount; filterIndex++)
   {
@@ -322,7 +321,6 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
       }
     }
   }
-  scope->Release();
   return rv;
 }
 
@@ -379,7 +377,7 @@ static const unsigned int sNumFilterFileAttribTable =
   MOZ_ARRAY_LENGTH(FilterFileAttribTable);
 
 // If we want to buffer file IO, wrap it in here.
-char nsMsgFilterList::ReadChar(nsIInputStream *aStream)
+int nsMsgFilterList::ReadChar(nsIInputStream *aStream)
 {
   char  newChar;
   uint32_t bytesRead;
@@ -394,13 +392,13 @@ char nsMsgFilterList::ReadChar(nsIInputStream *aStream)
   {
     if (m_startWritingToBuffer)
       m_unparsedFilterBuffer.Append(newChar);
-    return newChar;
+    return (unsigned char)newChar; // Make sure the char is unsigned.
   }
 }
 
-char nsMsgFilterList::SkipWhitespace(nsIInputStream *aStream)
+int nsMsgFilterList::SkipWhitespace(nsIInputStream *aStream)
 {
-  char ch;
+  int ch;
   do
   {
     ch = ReadChar(aStream);
@@ -414,10 +412,10 @@ bool nsMsgFilterList::StrToBool(nsCString &str)
   return str.Equals("yes") ;
 }
 
-char nsMsgFilterList::LoadAttrib(nsMsgFilterFileAttribValue &attrib, nsIInputStream *aStream)
+int nsMsgFilterList::LoadAttrib(nsMsgFilterFileAttribValue &attrib, nsIInputStream *aStream)
 {
   char  attribStr[100];
-  char  curChar;
+  int curChar;
   attrib = nsIMsgFilterList::attribNone;
 
   curChar = SkipWhitespace(aStream);
@@ -454,7 +452,7 @@ const char *nsMsgFilterList::GetStringForAttrib(nsMsgFilterFileAttribValue attri
 nsresult nsMsgFilterList::LoadValue(nsCString &value, nsIInputStream *aStream)
 {
   nsAutoCString  valueStr;
-  char  curChar;
+  int curChar;
   value = "";
   curChar = SkipWhitespace(aStream);
   if (curChar != '"')
@@ -467,7 +465,7 @@ nsresult nsMsgFilterList::LoadValue(nsCString &value, nsIInputStream *aStream)
   {
     if (curChar == '\\')
     {
-      char nextChar = ReadChar(aStream);
+      int nextChar = ReadChar(aStream);
       if (nextChar == '"')
         curChar = '"';
       else if (nextChar == '\\')  // replace "\\" with "\"
@@ -502,7 +500,7 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIInputStream *aStream)
   uint64_t bytesAvailable;
 
   nsCOMPtr<nsIInputStream> bufStream;
-  err = NS_NewBufferedInputStream(getter_AddRefs(bufStream), aStream, 10240);
+  err = NS_NewBufferedInputStream(getter_AddRefs(bufStream), aStream, FILE_IO_BUFFER_SIZE);
   NS_ENSURE_SUCCESS(err, err);
 
   nsMsgFilterFileAttribValue attrib;
@@ -513,7 +511,7 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIInputStream *aStream)
     nsAutoCString value;
     nsresult intToStringResult;
 
-    char curChar;
+    int curChar;
     curChar = LoadAttrib(attrib, bufStream);
     if (curChar == EOF_CHAR)  //reached eof
       break;
@@ -579,10 +577,10 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIInputStream *aStream)
         else
         {
           // ### fix me - this is silly.
-          char16_t *unicodeString =
-            nsTextFormatter::smprintf(unicodeFormatter, value.get());
-          filter->SetFilterName(nsDependentString(unicodeString));
-          nsTextFormatter::smprintf_free(unicodeString);
+          nsString unicodeString;
+          nsTextFormatter::ssprintf(unicodeString, unicodeFormatter,
+                                    value.get());
+          filter->SetFilterName(unicodeString);
         }
         m_curFilter = filter;
         m_filters.AppendElement(filter);
@@ -1146,11 +1144,11 @@ nsresult nsMsgFilterList::ComputeArbitraryHeaders()
     rv = GetFilterAt(index, getter_AddRefs(filter));
     if (!(NS_SUCCEEDED(rv) && filter)) continue;
 
-    nsCOMPtr <nsISupportsArray> searchTerms;
-    uint32_t numSearchTerms=0;
+    nsCOMPtr<nsIMutableArray> searchTerms;
+    uint32_t numSearchTerms = 0;
     filter->GetSearchTerms(getter_AddRefs(searchTerms));
     if (searchTerms)
-      searchTerms->Count(&numSearchTerms);
+      searchTerms->GetLength(&numSearchTerms);
     for (uint32_t i = 0; i < numSearchTerms; i++)
     {
       filter->GetTerm(i, &attrib, nullptr, nullptr, nullptr, arbitraryHeader);
@@ -1158,7 +1156,7 @@ nsresult nsMsgFilterList::ComputeArbitraryHeaders()
       {
         if (m_arbitraryHeaders.IsEmpty())
           m_arbitraryHeaders.Assign(arbitraryHeader);
-        else if (m_arbitraryHeaders.Find(arbitraryHeader, CaseInsensitiveCompare) == -1)
+        else if (m_arbitraryHeaders.Find(arbitraryHeader, /* ignoreCase = */ true) == -1)
         {
           m_arbitraryHeaders.Append(" ");
           m_arbitraryHeaders.Append(arbitraryHeader);
