@@ -4,6 +4,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"
+#include "nsArray.h"
 #include "nsMsgSearchCore.h"
 #include "nsMsgSearchAdapter.h"
 #include "nsMsgSearchBoolExpression.h"
@@ -32,7 +33,8 @@ nsMsgSearchSession::nsMsgSearchSession()
   m_handlingError = false;
   m_expressionTree = nullptr;
   m_searchPaused = false;
-  NS_NewISupportsArray(getter_AddRefs(m_termList));
+  m_termList = nsArray::Create();
+  NS_ASSERTION(m_termList, "Failed to allocate a nsIMutableArray for m_termList");
 }
 
 nsMsgSearchSession::~nsMsgSearchSession()
@@ -78,7 +80,7 @@ nsMsgSearchSession::AppendTerm(nsIMsgSearchTerm *aTerm)
 }
 
 NS_IMETHODIMP
-nsMsgSearchSession::GetSearchTerms(nsISupportsArray **aResult)
+nsMsgSearchSession::GetSearchTerms(nsIMutableArray **aResult)
 {
     NS_ENSURE_ARG_POINTER(aResult);
     *aResult = m_termList;
@@ -86,6 +88,13 @@ nsMsgSearchSession::GetSearchTerms(nsISupportsArray **aResult)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgSearchSession::SetSearchTerms(nsIMutableArray *aSearchTerms)
+{
+  m_termList = aSearchTerms;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMsgSearchSession::CreateTerm(nsIMsgSearchTerm **aResult)
 {
@@ -120,7 +129,7 @@ NS_IMETHODIMP nsMsgSearchSession::UnregisterListener(nsIMsgSearchNotify *aListen
     // Removal of something at a higher index than the iterator does not affect
     // it; we only care if the the index we were pointing at gets shifted down,
     // in which case we also want to shift down.
-    if (m_iListener != -1 && listenerIndex <= m_iListener)
+    if (m_iListener != -1 && (signed)listenerIndex <= m_iListener)
       m_iListener--;
   }
 
@@ -488,8 +497,9 @@ nsresult nsMsgSearchSession::StartTimer()
 
   m_backgroundTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
-  m_backgroundTimer->InitWithFuncCallback(TimerCallback, (void *) this, 0,
-                                          nsITimer::TYPE_REPEATING_SLACK);
+  m_backgroundTimer->InitWithNamedFuncCallback(TimerCallback, (void *) this, 0,
+                                               nsITimer::TYPE_REPEATING_SLACK,
+                                               "nsMsgSearchSession::TimerCallback");
   TimerCallback(m_backgroundTimer, this);
   return NS_OK;
 }
@@ -508,7 +518,7 @@ nsMsgSearchSession::GetRunningAdapter(nsIMsgSearchAdapter **aSearchAdapter)
   nsMsgSearchScopeTerm *scope = GetRunningScope();
   if (scope)
   {
-    NS_IF_ADDREF(*aSearchAdapter = scope->m_adapter);    
+    NS_IF_ADDREF(*aSearchAdapter = scope->m_adapter);
   }
   return NS_OK;
 }
@@ -533,7 +543,7 @@ nsresult nsMsgSearchSession::NotifyListenersDone(nsresult aStatus)
 {
   // need to stabilize "this" in case one of the listeners releases the last
   // reference to us.
-  nsRefPtr<nsIMsgSearchSession> kungFuDeathGrip(this);
+  RefPtr<nsIMsgSearchSession> kungFuDeathGrip(this);
 
   nsCOMPtr<nsIMsgSearchNotify> listener;
   m_iListener = 0;
@@ -647,7 +657,7 @@ nsMsgSearchSession::EnableFolderNotifications(bool aEnable)
     nsCOMPtr<nsIMsgFolder> folder;
     scope->GetFolder(getter_AddRefs(folder));
     if (folder)  //enable msg count notifications
-      folder->EnableNotifications(nsIMsgFolder::allMessageCountNotifications, aEnable, false);
+      folder->EnableNotifications(nsIMsgFolder::allMessageCountNotifications, aEnable);
   }
 }
 
