@@ -80,7 +80,7 @@ nsMsgSearchAttribEntry SearchAttribEntryTable[] =
     {nsMsgSearchAttrib::Size,       "size"},
     // this used to be nsMsgSearchAttrib::SenderInAddressBook
     // we used to have two Sender menuitems
-    // for backward compatability, we can still parse
+    // for backward compatibility, we can still parse
     // the old style.  see bug #179803
     {nsMsgSearchAttrib::Sender,     "from in ab"},
     {nsMsgSearchAttrib::JunkStatus, "junk status"},
@@ -415,20 +415,20 @@ NS_IMPL_ISUPPORTS(nsMsgSearchTerm, nsIMsgSearchTerm)
   for (const char *strPtr = str; *strPtr; strPtr++)
     if (*strPtr == '"')
       numQuotes++;
-    int escapedStrLen = PL_strlen(str) + numQuotes;
-    char  *escapedStr = (char *) PR_Malloc(escapedStrLen + 1);
-    if (escapedStr)
+  int escapedStrLen = PL_strlen(str) + numQuotes;
+  char  *escapedStr = (char *) PR_Malloc(escapedStrLen + 1);
+  if (escapedStr)
+  {
+    char *destPtr;
+    for (destPtr = escapedStr; *str; str++)
     {
-      char *destPtr;
-      for (destPtr = escapedStr; *str; str++)
-      {
-        if (*str == '"')
-          *destPtr++ = '\\';
-        *destPtr++ = *str;
-      }
-      *destPtr = '\0';
+      if (*str == '"')
+        *destPtr++ = '\\';
+      *destPtr++ = *str;
     }
-    return escapedStr;
+    *destPtr = '\0';
+  }
+  return escapedStr;
 }
 
 
@@ -440,8 +440,8 @@ nsresult nsMsgSearchTerm::OutputValue(nsCString &outputStr)
     // need to quote strings with ')' and strings starting with '"' or ' '
     // filter code will escape quotes
     if (PL_strchr(m_value.string, ')') ||
-      (m_value.string[0] == ' ') ||
-      (m_value.string[0] == '"'))
+        (m_value.string[0] == ' ') ||
+        (m_value.string[0] == '"'))
     {
       quoteVal = true;
       outputStr += "\"";
@@ -467,7 +467,7 @@ nsresult nsMsgSearchTerm::OutputValue(nsCString &outputStr)
   {
     switch (m_attribute)
     {
-    case nsMsgSearchAttrib::Date:
+      case nsMsgSearchAttrib::Date:
       {
         PRExplodedTime exploded;
         PR_ExplodeTime(m_value.u.date, PR_LocalTimeParameters, &exploded);
@@ -784,11 +784,12 @@ nsresult nsMsgSearchTerm::MatchArbitraryHeader (nsIMsgSearchScopeTerm *scope,
 
   // We will allow accumulation of received headers;
   bool isReceivedHeader = m_arbitraryHeader.EqualsLiteral("received");
-  
+
   while (searchingHeaders)
   {
     nsCString charsetIgnored;
     if (bodyHandler->GetNextLine(buf, charsetIgnored) < 0 || EMPTY_MESSAGE_LINE(buf))
+
       searchingHeaders = false;
     bool isContinuationHeader = searchingHeaders ? NS_IsAsciiWhitespace(buf.CharAt(0))
                                                    : false;
@@ -1483,7 +1484,7 @@ nsresult nsMsgSearchTerm::MatchJunkPercent(uint32_t aJunkPercent, bool *pResult)
   *pResult = result;
   return rv;
 }
-  
+
 
 nsresult nsMsgSearchTerm::MatchLabel(nsMsgLabelValue aLabelValue, bool *pResult)
 {
@@ -1510,7 +1511,7 @@ nsresult nsMsgSearchTerm::MatchLabel(nsMsgLabelValue aLabelValue, bool *pResult)
   return rv;
 }
 
-// MatchStatus () is not only used for nsMsgMessageFlags but also for 
+// MatchStatus () is not only used for nsMsgMessageFlags but also for
 // nsMsgFolderFlags (both being 'unsigned long')
 nsresult nsMsgSearchTerm::MatchStatus(uint32_t statusToMatch, bool *pResult)
 {
@@ -1912,7 +1913,7 @@ nsresult nsMsgSearchScopeTerm::TimeSlice (bool *aDone)
   return m_adapter->Search(aDone);
 }
 
-nsresult nsMsgSearchScopeTerm::InitializeAdapter (nsISupportsArray *termList)
+nsresult nsMsgSearchScopeTerm::InitializeAdapter (nsIArray *termList)
 {
   if (m_adapter)
     return NS_OK;
