@@ -159,7 +159,7 @@ bool
 nsMessengerUnixIntegration::BuildNotificationTitle(nsIMsgFolder *aFolder, nsIStringBundle *aBundle, nsString &aTitle)
 {
   nsString accountName;
-  aFolder->GetPrettiestName(accountName);
+  aFolder->GetPrettyName(accountName);
 
   int32_t numNewMessages = 0;
   aFolder->GetNumNewMessages(true, &numNewMessages);
@@ -176,9 +176,9 @@ nsMessengerUnixIntegration::BuildNotificationTitle(nsIMsgFolder *aFolder, nsIStr
   };
 
   aBundle->FormatStringFromName(numNewMessages == 1 ?
-                                  MOZ_UTF16("newMailNotification_message") :
-                                  MOZ_UTF16("newMailNotification_messages"),
-                                formatStrings, 2, getter_Copies(aTitle));
+                                  "newMailNotification_message" :
+                                  "newMailNotification_messages",
+                                formatStrings, 2, aTitle);
   return true;
 }
 
@@ -295,8 +295,8 @@ nsMessengerUnixIntegration::BuildNotificationBody(nsIMsgDBHdr *aHdr,
     {
       subject.get(), author.get()
     };
-    aBundle->FormatStringFromName(MOZ_UTF16("newMailNotification_messagetitle"),
-        formatStrings, 2, getter_Copies(msgTitle));
+    aBundle->FormatStringFromName("newMailNotification_messagetitle",
+        formatStrings, 2, msgTitle);
     alertBody.Append(msgTitle);
   }
   else if (showSubject)
@@ -349,6 +349,7 @@ nsresult nsMessengerUnixIntegration::ShowAlertMessage(const nsAString& aAlertTit
                                                 EmptyString(),
                                                 EmptyString(),
                                                 nullptr,
+                                                false,
                                                 false);
       if (NS_SUCCEEDED(rv))
         return rv;
@@ -384,7 +385,7 @@ nsresult nsMessengerUnixIntegration::ShowNewAlertNotification(bool aUserInitiate
   NS_ENSURE_SUCCESS(rv, rv);
   ifptr->SetData(mFoldersWithNewMail);
   ifptr->SetDataIID(&NS_GET_IID(nsIArray));
-  argsArray->AppendElement(ifptr, false);
+  argsArray->AppendElement(ifptr);
 
   // pass in the observer
   ifptr = do_CreateInstance(NS_SUPPORTS_INTERFACE_POINTER_CONTRACTID, &rv);
@@ -392,13 +393,13 @@ nsresult nsMessengerUnixIntegration::ShowNewAlertNotification(bool aUserInitiate
   nsCOMPtr <nsISupports> supports = do_QueryInterface(static_cast<nsIMessengerOSIntegration*>(this));
   ifptr->SetData(supports);
   ifptr->SetDataIID(&NS_GET_IID(nsIObserver));
-  argsArray->AppendElement(ifptr, false);
+  argsArray->AppendElement(ifptr);
 
   // pass in the animation flag
   nsCOMPtr<nsISupportsPRBool> scriptableUserInitiated (do_CreateInstance(NS_SUPPORTS_PRBOOL_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
   scriptableUserInitiated->SetData(aUserInitiated);
-  argsArray->AppendElement(scriptableUserInitiated, false);
+  argsArray->AppendElement(scriptableUserInitiated);
 
   nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   nsCOMPtr<mozIDOMWindowProxy> newWindow;
@@ -694,7 +695,7 @@ nsMessengerUnixIntegration::OnItemIntPropertyChanged(nsIMsgFolder *aItem, nsIAto
         return NS_OK; // kick out right now...
 
       if (!folderFound)
-        mFoldersWithNewMail->AppendElement(weakFolder, false);
+        mFoldersWithNewMail->AppendElement(weakFolder);
       // now regenerate the tooltip
       FillToolTipInfo();
     }
