@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -43,10 +43,9 @@
 #include "nsComponentManagerUtils.h"
 #include "nsNativeCharsetUtils.h"
 #include "nsMsgUtils.h"
-#ifdef MOZILLA_INTERNAL_API
 #include "mozilla/LookAndFeel.h"
-#endif
 #include "mozilla/Services.h"
+#include "mozilla/DebugOnly.h"
 #include "nsIMutableArray.h"
 #include "nsArrayUtils.h"
 
@@ -55,8 +54,7 @@
 #define PROFILE_COMMANDLINE_ARG " -profile "
 
 #define NOTIFICATIONCLASSNAME "MailBiffNotificationMessageWindow"
-#define UNREADMAILNODEKEY "Software\\Microsoft\\Windows\\CurrentVersion\\UnreadMail\\"
-#define SHELL32_DLL L"shell32.dll"
+#define UNREADMAILNODEKEY u"Software\\Microsoft\\Windows\\CurrentVersion\\UnreadMail\\"
 #define DOUBLE_QUOTE "\""
 #define MAIL_COMMANDLINE_ARG " -mail"
 #define IDI_MAILBIFF 32576
@@ -86,18 +84,10 @@
 #define NIN_BALLOONUSERCLICK (WM_USER + 5)
 #endif
 
-#ifndef MOZILLA_INTERNAL_API
-// from LookAndFeel.h
-#define NS_ALERT_HORIZONTAL 1
-#define NS_ALERT_LEFT       2
-#define NS_ALERT_TOP        4
-#endif
-
 using namespace mozilla;
 
 // begin shameless copying from nsNativeAppSupportWin
-HWND hwndForDOMWindow( mozIDOMWindowProxy *window )
-{
+static HWND hwndForDOMWindow(mozIDOMWindowProxy *window) {
   if ( !window ) {
     return 0;
   }
@@ -270,7 +260,7 @@ nsMessengerWinIntegration::~nsMessengerWinIntegration()
   }
 
   // one last attempt, update the registry
-  nsresult rv = UpdateRegistryWithCurrent();
+  mozilla::DebugOnly<nsresult> rv = UpdateRegistryWithCurrent();
   NS_ASSERTION(NS_SUCCEEDED(rv), "failed to update registry on shutdown");
   DestroyBiffIcon();
 }
@@ -299,7 +289,7 @@ nsMessengerWinIntegration::ResetCurrent()
   return NS_OK;
 }
 
-NOTIFYICONDATAW sBiffIconData = { NOTIFYICONDATAW_V2_SIZE,
+NOTIFYICONDATAW sBiffIconData = { (DWORD)NOTIFYICONDATAW_V2_SIZE,
                                   0,
                                   2,
                                   NIF_ICON | NIF_MESSAGE | NIF_TIP | NIF_INFO,
@@ -309,7 +299,7 @@ NOTIFYICONDATAW sBiffIconData = { NOTIFYICONDATAW_V2_SIZE,
                                   0,
                                   0,
                                   L"",
-                                  30000,
+                                  { 30000 },
                                   L"",
                                   NIIF_USER | NIIF_NOSOUND };
 // allow for the null terminator
@@ -336,14 +326,6 @@ nsMessengerWinIntegration::Init()
 {
   nsresult rv;
 
-  // Get shell32.dll handle
-  HMODULE hModule = ::GetModuleHandleW(SHELL32_DLL);
-
-  if (hModule) {
-    // SHQueryUserNotificationState is available from Vista
-    mSHQueryUserNotificationState = (fnSHQueryUserNotificationState)GetProcAddress(hModule, "SHQueryUserNotificationState");
-  }
-
   nsCOMPtr <nsIMsgAccountManager> accountManager =
     do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
@@ -452,6 +434,7 @@ nsresult nsMessengerWinIntegration::ShowAlertMessage(const nsString& aAlertTitle
                                                 NS_LITERAL_STRING("auto"),
                                                 EmptyString(), EmptyString(),
                                                 nullptr,
+                                                false,
                                                 false);
       mAlertInProgress = true;
     }
@@ -491,9 +474,10 @@ nsresult nsMessengerWinIntegration::ShowNewAlertNotification(bool aUserInitiated
     prefBranch->GetBoolPref(SHOW_ALERT_PREF, &showAlert);
 
   // check if we are allowed to show a notification
-  if (showAlert && mSHQueryUserNotificationState) {
-    MOZ_QUERY_USER_NOTIFICATION_STATE qstate;
-    if (SUCCEEDED(mSHQueryUserNotificationState(&qstate))) {
+  if (showAlert) {
+    QUERY_USER_NOTIFICATION_STATE qstate;
+
+    if (SUCCEEDED(SHQueryUserNotificationState(&qstate))) {
       if (qstate != QUNS_ACCEPTS_NOTIFICATIONS) {
         showAlert = false;
       }
@@ -510,7 +494,7 @@ nsresult nsMessengerWinIntegration::ShowNewAlertNotification(bool aUserInitiated
     NS_ENSURE_SUCCESS(rv, rv);
     ifptr->SetData(mFoldersWithNewMail);
     ifptr->SetDataIID(&NS_GET_IID(nsIArray));
-    rv = argsArray->AppendElement(ifptr, false);
+    rv = argsArray->AppendElement(ifptr);
     NS_ENSURE_SUCCESS(rv, rv);
 
     // pass in the observer
@@ -519,14 +503,14 @@ nsresult nsMessengerWinIntegration::ShowNewAlertNotification(bool aUserInitiated
     nsCOMPtr <nsISupports> supports = do_QueryInterface(static_cast<nsIMessengerOSIntegration*>(this));
     ifptr->SetData(supports);
     ifptr->SetDataIID(&NS_GET_IID(nsIObserver));
-    rv = argsArray->AppendElement(ifptr, false);
+    rv = argsArray->AppendElement(ifptr);
     NS_ENSURE_SUCCESS(rv, rv);
 
     // pass in the animation flag
     nsCOMPtr<nsISupportsPRBool> scriptableUserInitiated (do_CreateInstance(NS_SUPPORTS_PRBOOL_CONTRACTID, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
     scriptableUserInitiated->SetData(aUserInitiated);
-    rv = argsArray->AppendElement(scriptableUserInitiated, false);
+    rv = argsArray->AppendElement(scriptableUserInitiated);
     NS_ENSURE_SUCCESS(rv, rv);
 
     // pass in the alert origin
@@ -534,48 +518,10 @@ nsresult nsMessengerWinIntegration::ShowNewAlertNotification(bool aUserInitiated
     NS_ENSURE_TRUE(scriptableOrigin, NS_ERROR_FAILURE);
     scriptableOrigin->SetData(0);
     int32_t origin = 0;
-#ifdef MOZILLA_INTERNAL_API
     origin = LookAndFeel::GetInt(LookAndFeel::eIntID_AlertNotificationOrigin);
-#else
-    // Get task bar window handle
-    HWND shellWindow = FindWindowW(L"Shell_TrayWnd", NULL);
-
-    rv = prefBranch->GetIntPref(ALERT_ORIGIN_PREF, &origin);
-    if (NS_FAILED(rv) && (shellWindow != NULL))
-    {
-      // Determine position
-      APPBARDATA appBarData;
-      appBarData.hWnd = shellWindow;
-      appBarData.cbSize = sizeof(appBarData);
-      if (SHAppBarMessage(ABM_GETTASKBARPOS, &appBarData))
-      {
-        // Set alert origin as a bit field - see LookAndFeel.h
-        // 0 represents bottom right, sliding vertically.
-        switch(appBarData.uEdge)
-        {
-          case ABE_LEFT:
-            origin = NS_ALERT_HORIZONTAL | NS_ALERT_LEFT;
-            break;
-          case ABE_RIGHT:
-            origin = NS_ALERT_HORIZONTAL;
-            break;
-          case ABE_TOP:
-            origin = NS_ALERT_TOP;
-            // fall through for the right-to-left handling.
-          case ABE_BOTTOM:
-            // If the task bar is right-to-left,
-            // move the origin to the left
-            if (::GetWindowLong(shellWindow, GWL_EXSTYLE) &
-                  WS_EX_LAYOUTRTL)
-              origin |= NS_ALERT_LEFT;
-            break;
-        }
-      }
-    }
-#endif
     scriptableOrigin->SetData(origin);
 
-    rv = argsArray->AppendElement(scriptableOrigin, false);
+    rv = argsArray->AppendElement(scriptableOrigin);
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
@@ -647,7 +593,7 @@ nsresult nsMessengerWinIntegration::AlertClickedSimple()
   mSuppressBiffIcon = true;
   return NS_OK;
 }
-#endif MOZ_SUITE
+#endif
 
 NS_IMETHODIMP
 nsMessengerWinIntegration::Observe(nsISupports* aSubject, const char* aTopic, const char16_t* aData)
@@ -717,7 +663,7 @@ void nsMessengerWinIntegration::FillToolTipInfo()
     folder = do_QueryReferent(weakReference);
     if (folder)
     {
-      folder->GetPrettiestName(accountName);
+      folder->GetPrettyName(accountName);
 
       numNewMessages = 0;
       folder->GetNumNewMessages(true, &numNewMessages);
@@ -735,9 +681,9 @@ void nsMessengerWinIntegration::FillToolTipInfo()
 
         nsString finalText;
         if (numNewMessages == 1)
-          bundle->FormatStringFromName(MOZ_UTF16("biffNotification_message"), formatStrings, 1, getter_Copies(finalText));
+          bundle->FormatStringFromName("biffNotification_message", formatStrings, 1, finalText);
         else
-          bundle->FormatStringFromName(MOZ_UTF16("biffNotification_messages"), formatStrings, 1, getter_Copies(finalText));
+          bundle->FormatStringFromName("biffNotification_messages", formatStrings, 1, finalText);
 
         // the alert message is special...we actually only want to show the first account with
         // new mail in the alert.
@@ -925,7 +871,7 @@ nsMessengerWinIntegration::OnItemIntPropertyChanged(nsIMsgFolder *aItem, nsIAtom
           return NS_OK; // kick out right now...
       }
       if (!folderFound)
-        mFoldersWithNewMail->InsertElementAt(weakFolder, 0, false);
+        mFoldersWithNewMail->InsertElementAt(weakFolder, 0);
       // now regenerate the tooltip
       FillToolTipInfo();
     }
@@ -986,7 +932,7 @@ nsMessengerWinIntegration::OnUnreadCountUpdateTimer(nsITimer *timer, void *osInt
   nsMessengerWinIntegration *winIntegration = (nsMessengerWinIntegration*)osIntegration;
 
   winIntegration->mUnreadTimerActive = false;
-  nsresult rv = winIntegration->UpdateUnreadCount();
+  mozilla::DebugOnly<nsresult> rv = winIntegration->UpdateUnreadCount();
   NS_ASSERTION(NS_SUCCEEDED(rv), "updating unread count failed");
 }
 
@@ -1015,7 +961,7 @@ nsMessengerWinIntegration::RemoveCurrentFromRegistry()
   {
     if (wcscmp(registryUnreadMailCountKey, currentUnreadMailCountKey.get())==0) {
       nsAutoString deleteKey;
-      deleteKey.Assign(NS_LITERAL_STRING(UNREADMAILNODEKEY).get());
+      deleteKey.Assign(UNREADMAILNODEKEY);
       deleteKey.Append(currentUnreadMailCountKey.get());
 
       if (!deleteKey.IsEmpty()) {
@@ -1073,9 +1019,9 @@ nsMessengerWinIntegration::UpdateRegistryWithCurrent()
       CopyASCIItoUTF16(mEmail, pBuffer);
 
     // Write the info into the registry
-    HRESULT hr = SHSetUnreadMailCountW(pBuffer.get(),
-                                       mCurrentUnreadCount,
-                                       commandLinerForAppLaunch.get());
+    SHSetUnreadMailCountW(pBuffer.get(),
+                          mCurrentUnreadCount,
+                          commandLinerForAppLaunch.get());
   }
 
   // do this last
@@ -1183,8 +1129,11 @@ nsMessengerWinIntegration::SetupUnreadCountUpdateTimer()
   else
     mUnreadCountUpdateTimer = do_CreateInstance("@mozilla.org/timer;1");
 
-  mUnreadCountUpdateTimer->InitWithFuncCallback(OnUnreadCountUpdateTimer,
-    (void *)this, UNREAD_UPDATE_INTERVAL, nsITimer::TYPE_ONE_SHOT);
+  mUnreadCountUpdateTimer->InitWithNamedFuncCallback(OnUnreadCountUpdateTimer,
+                                                     (void *)this,
+                                                     UNREAD_UPDATE_INTERVAL,
+                                                     nsITimer::TYPE_ONE_SHOT,
+                                                     "nsMessengerWinIntegration::OnUnreadCountUpdateTimer");
 
   return NS_OK;
 }
