@@ -170,7 +170,7 @@ nsresult nsMsgSearchDBView::HashHdr(nsIMsgDBHdr *msgHdr, nsString& aHashKey)
     aHashKey.Truncate();
     nsCOMPtr<nsIMsgFolder> folder;
     msgHdr->GetFolder(getter_AddRefs(folder));
-    return folder->GetPrettiestName(aHashKey);
+    return folder->GetPrettyName(aHashKey);
   }
   return nsMsgGroupView::HashHdr(msgHdr, aHashKey);
 }
@@ -180,7 +180,7 @@ nsresult nsMsgSearchDBView::FetchLocation(int32_t aRow, nsAString& aLocationStri
   nsCOMPtr <nsIMsgFolder> folder;
   nsresult rv = GetFolderForViewIndex(aRow, getter_AddRefs(folder));
   NS_ENSURE_SUCCESS(rv,rv);
-  return folder->GetPrettiestName(aLocationString);
+  return folder->GetPrettyName(aLocationString);
 }
 
 nsresult nsMsgSearchDBView::OnNewHeader(nsIMsgDBHdr *newHdr, nsMsgKey aParentKey,
@@ -267,7 +267,7 @@ NS_IMETHODIMP nsMsgSearchDBView::OnHdrFlagsChanged(nsIMsgDBHdr *aHdrChanged, uin
 {
   // defer to base class if we're grouped or not threaded at all
   if (m_viewFlags & nsMsgViewFlagsType::kGroupBySort ||
-      !(m_viewFlags && nsMsgViewFlagsType::kThreadedDisplay))
+      !(m_viewFlags & nsMsgViewFlagsType::kThreadedDisplay))
     return nsMsgGroupView::OnHdrFlagsChanged(aHdrChanged, aOldFlags, 
                                              aNewFlags, aInstigator);
 
@@ -764,6 +764,7 @@ NS_IMETHODIMP nsMsgSearchDBView::DoCommand(nsMsgViewCommandTypeValue command)
       command == nsMsgViewCommandType::deleteNoTrash ||
       command == nsMsgViewCommandType::selectAll ||
       command == nsMsgViewCommandType::selectThread ||
+      command == nsMsgViewCommandType::selectFlagged ||
       command == nsMsgViewCommandType::expandAll ||
       command == nsMsgViewCommandType::collapseAll)
     return nsMsgDBView::DoCommand(command);
@@ -971,7 +972,7 @@ nsMsgSearchDBView::GetFoldersAndHdrsForSelection(nsMsgViewIndex *indices, int32_
         if (NS_SUCCEEDED(rv) && msgFolder && msgFolder == curFolder) 
         {
           nsCOMPtr<nsISupports> hdrSupports = do_QueryInterface(hdr);
-          msgHdrsForOneFolder->AppendElement(hdrSupports, false);
+          msgHdrsForOneFolder->AppendElement(hdrSupports);
         }
       }
     }
