@@ -12,8 +12,8 @@ load("../../../resources/asyncTestUtils.js");
 
 // javascript mime emitter functions
 var mimeMsg = {};
-Components.utils.import("resource:///modules/gloda/mimemsg.js", mimeMsg);
-Components.utils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource:///modules/gloda/mimemsg.js", mimeMsg);
+ChromeUtils.import("resource:///modules/mailServices.js");
 
 var tests = [
   startCopy,
@@ -22,7 +22,7 @@ var tests = [
   testDetach,
 ]
 
-function startCopy()
+function* startCopy()
 {
   // Get a message into the local filestore.
   let mailFile = do_get_file("../../../data/multipartmalt-detach");
@@ -32,7 +32,7 @@ function startCopy()
 }
 
 // process the message through mime
-function startMime()
+function* startMime()
 {
   let msgHdr = mailTestUtils.firstMsgHdr(localAccountUtils.inboxFolder);
 
@@ -42,7 +42,7 @@ function startMime()
 }
 
 // detach any found attachments
-function startDetach()
+function* startDetach()
 {
   let msgHdr = mailTestUtils.firstMsgHdr(localAccountUtils.inboxFolder);
   let msgURI = msgHdr.folder.generateMessageURI(msgHdr.messageKey);
@@ -57,7 +57,7 @@ function startDetach()
 }
 
 // test that the detachment was successful
-function testDetach()
+function* testDetach()
 {
   // This test seems to fail on Linux without the following delay.
   do_timeout(200, async_driver);
