@@ -11,7 +11,7 @@
 
 #include "nsIServiceManager.h"
 #include "nsCOMPtr.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsISupportsPrimitives.h"
 
 #include "nsIMsgBiffManager.h"
@@ -24,6 +24,7 @@
 #include "nsIMsgWindow.h"
 #include "nsIMsgFilterService.h"
 #include "nsIMsgProtocolInfo.h"
+#include "nsIMutableArray.h"
 #include "nsIPrefService.h"
 #include "nsIRelativeFilePref.h"
 #include "nsIDocShell.h"
@@ -46,6 +47,7 @@
 #include "nsIMsgSearchTerm.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "mozilla/Services.h"
+#include "Services.h"
 #include "nsIMsgFilter.h"
 #include "nsIArray.h"
 #include "nsArrayUtils.h"
@@ -643,13 +645,13 @@ nsMsgIncomingServer::ToString(nsAString& aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgIncomingServer::SetPassword(const nsACString& aPassword)
+NS_IMETHODIMP nsMsgIncomingServer::SetPassword(const nsAString& aPassword)
 {
   m_password = aPassword;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgIncomingServer::GetPassword(nsACString& aPassword)
+NS_IMETHODIMP nsMsgIncomingServer::GetPassword(nsAString& aPassword)
 {
   aPassword = m_password;
   return NS_OK;
@@ -718,7 +720,7 @@ nsresult nsMsgIncomingServer::GetPasswordWithoutUI()
         rv = logins[i]->GetPassword(password);
         NS_ENSURE_SUCCESS(rv, rv);
 
-        m_password = NS_LossyConvertUTF16toASCII(password);
+        m_password = password;
         break;
       }
     }
@@ -731,7 +733,7 @@ NS_IMETHODIMP
 nsMsgIncomingServer::GetPasswordWithUI(const nsAString& aPromptMessage, const
                                        nsAString& aPromptTitle,
                                        nsIMsgWindow* aMsgWindow,
-                                       nsACString& aPassword)
+                                       nsAString& aPassword)
 {
   nsresult rv = NS_OK;
 
@@ -786,7 +788,7 @@ nsMsgIncomingServer::GetPasswordWithUI(const nsAString& aPromptMessage, const
       // and getter_Copies.
       char16_t *uniPassword = nullptr;
       if (!aPassword.IsEmpty())
-        uniPassword = ToNewUnicode(NS_ConvertASCIItoUTF16(aPassword));
+        uniPassword = ToNewUnicode(aPassword);
 
       bool okayValue = true;
       rv = dialog->PromptPassword(PromiseFlatString(aPromptTitle).get(),
@@ -794,8 +796,6 @@ nsMsgIncomingServer::GetPasswordWithUI(const nsAString& aPromptMessage, const
                                   NS_ConvertASCIItoUTF16(serverUri).get(),
                                   nsIAuthPrompt::SAVE_PASSWORD_PERMANENTLY,
                                   &uniPassword, &okayValue);
-      nsAutoString uniPasswordAdopted;
-      uniPasswordAdopted.Adopt(uniPassword);
       NS_ENSURE_SUCCESS(rv, rv);
 
       if (!okayValue) // if the user pressed cancel, just return an empty string;
@@ -805,8 +805,10 @@ nsMsgIncomingServer::GetPasswordWithUI(const nsAString& aPromptMessage, const
       }
 
       // we got a password back...so remember it
-      rv = SetPassword(NS_LossyConvertUTF16toASCII(uniPasswordAdopted));
+      rv = SetPassword(nsDependentString(uniPassword));
       NS_ENSURE_SUCCESS(rv, rv);
+
+      PR_FREEIF(uniPassword);
     } // if we got a prompt dialog
     else
       return NS_ERROR_FAILURE;
@@ -865,7 +867,7 @@ nsMsgIncomingServer::ForgetPassword()
   }
   NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(count, logins);
 
-  return SetPassword(EmptyCString());
+  return SetPassword(EmptyString());
 }
 
 NS_IMETHODIMP
@@ -976,14 +978,14 @@ nsMsgIncomingServer::SetLocalPath(nsIFile *aLocalPath)
 NS_IMETHODIMP
 nsMsgIncomingServer::GetLocalStoreType(nsACString& aResult)
 {
-  NS_NOTYETIMPLEMENTED("nsMsgIncomingServer superclass not implementing GetLocalStoreType!");
+  MOZ_ASSERT_UNREACHABLE("nsMsgIncomingServer superclass not implementing GetLocalStoreType!");
   return NS_ERROR_UNEXPECTED;
 }
 
 NS_IMETHODIMP
 nsMsgIncomingServer::GetLocalDatabaseType(nsACString& aResult)
 {
-  NS_NOTYETIMPLEMENTED("nsMsgIncomingServer superclass not implementing GetLocalDatabaseType!");
+  MOZ_ASSERT_UNREACHABLE("nsMsgIncomingServer superclass not implementing GetLocalDatabaseType!");
   return NS_ERROR_UNEXPECTED;
 }
 
@@ -1202,14 +1204,21 @@ nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
   nsresult rv;
 
   // 1. Reset password so that users are prompted for new password for the new user/host.
-  ForgetPassword();
+  int32_t atPos = newName.FindChar('@');
+  // If only username changed and the new name just added a domain
+  // we can keep the password.
+  if (hostnameChanged || (atPos == kNotFound) ||
+      !StringHead(NS_ConvertASCIItoUTF16(newName), atPos)
+                 .Equals(NS_ConvertASCIItoUTF16(oldName))) {
+
+    ForgetPassword();
+  }
 
   // 2. Let the derived class close all cached connection to the old host.
   CloseCachedConnections();
 
   // 3. Notify any listeners for account server changes.
-  nsCOMPtr<nsIMsgAccountManager> accountManager = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIMsgAccountManager> accountManager(mozilla::services::GetAccountManager());
 
   rv = accountManager->NotifyServerChanged(this);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1219,13 +1228,11 @@ nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
   rv = GetPrettyName(acctName);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  NS_ENSURE_FALSE(acctName.IsEmpty(), NS_OK);
-
-  // exit if new name contains @ then better do not update the account name
-  if (!hostnameChanged && (newName.FindChar('@') != kNotFound))
+  // If new username contains @ then better do not update the account name.
+  if (acctName.IsEmpty() || (!hostnameChanged && (atPos != kNotFound)))
     return NS_OK;
 
-  int32_t atPos = acctName.FindChar('@');
+  atPos = acctName.FindChar('@');
 
   // get previous username and hostname
   nsCString userName, hostName;
@@ -1635,8 +1642,8 @@ NS_IMETHODIMP nsMsgIncomingServer::DisplayOfflineMsg(nsIMsgWindow *aMsgWindow)
   {
     nsString errorMsgTitle;
     nsString errorMsgBody;
-    bundle->GetStringFromName(u"nocachedbodybody2", getter_Copies(errorMsgBody));
-    bundle->GetStringFromName(u"nocachedbodytitle", getter_Copies(errorMsgTitle));
+    bundle->GetStringFromName("nocachedbodybody2", errorMsgBody);
+    bundle->GetStringFromName("nocachedbodytitle", errorMsgTitle);
     aMsgWindow->DisplayHTMLInMessagePane(errorMsgTitle, errorMsgBody, true);
   }
 
@@ -1875,7 +1882,7 @@ nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(nsIMsgFilterList *filte
      */
 
     // get the list of search terms from the filter
-    nsCOMPtr<nsISupportsArray> searchTerms;
+    nsCOMPtr<nsIMutableArray> searchTerms;
     rv = newFilter->GetSearchTerms(getter_AddRefs(searchTerms));
     NS_ENSURE_SUCCESS(rv, rv);
     uint32_t count = 0;
