@@ -4,7 +4,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsMsgProtocol.h"
 #include "nsIMsgMailNewsUrl.h"
 #include "nsIMsgMailSession.h"
@@ -35,6 +35,7 @@
 #include "nsIAsyncInputStream.h"
 #include "nsIMsgIncomingServer.h"
 #include "nsIInputStreamPump.h"
+#include "nsICancelable.h"
 #include "nsMimeTypes.h"
 #include "nsAlgorithm.h"
 #include "mozilla/Services.h"
@@ -78,6 +79,10 @@ nsresult nsMsgProtocol::InitFromURI(nsIURI *aUrl)
     mailUrl->GetStatusFeedback(getter_AddRefs(statusFeedback));
     mProgressEventSink = do_QueryInterface(statusFeedback);
   }
+
+  // Reset channel data in case the object is reused and initialised again.
+  mCharset.Truncate();
+
   return NS_OK;
 }
 
@@ -255,10 +260,10 @@ nsresult nsMsgProtocol::CloseSocket()
   if (m_request) {
     rv = m_request->Cancel(NS_BINDING_ABORTED);
   }
-  m_request = 0;
+  m_request = nullptr;
   if (m_transport) {
     m_transport->Close(NS_BINDING_ABORTED);
-    m_transport = 0;
+    m_transport = nullptr;
   }
 
   return rv;
@@ -323,6 +328,52 @@ NS_IMETHODIMP nsMsgProtocol::OnStartRequest(nsIRequest *request, nsISupports *ct
   return rv;
 }
 
+void nsMsgProtocol::ShowAlertMessage(nsIMsgMailNewsUrl *aMsgUrl, nsresult aStatus)
+{
+  const char16_t* errorString = nullptr;
+  switch (aStatus)
+  {
+  case NS_ERROR_UNKNOWN_HOST:
+  case NS_ERROR_UNKNOWN_PROXY_HOST:
+     errorString = u"unknownHostError";
+     break;
+  case NS_ERROR_CONNECTION_REFUSED:
+  case NS_ERROR_PROXY_CONNECTION_REFUSED:
+     errorString = u"connectionRefusedError";
+     break;
+  case NS_ERROR_NET_TIMEOUT:
+     errorString = u"netTimeoutError";
+     break;
+  case NS_ERROR_NET_RESET:
+     errorString = u"netResetError";
+     break;
+  case NS_ERROR_NET_INTERRUPT:
+     errorString = u"netInterruptError";
+     break;
+  default:
+     // Leave the string as nullptr.
+     break;
+  }
+
+  NS_ASSERTION(errorString, "unknown error, but don't alert user.");
+  if (errorString)
+  {
+    nsString errorMsg;
+    errorMsg.Adopt(FormatStringWithHostNameByName(errorString, aMsgUrl));
+    if (errorMsg.IsEmpty())
+    {
+      errorMsg.Assign(NS_LITERAL_STRING("[StringID "));
+      errorMsg.Append(errorString);
+      errorMsg.AppendLiteral("?]");
+    }
+
+    nsCOMPtr<nsIMsgMailSession> mailSession =
+      do_GetService(NS_MSGMAILSESSION_CONTRACTID);
+    if (mailSession)
+      mailSession->AlertUser(errorMsg, aMsgUrl);
+  }
+}
+
 // stop binding is a "notification" informing us that the stream associated with aURL is going away.
 NS_IMETHODIMP nsMsgProtocol::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult aStatus)
 {
@@ -337,7 +388,7 @@ NS_IMETHODIMP nsMsgProtocol::OnStopRequest(nsIRequest *request, nsISupports *ctx
   nsCOMPtr <nsIMsgMailNewsUrl> msgUrl = do_QueryInterface(ctxt, &rv);
   if (NS_SUCCEEDED(rv) && msgUrl)
   {
-    rv = msgUrl->SetUrlState(false, aStatus);
+    rv = msgUrl->SetUrlState(false, aStatus);  // Always returns NS_OK.
     if (m_loadGroup)
       m_loadGroup->RemoveRequest(static_cast<nsIRequest *>(this), nullptr, aStatus);
 
@@ -350,50 +401,12 @@ NS_IMETHODIMP nsMsgProtocol::OnStopRequest(nsIRequest *request, nsISupports *ctx
     // bugs #30775 and #30648 relate to this
     if (!m_channelContext && NS_FAILED(aStatus) &&
         (aStatus != NS_BINDING_ABORTED))
-    {
-      const char16_t* errorString = nullptr;
-      switch (aStatus)
-      {
-          case NS_ERROR_UNKNOWN_HOST:
-          case NS_ERROR_UNKNOWN_PROXY_HOST:
-             errorString = MOZ_UTF16("unknownHostError");
-             break;
-          case NS_ERROR_CONNECTION_REFUSED:
-          case NS_ERROR_PROXY_CONNECTION_REFUSED:
-             errorString = MOZ_UTF16("connectionRefusedError");
-             break;
-          case NS_ERROR_NET_TIMEOUT:
-             errorString = MOZ_UTF16("netTimeoutError");
-             break;
-          default:
-             // Leave the string as nullptr.
-             break;
-      }
-
-      NS_ASSERTION(errorString, "unknown error, but don't alert user.");
-      if (errorString)
-      {
-        nsString errorMsg;
-        errorMsg.Adopt(FormatStringWithHostNameByName(errorString, msgUrl));
-        if (errorMsg.IsEmpty())
-        {
-          errorMsg.Assign(NS_LITERAL_STRING("[StringID "));
-          errorMsg.Append(errorString);
-          errorMsg.AppendLiteral("?]");
-        }
-
-        nsCOMPtr<nsIMsgMailSession> mailSession =
-          do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv);
-        NS_ENSURE_SUCCESS(rv, rv);
-
-        rv = mailSession->AlertUser(errorMsg, msgUrl);
-      }
-    } // if we got an error code
+      ShowAlertMessage(msgUrl, aStatus);
   } // if we have a mailnews url.
 
   // Drop notification callbacks to prevent cycles.
-  mCallbacks = 0;
-  mProgressEventSink = 0;
+  mCallbacks = nullptr;
+  mProgressEventSink = nullptr;
   // Call CloseSocket(), in case we got here because the server dropped the
   // connection while reading, and we never get a chance to get back into
   // the protocol state machine via OnDataAvailable.
@@ -571,32 +584,32 @@ NS_IMETHODIMP nsMsgProtocol::GetContentType(nsACString &aContentType)
   // us to optimize the case where the message url actual refers to
   // a part in the message that has a content type that is not message/rfc822
 
-  if (m_ContentType.IsEmpty())
+  if (mContentType.IsEmpty())
     aContentType.AssignLiteral("message/rfc822");
   else
-    aContentType = m_ContentType;
+    aContentType = mContentType;
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgProtocol::SetContentType(const nsACString &aContentType)
 {
   nsAutoCString charset;
-  nsresult rv = NS_ParseResponseContentType(aContentType, m_ContentType, charset);
-  if (NS_FAILED(rv) || m_ContentType.IsEmpty())
-    m_ContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
+  nsresult rv = NS_ParseResponseContentType(aContentType, mContentType, charset);
+  if (NS_FAILED(rv) || mContentType.IsEmpty())
+    mContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
   return rv;
 }
 
 NS_IMETHODIMP nsMsgProtocol::GetContentCharset(nsACString &aContentCharset)
 {
-  aContentCharset.Truncate();
+  aContentCharset.Assign(mCharset);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgProtocol::SetContentCharset(const nsACString &aContentCharset)
 {
-  NS_WARNING("nsMsgProtocol::SetContentCharset() not implemented");
-  return NS_ERROR_NOT_IMPLEMENTED;
+  mCharset.Assign(aContentCharset);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -739,6 +752,12 @@ nsMsgProtocol::OnTransportStatus(nsITransport *transport, nsresult status,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgProtocol::GetIsDocument(bool *aIsDocument)
+{
+  return NS_GetIsDocumentChannel(this, aIsDocument);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // From nsIRequest
 ////////////////////////////////////////////////////////////////////////////////
@@ -760,11 +779,14 @@ NS_IMETHODIMP nsMsgProtocol::GetStatus(nsresult *status)
 
 NS_IMETHODIMP nsMsgProtocol::Cancel(nsresult status)
 {
-  NS_ASSERTION(m_request,"no channel");
-  if (!m_request)
-    return NS_ERROR_FAILURE;
+  if (m_proxyRequest)
+    m_proxyRequest->Cancel(status);
 
-  return m_request->Cancel(status);
+  if (m_request)
+    return m_request->Cancel(status);
+
+  NS_WARNING("no request to cancel");
+  return NS_ERROR_NOT_AVAILABLE;
 }
 
 NS_IMETHODIMP nsMsgProtocol::Suspend()
@@ -925,7 +947,7 @@ nsresult nsMsgProtocol::DoGSSAPIStep2(nsCString &commandResponse, nsCString &res
     return rv;
 }
 
-nsresult nsMsgProtocol::DoNtlmStep1(const char *username, const char *password, nsCString &response)
+nsresult nsMsgProtocol::DoNtlmStep1(const nsACString &username, const nsAString &password, nsCString &response)
 {
     nsresult rv;
 
@@ -935,7 +957,7 @@ nsresult nsMsgProtocol::DoNtlmStep1(const char *username, const char *password,
         return rv;
 
     m_authModule->Init(nullptr, 0, nullptr, NS_ConvertUTF8toUTF16(username).get(),
-                       NS_ConvertUTF8toUTF16(password).get());
+                       nsPromiseFlatString(password).get());
 
     void *outBuf;
     uint32_t outBufLen;
@@ -1012,7 +1034,7 @@ public:
     //
     // nsIOutputStreamCallback implementation ...
     //
-    NS_IMETHODIMP OnOutputStreamReady(nsIAsyncOutputStream *aOutStream)
+    NS_IMETHODIMP OnOutputStreamReady(nsIAsyncOutputStream *aOutStream) override
     {
         NS_ASSERTION(mInStream, "not initialized");
 
@@ -1044,7 +1066,7 @@ public:
 
         if (avail)
         {
-          rv = aOutStream->WriteFrom(mInStream, std::min(avail, uint64_t(4096)), &bytesWritten);
+          rv = aOutStream->WriteFrom(mInStream, std::min(avail, uint64_t(FILE_IO_BUFFER_SIZE)), &bytesWritten);
           // if were full at the time, the input stream may be backed up and we need to read any remains from the last ODA call
           // before we'll get more ODA calls
           if (protInst->mSuspendedRead)
@@ -1196,6 +1218,11 @@ NS_IMETHODIMP nsMsgAsyncWriteProtocol::Cancel(nsresult status)
 {
   mGenerateProgressNotifications = false;
 
+  if (m_proxyRequest)
+  {
+    m_proxyRequest->Cancel(status);
+  }
+
   if (m_request)
     m_request->Cancel(status);
 
@@ -1420,11 +1447,13 @@ nsresult nsMsgAsyncWriteProtocol::SetupTransportState()
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsIAsyncInputStream *inputStream = nullptr;
-    pipe->GetInputStream(&inputStream);
+    // This always succeeds because the pipe is initialized above.
+    MOZ_ALWAYS_SUCCEEDS(pipe->GetInputStream(&inputStream));
     mInStream = dont_AddRef(static_cast<nsIInputStream *>(inputStream));
 
     nsIAsyncOutputStream *outputStream = nullptr;
-    pipe->GetOutputStream(&outputStream);
+    // This always succeeds because the pipe is initialized above.
+    MOZ_ALWAYS_SUCCEEDS(pipe->GetOutputStream(&outputStream));
     m_outputStream = dont_AddRef(static_cast<nsIOutputStream *>(outputStream));
 
     mProviderThread = do_GetCurrentThread();
@@ -1489,7 +1518,8 @@ void nsMsgAsyncWriteProtocol::UpdateProgress(uint32_t aNewBytes)
     if (!webProgressListener) return;
 
     // XXX not sure if m_request is correct here
-    webProgressListener->OnProgressChange(nullptr, m_request, mNumBytesPosted, mFilePostSize, mNumBytesPosted, mFilePostSize);
+    webProgressListener->OnProgressChange(nullptr, m_request, mNumBytesPosted,
+      static_cast<uint32_t>(mFilePostSize), mNumBytesPosted, mFilePostSize);
   }
 
   return;
@@ -1518,7 +1548,6 @@ char16_t *FormatStringWithHostNameByName(const char16_t* stringName, nsIMsgMailN
   rv = sBundleService->CreateBundle(MSGS_URL, getter_AddRefs(sBundle));
   NS_ENSURE_SUCCESS(rv, nullptr);
 
-  char16_t *ptrv = nullptr;
   nsCOMPtr<nsIMsgIncomingServer> server;
   rv = msgUri->GetServer(getter_AddRefs(server));
   NS_ENSURE_SUCCESS(rv, nullptr);
@@ -1529,10 +1558,11 @@ char16_t *FormatStringWithHostNameByName(const char16_t* stringName, nsIMsgMailN
 
   NS_ConvertASCIItoUTF16 hostStr(hostName);
   const char16_t *params[] = { hostStr.get() };
-  rv = sBundle->FormatStringFromName(stringName, params, 1, &ptrv);
+  nsAutoString str;
+  rv = sBundle->FormatStringFromName(NS_ConvertUTF16toUTF8(stringName).get(), params, 1, str);
   NS_ENSURE_SUCCESS(rv, nullptr);
 
-  return ptrv;
+  return ToNewUnicode(str);
 }
 
 // vim: ts=2 sw=2
