@@ -49,11 +49,11 @@ typedef struct _findServerByHostnameEntry {
 static NS_DEFINE_CID(kCSmtpUrlCID, NS_SMTPURL_CID);
 static NS_DEFINE_CID(kCMailtoUrlCID, NS_MAILTOURL_CID);
 
-// foward declarations...
+// forward declarations...
 nsresult
 NS_MsgBuildSmtpUrl(nsIFile * aFilePath,
                    nsISmtpServer *aServer,
-                   const char* aRecipients, 
+                   const char* aRecipients,
                    nsIMsgIdentity * aSenderIdentity,
                    nsIUrlListener * aUrlListener,
                    nsIMsgStatusFeedback *aStatusFeedback,
@@ -78,10 +78,10 @@ NS_IMPL_ISUPPORTS(nsSmtpService, nsISmtpService, nsIProtocolHandler)
 
 
 NS_IMETHODIMP nsSmtpService::SendMailMessage(nsIFile * aFilePath,
-                                        const char * aRecipients, 
+                                        const char * aRecipients,
                                         nsIMsgIdentity * aSenderIdentity,
-                                        const char * aPassword,
-                                        nsIUrlListener * aUrlListener, 
+                                        const nsAString & aPassword,
+                                        nsIUrlListener * aUrlListener,
                                         nsIMsgStatusFeedback *aStatusFeedback,
                                         nsIInterfaceRequestor* aNotificationCallbacks,
                                         bool aRequestDSN,
@@ -96,14 +96,14 @@ NS_IMETHODIMP nsSmtpService::SendMailMessage(nsIFile * aFilePath,
 
   if (NS_SUCCEEDED(rv) && smtpServer)
   {
-    if (aPassword && *aPassword)
-      smtpServer->SetPassword(nsDependentCString(aPassword));
+    if (!aPassword.IsEmpty())
+      smtpServer->SetPassword(aPassword);
 
     // this ref counts urlToRun
     rv = NS_MsgBuildSmtpUrl(aFilePath, smtpServer, aRecipients, aSenderIdentity,
-                            aUrlListener, aStatusFeedback, 
+                            aUrlListener, aStatusFeedback,
                             aNotificationCallbacks, &urlToRun, aRequestDSN);
-    if (NS_SUCCEEDED(rv) && urlToRun)	
+    if (NS_SUCCEEDED(rv) && urlToRun)
       rv = NS_MsgLoadSmtpUrl(urlToRun, nullptr, aRequest);
 
     if (aURL) // does the caller want a handle on the url?
@@ -116,20 +116,20 @@ NS_IMETHODIMP nsSmtpService::SendMailMessage(nsIFile * aFilePath,
 }
 
 
-// The following are two convience functions I'm using to help expedite building and running a mail to url...
+// The following are two convenience functions I'm using to help expedite building and running a mail to url...
 
 // short cut function for creating a mailto url...
 nsresult NS_MsgBuildSmtpUrl(nsIFile * aFilePath,
                             nsISmtpServer *aSmtpServer,
-                            const char * aRecipients, 
+                            const char * aRecipients,
                             nsIMsgIdentity * aSenderIdentity,
-                            nsIUrlListener * aUrlListener, 
+                            nsIUrlListener * aUrlListener,
                             nsIMsgStatusFeedback *aStatusFeedback,
                             nsIInterfaceRequestor* aNotificationCallbacks,
                             nsIURI ** aUrl,
                             bool aRequestDSN)
 {
-  // mscott: this function is a convience hack until netlib actually dispatches
+  // mscott: this function is a convenience hack until netlib actually dispatches
   // smtp urls. in addition until we have a session to get a password, host and
   // other stuff from, we need to use default values....
   // ..for testing purposes....
@@ -197,7 +197,7 @@ nsresult NS_MsgBuildSmtpUrl(nsIFile * aFilePath,
       wwatch->GetNewAuthPrompter(0, getter_AddRefs(smtpAuthPrompt));
   }
 
-  smtpUrl->SetPrompt(smtpPrompt);            
+  smtpUrl->SetPrompt(smtpPrompt);
   smtpUrl->SetAuthPrompt(smtpAuthPrompt);
 
   if (aUrlListener)
@@ -218,7 +218,7 @@ nsresult NS_MsgLoadSmtpUrl(nsIURI * aUrl, nsISupports * aConsumer, nsIRequest **
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Create a smtp protocol instance to run the url in.
-  nsRefPtr<nsSmtpProtocol> smtpProtocol = new nsSmtpProtocol(aUrl);
+  RefPtr<nsSmtpProtocol> smtpProtocol = new nsSmtpProtocol(aUrl);
   if (!smtpProtocol)
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -257,7 +257,7 @@ NS_IMETHODIMP nsSmtpService::VerifyLogon(nsISmtpServer *aServer,
 NS_IMETHODIMP nsSmtpService::GetScheme(nsACString &aScheme)
 {
     aScheme = "mailto";
-    return NS_OK; 
+    return NS_OK;
 }
 
 NS_IMETHODIMP nsSmtpService::GetDefaultPort(int32_t *aDefaultPort)
@@ -270,7 +270,7 @@ NS_IMETHODIMP nsSmtpService::GetDefaultPort(int32_t *aDefaultPort)
     return rv;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsSmtpService::AllowPort(int32_t port, const char *scheme, bool *_retval)
 {
     // allow smtp to run on any port
@@ -334,11 +334,11 @@ NS_IMETHODIMP nsSmtpService::NewChannel2(nsIURI *aURI,
   nsCOMPtr<nsIAsyncOutputStream> pipeOut;
   nsCOMPtr<nsIPipe> pipe = do_CreateInstance("@mozilla.org/pipe;1");
   nsresult rv = pipe->Init(false, false, 0, 0);
-  if (NS_FAILED(rv)) 
-    return rv;
-  
-  pipe->GetInputStream(getter_AddRefs(pipeIn));
-  pipe->GetOutputStream(getter_AddRefs(pipeOut));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // These always succeed because the pipe is initialized above.
+  MOZ_ALWAYS_SUCCEEDS(pipe->GetInputStream(getter_AddRefs(pipeIn)));
+  MOZ_ALWAYS_SUCCEEDS(pipe->GetOutputStream(getter_AddRefs(pipeOut)));
 
   pipeOut->Close();
 
@@ -357,8 +357,11 @@ NS_IMETHODIMP nsSmtpService::NewChannel2(nsIURI *aURI,
   if (NS_FAILED(rv))
     return rv;
 
-  return NS_NewInputStreamChannel(_retval, aURI, pipeIn,
-                                  nullPrincipal, nsILoadInfo::SEC_NORMAL,
+  return NS_NewInputStreamChannel(_retval,
+                                  aURI,
+                                  pipeIn,
+                                  nullPrincipal,
+                                  nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
                                   nsIContentPolicy::TYPE_OTHER,
                                   NS_LITERAL_CSTRING("application/x-mailto"));
 }
@@ -382,7 +385,7 @@ nsSmtpService::loadSmtpServers()
 {
   if (mSmtpServersLoaded)
     return NS_OK;
-    
+
   nsresult rv;
   nsCOMPtr<nsIPrefService> prefService(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
   if (NS_FAILED(rv))
@@ -463,7 +466,7 @@ nsSmtpService::saveKeyList()
     nsresult rv;
     nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
-    
+
     return prefBranch->SetCharPref(PREF_MAIL_SMTPSERVERS, mServerKeyList.get());
 }
 
@@ -471,11 +474,11 @@ nsresult
 nsSmtpService::createKeyedServer(const char *key, nsISmtpServer** aResult)
 {
     if (!key) return NS_ERROR_NULL_POINTER;
-    
+
     nsresult rv;
     nsCOMPtr<nsISmtpServer> server = do_CreateInstance(NS_SMTPSERVER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
-    
+
     server->SetKey(key);
     mSmtpServers.AppendObject(server);
 
@@ -486,9 +489,9 @@ nsSmtpService::createKeyedServer(const char *key, nsISmtpServer** aResult)
         mServerKeyList += key;
     }
 
-    if (aResult) 
+    if (aResult)
        server.swap(*aResult);
-    
+
     return NS_OK;
 }
 
@@ -496,7 +499,7 @@ NS_IMETHODIMP
 nsSmtpService::GetSessionDefaultServer(nsISmtpServer **aServer)
 {
     NS_ENSURE_ARG_POINTER(aServer);
-    
+
     if (!mSessionDefaultServer)
         return GetDefaultServer(aServer);
 
@@ -517,7 +520,7 @@ nsSmtpService::GetDefaultServer(nsISmtpServer **aServer)
   NS_ENSURE_ARG_POINTER(aServer);
 
   loadSmtpServers();
-  
+
   *aServer = nullptr;
   // always returns NS_OK, just leaving *aServer at nullptr
   if (!mDefaultSmtpServer) {
@@ -548,7 +551,7 @@ nsSmtpService::GetDefaultServer(nsISmtpServer **aServer)
 
         mDefaultSmtpServer = mSmtpServers[0];
         NS_ENSURE_TRUE(mDefaultSmtpServer, NS_ERROR_NULL_POINTER);
-          
+
         // now we have a default server, set the prefs correctly
         nsCString serverKey;
         mDefaultSmtpServer->GetKey(getter_Copies(serverKey));
@@ -560,7 +563,7 @@ nsSmtpService::GetDefaultServer(nsISmtpServer **aServer)
   // at this point:
   // * mDefaultSmtpServer has a valid server
   // * the key has been set in the prefs
-    
+
   NS_IF_ADDREF(*aServer = mDefaultSmtpServer);
 
   return NS_OK;
@@ -576,7 +579,7 @@ nsSmtpService::SetDefaultServer(nsISmtpServer *aServer)
     nsCString serverKey;
     nsresult rv = aServer->GetKey(getter_Copies(serverKey));
     NS_ENSURE_SUCCESS(rv,rv);
-    
+
     nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     NS_ENSURE_SUCCESS(rv,rv);
     prefBranch->SetCharPref(PREF_MAIL_SMTP_DEFAULTSERVER, serverKey.get());
@@ -593,12 +596,12 @@ nsSmtpService::findServerByKey(nsISmtpServer *aServer, void *aData)
   if (NS_FAILED(rv))
     return true;
 
-  if (key.Equals(entry->key)) 
+  if (key.Equals(entry->key))
   {
     entry->server = aServer;
     return false;
   }
-    
+
   return true;
 }
 
@@ -621,7 +624,8 @@ nsSmtpService::CreateServer(nsISmtpServer **aResult)
         entry.key = key.get();
         entry.server = nullptr;
 
-        mSmtpServers.EnumerateForwards(findServerByKey, (void *)&entry);
+        for (nsISmtpServer* s : mSmtpServers)
+          findServerByKey(s, (void *)&entry);
         if (!entry.server) unique=true;
 
     } while (!unique);
@@ -645,7 +649,8 @@ nsSmtpService::GetServerByKey(const char* aKey, nsISmtpServer **aResult)
     findServerByKeyEntry entry;
     entry.key = aKey;
     entry.server = nullptr;
-    mSmtpServers.EnumerateForwards(findServerByKey, (void *)&entry);
+    for (nsISmtpServer* s : mSmtpServers)
+      findServerByKey(s, (void *)&entry);
 
     if (entry.server) {
         NS_ADDREF(*aResult = entry.server);
@@ -667,14 +672,14 @@ nsSmtpService::DeleteServer(nsISmtpServer *aServer)
 
     nsCString serverKey;
     aServer->GetKey(getter_Copies(serverKey));
-    
+
     mSmtpServers.RemoveObjectAt(idx);
 
     if (mDefaultSmtpServer.get() == aServer)
         mDefaultSmtpServer = nullptr;
     if (mSessionDefaultServer.get() == aServer)
         mSessionDefaultServer = nullptr;
-    
+
     nsAutoCString newServerList;
     nsCString tmpStr = mServerKeyList;
     char *newStr = tmpStr.BeginWriting();
@@ -717,7 +722,7 @@ nsSmtpService::findServerByHostname(nsISmtpServer *aServer, void *aData)
 
   bool checkHostname = !entry->hostname.IsEmpty();
   bool checkUsername = !entry->username.IsEmpty();
-    
+
   if ((!checkHostname ||
        (entry->hostname.Equals(hostname, nsCaseInsensitiveCStringComparator()))) &&
        (!checkUsername ||
@@ -740,12 +745,13 @@ nsSmtpService::FindServer(const char *aUsername,
     entry.hostname = aHostname;
     entry.username = aUsername;
 
-    mSmtpServers.EnumerateForwards(findServerByHostname, (void *)&entry);
+    for (nsISmtpServer* s : mSmtpServers)
+      findServerByHostname(s, (void *)&entry);
 
     // entry.server may be null, but that's ok.
     // just return null if no server is found
     NS_IF_ADDREF(*aResult = entry.server);
-    
+
     return NS_OK;
 }
 
@@ -757,7 +763,7 @@ nsSmtpService::GetServerByIdentity(nsIMsgIdentity *aSenderIdentity,
   nsresult rv = NS_ERROR_FAILURE;
 
   // First try the identity's preferred server
-  if (aSenderIdentity) 
+  if (aSenderIdentity)
   {
       nsCString smtpServerKey;
       rv = aSenderIdentity->GetSmtpServerKey(smtpServerKey);
