@@ -8,7 +8,7 @@
 #include "nsIURI.h"
 #include "nsNetCID.h"
 #include "nsSmtpUrl.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsMsgUtils.h"
 #include "nsIMimeConverter.h"
 #include "nsMsgMimeCID.h"
@@ -95,9 +95,12 @@ nsresult nsMailtoUrl::ParseMailtoUrl(char * searchPart)
       nsCString decodedName;
       MsgUnescapeString(nsDependentCString(token), 0, decodedName);
 
+      if (decodedName.IsEmpty())
+        break;
+
       switch (NS_ToUpper(decodedName.First()))
       {
-        /* DO NOT support attachment= in mailto urls. This poses a security fire hole!!! 
+        /* DO NOT support attachment= in mailto urls. This poses a security fire hole!!!
                           case 'A':
                           if (!PL_strcasecmp (token, "attachment"))
                           m_attachmentPart = value;
@@ -112,7 +115,7 @@ nsresult nsMailtoUrl::ParseMailtoUrl(char * searchPart)
             escapedBccPart += value;
           }
           else
-            escapedBccPart = value; 
+            escapedBccPart = value;
         }
         else if (decodedName.LowerCaseEqualsLiteral("body"))
         {
@@ -125,7 +128,7 @@ nsresult nsMailtoUrl::ParseMailtoUrl(char * searchPart)
             escapedBodyPart = value;
         }
         break;
-      case 'C': 
+      case 'C':
         if (decodedName.LowerCaseEqualsLiteral("cc"))
         {
           if (!escapedCcPart.IsEmpty())
@@ -137,7 +140,7 @@ nsresult nsMailtoUrl::ParseMailtoUrl(char * searchPart)
             escapedCcPart = value;
         }
         break;
-      case 'F': 
+      case 'F':
         if (decodedName.LowerCaseEqualsLiteral("followup-to"))
           escapedFollowUpToPart = value;
         else if (decodedName.LowerCaseEqualsLiteral("from"))
@@ -271,7 +274,7 @@ nsresult nsMailtoUrl::CleanupMailtoState()
     m_ccPart = "";
     m_subjectPart = "";
     m_newsgroupPart = "";
-    m_newsHostPart = ""; 
+    m_newsHostPart = "";
     m_referencePart = "";
     m_bodyPart = "";
     m_bccPart = "";
@@ -446,6 +449,12 @@ NS_IMETHODIMP nsMailtoUrl::SetHostPort(const nsACString &aHostPort)
 	return ParseUrl();
 }
 
+NS_IMETHODIMP nsMailtoUrl::SetHostAndPort(const nsACString &aHostPort)
+{
+	m_baseURL->SetHostAndPort(aHostPort);
+	return ParseUrl();
+}
+
 NS_IMETHODIMP nsMailtoUrl::GetHost(nsACString &aHost)
 {
 	return m_baseURL->GetHost(aHost);
@@ -515,22 +524,48 @@ NS_IMETHODIMP nsMailtoUrl::Equals(nsIURI *other, bool *_retval)
   return m_baseURL->Equals(other, _retval);
 }
 
-NS_IMETHODIMP nsMailtoUrl::Clone(nsIURI **_retval)
+nsresult
+nsMailtoUrl::CloneInternal(RefHandlingEnum aRefHandlingMode,
+                           const nsACString& newRef, nsIURI** _retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
 
-  nsRefPtr<nsMailtoUrl> clone = new nsMailtoUrl();
+  RefPtr<nsMailtoUrl> clone = new nsMailtoUrl();
 
   NS_ENSURE_TRUE(clone, NS_ERROR_OUT_OF_MEMORY);
 
-  nsresult rv = m_baseURL->Clone(getter_AddRefs(clone->m_baseURL));
+  nsresult rv;
+  if (aRefHandlingMode == eHonorRef) {
+    rv = m_baseURL->Clone(getter_AddRefs(clone->m_baseURL));
+  } else if (aRefHandlingMode == eReplaceRef) {
+    rv = m_baseURL->CloneWithNewRef(newRef, getter_AddRefs(clone->m_baseURL));
+  } else {
+    rv = m_baseURL->CloneIgnoringRef(getter_AddRefs(clone->m_baseURL));
+  }
   NS_ENSURE_SUCCESS(rv, rv);
-
   clone->ParseUrl();
   clone.forget(_retval);
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMailtoUrl::Clone(nsIURI **_retval)
+{
+  return CloneInternal(eHonorRef, EmptyCString(), _retval);
+}
+
+NS_IMETHODIMP
+nsMailtoUrl::CloneIgnoringRef(nsIURI** _retval)
+{
+  return CloneInternal(eIgnoreRef, EmptyCString(), _retval);
+}
+
+NS_IMETHODIMP
+nsMailtoUrl::CloneWithNewRef(const nsACString& newRef, nsIURI** _retval)
+{
+  return CloneInternal(eReplaceRef, newRef, _retval);
+}
+
 NS_IMETHODIMP nsMailtoUrl::Resolve(const nsACString &relativePath, nsACString &result)
 {
   return m_baseURL->Resolve(relativePath, result);
@@ -558,32 +593,60 @@ NS_IMETHODIMP nsMailtoUrl::EqualsExceptRef(nsIURI *other, bool *result)
   return m_baseURL->EqualsExceptRef(other, result);
 }
 
-NS_IMETHODIMP
-nsMailtoUrl::CloneIgnoringRef(nsIURI** result)
-{
-  nsCOMPtr<nsIURI> clone;
-  nsresult rv = Clone(getter_AddRefs(clone));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = clone->SetRef(EmptyCString());
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  clone.forget(result);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsMailtoUrl::GetSpecIgnoringRef(nsACString &result)
 {
   return m_baseURL->GetSpecIgnoringRef(result);
 }
 
+NS_IMETHODIMP
+nsMailtoUrl::GetDisplaySpec(nsACString& aUnicodeSpec)
+{
+  return GetSpec(aUnicodeSpec);
+}
+
+NS_IMETHODIMP
+nsMailtoUrl::GetDisplayHostPort(nsACString& aUnicodeHostPort)
+{
+  return GetHostPort(aUnicodeHostPort);
+}
+
+NS_IMETHODIMP
+nsMailtoUrl::GetDisplayHost(nsACString& aUnicodeHost)
+{
+  return GetHost(aUnicodeHost);
+}
+
 NS_IMETHODIMP
 nsMailtoUrl::GetHasRef(bool *result)
 {
   return m_baseURL->GetHasRef(result);
 }
 
+NS_IMETHODIMP
+nsMailtoUrl::GetFilePath(nsACString &aFilePath)
+{
+  return m_baseURL->GetFilePath(aFilePath);
+}
+
+NS_IMETHODIMP
+nsMailtoUrl::SetFilePath(const nsACString &aFilePath)
+{
+  return m_baseURL->SetFilePath(aFilePath);
+}
+
+NS_IMETHODIMP
+nsMailtoUrl::GetQuery(nsACString &aQuery)
+{
+  return m_baseURL->GetQuery(aQuery);
+}
+
+NS_IMETHODIMP
+nsMailtoUrl::SetQuery(const nsACString &aQuery)
+{
+  return m_baseURL->SetQuery(aQuery);
+}
+
 /////////////////////////////////////////////////////////////////////////////////////
 // smtp url definition
 /////////////////////////////////////////////////////////////////////////////////////
@@ -596,12 +659,12 @@ nsSmtpUrl::nsSmtpUrl() : nsMsgMailNewsUrl()
   m_requestDSN = false;
   m_verifyLogon = false;
 }
- 
+
 nsSmtpUrl::~nsSmtpUrl()
 {
 }
-  
-NS_IMPL_ISUPPORTS_INHERITED(nsSmtpUrl, nsMsgMailNewsUrl, nsISmtpUrl)  
+
+NS_IMPL_ISUPPORTS_INHERITED(nsSmtpUrl, nsMsgMailNewsUrl, nsISmtpUrl)
 
 ////////////////////////////////////////////////////////////////////////////////////
 // Begin nsISmtpUrl specific support
@@ -653,7 +716,7 @@ NS_IMETHODIMP nsSmtpUrl::GetPostMessageFile(nsIFile ** aFile)
 
 NS_IMPL_GETSET(nsSmtpUrl, RequestDSN, bool, m_requestDSN)
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsSmtpUrl::SetDsnEnvid(const nsACString &aDsnEnvid)
 {
     m_dsnEnvid = aDsnEnvid;
