@@ -2,35 +2,30 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const EXPORTED_SYMBOLS = ['Log4Moz'];
+this.EXPORTED_SYMBOLS = ['Log4Moz'];
 
-Components.utils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
 
-const Cc = Components.classes;
-const Ci = Components.interfaces;
-const Cr = Components.results;
-const Cu = Components.utils;
+var MODE_RDONLY   = 0x01;
+var MODE_WRONLY   = 0x02;
+var MODE_CREATE   = 0x08;
+var MODE_APPEND   = 0x10;
+var MODE_TRUNCATE = 0x20;
 
-const MODE_RDONLY   = 0x01;
-const MODE_WRONLY   = 0x02;
-const MODE_CREATE   = 0x08;
-const MODE_APPEND   = 0x10;
-const MODE_TRUNCATE = 0x20;
+var PERMS_FILE      = parseInt("0644", 8);
+var PERMS_DIRECTORY = parseInt("0755", 8);
 
-const PERMS_FILE      = parseInt("0644", 8);
-const PERMS_DIRECTORY = parseInt("0755", 8);
+var ONE_BYTE = 1;
+var ONE_KILOBYTE = 1024 * ONE_BYTE;
+var ONE_MEGABYTE = 1024 * ONE_KILOBYTE;
 
-const ONE_BYTE = 1;
-const ONE_KILOBYTE = 1024 * ONE_BYTE;
-const ONE_MEGABYTE = 1024 * ONE_KILOBYTE;
+var DEFAULT_NETWORK_TIMEOUT_DELAY = 5;
 
-const DEFAULT_NETWORK_TIMEOUT_DELAY = 5;
+var CDATA_START = "<![CDATA[";
+var CDATA_END = "]]>";
+var CDATA_ESCAPED_END = CDATA_END + "]]&gt;" + CDATA_START;
 
-const CDATA_START = "<![CDATA[";
-const CDATA_END = "]]>";
-const CDATA_ESCAPED_END = CDATA_END + "]]&gt;" + CDATA_START;
-
-let Log4Moz = {
+var Log4Moz = {
   Level: {
     Fatal:  70,
     Error:  60,
@@ -374,8 +369,7 @@ Logger.prototype = {
         continue;
 
       if (!message)
-        message = new LogMessage(this._name, level,
-                                 Array.prototype.slice.call(args));
+        message = new LogMessage(this._name, level, Array.from(args));
 
       appender.append(message);
     }
@@ -476,33 +470,26 @@ Formatter.prototype = {
 };
 
 // services' log4moz lost the date formatting default...
-function BasicFormatter(dateFormat) {
-  if (dateFormat)
-    this.dateFormat = dateFormat;
+function BasicFormatter() {
 }
 BasicFormatter.prototype = {
   __proto__: Formatter.prototype,
 
-  _dateFormat: null,
-
-  get dateFormat() {
-    if (!this._dateFormat)
-      this._dateFormat = "%Y-%m-%d %H:%M:%S";
-    return this._dateFormat;
-  },
-
-  set dateFormat(format) {
-    this._dateFormat = format;
-  },
-
   format: function BF_format(message) {
     let date = new Date(message.time);
+    // Format timestamp as: "%Y-%m-%d %H:%M:%S"
+    let year = date.getFullYear().toString();
+    let month = (date.getMonth() + 1).toString().padStart(2, "0");
+    let day = date.getDate().toString().padStart(2, "0");
+    let hours = date.getHours().toString().padStart(2, "0");
+    let minutes = date.getMinutes().toString().padStart(2, "0");
+    let seconds = date.getSeconds().toString().padStart(2, "0");
+
+    let timeStamp = `${year}-${month}-${day} ${hours}:${minutes}:${seconds}`;
     // The trick below prevents errors further down because mo is null or
     //  undefined.
-    let messageString = [
-      ("" + mo) for each
-      ([,mo] in Iterator(message.messageObjects))].join(" ");
-    return date.toLocaleFormat(this.dateFormat) + "\t" +
+    let messageString = message.messageObjects.map(mo => "" + mo).join(" ");
+    return timeStamp + "\t" +
       message.loggerName + "\t" + message.levelDesc + "\t" +
       messageString + "\n";
   }
@@ -523,8 +510,8 @@ XMLFormatter.prototype = {
 
   format: function XF_format(message) {
     let cdataEscapedMessage =
-      [((typeof(mo) == "object") ? mo.toString() : mo) for each
-       ([,mo] in Iterator(message.messageObjects))]
+      message.messageObjects
+        .map(mo => (typeof(mo) == "object") ? mo.toString() : mo)
         .join(" ")
         .split(CDATA_END).join(CDATA_ESCAPED_END);
     return "<log4j:event logger='" + message.loggerName + "' " +
@@ -550,7 +537,7 @@ JSONFormatter.prototype = {
     let origMessageObjects = message.messageObjects;
     message.messageObjects = [];
     let reProto = [];
-    for each (let [, messageObject] in Iterator(origMessageObjects)) {
+    for (let messageObject of origMessageObjects) {
       if (messageObject)
         if (messageObject._jsonMe) {
           message.messageObjects.push(messageObject);
@@ -567,7 +554,7 @@ JSONFormatter.prototype = {
     }
     let encoded = JSON.stringify(message) + "\r\n";
     message.msgObjects = origMessageObjects;
-//    for each (let [,objectAndProtoPair] in Iterator (reProto)) {
+//    for (let objectAndProtoPair of reProto) {
 //      objectAndProtoPair[0].__proto__ = objectAndProtoPair[1];
 //    }
     return encoded;
@@ -686,7 +673,7 @@ function ConsoleAppender(formatter) {
 ConsoleAppender.prototype = {
   __proto__: Appender.prototype,
 
-  // override to send Error and higher level messages to Components.utils.reportError()
+  // override to send Error and higher level messages to Cu.reportError()
   append: function CApp_append(message) {
     let stringMessage = this._formatter.format(message);
     if (message.level > Log4Moz.Level.Warn) {
@@ -863,7 +850,7 @@ SocketAppender.prototype = {
 
       let uniOutputStream = Cc["@mozilla.org/intl/converter-output-stream;1"]
                               .createInstance(Ci.nsIConverterOutputStream);
-      uniOutputStream.init(outputStream, "utf-8", 0, 0x0000);
+      uniOutputStream.init(outputStream, "utf-8");
 
       this.__nos = uniOutputStream;
     } catch (ex) {
