@@ -11,6 +11,7 @@
 #include "nsIMsgThread.h"
 #include "nsMsgMimeCID.h"
 #include "nsIMimeConverter.h"
+#include "mozilla/Attributes.h"
 
 using namespace mozilla::mailnews;
 
@@ -702,7 +703,7 @@ nsresult nsMsgHdr::GetUInt64Column(mdb_token token, uint64_t *pvalue, uint64_t d
  *  and hand you everything after that.  We change things at all because that
  *  same behaviour does not make sense if we have already seen a proper message
  *  id.  We keep the old behaviour at all because it would seem to have
- *  benefits.  (See jwz's non-zero stats: http://www.jwz.org/doc/threading.html) 
+ *  benefits.  (See jwz's non-zero stats: http://www.jwz.org/doc/threading.html)
  * So, to re-state, if there is a valid message-id in there at all, we only
  *  return valid message-id's (sans bracketing '<' and '>').  If there isn't,
  *  our result (via "references") is a left-trimmed copy of the string.  If
@@ -713,11 +714,11 @@ nsresult nsMsgHdr::GetUInt64Column(mdb_token token, uint64_t *pvalue, uint64_t d
  *  provide "valid-stuff" and an empty string (which you should ignore) as
  *  results.  However "this stuff is invalid" would return itself, allowing
  *  anything relying on that behaviour to keep working.
- * 
+ *
  * Note: We accept anything inside the '<' and '>'; technically, we should want
  *  at least a '@' in there (per rfc 2822).  But since we're going out of our
  *  way to support weird things...
- * 
+ *
  * @param startNextRef The position to start at; this should either be the start
  *     of your references string or our return value from a previous call.
  * @param reference You pass a nsCString by reference, we put the reference we
@@ -731,7 +732,7 @@ nsresult nsMsgHdr::GetUInt64Column(mdb_token token, uint64_t *pvalue, uint64_t d
  *     properly formatted message-ids.
  * @returns The next starting position of this routine, which may be pointing at
  *     a nul '\0' character to indicate termination.
- */ 
+ */
 const char *nsMsgHdr::GetNextReference(const char *startNextRef,
                                        nsCString &reference,
                                        bool acceptNonDelimitedReferences)
@@ -769,6 +770,7 @@ const char *nsMsgHdr::GetNextReference(const char *startNextRef,
         // intentional fallthrough so whitespaceEndedAt will definitely have
         //  a non-NULL value, just in case the message-id is not valid (no '>')
         //  and the old-school support is desired.
+        MOZ_FALLTHROUGH;
       default:
         if (!whitespaceEndedAt)
             whitespaceEndedAt = ptr;
@@ -776,7 +778,7 @@ const char *nsMsgHdr::GetNextReference(const char *startNextRef,
     }
   }
 
-  // keep going until we hit a '>' or hit the end of the string 
+  // keep going until we hit a '>' or hit the end of the string
   for(; *ptr ; ptr++)
   {
     if (*ptr == '>')
