@@ -8,17 +8,17 @@
 #include "nsCOMPtr.h"
 #include "nsMsgThread.h"
 #include "MailNewsTypes2.h"
+#include "mozilla/DebugOnly.h"
 
 NS_IMPL_ISUPPORTS(nsMsgThread, nsIMsgThread)
 
 nsMsgThread::nsMsgThread()
 {
-  MOZ_COUNT_CTOR(nsMsgThread);
   Init();
 }
+
 nsMsgThread::nsMsgThread(nsMsgDatabase *db, nsIMdbTable *table)
 {
-  MOZ_COUNT_CTOR(nsMsgThread);
   Init();
   m_mdbTable = table;
   m_mdbDB = db;
@@ -50,10 +50,9 @@ void nsMsgThread::Init()
 
 nsMsgThread::~nsMsgThread()
 {
-  MOZ_COUNT_DTOR(nsMsgThread);
   if (m_mdbDB)
   {
-    bool found = m_mdbDB->m_threads.RemoveElement(this);
+    mozilla::DebugOnly<bool> found = m_mdbDB->m_threads.RemoveElement(this);
     NS_ASSERTION(found, "removing thread not in threads array");
   }
   else // This can happen if db is forced closed
@@ -140,7 +139,7 @@ NS_IMETHODIMP nsMsgThread::SetSubject(const nsACString& aSubject)
 NS_IMETHODIMP nsMsgThread::GetSubject(nsACString& aSubject)
 {
   nsCString subjectStr;
-  nsresult rv = m_mdbDB->RowCellColumnToCharPtr(m_metaRow, m_mdbDB->m_threadSubjectColumnToken, 
+  nsresult rv = m_mdbDB->RowCellColumnToCharPtr(m_metaRow, m_mdbDB->m_threadSubjectColumnToken,
                                                 getter_Copies(subjectStr));
 
   aSubject.Assign(subjectStr);
@@ -154,7 +153,6 @@ NS_IMETHODIMP nsMsgThread::GetNumChildren(uint32_t *result)
   return NS_OK;
 }
 
-
 NS_IMETHODIMP nsMsgThread::GetNumUnreadChildren (uint32_t *result)
 {
   NS_ENSURE_ARG_POINTER(result);
@@ -167,7 +165,7 @@ nsresult nsMsgThread::RerootThread(nsIMsgDBHdr *newParentOfOldRoot, nsIMsgDBHdr
   nsresult rv = NS_OK;
   mdb_pos outPos;
   nsMsgKey newHdrAncestor;
-  nsCOMPtr <nsIMsgDBHdr> ancestorHdr = newParentOfOldRoot;
+  nsCOMPtr<nsIMsgDBHdr> ancestorHdr = newParentOfOldRoot;
   nsMsgKey newRoot;
 
   ancestorHdr->GetMessageKey(&newRoot);
@@ -210,6 +208,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
   bool parentKeyNeedsSetting = true;
 
   nsIMdbRow *hdrRow = hdr->GetMDBRow();
+  NS_ENSURE_STATE(hdrRow);
   hdr->GetRawFlags(&newHdrFlags);
   hdr->GetMessageKey(&newHdrKey);
   hdr->GetDateInSeconds(&msgDate);
@@ -220,7 +219,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
     SetFlags(m_flags | nsMsgMessageFlags::Watched);
 
   child->AndFlags(~(nsMsgMessageFlags::Watched), &newHdrFlags);
-  
+
   // These are threading flags that the child may have set before being added
   // to the database.
   uint32_t protoThreadFlags;
@@ -229,9 +228,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
   // Clear the flag so that it doesn't fudge anywhere else
   child->SetUint32Property("ProtoThreadFlags", 0);
 
-  uint32_t numChildren;
-  uint32_t childIndex = 0;
-
+  uint32_t numChildren = 0;
   // get the num children before we add the new header.
   GetNumChildren(&numChildren);
 
@@ -253,10 +250,10 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
     child->SetThreadParent(parentKey);
     parentKeyNeedsSetting = false;
   }
-  // check if this header is a parent of one of the messages in this thread
 
+  // check if this header is a parent of one of the messages in this thread
   bool hdrMoved = false;
-  nsCOMPtr <nsIMsgDBHdr> curHdr;
+  nsCOMPtr<nsIMsgDBHdr> curHdr;
   uint32_t moveIndex = 0;
 
   PRTime newHdrDate;
@@ -274,9 +271,9 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
 
   if (numChildren < 1000)
   {
-    for (childIndex = 0; childIndex < numChildren; childIndex++)
+    for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
     {
-      nsMsgKey msgKey;
+      nsMsgKey msgKey = nsMsgKey_None;
 
       rv = GetChildHdrAt(childIndex, getter_AddRefs(curHdr));
       if (NS_SUCCEEDED(rv) && curHdr)
@@ -292,7 +289,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
             hdrMoved = true;
             curHdr->GetThreadParent(&oldThreadParent);
             curHdr->GetMessageKey(&msgKey);
-            nsCOMPtr <nsIMsgDBHdr> curParent;
+            nsCOMPtr<nsIMsgDBHdr> curParent;
             m_mdbDB->GetMsgHdrForKey(oldThreadParent, getter_AddRefs(curParent));
             if (curParent && hdr->IsAncestorOf(curParent))
             {
@@ -312,6 +309,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
             }
           }
           curHdr->SetThreadParent(newHdrKey);
+          // TODO: what should be msgKey if hdrMoved was true above?
           if (msgKey == newHdrKey)
             parentKeyNeedsSetting = false;
 
@@ -343,7 +341,7 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
   {
     PRTime topLevelHdrDate;
 
-    nsCOMPtr <nsIMsgDBHdr> topLevelHdr;
+    nsCOMPtr<nsIMsgDBHdr> topLevelHdr;
     rv = GetRootHdr(nullptr, getter_AddRefs(topLevelHdr));
     if (NS_SUCCEEDED(rv) && topLevelHdr)
     {
@@ -399,12 +397,11 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
 nsresult nsMsgThread::ReparentNonReferenceChildrenOf(nsIMsgDBHdr *oldTopLevelHdr, nsMsgKey newParentKey,
                                                             nsIDBChangeAnnouncer *announcer)
 {
-  nsCOMPtr <nsIMsgDBHdr> curHdr;
-  uint32_t numChildren;
-  uint32_t childIndex = 0;
+  nsCOMPtr<nsIMsgDBHdr> curHdr;
+  uint32_t numChildren = 0;
 
   GetNumChildren(&numChildren);
-  for (childIndex = 0; childIndex < numChildren; childIndex++)
+  for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
   {
     nsMsgKey oldTopLevelHdrKey;
 
@@ -550,15 +547,13 @@ nsresult nsMsgThread::ReparentChildrenOf(nsMsgKey oldParent, nsMsgKey newParent,
 {
   nsresult rv = NS_OK;
 
-  uint32_t numChildren;
-  uint32_t childIndex = 0;
-
+  uint32_t numChildren = 0;
   GetNumChildren(&numChildren);
 
-  nsCOMPtr <nsIMsgDBHdr> curHdr;
+  nsCOMPtr<nsIMsgDBHdr> curHdr;
   if (numChildren > 0)
   {
-    for (childIndex = 0; childIndex < numChildren; childIndex++)
+    for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
     {
       rv = GetChildHdrAt(childIndex, getter_AddRefs(curHdr));
       if (NS_SUCCEEDED(rv) && curHdr)
@@ -614,7 +609,7 @@ protected:
   nsresult                Prefetch();
 
   nsIMdbTableRowCursor*   mRowCursor;
-  nsCOMPtr <nsIMsgDBHdr>  mResultHdr;
+  nsCOMPtr<nsIMsgDBHdr>  mResultHdr;
   nsMsgThread*            mThread;
   nsMsgKey                mThreadParentKey;
   nsMsgKey                mFirstMsgKey;
@@ -642,16 +637,13 @@ nsMsgThreadEnumerator::nsMsgThreadEnumerator(nsMsgThread *thread, nsMsgKey start
   if (NS_SUCCEEDED(rv) && mResultHdr)
     mResultHdr->GetMessageKey(&mFirstMsgKey);
 
-  uint32_t numChildren;
+  uint32_t numChildren = 0;
   mThread->GetNumChildren(&numChildren);
 
   if (mThreadParentKey != nsMsgKey_None)
   {
     nsMsgKey msgKey = nsMsgKey_None;
-    uint32_t childIndex = 0;
-
-
-    for (childIndex = 0; childIndex < numChildren; childIndex++)
+    for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
     {
       rv = mThread->GetChildHdrAt(childIndex, getter_AddRefs(mResultHdr));
       if (NS_SUCCEEDED(rv) && mResultHdr)
@@ -667,7 +659,6 @@ nsMsgThreadEnumerator::nsMsgThreadEnumerator(nsMsgThread *thread, nsMsgKey start
 
         if (mDone)
           break;
-
       }
       else
         NS_ASSERTION(false, "couldn't get child from thread");
@@ -675,7 +666,7 @@ nsMsgThreadEnumerator::nsMsgThreadEnumerator(nsMsgThread *thread, nsMsgKey start
   }
 
 #ifdef DEBUG_bienvenu1
-  nsCOMPtr <nsIMsgDBHdr> child;
+  nsCOMPtr<nsIMsgDBHdr> child;
   for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
   {
     rv = mThread->GetChildHdrAt(childIndex, getter_AddRefs(child));
@@ -693,6 +684,7 @@ nsMsgThreadEnumerator::nsMsgThreadEnumerator(nsMsgThread *thread, nsMsgKey start
     }
   }
 #endif
+
   NS_ADDREF(thread);
 }
 
@@ -703,7 +695,6 @@ nsMsgThreadEnumerator::~nsMsgThreadEnumerator()
 
 NS_IMPL_ISUPPORTS(nsMsgThreadEnumerator, nsISimpleEnumerator)
 
-
 int32_t nsMsgThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
 {
   //	if (msgKey != mThreadParentKey)
@@ -712,10 +703,9 @@ int32_t nsMsgThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
   // If the inMsgKey is the first message in the thread, then all children
   // without parents are considered to be children of inMsgKey.
   // Otherwise, only true children qualify.
-  uint32_t numChildren;
-  nsCOMPtr <nsIMsgDBHdr> curHdr;
-  int32_t firstChildIndex = -1;
 
+  int32_t firstChildIndex = -1;
+  uint32_t numChildren = 0;
   mThread->GetNumChildren(&numChildren);
 
   // if this is the first message in the thread, just check if there's more than
@@ -725,6 +715,7 @@ int32_t nsMsgThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
 
   for (uint32_t curChildIndex = 0; curChildIndex < numChildren; curChildIndex++)
   {
+    nsCOMPtr<nsIMsgDBHdr> curHdr;
     nsresult rv = mThread->GetChildHdrAt(curChildIndex, getter_AddRefs(curHdr));
     if (NS_SUCCEEDED(rv) && curHdr)
     {
@@ -753,12 +744,11 @@ NS_IMETHODIMP nsMsgThreadEnumerator::GetNext(nsISupports **aItem)
   {
     rv = Prefetch();
     NS_ENSURE_SUCCESS(rv, rv);
-  }  
+  }
 
   if (mResultHdr)
   {
-    *aItem = mResultHdr;
-    NS_ADDREF(*aItem);
+    NS_ADDREF(*aItem = mResultHdr);
     mNeedToPrefetch = true;
   }
   return NS_OK;
@@ -776,7 +766,7 @@ nsresult nsMsgThreadEnumerator::Prefetch()
   }
   else if (!mDone)
   {
-    uint32_t numChildren;
+    uint32_t numChildren = 0;
     mThread->GetNumChildren(&numChildren);
 
     while (mChildIndex < (int32_t) numChildren)
@@ -844,7 +834,6 @@ NS_IMETHODIMP nsMsgThreadEnumerator::HasMoreElements(bool *aResult)
 NS_IMETHODIMP nsMsgThread::EnumerateMessages(nsMsgKey parentKey, nsISimpleEnumerator* *result)
 {
   nsMsgThreadEnumerator* e = new nsMsgThreadEnumerator(this, parentKey, nullptr, nullptr);
-  NS_ENSURE_TRUE(e, NS_ERROR_OUT_OF_MEMORY);
   NS_ADDREF(*result = e);
   return NS_OK;
 }
@@ -856,12 +845,12 @@ nsresult nsMsgThread::ReparentMsgsWithInvalidParent(uint32_t numChildren, nsMsgK
   // i.e., a parent that's in the thread!
   for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
   {
-    nsCOMPtr <nsIMsgDBHdr> curChild;
+    nsCOMPtr<nsIMsgDBHdr> curChild;
     rv  = GetChildHdrAt(childIndex, getter_AddRefs(curChild));
     if (NS_SUCCEEDED(rv) && curChild)
     {
       nsMsgKey parentKey;
-      nsCOMPtr <nsIMsgDBHdr> parent;
+      nsCOMPtr<nsIMsgDBHdr> parent;
 
       curChild->GetThreadParent(&parentKey);
 
@@ -877,7 +866,7 @@ nsresult nsMsgThread::ReparentMsgsWithInvalidParent(uint32_t numChildren, nsMsgK
           // can't be your own parent; set parent to thread parent,
           // or make ourselves the root if we are the root.
           if (childKey == parentKey)
-            curChild->SetThreadParent(m_threadRootKey == childKey ? 
+            curChild->SetThreadParent(m_threadRootKey == childKey ?
                                       nsMsgKey_None : m_threadRootKey);
         }
       }
@@ -903,18 +892,20 @@ NS_IMETHODIMP nsMsgThread::GetRootHdr(int32_t *resultIndex, nsIMsgDBHdr **result
       (*result)->GetThreadParent(&parentKey);
       if (parentKey == nsMsgKey_None)
         return rv;
+      // XXX Hack: since GetChildHdrForKey() addref'ed result, we need to
+      // release any unwanted result before continuing.
       NS_RELEASE(*result);
     }
 #ifdef DEBUG_David_Bienvenu
     printf("need to reset thread root key\n");
 #endif
-    uint32_t numChildren;
     nsMsgKey threadParentKey = nsMsgKey_None;
+    uint32_t numChildren = 0;
     GetNumChildren(&numChildren);
 
     for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
     {
-      nsCOMPtr <nsIMsgDBHdr> curChild;
+      nsCOMPtr<nsIMsgDBHdr> curChild;
       rv  = GetChildHdrAt(childIndex, getter_AddRefs(curChild));
       if (NS_SUCCEEDED(rv) && curChild)
       {
@@ -932,9 +923,8 @@ NS_IMETHODIMP nsMsgThread::GetRootHdr(int32_t *resultIndex, nsIMsgDBHdr **result
           SetThreadRootKey(threadParentKey);
           if (resultIndex)
             *resultIndex = childIndex;
-          NS_ADDREF(*result = curChild);
+          curChild.forget(result);
           ReparentMsgsWithInvalidParent(numChildren, threadParentKey);
-          //            return NS_OK;
         }
       }
     }
@@ -965,10 +955,10 @@ nsresult nsMsgThread::ChangeChildCount(int32_t delta)
   uint32_t childCount = 0;
   m_mdbDB->RowCellColumnToUInt32(m_metaRow, m_mdbDB->m_threadChildrenColumnToken, childCount);
 
-  NS_WARN_IF_FALSE(childCount != 0 || delta > 0, "child count gone negative");
+  NS_WARNING_ASSERTION(childCount != 0 || delta > 0, "child count gone negative");
   childCount += delta;
 
-  NS_WARN_IF_FALSE((int32_t) childCount >= 0, "child count gone to 0 or below");
+  NS_WARNING_ASSERTION((int32_t) childCount >= 0, "child count gone to 0 or below");
   if ((int32_t) childCount < 0)	// force child count to >= 0
     childCount = 0;
 
@@ -1004,17 +994,12 @@ nsresult nsMsgThread::SetThreadRootKey(nsMsgKey threadRootKey)
 
 nsresult nsMsgThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **result, int32_t *resultIndex)
 {
-  uint32_t numChildren;
-  uint32_t childIndex = 0;
-  nsresult rv = NS_OK;        // XXX or should this default to an error?
-
   NS_ENSURE_ARG_POINTER(result);
 
+  nsresult rv = NS_OK;        // XXX or should this default to an error?
+  uint32_t numChildren = 0;
   GetNumChildren(&numChildren);
-
-  if ((int32_t) numChildren < 0)
-    numChildren = 0;
-
+  uint32_t childIndex;
   for (childIndex = 0; childIndex < numChildren; childIndex++)
   {
     rv = GetChildHdrAt(childIndex, result);
@@ -1044,7 +1029,7 @@ nsresult nsMsgThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **resul
           {
             // otherwise, let's try to figure out which thread
             // this message really belongs to.
-            nsCOMPtr<nsIMsgThread> threadKeyThread = 
+            nsCOMPtr<nsIMsgThread> threadKeyThread =
                   dont_AddRef(m_mdbDB->GetThreadForThreadId(threadKey));
             if (threadKeyThread)
             {
@@ -1069,6 +1054,8 @@ nsresult nsMsgThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **resul
         }
         break;
       }
+      // XXX Hack: since GetChildHdrAt() addref'ed result, we need to
+      // release any unwanted result before continuing in the loop.
       NS_RELEASE(*result);
     }
   }
@@ -1081,20 +1068,18 @@ nsresult nsMsgThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **resul
 NS_IMETHODIMP nsMsgThread::GetFirstUnreadChild(nsIMsgDBHdr **result)
 {
   NS_ENSURE_ARG_POINTER(result);
-  uint32_t numChildren;
+
   nsresult rv = NS_OK;
   uint8_t minLevel = 0xff;
 
+  uint32_t numChildren = 0;
   GetNumChildren(&numChildren);
 
-  if ((int32_t) numChildren < 0)
-    numChildren = 0;
-
-  nsCOMPtr <nsIMsgDBHdr> retHdr;
+  nsCOMPtr<nsIMsgDBHdr> retHdr;
 
   for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
   {
-    nsCOMPtr <nsIMsgDBHdr> child;
+    nsCOMPtr<nsIMsgDBHdr> child;
     rv = GetChildHdrAt(childIndex, getter_AddRefs(child));
     if (NS_SUCCEEDED(rv) && child)
     {
@@ -1114,7 +1099,7 @@ NS_IMETHODIMP nsMsgThread::GetFirstUnreadChild(nsIMsgDBHdr **result)
         uint8_t level = 0;
         nsMsgKey parentId;
         child->GetThreadParent(&parentId);
-        nsCOMPtr <nsIMsgDBHdr> parent;
+        nsCOMPtr<nsIMsgDBHdr> parent;
         // count number of ancestors - that's our level
         while (parentId != nsMsgKey_None)
         {
@@ -1134,7 +1119,7 @@ NS_IMETHODIMP nsMsgThread::GetFirstUnreadChild(nsIMsgDBHdr **result)
     }
   }
 
-  NS_IF_ADDREF(*result = retHdr);
+  retHdr.forget(result);
   return rv;
 }
 
@@ -1143,17 +1128,12 @@ NS_IMETHODIMP nsMsgThread::GetNewestMsgDate(uint32_t *aResult)
   // if this hasn't been set, figure it out by enumerating the msgs in the thread.
   if (!m_newestMsgDate)
   {
-    uint32_t numChildren;
     nsresult rv;
-
+    uint32_t numChildren;
     GetNumChildren(&numChildren);
-
-    if ((int32_t) numChildren < 0)
-      numChildren = 0;
-
     for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
     {
-      nsCOMPtr <nsIMsgDBHdr> child;
+      nsCOMPtr<nsIMsgDBHdr> child;
       rv = GetChildHdrAt(childIndex, getter_AddRefs(child));
       if (NS_SUCCEEDED(rv))
       {
@@ -1163,13 +1143,11 @@ NS_IMETHODIMP nsMsgThread::GetNewestMsgDate(uint32_t *aResult)
           m_newestMsgDate = msgDate;
       }
     }
-
   }
   *aResult = m_newestMsgDate;
   return NS_OK;
 }
 
-
 NS_IMETHODIMP nsMsgThread::SetNewestMsgDate(uint32_t aNewestMsgDate)
 {
   m_newestMsgDate = aNewestMsgDate;
