@@ -57,7 +57,7 @@ public:
 
     nsCOMPtr<nsISupports> m_srcSupport; // source file spec or folder
     nsCOMPtr<nsIArray> m_messages; // array of source messages
-    nsRefPtr<nsImapMoveCopyMsgTxn> m_undoMsgTxn; // undo object with this copy operation
+    RefPtr<nsImapMoveCopyMsgTxn> m_undoMsgTxn; // undo object with this copy operation
     nsCOMPtr<nsIMsgDBHdr> m_message; // current message to be copied
     nsCOMPtr<nsIMsgCopyServiceListener> m_listener; // listener of this copy
                                                     // operation
@@ -81,7 +81,7 @@ public:
     bool m_allowUndo;
     bool m_eatLF;
     uint32_t m_newMsgFlags; // only used if there's no m_message
-    nsCString m_newMsgKeywords; // ditto 
+    nsCString m_newMsgKeywords; // ditto
     // If the server supports UIDPLUS, this is the UID for the append,
     // if we're doing an append.
     nsMsgKey m_appendUID;
@@ -196,7 +196,7 @@ class nsImapMailFolder :  public nsMsgDBFolder,
                           public nsICopyMessageListener,
                           public nsIMsgFilterHitNotify
 {
- static const uint32_t PLAYBACK_TIMER_INTERVAL_IN_MS = 500; 
+ static const uint32_t PLAYBACK_TIMER_INTERVAL_IN_MS = 500;
 public:
   nsImapMailFolder();
 
@@ -210,7 +210,7 @@ public:
 
   NS_IMETHOD CreateSubfolder(const nsAString& folderName,nsIMsgWindow *msgWindow ) override;
   NS_IMETHOD AddSubfolder(const nsAString& aName, nsIMsgFolder** aChild) override;
-  NS_IMETHODIMP CreateStorageIfMissing(nsIUrlListener* urlListener);
+  NS_IMETHODIMP CreateStorageIfMissing(nsIUrlListener* urlListener) override;
 
   NS_IMETHOD Compact(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow) override;
   NS_IMETHOD CompactAll(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow,
@@ -322,6 +322,8 @@ public:
   */
   NS_IMETHOD GetOfflineMsgFolder(nsMsgKey msgKey, nsIMsgFolder **aMsgFolder) override;
 
+  NS_IMETHOD GetIncomingServerType(nsACString& serverType) override;
+
   nsresult AddSubfolderWithPath(nsAString& name, nsIFile *dbPath, nsIMsgFolder **child, bool brandNew = false);
   nsresult MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
                                   nsIMsgDatabase *sourceDB,
@@ -372,7 +374,7 @@ protected:
    *
    * @param srcFile       file containing the message key
    * @param msgKey        key to use for the new messages
-   */   
+   */
   nsresult CopyFileToOfflineStore(nsIFile *srcFile, nsMsgKey msgKey);
 
   nsresult MarkMessagesImapDeleted(nsTArray<nsMsgKey> *keyArray, bool deleted, nsIMsgDatabase *db);
@@ -395,7 +397,6 @@ protected:
   nsresult AddDirectorySeparator(nsIFile *path);
   nsresult CreateSubFolders(nsIFile *path);
   nsresult GetDatabase() override;
-  virtual void GetIncomingServerType(nsCString& serverType) override { serverType.AssignLiteral("imap");}
 
   nsresult        GetFolderOwnerUserName(nsACString& userName);
   nsIMAPNamespace *GetNamespaceForFolder();
@@ -458,7 +459,7 @@ protected:
   nsCOMPtr<nsIMsgFilterPlugin> m_filterPlugin;  // XXX should be a list
   // used with filter plugins to know when we've finished classifying and can playback moves
   bool m_msgMovedByFilter;
-  nsImapMoveCoalescer *m_moveCoalescer; // strictly owned by the nsImapMailFolder
+  RefPtr<nsImapMoveCoalescer> m_moveCoalescer; // strictly owned by the nsImapMailFolder
   nsCOMPtr<nsIMutableArray> m_junkMessagesToMarkAsRead;
   /// list of keys to be moved to the junk folder
   nsTArray<nsMsgKey> mSpamKeysToMove;
@@ -481,8 +482,8 @@ protected:
   bool m_urlRunning;
 
   // undo move/copy transaction support
-  nsRefPtr<nsMsgTxn> m_pendingUndoTxn;
-  nsRefPtr<nsImapMailCopyState> m_copyState;
+  RefPtr<nsMsgTxn> m_pendingUndoTxn;
+  RefPtr<nsImapMailCopyState> m_copyState;
   char m_hierarchyDelimiter;
   int32_t m_boxFlags;
   nsCString m_onlineFolderName;
@@ -517,7 +518,7 @@ protected:
   bool m_filterListRequiresBody;
 
   // auto-sync (automatic message download) support
-  nsRefPtr<nsAutoSyncState> m_autoSyncStateObj;
+  RefPtr<nsAutoSyncState> m_autoSyncStateObj;
 
   // Quota support
   nsCString m_folderQuotaRoot;
@@ -527,7 +528,7 @@ protected:
   // Pseudo-Offline Playback support
   nsPlaybackRequest *m_pendingPlaybackReq;
   nsCOMPtr<nsITimer> m_playbackTimer;
-  nsTArray<nsRefPtr<nsImapMoveCopyMsgTxn> > m_pendingOfflineMoves;
+  nsTArray<RefPtr<nsImapMoveCopyMsgTxn> > m_pendingOfflineMoves;
   // hash table of mapping between messageids and message keys
   // for pseudo hdrs.
   nsDataHashtable<nsCStringHashKey, nsMsgKey> m_pseudoHdrs;
