@@ -15,7 +15,7 @@
 #include "nsIAsyncOutputStream.h"
 #include "nsIAsyncInputStream.h"
 #include "nsImapCore.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsIProgressEventSink.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
@@ -46,7 +46,6 @@
 #include "nsCOMPtr.h"
 #include "nsIImapIncomingServer.h"
 #include "nsIMsgWindow.h"
-#include "nsICacheListener.h"
 #include "nsIImapHeaderXferInfo.h"
 #include "nsMsgLineBuffer.h"
 #include "nsIAsyncInputStream.h"
@@ -56,6 +55,10 @@
 #include "nsIMsgAsyncPrompter.h"
 #include "mozilla/ReentrantMonitor.h"
 #include "nsSyncRunnableHelpers.h"
+#include "nsICacheEntryOpenCallback.h"
+#include "nsIProtocolProxyCallback.h"
+#include "nsICancelable.h"
+#include "nsImapStringBundle.h"
 
 class nsIMAPMessagePartIDArray;
 class nsIMsgIncomingServer;
@@ -124,18 +127,29 @@ private:
 #define IMAP_ISSUED_LANGUAGE_REQUEST  0x00000020 // make sure we only issue the language request once per connection...
 #define IMAP_ISSUED_COMPRESS_REQUEST  0x00000040 // make sure we only request compression once
 
+// There are 3 types of progress strings for items downloaded from IMAP servers.
+// An index is needed to keep track of the current count of the number of each
+// item type downloaded. The IMAP_EMPTY_STRING_INDEX means no string displayed.
+#define IMAP_NUMBER_OF_PROGRESS_STRINGS 4
+#define IMAP_HEADERS_STRING_INDEX       0
+#define IMAP_FLAGS_STRING_INDEX         1
+#define IMAP_MESSAGES_STRING_INDEX      2
+#define IMAP_EMPTY_STRING_INDEX         3
+
 class nsImapProtocol : public nsIImapProtocol,
                        public nsIRunnable,
                        public nsIInputStreamCallback,
                        public nsSupportsWeakReference,
                        public nsMsgProtocol,
                        public nsIImapProtocolSink,
-                       public nsIMsgAsyncPromptListener
+                       public nsIMsgAsyncPromptListener,
+                       public nsIProtocolProxyCallback
 {
 public:
 
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSIINPUTSTREAMCALLBACK
+  NS_DECL_NSIPROTOCOLPROXYCALLBACK
   nsImapProtocol();
 
   virtual nsresult ProcessProtocolState(nsIURI * url, nsIInputStream * inputStream,
@@ -166,7 +180,7 @@ public:
   void GetShouldDownloadAllHeaders(bool *aResult);
   void GetArbitraryHeadersToDownload(nsCString &aResult);
   virtual void AdjustChunkSize();
-  virtual void FetchMessage(const nsCString &messageIds, 
+  virtual void FetchMessage(const nsCString &messageIds,
     nsIMAPeFetchFields whatToFetch,
     const char *fetchModifier = nullptr,
     uint32_t startByte = 0, uint32_t numBytes = 0,
@@ -242,7 +256,7 @@ public:
   void ProgressEventFunctionUsingName(const char* aMsgId);
   void ProgressEventFunctionUsingNameWithString(const char* aMsgName, const char *
     aExtraInfo);
-  void PercentProgressUpdateEvent(char16_t *message, int64_t currentProgress, int64_t maxProgress);
+  void PercentProgressUpdateEvent(const char16_t *message, int64_t currentProgress, int64_t maxProgress);
   void ShowProgress();
 
   // utility function calls made by the server
@@ -313,10 +327,10 @@ public:
 private:
   virtual ~nsImapProtocol();
   // the following flag is used to determine when a url is currently being run. It is cleared when we
-  // finish processng a url and it is set whenever we call Load on a url
+  // finish processing a url and it is set whenever we call Load on a url
   bool m_urlInProgress;
   nsCOMPtr<nsIImapUrl> m_runningUrl; // the nsIImapURL that is currently running
-  nsImapAction m_imapAction;  // current imap action associated with this connnection...
+  nsImapAction m_imapAction;  // current imap action associated with this connection...
 
   nsCString             m_hostName;
   nsCString             m_userName;
@@ -329,7 +343,7 @@ private:
   uint32_t        m_curReadIndex;  // current read index
   nsCString       m_trashFolderName;
 
-  // Ouput stream for writing commands to the socket
+  // Output stream for writing commands to the socket
   nsCOMPtr<nsISocketTransport>  m_transport;
 
   nsCOMPtr<nsIAsyncInputStream>   m_channelInputStream;
@@ -355,7 +369,7 @@ private:
   // If we get an async password prompt, this is where the UI thread
   // stores the password, before notifying the imap thread of the password
   // via the m_passwordReadyMonitor.
-  nsCString m_password;
+  nsString m_password;
   // Set to the result of nsImapServer::PromptPassword
   nsresult    m_passwordStatus;
 
@@ -367,14 +381,15 @@ private:
   bool        m_nextUrlReadyToRun;
   nsWeakPtr   m_server;
 
-  nsRefPtr<ImapMailFolderSinkProxy> m_imapMailFolderSink;
-  nsRefPtr<ImapMessageSinkProxy>    m_imapMessageSink;
-  nsRefPtr<ImapServerSinkProxy>     m_imapServerSink;
-  nsRefPtr<ImapProtocolSinkProxy>   m_imapProtocolSink;
+  RefPtr<ImapMailFolderSinkProxy> m_imapMailFolderSink;
+  RefPtr<ImapMessageSinkProxy>    m_imapMessageSink;
+  RefPtr<ImapServerSinkProxy>     m_imapServerSink;
+  RefPtr<ImapProtocolSinkProxy>   m_imapProtocolSink;
 
   // helper function to setup imap sink interface proxies
-  void SetupSinkProxy();
+  nsresult SetupSinkProxy();
   // End thread support stuff
+  nsresult LoadImapUrlInternal();
 
   bool GetDeleteIsMoveToTrash();
   bool GetShowDeletedMessages();
@@ -390,7 +405,6 @@ private:
   // biff
   void  PeriodicBiff();
   void  SendSetBiffIndicatorEvent(nsMsgBiffState newState);
-  bool  CheckNewMail();
 
   // folder opening and listing header functions
   void FolderHeaderDump(uint32_t *msgUids, uint32_t msgCount);
@@ -414,6 +428,7 @@ private:
 
   // initialization function given a new url and transport layer
   nsresult  SetupWithUrl(nsIURI * aURL, nsISupports* aConsumer);
+  nsresult  SetupWithUrlCallback(nsIProxyInfo* proxyInfo);
   void ReleaseUrlState(bool rerunningUrl); // release any state that is stored on a per action basis.
   /**
    * Last ditch effort to run the url without using an imap connection.
@@ -432,7 +447,7 @@ private:
   // SendData not only writes the NULL terminated data in dataBuffer to our output stream
   // but it also informs the consumer that the data has been written to the stream.
   // aSuppressLogging --> set to true if you wish to suppress logging for this particular command.
-  // this is useful for making sure we don't log authenication information like the user's password (which was
+  // this is useful for making sure we don't log authentication information like the user's password (which was
   // encoded anyway), but still we shouldn't add that information to the log.
   nsresult SendData(const char * dataBuffer, bool aSuppressLogging = false) override;
 
@@ -446,7 +461,7 @@ private:
 
   // use to prevent re-entering TellThreadToDie.
   bool m_inThreadShouldDie;
-  // if the UI thread has signalled the IMAP thread to die, and the
+  // if the UI thread has signaled the IMAP thread to die, and the
   // connection has timed out, this will be set to FALSE.
   bool m_safeToCloseConnection;
 
@@ -462,7 +477,7 @@ private:
   void StartTLS();
 
   // login related methods.
-  nsresult GetPassword(nsCString &password, bool aNewPasswordRequested);
+  nsresult GetPassword(nsString &password, bool aNewPasswordRequested);
   void InitPrefAuthMethods(int32_t authMethodPrefValue,
                            nsIMsgIncomingServer *aServer);
   nsresult ChooseAuthMethod();
@@ -472,13 +487,13 @@ private:
   // All of these methods actually issue protocol
   void Capability(); // query host for capabilities.
   void ID(); // send RFC 2971 app info to server
-  void EnableCondStore(); 
+  void EnableCondStore();
   void StartCompressDeflate();
   nsresult BeginCompressing();
   void Language(); // set the language on the server if it supports it
   void Namespace();
   void InsecureLogin(const char *userName, const nsCString &password);
-  nsresult AuthLogin(const char *userName, const nsCString &password, eIMAPCapabilityFlag flag);
+  nsresult AuthLogin(const char *userName, const nsString &password, eIMAPCapabilityFlag flag);
   void ProcessAuthenticatedStateURL();
   void ProcessAfterAuthenticated();
   void ProcessSelectedStateURL();
@@ -540,6 +555,7 @@ private:
   // LIST SUBSCRIBED command (from RFC 5258) crashes some servers. so we need to
   // identify those servers
   bool GetListSubscribedIsBrokenOnServer();
+  bool IsExtraSelectNeeded();
   void Lsub(const char *mailboxPattern, bool addDirectoryIfNecessary);
   void List(const char *mailboxPattern, bool addDirectoryIfNecessary,
             bool useXLIST = false);
@@ -562,7 +578,7 @@ private:
   nsresult GetMsgWindow(nsIMsgWindow ** aMsgWindow);
   // End Process AuthenticatedState Url helper methods
 
-  virtual char const *GetType() {return "imap";}
+  virtual char const *GetType() override {return "imap";}
 
   // Quota support
   void GetQuotaDataIfSupported(const char *aBoxName);
@@ -570,11 +586,11 @@ private:
   // CondStore support - true if server supports it, and the user hasn't disabled it.
   bool UseCondStore();
   // false if pref "mail.server.serverxxx.use_condstore" is false;
-  bool m_useCondStore; 
+  bool m_useCondStore;
   // COMPRESS=DEFLATE support - true if server supports it, and the user hasn't disabled it.
   bool UseCompressDeflate();
   // false if pref "mail.server.serverxxx.use_compress_deflate" is false;
-  bool m_useCompressDeflate; 
+  bool m_useCompressDeflate;
   // these come from the nsIDBFolderInfo in the msgDatabase and
   // are initialized in nsImapProtocol::SetupWithUrl.
   uint64_t mFolderLastModSeq;
@@ -603,8 +619,8 @@ private:
   int32_t m_socketType;
   int32_t m_chunkSize;
   int32_t m_chunkThreshold;
-  nsRefPtr <nsMsgImapLineDownloadCache> m_downloadLineCache;
-  nsRefPtr <nsMsgImapHdrXferInfo> m_hdrDownloadCache;
+  RefPtr<nsMsgImapLineDownloadCache> m_downloadLineCache;
+  RefPtr<nsMsgImapHdrXferInfo> m_hdrDownloadCache;
   nsCOMPtr <nsIImapHeaderInfo> m_curHdrInfo;
   // mapping between mailboxes and the corresponding folder flags
   nsDataHashtable<nsCStringHashKey, int32_t> m_standardListMailboxes;
@@ -616,28 +632,21 @@ private:
 
   bool m_fromHeaderSeen;
 
-  // these settings allow clients to override various pieces of the connection info from the url
-  bool m_overRideUrlConnectionInfo;
-
-  nsCString m_logonHost;
-  nsCString m_logonCookie;
-  int16_t m_logonPort;
-  
   nsString mAcceptLanguages;
-  
+
   // progress stuff
-  void SetProgressString(const char* stringName);
-  
-  nsString m_progressString;
+  void SetProgressString(uint32_t aStringIndex);
+
   nsCString     m_progressStringName;
-  int32_t       m_progressIndex;
-  int32_t       m_progressCount;
+  uint32_t      m_stringIndex;
+  int32_t       m_progressCurrentNumber[IMAP_NUMBER_OF_PROGRESS_STRINGS];
+  int32_t       m_progressExpectedNumber;
   nsCString     m_lastProgressStringName;
   int32_t       m_lastPercent;
   int64_t       m_lastProgressTime;
+  nsCOMPtr<nsIStringBundle> m_bundle;
 
   bool m_notifySearchHit;
-  bool m_checkForNewMailDownloadsHeaders;
   bool m_needNoop;
   bool m_idle;
   bool m_useIdle;
@@ -646,6 +655,8 @@ private:
   bool m_closeNeededBeforeSelect;
   bool m_retryUrlOnError;
   bool m_preferPlainText;
+  nsCString m_forceSelectValue;
+  bool m_forceSelect;
 
   int32_t m_uidValidity; // stored uid validity for the selected folder.
 
@@ -672,7 +683,7 @@ private:
 
   nsString m_emptyMimePartString;
 
-  nsRefPtr<mozilla::mailnews::OAuth2ThreadHelper> mOAuth2Support;
+  RefPtr<mozilla::mailnews::OAuth2ThreadHelper> mOAuth2Support;
 };
 
 // This small class is a "mock" channel because it is a mockery of the imap channel's implementation...
@@ -682,10 +693,10 @@ private:
 //
 // Threading concern: This class lives entirely in the UI thread.
 
-class nsICacheEntryDescriptor;
+class nsICacheEntry;
 
 class nsImapMockChannel : public nsIImapMockChannel
-                        , public nsICacheListener
+                        , public nsICacheEntryOpenCallback
                         , public nsITransportEventSink
                         , public nsSupportsWeakReference
 {
@@ -696,11 +707,12 @@ public:
   NS_DECL_NSIIMAPMOCKCHANNEL
   NS_DECL_NSICHANNEL
   NS_DECL_NSIREQUEST
-  NS_DECL_NSICACHELISTENER
+  NS_DECL_NSICACHEENTRYOPENCALLBACK
   NS_DECL_NSITRANSPORTEVENTSINK
 
   nsImapMockChannel();
   static nsresult Create (const nsIID& iid, void **result);
+  nsresult RunOnStopRequestFailure();
 
 protected:
   virtual ~nsImapMockChannel();
@@ -710,7 +722,7 @@ protected:
   nsCOMPtr<nsILoadGroup> m_loadGroup;
   nsCOMPtr<nsILoadInfo> m_loadInfo;
   nsCOMPtr<nsIStreamListener> m_channelListener;
-  nsISupports * m_channelContext; 
+  nsISupports * m_channelContext;
   nsresult m_cancelStatus;
   nsLoadFlags mLoadFlags;
   nsCOMPtr<nsIProgressEventSink> mProgressEventSink;
@@ -718,8 +730,9 @@ protected:
   nsCOMPtr<nsISupports> mOwner;
   nsCOMPtr<nsISupports> mSecurityInfo;
   nsCOMPtr<nsIRequest> mCacheRequest; // the request associated with a read from the cache
-  nsCString m_ContentType;
-  nsWeakPtr   m_protocol;
+  nsCString mContentType;
+  nsCString mCharset;
+  nsWeakPtr mProtocol;
 
   bool mChannelClosed;
   bool mReadingFromCache;
@@ -730,7 +743,7 @@ protected:
   nsresult OpenCacheEntry(); // makes a request to the cache service for a cache entry for a url
   bool ReadFromLocalCache(); // attempts to read the url out of our local (offline) cache....
   nsresult ReadFromImapConnection(); // creates a new imap connection to read the url
-  nsresult ReadFromMemCache(nsICacheEntryDescriptor *entry); // attempts to read the url out of our memory cache
+  nsresult ReadFromMemCache(nsICacheEntry *entry); // attempts to read the url out of our memory cache
   nsresult NotifyStartEndReadFromCache(bool start);
 
   // we end up daisy chaining multiple nsIStreamListeners into the load process.
