@@ -10,7 +10,7 @@
 #include "nsImapUrl.h"
 #include "nsIIMAPHostSessionList.h"
 #include "nsThreadUtils.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "prmem.h"
 #include "plstr.h"
 #include "prprf.h"
@@ -20,7 +20,7 @@
 #include "nsMsgBaseCID.h"
 #include "nsImapUtils.h"
 #include "nsIMAPNamespace.h"
-#include "nsICacheEntryDescriptor.h"
+#include "nsICacheEntry.h"
 #include "nsIMsgFolder.h"
 #include "nsIDocShell.h"
 #include "nsIInterfaceRequestor.h"
@@ -335,7 +335,7 @@ NS_IMETHODIMP nsImapUrl::GetImapPartToFetch(char **result)
 {
   //  here's the old code....
 
-  // unforunately an imap part can have the form: /;section= OR
+  // unfortunately an imap part can have the form: /;section= OR
   // it can have the form ?section=. We need to look for both.
   if (m_listOfMessageIds)
   {
@@ -371,10 +371,7 @@ NS_IMETHODIMP nsImapUrl::GetOnlineSubDirSeparator(char* separator)
       *separator = m_onlineSubDirSeparator;
       return NS_OK;
   }
-  else
-  {
-      return NS_ERROR_NULL_POINTER;
-  }
+  return NS_ERROR_NULL_POINTER;
 }
 
 NS_IMETHODIMP nsImapUrl::GetNumBytesToFetch(int32_t *aNumBytesToFetch)
@@ -865,7 +862,7 @@ NS_IMETHODIMP nsImapUrl::AllocateServerPath(const char * canonicalPath, char onl
   if (!result)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  register unsigned char* dst = (unsigned char *) result;
+  unsigned char* dst = (unsigned char *) result;
   src = sourcePath;
   for (i = 0; i < len; i++)
   {
@@ -887,8 +884,8 @@ NS_IMETHODIMP nsImapUrl::AllocateServerPath(const char * canonicalPath, char onl
 
 /* static */ nsresult nsImapUrl::UnescapeSlashes(char *sourcePath)
 {
-    register char *src = sourcePath;
-    register char *dst = sourcePath;
+    char *src = sourcePath;
+    char *dst = sourcePath;
 
     while (*src)
     {
@@ -1043,7 +1040,7 @@ NS_IMETHODIMP nsImapUrl::SetAllowContentChange(bool allowContentChange)
 NS_IMETHODIMP nsImapUrl::SetContentModified(nsImapContentModifiedType contentModified)
 {
   m_contentModified = contentModified;
-  nsCOMPtr<nsICacheEntryDescriptor>  cacheEntry;
+  nsCOMPtr<nsICacheEntry> cacheEntry;
   nsresult res = GetMemCacheEntry(getter_AddRefs(cacheEntry));
   if (NS_SUCCEEDED(res) && cacheEntry)
   {
@@ -1149,7 +1146,7 @@ nsImapUrl::GetMsgFile(nsIFile** aFile)
 NS_IMETHODIMP nsImapUrl::GetMockChannel(nsIImapMockChannel ** aChannel)
 {
   NS_ENSURE_ARG_POINTER(aChannel);
-  NS_WARN_IF_FALSE(NS_IsMainThread(), "should only access mock channel on ui thread");
+  NS_WARNING_ASSERTION(NS_IsMainThread(), "should only access mock channel on ui thread");
   *aChannel = nullptr;
   nsCOMPtr<nsIImapMockChannel> channel(do_QueryReferent(m_channelWeakPtr));
   channel.swap(*aChannel);
@@ -1158,7 +1155,7 @@ NS_IMETHODIMP nsImapUrl::GetMockChannel(nsIImapMockChannel ** aChannel)
 
 NS_IMETHODIMP nsImapUrl::SetMockChannel(nsIImapMockChannel * aChannel)
 {
-  NS_WARN_IF_FALSE(NS_IsMainThread(), "should only access mock channel on ui thread");
+  NS_WARNING_ASSERTION(NS_IsMainThread(), "should only access mock channel on ui thread");
   m_channelWeakPtr = do_GetWeakReference(aChannel);
   return NS_OK;
 }
@@ -1170,9 +1167,12 @@ NS_IMETHODIMP nsImapUrl::GetAllowContentChange(bool *result)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImapUrl::Clone(nsIURI **_retval)
+NS_IMETHODIMP nsImapUrl::CloneInternal(uint32_t aRefHandlingMode,
+                                       const nsACString& newRef,
+                                       nsIURI** _retval)
 {
-  nsresult rv = nsMsgMailNewsUrl::Clone(_retval);
+  nsresult rv =
+    nsMsgMailNewsUrl::CloneInternal(aRefHandlingMode, newRef, _retval);
   NS_ENSURE_SUCCESS(rv, rv);
   // also clone the mURI member, because GetUri below won't work if
   // mURI isn't set due to escaping issues.
@@ -1182,6 +1182,30 @@ NS_IMETHODIMP nsImapUrl::Clone(nsIURI **_retval)
   return rv;
 }
 
+NS_IMETHODIMP nsImapUrl::GetPrincipalSpec(nsACString& aPrincipalSpec)
+{
+  // URLs look like this:
+  // imap://user@domain@server:port/fetch>UID>folder>nn
+  // We simply strip any query part beginning with ? & or /;
+  // Normalised spec: imap://user@domain@server:port/fetch>UID>folder>nn
+  nsCOMPtr<nsIMsgMailNewsUrl> mailnewsURL;
+  QueryInterface(NS_GET_IID(nsIMsgMailNewsUrl), getter_AddRefs(mailnewsURL));
+
+  nsAutoCString spec;
+  mailnewsURL->GetSpecIgnoringRef(spec);
+
+  // Strip any query part beginning with ? or /;
+  int32_t ind = spec.Find("/;");
+  if (ind != kNotFound)
+    spec.SetLength(ind);
+
+  ind = spec.FindChar('?');
+  if (ind != kNotFound)
+    spec.SetLength(ind);
+
+  aPrincipalSpec.Assign(spec);
+  return NS_OK;
+}
 
 NS_IMETHODIMP nsImapUrl::SetUri(const char * aURI)
 {
@@ -1197,7 +1221,7 @@ NS_IMETHODIMP nsImapUrl::GetUri(char** aURI)
   else
   {
     *aURI = nullptr;
-    uint32_t key = m_listOfMessageIds ? atoi(m_listOfMessageIds) : 0;
+    uint32_t key = m_listOfMessageIds ? strtoul(m_listOfMessageIds, nullptr, 10) : 0;
     nsCString canonicalPath;
     AllocateCanonicalPath(m_sourceCanonicalFolderPathSubString, m_onlineSubDirSeparator, (getter_Copies(canonicalPath)));
     nsCString fullFolderPath("/");
