@@ -26,7 +26,7 @@ load("../../../resources/alertTestUtils.js");
 load("../../../resources/messageGenerator.js");
 
 // Globals
-Components.utils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource:///modules/mailServices.js");
 
 setupIMAPPump();
 
@@ -50,7 +50,7 @@ var tests = [
 
 var gTargetFolder;
 
-function test_createTargetFolder()
+function* test_createTargetFolder()
 {
   gAutoSyncManager.addListener(gAutoSyncListener);
 
@@ -62,7 +62,7 @@ function test_createTargetFolder()
   gTargetFolder.setFlag(Ci.nsMsgFolderFlags.CheckNew);
 }
 
-function test_checkForNewMessages()
+function* test_checkForNewMessages()
 {
   addMessageToFolder(gTargetFolder);
   // This will update the INBOX and STATUS targetFolder. We only care about
@@ -86,23 +86,23 @@ function test_triggerAutoSyncIdle()
 }
 
 // move the message to a diffent folder
-function test_moveMessageToTargetFolder()
+function* test_moveMessageToTargetFolder()
 {
   let observer = gAutoSyncManager.QueryInterface(Ci.nsIObserver);
   observer.observe(null, "mail:appIdle", "back");
   let msgHdr = mailTestUtils.firstMsgHdr(IMAPPump.inbox);
-  do_check_neq(msgHdr, null);
+  do_check_true(msgHdr !== null);
 
   // Now move this message to the target folder.
   let messages = Cc["@mozilla.org/array;1"]
                    .createInstance(Ci.nsIMutableArray);
-  messages.appendElement(msgHdr, false);
+  messages.appendElement(msgHdr);
   MailServices.copy.CopyMessages(IMAPPump.inbox, messages, gTargetFolder, true,
                                  asyncCopyListener, null, false);
   yield false;
 }
 
-function test_waitForTargetUpdate()
+function* test_waitForTargetUpdate()
 {
   // After the copy, now we expect to get notified of the gTargetFolder
   // getting updated, after we simulate going idle.
@@ -188,7 +188,7 @@ var gAutoSyncListener =
     try {
       let queueName = "";
       dump("folder added into Q " + this.qName(queue) + " " + folder.URI + "\n");
-      if (folder instanceof Components.interfaces.nsIMsgFolder &&
+      if (folder instanceof Ci.nsIMsgFolder &&
           queue == nsIAutoSyncMgrListener.PriorityQueue) {
       }
     } catch (e) {
@@ -198,7 +198,7 @@ var gAutoSyncListener =
   onFolderRemovedFromQ : function(queue, folder) {
     try {
       dump("folder removed from Q " + this.qName(queue) + " " + folder.URI + "\n");
-      if (folder instanceof Components.interfaces.nsIMsgFolder &&
+      if (folder instanceof Ci.nsIMsgFolder &&
           queue == nsIAutoSyncMgrListener.PriorityQueue) {
       }
     } catch (e) {
@@ -216,7 +216,7 @@ var gAutoSyncListener =
   onDownloadCompleted : function(folder) {
     try {
       dump("folder download completed" + folder.URI + "\n");
-      if (folder instanceof Components.interfaces.nsIMsgFolder) {
+      if (folder instanceof Ci.nsIMsgFolder) {
         let index = mailTestUtils.non_strict_index_of(this._waitingForUpdateList, folder);
         if (index != -1)
           this._waitingForUpdateList.splice(index, 1);
@@ -232,13 +232,13 @@ var gAutoSyncListener =
   },
 
   onDownloadError : function(folder) {
-    if (folder instanceof Components.interfaces.nsIMsgFolder) {
-      dump("OnDownloadError: " + folder.prettiestName + "\n");
+    if (folder instanceof Ci.nsIMsgFolder) {
+      dump("OnDownloadError: " + folder.prettyName + "\n");
     }
   },
 
   onDiscoveryQProcessed : function (folder, numOfHdrsProcessed, leftToProcess) {
-    dump("onDiscoveryQProcessed: " + folder.prettiestName + "\n");
+    dump("onDiscoveryQProcessed: " + folder.prettyName + "\n");
     let index = mailTestUtils.non_strict_index_of(this._waitingForDiscoveryList, folder);
     if (index != -1)
       this._waitingForDiscoveryList.splice(index, 1);
@@ -275,8 +275,7 @@ function addMessageToFolder(folder)
 
   let msgURI =
     Services.io.newURI("data:text/plain;base64," +
-                       btoa(messages[0].toMessageString()),
-                       null, null);
+                       btoa(messages[0].toMessageString()));
   let imapMailbox =  IMAPPump.daemon.getMailbox(folder.name);
   // We add messages with \Seen flag set so that we won't accidentally
   // trigger the code that updates imap folders that have unread messages moved
