@@ -144,7 +144,7 @@ nsBeckyMail::AppendMailboxDescriptor(nsIFile *aEntry,
   descriptor->SetDepth(aDepth);
 
   mailboxFile->InitWithFile(aEntry);
-  aCollected->AppendElement(descriptor, false);
+  aCollected->AppendElement(descriptor);
 
   return NS_OK;
 }
@@ -389,6 +389,7 @@ private:
 
 ImportMessageRunnable::ImportMessageRunnable(nsIFile *aMessageFile,
                                              nsIMsgFolder *aFolder) :
+  mozilla::Runnable("ImportMessageRunnable"),
   mMessageFile(aMessageFile), mFolder(aFolder)
 {
 }
@@ -482,7 +483,7 @@ ImportMessageRunnable::WriteAttachmentFile(nsIFile *aMailboxFile,
                                   attachmentFile);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  char buffer[4096];
+  char buffer[FILE_IO_BUFFER_SIZE];
   uint32_t readBytes = 0;
   uint32_t writtenBytes = 0;
   rv = aOutputStream->Write(MSG_LINEBREAK, strlen(MSG_LINEBREAK), &writtenBytes);
@@ -614,10 +615,10 @@ nsBeckyMail::ImportMailbox(nsIImportMailboxDescriptor *aSource,
   nsAutoString successMessage;
   const char16_t *format = { name.get() };
   rv =
-      nsBeckyStringBundle::FormatStringFromName(MOZ_UTF16("BeckyImportMailboxSuccess"),
+      nsBeckyStringBundle::FormatStringFromName("BeckyImportMailboxSuccess",
                                                 &format,
                                                 1,
-                                                getter_Copies(successMessage));
+                                                successMessage);
   successMessage.AppendLiteral("\n");
   *aSuccessLog = ToNewUnicode(successMessage);
 
