@@ -13,7 +13,7 @@
 
 #include "nscore.h"
 #include <time.h>
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsMsgUtils.h"
 #include "nsMimeTypes.h"
@@ -560,11 +560,6 @@ const char* CpToCharset(unsigned int cp)
   return 0; // not found
 }
 
-// We don't use nsMsgI18Ncheck_data_in_charset_range because it returns true
-// even if there's no such charset:
-// 1. result initialized by true and returned if, eg, GetUnicodeEncoderRaw fail
-// 2. it uses GetUnicodeEncoderRaw(), not GetUnicodeEncoder() (to normalize the
-//    charset string) (see nsMsgI18N.cpp)
 // This function returns true only if the unicode (utf-16) text can be
 // losslessly represented in specified charset
 bool CMapiMessage::CheckBodyInCharsetRange(const char* charset)
@@ -573,42 +568,33 @@ bool CMapiMessage::CheckBodyInCharsetRange(const char* charset)
     return true;
   if (!_stricmp(charset, "utf-8"))
     return true;
-  if (!_stricmp(charset, "utf-7"))
-    return true;
 
-  nsresult rv;
-  static nsCOMPtr<nsICharsetConverterManager> ccm =
-    do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, false);
-  nsCOMPtr<nsIUnicodeEncoder> encoder;
+  auto encoding = mozilla::Encoding::ForLabelNoReplacement(nsDependentCString(charset));
+  if (!encoding)
+    return false;
+  auto encoder = encoding->NewEncoder();
 
-  // get an unicode converter
-  rv = ccm->GetUnicodeEncoder(charset, getter_AddRefs(encoder));
-  NS_ENSURE_SUCCESS(rv, false);
-  rv = encoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Signal, nullptr, 0);
-  NS_ENSURE_SUCCESS(rv, false);
-
-  const char16_t *txt = m_body.get();
-  int32_t txtLen = m_body.Length();
-  const char16_t *currentSrcPtr = txt;
-  int srcLength;
-  int dstLength;
-  char localbuf[512];
-  int consumedLen = 0;
-
-  // convert
-  while (consumedLen < txtLen) {
-    srcLength = txtLen - consumedLen;
-    dstLength = sizeof(localbuf)/sizeof(localbuf[0]);
-    rv = encoder->Convert(currentSrcPtr, &srcLength, localbuf, &dstLength);
-    if (rv == NS_ERROR_UENC_NOMAPPING)
-      return false;
-    if (NS_FAILED(rv) || dstLength == 0)
+  uint8_t buffer[512];
+  auto src = mozilla::MakeSpan(m_body);
+  auto dst = mozilla::MakeSpan(buffer);
+  while (true) {
+    uint32_t result;
+    size_t read;
+    size_t written;
+    mozilla::Tie(result, read, written) =
+      encoder->EncodeFromUTF16WithoutReplacement(src, dst, false);
+    if (result == mozilla::kInputEmpty) {
+      // All converted successfully.
       break;
-
-    currentSrcPtr += srcLength;
-    consumedLen = currentSrcPtr - txt; // src length used so far
+    } else if (result != mozilla::kOutputFull) {
+      // Didn't use all the input but the outout isn't full, hence
+      // there was an unencodable character.
+      return false;
+    }
+    src = src.From(read);
+    // dst = dst.From(written); // Just overwrite output since we don't need it.
   }
+
   return true;
 }
 
@@ -638,7 +624,7 @@ void ExtractMetaCharset(const wchar_t* body, int bodySz, /*out*/nsCString& chars
 
   // remove everything from the string after the next ; or " or space,
   // whichever comes first.
-  // The inital sting looks something like
+  // The initial string looks something like
   // <META content="text/html; charset=utf-8" http-equiv=Content-Type>
   // <META content="text/html; charset=utf-8;" http-equiv=Content-Type>
   // <META content="text/html; charset=utf-8 ;" http-equiv=Content-Type>
@@ -647,8 +633,8 @@ void ExtractMetaCharset(const wchar_t* body, int bodySz, /*out*/nsCString& chars
   const wchar_t* chset_end = std::find_first_of(chset_pos, eohd_pos, term,
                                                 term_end);
   if (chset_end != eohd_pos)
-    LossyCopyUTF16toASCII(Substring(wwc(const_cast<wchar_t *>(chset_pos)),
-                                    wwc(const_cast<wchar_t *>(chset_end))),
+    LossyCopyUTF16toASCII(Substring(char16ptr_t(chset_pos),
+                                    char16ptr_t(chset_end)),
                                     charset);
 }
 
@@ -851,7 +837,6 @@ bool CMapiMessage::GetTmpFile(/*out*/ nsIFile **aResult)
 
 bool CMapiMessage::CopyMsgAttachToFile(LPATTACH lpAttach, /*out*/ nsIFile **tmp_file)
 {
-  bool bResult = true;
   LPMESSAGE  lpMsg;
   HRESULT hr = lpAttach->OpenProperty(PR_ATTACH_DATA_OBJ, &IID_IMessage, 0, 0,
                                       reinterpret_cast<LPUNKNOWN *>(&lpMsg));
@@ -1093,7 +1078,7 @@ bool CMapiMessage::AddAttachment(DWORD aNum)
       // reference to an embedded object is not in the main body, but in another
       // embedded object - like body references a CSS attachment that in turn references
       // a picture as a background of its element). From the other hand, it's unreliable
-      // to relax the search criteria to any occurence of the Content-Id string in the body -
+      // to relax the search criteria to any occurrence of the Content-Id string in the body -
       // partly because the string may be simply in a text or other non-referencing part,
       // partly because of the abovementioned possibility that the reference is outside
       // the body at all.
@@ -1195,7 +1180,7 @@ nsresult CMapiMessage::GetAttachments(nsIArray **aArray)
     a->SetEncoding(nsDependentCString((*it)->encoding));
     a->SetRealName(nsDependentCString((*it)->real_name));
     a->SetType(nsDependentCString((*it)->type));
-    attachments->AppendElement(a, false);
+    attachments->AppendElement(a);
   }
   return rv;
 }
@@ -1313,7 +1298,7 @@ CMapiMessageHeaders::~CMapiMessageHeaders()
   ClearHeaderFields();
 }
 
-void Delete(void* p) { delete p; }
+void CMapiMessageHeaders::Delete(CHeaderField* p) { delete p; }
 
 void CMapiMessageHeaders::ClearHeaderFields()
 {
