@@ -6,7 +6,7 @@
 
 #include "nscore.h"
 #include "prthread.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsMsgUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsCOMPtr.h"
@@ -33,7 +33,7 @@
 #include "nsCRT.h"
 #include "nsOutlookCompose.h"
 
-#include "OutlookDebugLog.h"
+#include "ImportDebug.h"
 
 #include "nsMimeTypes.h"
 #include "nsMsgUtils.h"
@@ -45,7 +45,6 @@
 
 #include <algorithm>
 
-static NS_DEFINE_CID(kMsgSendCID, NS_MSGSEND_CID);
 static NS_DEFINE_CID(kMsgCompFieldsCID, NS_MSGCOMPFIELDS_CID);
 
 // We need to do some calculations to set these numbers to something reasonable!
@@ -310,7 +309,7 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode, CMapiMessage
         nsCOMPtr<nsIMsgEmbeddedImageData> imageData =
           new nsImportEmbeddedImageData(uri, nsDependentCString(cid),
                                      nsDependentCString(name));
-        embeddedObjects->AppendElement(imageData, false);
+        embeddedObjects->AppendElement(imageData);
       }
     }
   }
@@ -319,7 +318,6 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode, CMapiMessage
   const char *charset = msg.GetBodyCharset();
   nsMsgI18NConvertFromUnicode(charset ? nsDependentCString(charset) : EmptyCString(), bodyW, bodyA);
 
-
   nsCOMPtr<nsIImportService> impService(do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -560,9 +558,7 @@ void nsOutlookCompose::UpdateHeaders(CMapiMessageHeaders& oldHeaders, const CMap
 
 void nsOutlookCompose::HackBody(const wchar_t* orig, size_t origLen, nsString& hack)
 {
-#ifdef MOZILLA_INTERNAL_API
   hack.SetCapacity(static_cast<size_t>(origLen*1.4));
-#endif
   hack.Assign(hackBeginW);
   hack.Append(m_hackedPostfix);
 
@@ -724,11 +720,9 @@ nsresult CCompositionFile::ToDest(_OutFn dest, const char* term, int termSize)
 {
   CTermGuard guard(term, termSize);
 
-#ifdef MOZILLA_INTERNAL_API
   // We already know the required string size, so reduce future reallocations
   if (!guard.IsChecking() && !m_convertCRs)
     dest.SetCapacity(m_fileSize - m_fileReadPos);
-#endif
 
   bool wasCR = false;
   char c = 0;
@@ -781,9 +775,7 @@ nsresult CCompositionFile::ToDest(_OutFn dest, const char* term, int termSize)
 class dest_nsCString {
 public:
   dest_nsCString(nsCString& str) : m_str(str) { m_str.Truncate(); }
-#ifdef MOZILLA_INTERNAL_API
   void SetCapacity(int32_t sz) { m_str.SetCapacity(sz); }
-#endif
   nsresult Append(const char* buf, uint32_t count) {
     m_str.Append(buf, count); return NS_OK; }
 private:
@@ -793,9 +785,7 @@ private:
 class dest_Stream {
 public:
   dest_Stream(nsIOutputStream *dest) : m_stream(dest) {}
-#ifdef MOZILLA_INTERNAL_API
   void SetCapacity(int32_t) { /*do nothing*/ }
-#endif
   // const_cast here is due to the poor design of the EscapeFromSpaceLine()
   // that requires a non-constant pointer while doesn't modify its data
   nsresult Append(const char* buf, uint32_t count) {
