@@ -19,7 +19,7 @@
 #include "nsISupportsPrimitives.h"
 #include "nsIImportMailboxDescriptor.h"
 
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsUnicharUtils.h"
 
 #include "nsMsgUtils.h"
@@ -55,21 +55,21 @@ public:
   NS_DECL_THREADSAFE_ISUPPORTS
 
   /* nsISupports GetData (in string dataId); */
-  NS_IMETHOD GetData(const char *dataId, nsISupports **_retval);
+  NS_IMETHOD GetData(const char *dataId, nsISupports **_retval) override;
 
-  NS_IMETHOD SetData(const char *dataId, nsISupports *pData);
+  NS_IMETHOD SetData(const char *dataId, nsISupports *pData) override;
 
-  NS_IMETHOD GetStatus(const char *statusKind, int32_t *_retval);
+  NS_IMETHOD GetStatus(const char *statusKind, int32_t *_retval) override;
 
-  NS_IMETHOD WantsProgress(bool *_retval);
+  NS_IMETHOD WantsProgress(bool *_retval) override;
 
-  NS_IMETHODIMP BeginImport(nsISupportsString *successLog, nsISupportsString *errorLog, bool *_retval) ;
+  NS_IMETHODIMP BeginImport(nsISupportsString *successLog, nsISupportsString *errorLog, bool *_retval) override;
 
-  NS_IMETHOD ContinueImport(bool *_retval);
+  NS_IMETHOD ContinueImport(bool *_retval) override;
 
-  NS_IMETHOD GetProgress(int32_t *_retval);
+  NS_IMETHOD GetProgress(int32_t *_retval) override;
 
-  NS_IMETHOD CancelImport(void);
+  NS_IMETHOD CancelImport(void) override;
 
 private:
   virtual ~nsImportGenericMail();
@@ -144,9 +144,9 @@ nsresult ProxyForceDBClosed(nsIMsgFolder *aFolder);
 
 nsresult NS_NewGenericMail(nsIImportGeneric** aImportGeneric)
 {
-    NS_PRECONDITION(aImportGeneric != nullptr, "null ptr");
-    if (! aImportGeneric)
-        return NS_ERROR_NULL_POINTER;
+    NS_ASSERTION(aImportGeneric != nullptr, "null ptr");
+    if (!aImportGeneric)
+      return NS_ERROR_NULL_POINTER;
 
   nsImportGenericMail *pGen = new nsImportGenericMail();
 
@@ -178,10 +178,6 @@ nsImportGenericMail::nsImportGenericMail()
   m_createdFolder = false;
     m_performingMigration = false;
 
-  // Init logging module.
-  if (!IMPORTLOGMODULE)
-    IMPORTLOGMODULE = PR_NewLogModule("IMPORT");
-
   nsresult rv = nsImportStringBundle::GetStringBundle(IMPORT_MSGS_URL, getter_AddRefs(m_stringBundle));
   if (NS_FAILED(rv))
     IMPORT_LOG0("Failed to get string bundle for Importing Mail");
@@ -211,7 +207,7 @@ NS_IMETHODIMP nsImportGenericMail::GetData(const char *dataId, nsISupports **_re
 {
   nsresult rv = NS_OK;
 
-  NS_PRECONDITION(_retval != nullptr, "null ptr");
+  NS_ASSERTION(_retval != nullptr, "null ptr");
   if (!_retval)
     return NS_ERROR_NULL_POINTER;
 
@@ -265,7 +261,7 @@ NS_IMETHODIMP nsImportGenericMail::GetData(const char *dataId, nsISupports **_re
 NS_IMETHODIMP nsImportGenericMail::SetData(const char *dataId, nsISupports *item)
 {
   nsresult rv = NS_OK;
-  NS_PRECONDITION(dataId != nullptr, "null ptr");
+  NS_ASSERTION(dataId != nullptr, "null ptr");
   if (!dataId)
     return NS_ERROR_NULL_POINTER;
 
@@ -318,8 +314,8 @@ NS_IMETHODIMP nsImportGenericMail::SetData(const char *dataId, nsISupports *item
 
 NS_IMETHODIMP nsImportGenericMail::GetStatus(const char *statusKind, int32_t *_retval)
 {
-  NS_PRECONDITION(statusKind != nullptr, "null ptr");
-  NS_PRECONDITION(_retval != nullptr, "null ptr");
+  NS_ASSERTION(statusKind != nullptr, "null ptr");
+  NS_ASSERTION(_retval != nullptr, "null ptr");
   if (!statusKind || !_retval)
     return NS_ERROR_NULL_POINTER;
 
@@ -384,7 +380,7 @@ void nsImportGenericMail::GetDefaultDestination(void)
 
 NS_IMETHODIMP nsImportGenericMail::WantsProgress(bool *_retval)
 {
-  NS_PRECONDITION(_retval != nullptr, "null ptr");
+  NS_ASSERTION(_retval != nullptr, "null ptr");
   NS_ENSURE_ARG_POINTER(_retval);
 
   if (m_pThreadData) {
@@ -452,9 +448,9 @@ void nsImportGenericMail::GetMailboxName(uint32_t index, nsISupportsString *pStr
 
 NS_IMETHODIMP nsImportGenericMail::BeginImport(nsISupportsString *successLog, nsISupportsString *errorLog, bool *_retval)
 {
-  NS_PRECONDITION(_retval != nullptr, "null ptr");
-    if (!_retval)
-        return NS_ERROR_NULL_POINTER;
+  NS_ASSERTION(_retval != nullptr, "null ptr");
+  if (!_retval)
+    return NS_ERROR_NULL_POINTER;
 
   nsString  success;
   nsString  error;
@@ -541,9 +537,9 @@ NS_IMETHODIMP nsImportGenericMail::BeginImport(nsISupportsString *successLog, ns
 
 NS_IMETHODIMP nsImportGenericMail::ContinueImport(bool *_retval)
 {
-    NS_PRECONDITION(_retval != nullptr, "null ptr");
-    if (!_retval)
-        return NS_ERROR_NULL_POINTER;
+  NS_ASSERTION(_retval != nullptr, "null ptr");
+  if (!_retval)
+    return NS_ERROR_NULL_POINTER;
 
   *_retval = true;
   if (m_pThreadData) {
@@ -559,9 +555,9 @@ NS_IMETHODIMP nsImportGenericMail::GetProgress(int32_t *_retval)
 {
   // This returns the progress from the the currently
   // running import mail or import address book thread.
-    NS_PRECONDITION(_retval != nullptr, "null ptr");
-    if (!_retval)
-        return NS_ERROR_NULL_POINTER;
+  NS_ASSERTION(_retval != nullptr, "null ptr");
+  if (!_retval)
+    return NS_ERROR_NULL_POINTER;
 
   if (!m_pThreadData || !(m_pThreadData->threadAlive)) {
     *_retval = 100;
@@ -604,9 +600,9 @@ void nsImportGenericMail::ReportError(int32_t id, const char16_t *pName, nsStrin
 
   // load the error string
   char16_t *pFmt = nsImportStringBundle::GetStringByID(id, aBundle);
-  char16_t *pText = nsTextFormatter::smprintf(pFmt, pName);
+  nsString pText;
+  nsTextFormatter::ssprintf(pText, pFmt, pName);
   pStream->Append(pText);
-  nsTextFormatter::smprintf_free(pText);
   NS_Free(pFmt);
   pStream->Append(NS_ConvertASCIItoUTF16(MSG_LINEBREAK));
 }
@@ -771,7 +767,7 @@ ImportMailThread(void *stuff)
         while ((newDepth < depth) && NS_SUCCEEDED(rv)) {
           rv = curFolder->GetParent(getter_AddRefs(curFolder));
           if (NS_FAILED(rv)) {
-            IMPORT_LOG1("*** ImportMailThread: Failed to get the interface for parent folder '%s'.", lastName.get());
+            IMPORT_LOG1("*** ImportMailThread: Failed to get the interface for parent folder '%s'.", NS_ConvertUTF16toUTF8(lastName).get());
             nsImportGenericMail::ReportError(IMPORT_ERROR_MB_FINDCHILD,
                                              lastName.get(), &error,
                                              pData->stringBundle);
@@ -781,7 +777,7 @@ ImportMailThread(void *stuff)
           depth--;
         }
         if (NS_FAILED(rv)) {
-          IMPORT_LOG1("*** ImportMailThread: Failed to get the proxy interface for parent folder '%s'.", lastName.get());
+          IMPORT_LOG1("*** ImportMailThread: Failed to get the proxy interface for parent folder '%s'.", NS_ConvertUTF16toUTF8(lastName).get());
           nsImportStringBundle::GetStringByID(IMPORT_ERROR_MB_NOPROXY,
                                               pData->stringBundle, error);
           pData->fatalError = true;
@@ -822,7 +818,7 @@ ImportMailThread(void *stuff)
 
       rv = ProxyGetChildNamed(curFolder, lastName, getter_AddRefs(newFolder));
       if (NS_FAILED(rv)) {
-        IMPORT_LOG1("*** ImportMailThread: Failed to locate subfolder '%s' after it's been created.", lastName.get());
+        IMPORT_LOG1("*** ImportMailThread: Failed to locate subfolder '%s' after it's been created.", NS_ConvertUTF16toUTF8(lastName).get());
         nsImportGenericMail::ReportError(IMPORT_ERROR_MB_CREATE, lastName.get(),
                                          &error, pData->stringBundle);
       }
@@ -844,7 +840,7 @@ ImportMailThread(void *stuff)
 
         pData->currentSize = 0;
         pData->currentTotal += size;
-        
+
         // commit to the db synchronously, but using a proxy since it doesn't like being used
         // elsewhere than from the main thread.
         // OK, we've copied the actual folder/file over if the folder size is not 0
@@ -865,7 +861,7 @@ ImportMailThread(void *stuff)
 
   // Now save the new acct info to pref file.
   nsCOMPtr <nsIMsgAccountManager> accMgr = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
-        if (NS_SUCCEEDED(rv) && accMgr) {
+  if (NS_SUCCEEDED(rv) && accMgr) {
     rv = accMgr->SaveAccountInfo();
     NS_ASSERTION(NS_SUCCEEDED(rv), "Can't save account info to pref file");
   }
@@ -886,7 +882,6 @@ ImportMailThread(void *stuff)
   IMPORT_LOG1("Import mailbox thread done: %d\n", (int) pData->currentTotal);
 
   pData->ThreadDelete();
-
 }
 
 // Creates a folder in Local Folders with the module name + mail
@@ -907,13 +902,12 @@ bool nsImportGenericMail::CreateFolder(nsIMsgFolder **ppFolder)
   nsString folderName;
   if (!m_pName.IsEmpty()) {
     const char16_t *moduleName[] = { m_pName.get() };
-    rv = bundle->FormatStringFromName(MOZ_UTF16("ImportModuleFolderName"),
+    rv = bundle->FormatStringFromName("ImportModuleFolderName",
                                       moduleName, 1,
-                                      getter_Copies(folderName));
+                                      folderName);
   }
   else {
-    rv = bundle->GetStringFromName(MOZ_UTF16("DefaultFolderName"),
-                                   getter_Copies(folderName));
+    rv = bundle->GetStringFromName("DefaultFolderName", folderName);
   }
   if (NS_FAILED(rv)) {
       IMPORT_LOG0("*** Failed to get Folder Name!\n");
@@ -1000,7 +994,7 @@ private:
 };
 
 GetSubFoldersRunnable::GetSubFoldersRunnable(nsIMsgFolder *aFolder) :
-  m_folder(aFolder)
+  mozilla::Runnable("GetSubFoldersRunnable"), m_folder(aFolder)
 {
 }
 
@@ -1032,6 +1026,7 @@ protected:
 GetChildNamedRunnable::GetChildNamedRunnable(nsIMsgFolder *aFolder,
                                              const nsAString & aName,
                                              nsIMsgFolder **aChild) :
+  mozilla::Runnable("GetChildNamedRunnable"),
   m_folder(aFolder), m_name(aName), m_child(aChild)
 {
 }
@@ -1061,7 +1056,7 @@ protected:
 };
 
 GetParentRunnable::GetParentRunnable(nsIMsgFolder *aFolder, nsIMsgFolder **aParent) :
-  m_folder(aFolder), m_parent(aParent)
+  mozilla::Runnable("GetParentRunnable"), m_folder(aFolder), m_parent(aParent)
 {
 }
 
@@ -1092,6 +1087,7 @@ protected:
 ContainsChildNamedRunnable::ContainsChildNamedRunnable(nsIMsgFolder *aFolder,
                                                        const nsAString &aName,
                                                        bool *aResult) :
+  mozilla::Runnable("ContainsChildNamedRunnable"),
   m_folder(aFolder), m_name(aName), m_result(aResult)
 {
 }
@@ -1129,7 +1125,8 @@ protected:
 GenerateUniqueSubfolderNameRunnable::GenerateUniqueSubfolderNameRunnable(
   nsIMsgFolder *aFolder, const nsAString& aPrefix, nsIMsgFolder *aOtherFolder,
   nsAString& aName)
-  : m_folder(aFolder), m_prefix(aPrefix), m_otherFolder(aOtherFolder), m_name(aName)
+  : mozilla::Runnable("GenerateUniqueSubfolderNameRunnable")
+  , m_folder(aFolder), m_prefix(aPrefix), m_otherFolder(aOtherFolder), m_name(aName)
 {
 }
 
@@ -1162,7 +1159,7 @@ protected:
 
 CreateSubfolderRunnable::CreateSubfolderRunnable(nsIMsgFolder *aFolder,
                                                  const nsAString &aName) :
-  m_folder(aFolder), m_name(aName)
+  mozilla::Runnable("CreateSubfolderRunnable"), m_folder(aFolder), m_name(aName)
 {
 }
 
@@ -1189,7 +1186,7 @@ protected:
 };
 
 ForceDBClosedRunnable::ForceDBClosedRunnable(nsIMsgFolder *aFolder) :
-  m_folder(aFolder)
+  mozilla::Runnable("ForceDBClosedRunnable"), m_folder(aFolder)
 {
 }
 
