@@ -18,17 +18,11 @@
 #include "nsIUnicharLineInputStream.h"
 #include "nsMsgUtils.h"
 
-#include "TextDebugLog.h"
+#include "ImportDebug.h"
 #include "plstr.h"
 #include "msgCore.h"
 #include <algorithm>
 
-#ifndef MOZILLA_INTERNAL_API
-#include "nsMsgI18N.h"
-#define NS_CopyNativeToUnicode(source, dest) \
-        nsMsgI18NConvertToUnicode(nsMsgI18NFileSystemCharset(), source, dest)
-#endif
-
 #define kWhitespace    " \t\b\r\n"
 
 nsTextAddress::nsTextAddress()
@@ -122,7 +116,7 @@ nsresult nsTextAddress::ImportAddresses(bool *pAbort, const char16_t *pName, nsI
     // Read the line in
     rv = ReadRecord(lineStream, line, &more);
     if (NS_SUCCEEDED(rv)) {
-      // Now proces it to add it to the database
+      // Now process it to add it to the database
       rv = ProcessLine(line, errors);
 
       if (NS_FAILED(rv)) {
