@@ -26,7 +26,7 @@
 #include "prprf.h"
 #include "nsIMsgHdr.h"
 #include "nsIFileURL.h"
-#include "mozilla/nsRefPtr.h"
+#include "mozilla/RefPtr.h"
 
 nsMailboxService::nsMailboxService()
 {
@@ -331,7 +331,6 @@ NS_IMETHODIMP nsMailboxService::StreamHeaders(const char *aMessageURI,
 
 NS_IMETHODIMP nsMailboxService::IsMsgInMemCache(nsIURI *aUrl,
                                                 nsIMsgFolder *aFolder,
-                                                nsICacheEntryDescriptor **aCacheEntry,
                                                 bool *aResult)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
@@ -351,8 +350,8 @@ NS_IMETHODIMP nsMailboxService::OpenAttachment(const char *aContentType,
   urlString += aContentType;
   urlString += "&filename=";
   urlString += aFileName;
-  CreateStartupUrl(urlString.get(), getter_AddRefs(URL));
-  nsresult rv;
+  nsresult rv = NS_NewURI(getter_AddRefs(URL), urlString);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // try to run the url in the docshell...
   nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(aDisplayConsumer, &rv));
@@ -584,7 +583,8 @@ NS_IMETHODIMP nsMailboxService::NewChannel2(nsIURI *aURI,
   NS_ENSURE_ARG_POINTER(_retval);
   nsresult rv = NS_OK;
   nsAutoCString spec;
-  aURI->GetSpec(spec);
+  rv = aURI->GetSpec(spec);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   if (spec.Find("?uidl=") >= 0 || spec.Find("&uidl=") >= 0)
   {
@@ -600,7 +600,7 @@ NS_IMETHODIMP nsMailboxService::NewChannel2(nsIURI *aURI,
     }
   }
 
-  nsRefPtr<nsMailboxProtocol> protocol = new nsMailboxProtocol(aURI);
+  RefPtr<nsMailboxProtocol> protocol = new nsMailboxProtocol(aURI);
   if (!protocol) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
