@@ -294,8 +294,8 @@ void nsMsgMailboxParser::UpdateStatusText (const char* stringName)
       return;
     nsString finalString;
     const char16_t * stringArray[] = { m_folderName.get() };
-    rv = bundle->FormatStringFromName(NS_ConvertASCIItoUTF16(stringName).get(),
-                                      stringArray, 1, getter_Copies(finalString));
+    rv = bundle->FormatStringFromName(stringName,
+                                      stringArray, 1, finalString);
     m_statusFeedback->ShowStatusString(finalString);
   }
 }
@@ -1058,11 +1058,7 @@ nsresult nsParseMailMessageState::ParseHeaders ()
     }
     if (!header && m_customDBHeaders.Length())
     {
-#ifdef MOZILLA_INTERNAL_API
       nsDependentCSubstring headerStr(buf, end);
-#else
-      nsDependentCSubstring headerStr(buf, end - buf);
-#endif
 
       ToLowerCase(headerStr);
       size_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
@@ -1360,8 +1356,10 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
     /* Take off <> around message ID. */
     if (id)
     {
-      if (id->length > 0 && id->value[0] == '<')
-        id->length--, id->value++;
+      if (id->length > 0 && id->value[0] == '<') {
+        id->length--;
+        id->value++;
+      }
 
       NS_WARNING_ASSERTION(id->length > 0, "id->length failure in FinalizeHeaders().");
 
@@ -1979,9 +1977,8 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
   bool msgIsNew = true;
   for (uint32_t actionIndex = 0; actionIndex < numActions && *applyMore; actionIndex++)
   {
-    nsCOMPtr<nsIMsgRuleAction> filterAction;
-    rv = filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction),
-                                                       getter_AddRefs(filterAction));
+    nsCOMPtr<nsIMsgRuleAction> filterAction =
+      do_QueryElementAt(filterActionList, actionIndex, &rv);
     if (NS_FAILED(rv) || !filterAction)
       continue;
 
@@ -2080,7 +2077,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
           if (!actionTargetFolderUri.IsEmpty() && !actionTargetFolderUri.Equals(uri))
           {
             nsCOMPtr<nsIMutableArray> messageArray(do_CreateInstance(NS_ARRAY_CONTRACTID));
-            messageArray->AppendElement(msgHdr, false);
+            messageArray->AppendElement(msgHdr);
 
             nsCOMPtr<nsIMsgFolder> dstFolder;
             nsCOMPtr<nsIMsgCopyService> copyService;
@@ -2128,7 +2125,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
       case nsMsgFilterAction::MarkFlagged:
         {
           nsCOMPtr<nsIMutableArray> messageArray(do_CreateInstance(NS_ARRAY_CONTRACTID));
-          messageArray->AppendElement(msgHdr, false);
+          messageArray->AppendElement(msgHdr);
           m_downloadFolder->MarkMessagesFlagged(messageArray, true);
         }
         break;
@@ -2142,7 +2139,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
         nsCString keyword;
         filterAction->GetStrValue(keyword);
         nsCOMPtr<nsIMutableArray> messageArray(do_CreateInstance(NS_ARRAY_CONTRACTID));
-        messageArray->AppendElement(msgHdr, false);
+        messageArray->AppendElement(msgHdr);
         m_downloadFolder->AddKeywordsToMessages(messageArray, keyword);
         break;
       }
@@ -2196,7 +2193,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
           {
             nsCOMPtr<nsIMutableArray> messages = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
             NS_ENSURE_SUCCESS(rv, rv);
-            messages->AppendElement(msgHdr, false);
+            messages->AppendElement(msgHdr);
             // This action ignores the deleteMailLeftOnServer preference
             localFolder->MarkMsgsOnPop3Server(messages, POP3_FORCE_DEL);
 
@@ -2221,7 +2218,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
           {
             nsCOMPtr<nsIMutableArray> messages = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
             NS_ENSURE_SUCCESS(rv, rv);
-            messages->AppendElement(msgHdr, false);
+            messages->AppendElement(msgHdr);
             localFolder->MarkMsgsOnPop3Server(messages, POP3_FETCH_BODY);
             // Don't add this header to the DB, we're going to replace it
             // with the full message.
@@ -2254,7 +2251,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
             do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
         NS_ENSURE_TRUE(messageArray, rv);
         if (NS_SUCCEEDED(rv))
-          rv = messageArray->AppendElement(msgHdr, false);
+          rv = messageArray->AppendElement(msgHdr);
 
 
         if (NS_SUCCEEDED(rv))
@@ -2351,7 +2348,7 @@ nsresult nsParseNewMailState::ApplyForwardAndReplyFilter(nsIMsgWindow *msgWindow
 void nsParseNewMailState::MarkFilteredMessageRead(nsIMsgDBHdr *msgHdr)
 {
   nsCOMPtr<nsIMutableArray> messageArray(do_CreateInstance(NS_ARRAY_CONTRACTID));
-  messageArray->AppendElement(msgHdr, false);
+  messageArray->AppendElement(msgHdr);
   m_downloadFolder->MarkMessagesRead(messageArray, true);
 }
 
@@ -2369,7 +2366,7 @@ void nsParseNewMailState::MarkFilteredMessageUnread(nsIMsgDBHdr *msgHdr)
     msgHdr->OrFlags(nsMsgMessageFlags::New, &newFlags);
   }
   nsCOMPtr<nsIMutableArray> messageArray(do_CreateInstance(NS_ARRAY_CONTRACTID));
-  messageArray->AppendElement(msgHdr, false);
+  messageArray->AppendElement(msgHdr);
   m_downloadFolder->MarkMessagesRead(messageArray, false);
 }
 
