@@ -466,11 +466,11 @@ void nsPop3Service::AlertServerBusy(nsIMsgMailNewsUrl *url)
   nsString alertString;
   nsString dialogTitle;
   bundle->FormatStringFromName(
-    MOZ_UTF16("pop3ServerBusy"),
-    params, 1, getter_Copies(alertString));
+    "pop3ServerBusy",
+    params, 1, alertString);
   bundle->FormatStringFromName(
-    MOZ_UTF16("pop3ErrorDialogTitle"),
-    params, 1, getter_Copies(dialogTitle));
+    "pop3ErrorDialogTitle",
+    params, 1, dialogTitle);
   if (!alertString.IsEmpty())
     dialog->Alert(dialogTitle.get(), alertString.get());
 }
