@@ -2,10 +2,10 @@
  * The intent of this file is to test that movemail download code
  * works correctly.
  */
-Components.utils.import("resource:///modules/mailServices.js");
-Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource://gre/modules/Task.jsm");
-Components.utils.import("resource://testing-common/mailnews/PromiseTestUtils.jsm");
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/Task.jsm");
+ChromeUtils.import("resource://testing-common/mailnews/PromiseTestUtils.jsm");
 
 var gPluggableStores = [
   "@mozilla.org/msgstore/berkeleystore;1",
@@ -75,7 +75,7 @@ function run_test()
   for (let index = 0; index < localAccountUtils.pluggableStores.length; index++) {
     add_task(setup(localAccountUtils.pluggableStores[index],
                    hostName + "-" + index));
-    gTestArray.forEach(add_task);
+    gTestArray.forEach(x => add_task(x));
   }
 
   run_next_test();
