@@ -9,9 +9,9 @@
  *     we get a new password prompt and can enter the password.
  */
 
-Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
-Components.utils.import("resource:///modules/mailServices.js");
-Components.utils.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource:///modules/mailServices.js");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
 load("../../../resources/logHelper.js");
 load("../../../resources/asyncTestUtils.js");
 load("../../../resources/alertTestUtils.js");
@@ -121,7 +121,7 @@ function actually_run_test() {
   async_run_tests(tests);
 }
 
-function getMail1() {
+function* getMail1() {
   dump("\nGet Mail 1\n");
 
   // Now get mail
@@ -144,7 +144,7 @@ function getMail1() {
   yield true;
 }
 
-function getMail2() {
+function* getMail2() {
   dump("\nGet Mail 2\n");
 
   // Now get the mail
@@ -153,7 +153,7 @@ function getMail2() {
   dump("\nGot Mail 2\n");
 }
 
-function endTest() {
+function* endTest() {
   // Now check the new one has been saved.
   logins = Services.logins.findLogins(count, "mailbox://localhost", null,
                                       "mailbox://localhost");
