@@ -6,7 +6,7 @@
 #include "nspr.h"
 #include "plstr.h"
 #include "nsCOMPtr.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "mozilla/Services.h"
 #include "nsIRequest.h"
 #include "nsIStringBundle.h"
@@ -87,8 +87,8 @@ static void PgpMimeGetNeedsAddonString(nsCString &aResult)
   const char16_t *formatStrings[] = { url16.get() };
 
   nsString result;
-  rv = stringBundle->FormatStringFromName(MOZ_UTF16(PGPMIME_STR_NOT_SUPPORTED_ID),
-                                          formatStrings, 1, getter_Copies(result));
+  rv = stringBundle->FormatStringFromName(PGPMIME_STR_NOT_SUPPORTED_ID,
+                                          formatStrings, 1, result);
   if (NS_FAILED(rv))
     return;
   aResult = NS_ConvertUTF16toUTF8(result);
