@@ -25,7 +25,6 @@
 #include "nsIMimeHeaders.h"
 #include "nsIMsgWindow.h"
 #include "nsIMsgMailNewsUrl.h"
-#include "nsDateTimeFormatCID.h"
 #include "nsServiceManagerUtils.h"
 #include "nsComponentManagerUtils.h"
 #include "nsMsgUtils.h"
@@ -33,7 +32,7 @@
 #include "mozilla/Services.h"
 #include <algorithm>
 
-static PRLogModuleInfo * gMimeEmitterLogModule = nullptr;
+static mozilla::LazyLogModule gMimeEmitterLogModule("MIME");
 
 #define   MIME_HEADER_URL      "chrome://messenger/locale/mimeheader.properties"
 #define   MIME_URL             "chrome://messenger/locale/mime.properties"
@@ -80,9 +79,6 @@ nsMimeBaseEmitter::nsMimeBaseEmitter()
   // This is needed for conversion of I18N Strings...
   mUnicodeConverter = do_GetService(NS_MIME_CONVERTER_CONTRACTID);
 
-  if (!gMimeEmitterLogModule)
-    gMimeEmitterLogModule = PR_NewLogModule("MIME");
-
   // Do prefs last since we can live without this if it fails...
   nsCOMPtr<nsIPrefBranch> pPrefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
   if (pPrefBranch)
@@ -212,8 +208,7 @@ nsMimeBaseEmitter::MimeGetStringByName(const char *aHeaderName)
   {
     nsString val;
 
-    res = m_headerStringBundle->GetStringFromName(NS_ConvertASCIItoUTF16(aHeaderName).get(),
-                                                  getter_Copies(val));
+    res = m_headerStringBundle->GetStringFromName(aHeaderName, val);
 
     if (NS_FAILED(res))
       return nullptr;
@@ -248,8 +243,7 @@ nsMimeBaseEmitter::MimeGetStringByID(int32_t aID)
   if (m_stringBundle)
   {
     nsString val;
-
-    res = m_stringBundle->GetStringFromID(aID, getter_Copies(val));
+    res = m_stringBundle->GetStringFromID(aID, val);
 
     if (NS_FAILED(res))
       return nullptr;
@@ -482,7 +476,7 @@ nsMimeBaseEmitter::WriteHelper(const nsACString &buf, uint32_t *countWritten)
     uint64_t avail;
     rv = mInputStream->Available(&avail);
     if (NS_SUCCEEDED(rv) && avail) {
-      mOutListener->OnDataAvailable(mChannel, mURL, mInputStream, 0, 
+      mOutListener->OnDataAvailable(mChannel, mURL, mInputStream, 0,
                                     std::min(avail, uint64_t(PR_UINT32_MAX)));
 
       // try writing again...
@@ -563,9 +557,7 @@ nsMimeBaseEmitter::StartHeader(bool rootMailHeader, bool headerOnly, const char
 NS_IMETHODIMP
 nsMimeBaseEmitter::UpdateCharacterSet(const char *aCharset)
 {
-  if ( (aCharset) && (PL_strcasecmp(aCharset, "US-ASCII")) &&
-        (PL_strcasecmp(aCharset, "ISO-8859-1")) &&
-        (PL_strcasecmp(aCharset, "UTF-8")) )
+  if (aCharset)
   {
     nsAutoCString contentType;
 
@@ -595,7 +587,11 @@ nsMimeBaseEmitter::UpdateCharacterSet(const char *aCharset)
 
       // have to set content-type since it could have an embedded null byte
       mChannel->SetContentType(nsDependentCString(cBegin));
-      mChannel->SetContentCharset(nsDependentCString(aCharset));
+      if (PL_strcasecmp(aCharset, "US-ASCII") == 0) {
+        mChannel->SetContentCharset(NS_LITERAL_CSTRING("ISO-8859-1"));
+      } else {
+        mChannel->SetContentCharset(nsDependentCString(aCharset));
+      }
     }
   }
 
@@ -661,11 +657,6 @@ nsMimeBaseEmitter::GenerateDateString(const char * dateString,
 {
   nsresult rv = NS_OK;
 
-  if (!mDateFormatter) {
-    mDateFormatter = do_CreateInstance(NS_DATETIMEFORMAT_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
   /**
    * See if the user wants to have the date displayed in the senders
    * timezone (including the timezone offset).
@@ -689,9 +680,13 @@ nsMimeBaseEmitter::GenerateDateString(const char * dateString,
     dateFormatPrefs->SetBoolPref("date_senders_timezone", true);
 
   PRExplodedTime explodedMsgTime;
-  // XXX Casting PRStatus to nsresult
-  rv = static_cast<nsresult>(
-    PR_ParseTimeStringToExplodedTime(dateString, false, &explodedMsgTime));
+
+  // Bogus date string may leave some fields uninitialized, so take precaution.
+  memset(&explodedMsgTime, 0, sizeof (PRExplodedTime));
+
+  if (PR_ParseTimeStringToExplodedTime(dateString, false, &explodedMsgTime) != PR_SUCCESS)
+    return NS_ERROR_FAILURE;
+
   /**
    * To determine the date format to use, comparison of current and message
    * time has to be made. If displaying in local time, both timestamps have
@@ -723,32 +718,28 @@ nsMimeBaseEmitter::GenerateDateString(const char * dateString,
   }
 
   nsAutoString formattedDateString;
+
+  rv = mozilla::DateTimeFormat::FormatPRExplodedTime(dateFormat,
+                                                     kTimeFormatNoSeconds,
+                                                     &explodedCompTime,
+                                                     formattedDateString);
+
   if (NS_SUCCEEDED(rv))
   {
-    rv = mDateFormatter->FormatPRExplodedTime(nullptr /* nsILocale* locale */,
-                                              dateFormat,
-                                              kTimeFormatNoSeconds,
-                                              &explodedCompTime,
-                                              formattedDateString);
-
-    if (NS_SUCCEEDED(rv))
+    if (displaySenderTimezone)
     {
-      if (displaySenderTimezone)
-      {
-        // offset of local time from UTC in minutes
-        int32_t senderoffset = (explodedMsgTime.tm_params.tp_gmt_offset +
-                                explodedMsgTime.tm_params.tp_dst_offset) / 60;
-        // append offset to date string
-        char16_t *tzstring =
-          nsTextFormatter::smprintf(MOZ_UTF16(" %+05d"),
-                                    (senderoffset / 60 * 100) +
-                                    (senderoffset % 60));
-        formattedDateString.Append(tzstring);
-        nsTextFormatter::smprintf_free(tzstring);
-      }
-
-      CopyUTF16toUTF8(formattedDateString, formattedDate);
+      // offset of local time from UTC in minutes
+      int32_t senderoffset = (explodedMsgTime.tm_params.tp_gmt_offset +
+                              explodedMsgTime.tm_params.tp_dst_offset) / 60;
+      // append offset to date string
+      nsString tzstring;
+      nsTextFormatter::ssprintf(tzstring, u" %+05d",
+                                (senderoffset / 60 * 100) +
+                                (senderoffset % 60));
+      formattedDateString.Append(tzstring);
     }
+
+    CopyUTF16toUTF8(formattedDateString, formattedDate);
   }
 
   return rv;
@@ -784,7 +775,7 @@ nsMimeBaseEmitter::GetLocalizedDateString(const char * dateString)
 nsresult
 nsMimeBaseEmitter::WriteHeaderFieldHTML(const char *field, const char *value)
 {
-  char *newValue = nullptr;
+  nsCString newValue;
   char *i18nValue = nullptr;
 
   if ( (!field) || (!value) )
@@ -814,18 +805,18 @@ nsMimeBaseEmitter::WriteHeaderFieldHTML(const char *field, const char *value)
     nsresult rv = mUnicodeConverter->DecodeMimeHeaderToUTF8(
       nsDependentCString(i18nValue), nullptr, false, true, tValue);
     if (NS_SUCCEEDED(rv) && !tValue.IsEmpty())
-      newValue = MsgEscapeHTML(tValue.get());
+      nsAppendEscapedHTML(tValue, newValue);
     else
-      newValue = MsgEscapeHTML(i18nValue);
+      nsAppendEscapedHTML(nsDependentCString(i18nValue), newValue);
   }
   else
   {
-    newValue = MsgEscapeHTML(i18nValue);
+    nsAppendEscapedHTML(nsDependentCString(i18nValue), newValue);
   }
 
   free(i18nValue);
 
-  if (!newValue)
+  if (newValue.IsEmpty())
     return NS_OK;
 
   mHTMLHeaders.Append("<tr>");
@@ -866,7 +857,6 @@ nsMimeBaseEmitter::WriteHeaderFieldHTML(const char *field, const char *value)
 
   mHTMLHeaders.Append("</tr>");
 
-  PR_FREEIF(newValue);
   return NS_OK;
 }
 
@@ -883,9 +873,7 @@ nsMimeBaseEmitter::WriteHeaderFieldHTMLPrefix(const nsACString &name)
     mHTMLHeaders.Append("<br><fieldset class=\"mimeAttachmentHeader\">");
     if (!name.IsEmpty()) {
       mHTMLHeaders.Append("<legend class=\"mimeAttachmentHeaderName\">");
-      nsCString escapedName;
-      escapedName.Adopt(MsgEscapeHTML(nsCString(name).get()));
-      mHTMLHeaders.Append(escapedName);
+      nsAppendEscapedHTML(name, mHTMLHeaders);
       mHTMLHeaders.Append("</legend>");
     }
     mHTMLHeaders.Append("</fieldset>");
@@ -1045,7 +1033,7 @@ nsMimeBaseEmitter::Complete()
   if (mOutListener)
   {
     uint64_t bytesInStream = 0;
-    nsresult rv2 = mInputStream->Available(&bytesInStream);
+    mozilla::DebugOnly<nsresult> rv2 = mInputStream->Available(&bytesInStream);
     NS_ASSERTION(NS_SUCCEEDED(rv2), "Available failed");
     if (bytesInStream)
     {
