@@ -39,6 +39,7 @@
 #include "mimetpfl.h"   /*   |     |     |--- MimeInlineTextPlainFlowed     */
 #include "mimethtm.h"  /*   |     |     |--- MimeInlineTextHTML      */
 #include "mimethsa.h"  /*   |     |     |     |--- M.I.TextHTMLSanitized   */
+#include "mimeTextHTMLParsed.h" /*|     |     |--- M.I.TextHTMLParsed   */
 #include "mimetric.h"  /*   |     |     |--- MimeInlineTextRichtext    */
 #include "mimetenr.h"  /*   |     |     |     |--- MimeInlineTextEnriched  */
 /* SUPPORTED VIA PLUGIN      |     |     |--- MimeInlineTextVCard           */
@@ -67,11 +68,14 @@
 #include "nsMimeTypes.h"
 #include "nsMsgUtils.h"
 #include "nsIPrefBranch.h"
+#include "mozilla/Preferences.h"
 #include "imgLoader.h"
 
 #include "nsIMsgMailNewsUrl.h"
 #include "nsIMsgHdr.h"
 
+using namespace mozilla;
+
 // forward declaration
 void getMsgHdrForCurrentURL(MimeDisplayOptions *opts, nsIMsgDBHdr ** aMsgHdr);
 
@@ -107,7 +111,7 @@ find_content_type_attribs(const char *content_type,
   if (!ctHandlerList)
     return false;
 
-  for (size_t i = 0; i < ctHandlerList->Length(); i++) 
+  for (size_t i = 0; i < ctHandlerList->Length(); i++)
   {
     cthandler_struct *ptr = ctHandlerList->ElementAt(i);
     if (PL_strcasecmp(content_type, ptr->content_type) == 0)
@@ -149,7 +153,7 @@ add_content_type_attribs(const char *content_type,
 }
 
 /*
- * This routine will find all content type handler for a specifc content
+ * This routine will find all content type handler for a specific content
  * type (if it exists)
  */
 bool
@@ -162,7 +166,7 @@ force_inline_display(const char *content_type)
 }
 
 /*
- * This routine will find all content type handler for a specifc content
+ * This routine will find all content type handler for a specific content
  * type (if it exists) and is defined to the nsRegistry
  */
 MimeObjectClass *
@@ -176,9 +180,9 @@ mime_locate_external_content_handler(const char *content_type,
   nsresult rv;
 
   nsAutoCString lookupID("@mozilla.org/mimecth;1?type=");
-  nsAutoCString lowerCaseContentType;
-  ToLowerCase(nsDependentCString(content_type), lowerCaseContentType);
-  lookupID += lowerCaseContentType;
+  nsAutoCString contentType;
+  ToLowerCase(nsDependentCString(content_type), contentType);
+  lookupID += contentType;
 
   nsCOMPtr<nsIMimeContentTypeHandler> ctHandler = do_CreateInstance(lookupID.get(), &rv);
   if (NS_FAILED(rv) || !ctHandler) {
@@ -189,20 +193,20 @@ mime_locate_external_content_handler(const char *content_type,
 
     nsCString value;
     rv = catman->GetCategoryEntry(NS_SIMPLEMIMECONVERTERS_CATEGORY,
-                                  content_type, getter_Copies(value));
+                                  contentType.get(), getter_Copies(value));
     if (NS_FAILED(rv) || value.IsEmpty())
       return nullptr;
-    rv = MIME_NewSimpleMimeConverterStub(content_type,
+    rv = MIME_NewSimpleMimeConverterStub(contentType.get(),
                                          getter_AddRefs(ctHandler));
     if (NS_FAILED(rv) || !ctHandler)
       return nullptr;
   }
 
-  rv = ctHandler->CreateContentTypeHandlerClass(content_type, ctHandlerInfo, &newObj);
+  rv = ctHandler->CreateContentTypeHandlerClass(contentType.get(), ctHandlerInfo, &newObj);
   if (NS_FAILED(rv))
     return nullptr;
 
-  add_content_type_attribs(content_type, ctHandlerInfo);
+  add_content_type_attribs(contentType.get(), ctHandlerInfo);
   return newObj;
 }
 
@@ -329,7 +333,7 @@ bool mime_is_allowed_class(const MimeObjectClass *clazz,
      !(
         (avoid_html
          && (
-              clazz == (MimeObjectClass *)&mimeInlineTextHTMLClass
+              clazz == (MimeObjectClass *)&mimeInlineTextHTMLParsedClass
                          /* Should not happen - we protect against that in
                             mime_find_class(). Still for safety... */
             )) ||
@@ -485,7 +489,7 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
        )
       /* Use a little hack to prevent some dangerous plugins, which ship
          with Mozilla, to run.
-         For the truely user-installed plugins, we rely on the judgement
+         For the truly user-installed plugins, we rely on the judgement
          of the user. */
     {
       if (!exact_match_p)
@@ -506,15 +510,23 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
     {
       if      (!PL_strcasecmp(content_type+5,    "html"))
       {
-        if (opts
-            && opts->format_out == nsMimeOutput::nsMimeMessageSaveAs)
+        if (opts &&
+            (opts->format_out == nsMimeOutput::nsMimeMessageSaveAs ||
+             opts->format_out == nsMimeOutput::nsMimeMessageFilterSniffer ||
+             opts->format_out == nsMimeOutput::nsMimeMessageDecrypt ||
+             opts->format_out == nsMimeOutput::nsMimeMessageAttach))
           // SaveAs in new modes doesn't work yet.
         {
+          // Don't use the parsed HTML class if we're ...
+          // - saving the HTML of a message
+          // - getting message content for filtering
+          // - snarfing attachments (nsMimeMessageDecrypt used in SnarfMsgAttachment)
+          // - processing attachments (like deleting attachments).
           clazz = (MimeObjectClass *)&mimeInlineTextHTMLClass;
           types_of_classes_to_disallow = 0;
         }
         else if (html_as == 0 || html_as == 4) // Render sender's HTML
-          clazz = (MimeObjectClass *)&mimeInlineTextHTMLClass;
+          clazz = (MimeObjectClass *)&mimeInlineTextHTMLParsedClass;
         else if (html_as == 1) // convert HTML to plaintext
           // Do a HTML->TXT->HTML conversion, see mimethpl.h.
           clazz = (MimeObjectClass *)&mimeInlineTextHTMLAsPlaintextClass;
@@ -703,7 +715,7 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
     /* The magic image types which we are able to display internally...
     */
     else if (!PL_strncasecmp(content_type,    "image/", 6)) {
-        if (imgLoader::SupportImageWithMimeType(content_type))
+        if (imgLoader::SupportImageWithMimeType(content_type, AcceptedMimeTypes::IMAGES_AND_DOCUMENTS))
           clazz = (MimeObjectClass *)&mimeInlineImageClass;
         else
           clazz = (MimeObjectClass *)&mimeExternalObjectClass;
@@ -712,7 +724,18 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
 #ifdef ENABLE_SMIME
     else if (!PL_strcasecmp(content_type, APPLICATION_XPKCS7_MIME)
              || !PL_strcasecmp(content_type, APPLICATION_PKCS7_MIME)) {
-        char *ct = (hdrs ? MimeHeaders_get(hdrs, HEADER_CONTENT_TYPE,
+
+        if (!Preferences::GetBool("mailnews.p7m_subparts_external", false) &&
+            opts->is_child) {
+          // We do not allow encrypted parts except as top level.
+          // Allowing them would leak the plain text in case the part is
+          // cleverly hidden and the decrypted content gets included in
+          // replies and forwards.
+          clazz = (MimeObjectClass *)&mimeExternalObjectClass;
+          return clazz;
+        }
+
+         char *ct = (hdrs ? MimeHeaders_get(hdrs, HEADER_CONTENT_TYPE,
                                            false, false)
                            : nullptr);
         char *st = (ct ? MimeHeaders_get_parameter(ct, "smime-type", NULL, NULL)
@@ -858,7 +881,7 @@ mime_create (const char *content_type, MimeHeaders *hdrs,
         override_content_type = nullptr;
       PR_FREEIF(name);
 
-      // Workaroung for saving '.eml" file encoded with base64.
+      // Workaround for saving '.eml" file encoded with base64.
       // Do not override with message/rfc822 whenever Transfer-Encoding is
       // base64 since base64 encoding of message/rfc822 is invalid.
       // Our MimeMessageClass has no capability to decode it.
@@ -916,22 +939,20 @@ mime_create (const char *content_type, MimeHeaders *hdrs,
     ;  /* Use the class we've got. */
   else
   {
-    //
-    // rhp: Ok, this is a modification to try to deal with messages
-    //      that have content disposition set to "attachment" even though
-    //      we probably should show them inline.
-    //
-    if (  (clazz != (MimeObjectClass *)&mimeInlineTextHTMLClass) &&
-          (clazz != (MimeObjectClass *)&mimeInlineTextClass) &&
+    // override messages that have content disposition set to "attachment"
+    // even though we probably should show them inline.
+    if (  (clazz != (MimeObjectClass *)&mimeInlineTextClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextPlainClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextPlainFlowedClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextHTMLClass) &&
+          (clazz != (MimeObjectClass *)&mimeInlineTextHTMLParsedClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextHTMLSanitizedClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextHTMLAsPlaintextClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextRichtextClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextEnrichedClass) &&
           (clazz != (MimeObjectClass *)&mimeMessageClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineImageClass) )
+      // not a special inline type, so show as attachment
       clazz = (MimeObjectClass *)&mimeExternalObjectClass;
   }
 
@@ -1169,7 +1190,7 @@ mime_external_attachment_url(MimeObject *obj)
    to decide if the headers need to be presented differently.)
  */
 bool
-mime_crypto_object_p(MimeHeaders *hdrs, bool clearsigned_counts)
+mime_crypto_object_p(MimeHeaders *hdrs, bool clearsigned_counts, MimeDisplayOptions *opts)
 {
   char *ct;
   MimeObjectClass *clazz;
@@ -1188,7 +1209,7 @@ mime_crypto_object_p(MimeHeaders *hdrs, bool clearsigned_counts)
   }
 
   /* It's a candidate for being a crypto object.  Let's find out for sure... */
-  clazz = mime_find_class (ct, hdrs, 0, true);
+  clazz = mime_find_class(ct, hdrs, opts, true);
   PR_Free(ct);
 
   if (clazz == ((MimeObjectClass *)&mimeEncryptedCMSClass))
@@ -1680,7 +1701,7 @@ mime_parse_url_options(const char *url, MimeDisplayOptions *options)
  */
 
 int
-MimeOptions_write(MimeDisplayOptions *opt, nsCString &name, const char *data,
+MimeOptions_write(MimeHeaders *hdrs, MimeDisplayOptions *opt, const char *data,
                   int32_t length, bool user_visible_p)
 {
   int status = 0;
@@ -1704,6 +1725,10 @@ MimeOptions_write(MimeDisplayOptions *opt, nsCString &name, const char *data,
       opt->state->separator_suppressed_p = false;
       if (lstatus < 0) return lstatus;
 
+      nsCString name;
+      name.Adopt(MimeHeaders_get_name(hdrs, opt));
+      MimeHeaders_convert_header_value(opt, name, false);
+
       if (!name.IsEmpty()) {
           sep = "<LEGEND CLASS=\"mimeAttachmentHeaderName\">";
           lstatus = opt->output_fn(sep, strlen(sep), closure);
@@ -1711,7 +1736,7 @@ MimeOptions_write(MimeDisplayOptions *opt, nsCString &name, const char *data,
           if (lstatus < 0) return lstatus;
 
           nsCString escapedName;
-          escapedName.Adopt(MsgEscapeHTML(name.get()));
+          nsAppendEscapedHTML(name, escapedName);
 
           lstatus = opt->output_fn(escapedName.get(),
                                    escapedName.Length(), closure);
@@ -1724,7 +1749,7 @@ MimeOptions_write(MimeDisplayOptions *opt, nsCString &name, const char *data,
           if (lstatus < 0) return lstatus;
       }
 
-      sep = "</FIELDSET><BR/>";
+      sep = "</FIELDSET>";
       lstatus = opt->output_fn(sep, strlen(sep), closure);
       opt->state->separator_suppressed_p = false;
       if (lstatus < 0) return lstatus;
@@ -1748,7 +1773,7 @@ MimeObject_write(MimeObject *obj, const char *output, int32_t length,
 {
   if (!obj->output_p) return 0;
 
-  // if we're stripping attachments, check if any parent is not being ouput
+  // if we're stripping attachments, check if any parent is not being output
   if (obj->options->format_out == nsMimeOutput::nsMimeMessageAttach)
   {
     // if true, mime generates a separator in html - we don't want that.
@@ -1767,10 +1792,7 @@ MimeObject_write(MimeObject *obj, const char *output, int32_t length,
     NS_ASSERTION(obj->options->state->first_data_written_p, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
   }
 
-  nsCString name;
-  name.Adopt(MimeHeaders_get_name(obj->headers, obj->options));
-  MimeHeaders_convert_header_value(obj->options, name, false);
-  return MimeOptions_write(obj->options, name, output, length, user_visible_p);
+  return MimeOptions_write(obj->headers, obj->options, output, length, user_visible_p);
 }
 
 int
