@@ -269,8 +269,10 @@ nsresult nsNntpIncomingServer::SetupNewsrcSaveTimer()
   if(mNewsrcSaveTimer)
     mNewsrcSaveTimer->Cancel();
   mNewsrcSaveTimer = do_CreateInstance("@mozilla.org/timer;1");
-  mNewsrcSaveTimer->InitWithFuncCallback(OnNewsrcSaveTimer, (void*)this, timeInMSUint32,
-                                           nsITimer::TYPE_REPEATING_SLACK);
+  mNewsrcSaveTimer->InitWithNamedFuncCallback(OnNewsrcSaveTimer, (void*)this,
+                                              timeInMSUint32,
+                                              nsITimer::TYPE_REPEATING_SLACK,
+                                              "nsNntpIncomingServer::OnNewsrcSaveTimer");
   return NS_OK;
 }
 
@@ -634,7 +636,7 @@ nsNntpIncomingServer::PrepareForNextUrl(nsNNTPProtocol *aConnection)
     if (NS_SUCCEEDED(rv))
       return NS_OK;
   }
-  
+
   // No queued uris.
   return NS_OK;
 }
@@ -1578,9 +1580,9 @@ nsNntpIncomingServer::GroupNotFound(nsIMsgWindow *aMsgWindow,
   const char16_t *formatStrings[2] = { groupName.get(), hostStr.get() };
   nsString confirmText;
   rv = bundle->FormatStringFromName(
-                    u"autoUnsubscribeText",
+                    "autoUnsubscribeText",
                     formatStrings, 2,
-                    getter_Copies(confirmText));
+                    confirmText);
   NS_ENSURE_SUCCESS(rv,rv);
 
   bool confirmResult = false;
