@@ -1,7 +1,7 @@
 /* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 // This file implements test NNTP servers
 
-Components.utils.import("resource:///modules/mimeParser.jsm");
+ChromeUtils.import("resource:///modules/mimeParser.jsm");
 
 var EXPORTED_SYMBOLS = [
   'nntpDaemon',
@@ -126,9 +126,9 @@ function wildmat2regex(wildmat) {
 }
 
 // NNTP FLAGS
-const NNTP_POSTABLE = 0x0001;
+var NNTP_POSTABLE = 0x0001;
 
-const NNTP_REAL_LENGTH = 0x0100;
+var NNTP_REAL_LENGTH = 0x0100;
 
 function hasFlag(flags, flag) {
   return (flags & flag) == flag;
@@ -551,8 +551,8 @@ subclass(NNTP_RFC4643_extension, NNTP_RFC2980_handler, {
     }
     else if (action == "pass") {
       if (!this.requireBoth || !this.usernameReceived)
-        return "482 Authetication commands issued out of sequence";
-      
+        return "482 Authentication commands issued out of sequence";
+
       this.usernameReceived = false;
 
       var expectPassword = this.lastGroupTried
