@@ -1,58 +1,58 @@
 ////////////////////////////////////////////////////////////////////////////////
 // Interfaces
 
-const nsIAccessibleRetrieval = Components.interfaces.nsIAccessibleRetrieval;
+const nsIAccessibilityService = Ci.nsIAccessibilityService;
 
-const nsIAccessibleEvent = Components.interfaces.nsIAccessibleEvent;
+const nsIAccessibleEvent = Ci.nsIAccessibleEvent;
 const nsIAccessibleStateChangeEvent =
-  Components.interfaces.nsIAccessibleStateChangeEvent;
+  Ci.nsIAccessibleStateChangeEvent;
 const nsIAccessibleCaretMoveEvent =
-  Components.interfaces.nsIAccessibleCaretMoveEvent;
+  Ci.nsIAccessibleCaretMoveEvent;
 const nsIAccessibleTextChangeEvent =
-  Components.interfaces.nsIAccessibleTextChangeEvent;
+  Ci.nsIAccessibleTextChangeEvent;
 const nsIAccessibleVirtualCursorChangeEvent =
-  Components.interfaces.nsIAccessibleVirtualCursorChangeEvent;
+  Ci.nsIAccessibleVirtualCursorChangeEvent;
 const nsIAccessibleObjectAttributeChangedEvent =
-  Components.interfaces.nsIAccessibleObjectAttributeChangedEvent;
+  Ci.nsIAccessibleObjectAttributeChangedEvent;
 
-const nsIAccessibleStates = Components.interfaces.nsIAccessibleStates;
-const nsIAccessibleRole = Components.interfaces.nsIAccessibleRole;
-const nsIAccessibleScrollType = Components.interfaces.nsIAccessibleScrollType;
-const nsIAccessibleCoordinateType = Components.interfaces.nsIAccessibleCoordinateType;
+const nsIAccessibleStates = Ci.nsIAccessibleStates;
+const nsIAccessibleRole = Ci.nsIAccessibleRole;
+const nsIAccessibleScrollType = Ci.nsIAccessibleScrollType;
+const nsIAccessibleCoordinateType = Ci.nsIAccessibleCoordinateType;
 
-const nsIAccessibleRelation = Components.interfaces.nsIAccessibleRelation;
-const nsIAccessibleTextRange = Components.interfaces.nsIAccessibleTextRange;
+const nsIAccessibleRelation = Ci.nsIAccessibleRelation;
+const nsIAccessibleTextRange = Ci.nsIAccessibleTextRange;
 
-const nsIAccessible = Components.interfaces.nsIAccessible;
+const nsIAccessible = Ci.nsIAccessible;
 
-const nsIAccessibleDocument = Components.interfaces.nsIAccessibleDocument;
-const nsIAccessibleApplication = Components.interfaces.nsIAccessibleApplication;
+const nsIAccessibleDocument = Ci.nsIAccessibleDocument;
+const nsIAccessibleApplication = Ci.nsIAccessibleApplication;
 
-const nsIAccessibleText = Components.interfaces.nsIAccessibleText;
-const nsIAccessibleEditableText = Components.interfaces.nsIAccessibleEditableText;
+const nsIAccessibleText = Ci.nsIAccessibleText;
+const nsIAccessibleEditableText = Ci.nsIAccessibleEditableText;
 
-const nsIAccessibleHyperLink = Components.interfaces.nsIAccessibleHyperLink;
-const nsIAccessibleHyperText = Components.interfaces.nsIAccessibleHyperText;
+const nsIAccessibleHyperLink = Ci.nsIAccessibleHyperLink;
+const nsIAccessibleHyperText = Ci.nsIAccessibleHyperText;
 
-const nsIAccessibleImage = Components.interfaces.nsIAccessibleImage;
-const nsIAccessiblePivot = Components.interfaces.nsIAccessiblePivot;
-const nsIAccessibleSelectable = Components.interfaces.nsIAccessibleSelectable;
-const nsIAccessibleTable = Components.interfaces.nsIAccessibleTable;
-const nsIAccessibleTableCell = Components.interfaces.nsIAccessibleTableCell;
-const nsIAccessibleTraversalRule = Components.interfaces.nsIAccessibleTraversalRule;
-const nsIAccessibleValue = Components.interfaces.nsIAccessibleValue;
+const nsIAccessibleImage = Ci.nsIAccessibleImage;
+const nsIAccessiblePivot = Ci.nsIAccessiblePivot;
+const nsIAccessibleSelectable = Ci.nsIAccessibleSelectable;
+const nsIAccessibleTable = Ci.nsIAccessibleTable;
+const nsIAccessibleTableCell = Ci.nsIAccessibleTableCell;
+const nsIAccessibleTraversalRule = Ci.nsIAccessibleTraversalRule;
+const nsIAccessibleValue = Ci.nsIAccessibleValue;
 
-const nsIObserverService = Components.interfaces.nsIObserverService;
+const nsIObserverService = Ci.nsIObserverService;
 
-const nsIDOMDocument = Components.interfaces.nsIDOMDocument;
-const nsIDOMEvent = Components.interfaces.nsIDOMEvent;
-const nsIDOMHTMLDocument = Components.interfaces.nsIDOMHTMLDocument;
-const nsIDOMNode = Components.interfaces.nsIDOMNode;
-const nsIDOMHTMLElement = Components.interfaces.nsIDOMHTMLElement;
-const nsIDOMWindow = Components.interfaces.nsIDOMWindow;
-const nsIDOMXULElement = Components.interfaces.nsIDOMXULElement;
+const nsIDOMDocument = Ci.nsIDOMDocument;
+const nsIDOMEvent = Ci.nsIDOMEvent;
+const nsIDOMHTMLDocument = Ci.nsIDOMHTMLDocument;
+const nsIDOMNode = Ci.nsIDOMNode;
+const nsIDOMHTMLElement = Ci.nsIDOMHTMLElement;
+const nsIDOMWindow = Ci.nsIDOMWindow;
+const nsIDOMXULElement = Ci.nsIDOMXULElement;
 
-const nsIPropertyElement = Components.interfaces.nsIPropertyElement;
+const nsIPropertyElement = Ci.nsIPropertyElement;
 
 ////////////////////////////////////////////////////////////////////////////////
 // OS detect
@@ -91,28 +91,28 @@ const MAX_TRIM_LENGTH = 100;
 /**
  * Services to determine if e10s is enabled.
  */
-Components.utils.import('resource://gre/modules/Services.jsm');
+ChromeUtils.import('resource://gre/modules/Services.jsm');
 
 /**
- * nsIAccessibleRetrieval service.
+ * nsIAccessibilityService service.
  */
-var gAccRetrieval = Components.classes["@mozilla.org/accessibleRetrieval;1"].
-  getService(nsIAccessibleRetrieval);
+var gAccService = Cc["@mozilla.org/accessibilityService;1"].
+  getService(nsIAccessibilityService);
 
 /**
  * Enable/disable logging.
  */
 function enableLogging(aModules)
 {
-  gAccRetrieval.setLogging(aModules);
+  gAccService.setLogging(aModules);
 }
 function disableLogging()
 {
-  gAccRetrieval.setLogging("");
+  gAccService.setLogging("");
 }
 function isLogged(aModule)
 {
-  return gAccRetrieval.isLogged(aModule);
+  return gAccService.isLogged(aModule);
 }
 
 /**
@@ -253,7 +253,7 @@ function getAccessible(aAccOrElmOrID, aInterfaces, aElmObj, aDoNotFailIf)
   var elm = null;
 
   if (aAccOrElmOrID instanceof nsIAccessible) {
-    try { elm = aAccOrElmOrID.DOMNode; } catch(e) { }
+    try { elm = aAccOrElmOrID.DOMNode; } catch (e) { }
 
   } else if (aAccOrElmOrID instanceof nsIDOMNode) {
     elm = aAccOrElmOrID;
@@ -272,7 +272,7 @@ function getAccessible(aAccOrElmOrID, aInterfaces, aElmObj, aDoNotFailIf)
   var acc = (aAccOrElmOrID instanceof nsIAccessible) ? aAccOrElmOrID : null;
   if (!acc) {
     try {
-      acc = gAccRetrieval.getAccessibleFor(elm);
+      acc = gAccService.getAccessibleFor(elm);
     } catch (e) {
     }
 
@@ -291,6 +291,9 @@ function getAccessible(aAccOrElmOrID, aInterfaces, aElmObj, aDoNotFailIf)
     aInterfaces = [ aInterfaces ];
 
   for (var index = 0; index < aInterfaces.length; index++) {
+    if (acc instanceof aInterfaces[index]) {
+      continue;
+    }
     try {
       acc.QueryInterface(aInterfaces[index]);
     } catch (e) {
@@ -310,9 +313,8 @@ function getAccessible(aAccOrElmOrID, aInterfaces, aElmObj, aDoNotFailIf)
  */
 function isAccessible(aAccOrElmOrID, aInterfaces)
 {
-  return getAccessible(aAccOrElmOrID, aInterfaces, null,
-                       DONOTFAIL_IF_NO_ACC | DONOTFAIL_IF_NO_INTERFACE) ?
-    true : false;
+  return !!getAccessible(aAccOrElmOrID, aInterfaces, null,
+                         DONOTFAIL_IF_NO_ACC | DONOTFAIL_IF_NO_INTERFACE);
 }
 
 /**
@@ -360,7 +362,7 @@ function getTabDocAccessible(aAccOrElmOrID)
  */
 function getApplicationAccessible()
 {
-  return gAccRetrieval.getApplicationAccessible().
+  return gAccService.getApplicationAccessible().
     QueryInterface(nsIAccessibleApplication);
 }
 
@@ -548,7 +550,7 @@ function testAccessibleTree(aAccOrElmOrID, aAccTree, aFlags)
 
         // nsIAccessible::indexInParent
         var indexInParent = -1;
-        try { indexInParent = child.indexInParent; } catch(e) {}
+        try { indexInParent = child.indexInParent; } catch (e) {}
         is(indexInParent, i,
            "Wrong index in parent of " + prettyName(child));
 
@@ -581,7 +583,7 @@ function testAccessibleTree(aAccOrElmOrID, aAccTree, aFlags)
 function isAccessibleInCache(aNodeOrId)
 {
   var node = getNode(aNodeOrId);
-  return gAccRetrieval.getAccessibleFromCache(node) ? true : false;
+  return !!gAccService.getAccessibleFromCache(node);
 }
 
 /**
@@ -604,7 +606,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
   try {
     aAcc.firstChild;
   } catch (e) {
-    success = (e.result == Components.results.NS_ERROR_FAILURE)
+    success = (e.result == Cr.NS_ERROR_FAILURE);
   }
   ok(success, "firstChild" + msg);
 
@@ -613,7 +615,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
   try {
     aAcc.lastChild;
   } catch (e) {
-    success = (e.result == Components.results.NS_ERROR_FAILURE)
+    success = (e.result == Cr.NS_ERROR_FAILURE);
   }
   ok(success, "lastChild" + msg);
 
@@ -622,7 +624,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
   try {
     aAcc.childCount;
   } catch (e) {
-    success = (e.result == Components.results.NS_ERROR_FAILURE)
+    success = (e.result == Cr.NS_ERROR_FAILURE);
   }
   ok(success, "childCount" + msg);
 
@@ -631,7 +633,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
   try {
     aAcc.children;
   } catch (e) {
-    success = (e.result == Components.results.NS_ERROR_FAILURE)
+    success = (e.result == Cr.NS_ERROR_FAILURE);
   }
   ok(success, "children" + msg);
 
@@ -640,7 +642,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
   try {
     aAcc.nextSibling;
   } catch (e) {
-    success = (e.result == Components.results.NS_ERROR_FAILURE);
+    success = (e.result == Cr.NS_ERROR_FAILURE);
   }
   ok(success, "nextSibling" + msg);
 
@@ -649,7 +651,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
   try {
     aAcc.previousSibling;
   } catch (e) {
-    success = (e.result == Components.results.NS_ERROR_FAILURE);
+    success = (e.result == Cr.NS_ERROR_FAILURE);
   }
   ok(success, "previousSibling" + msg);
 
@@ -658,7 +660,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
   try {
     aAcc.parent;
   } catch (e) {
-    success = (e.result == Components.results.NS_ERROR_FAILURE);
+    success = (e.result == Cr.NS_ERROR_FAILURE);
   }
   ok(success, "parent" + msg);
 }
@@ -668,7 +670,7 @@ function testDefunctAccessible(aAcc, aNodeOrId)
  */
 function roleToString(aRole)
 {
-  return gAccRetrieval.getStringRole(aRole);
+  return gAccService.getStringRole(aRole);
 }
 
 /**
@@ -676,14 +678,14 @@ function roleToString(aRole)
  */
 function statesToString(aStates, aExtraStates)
 {
-  var list = gAccRetrieval.getStringStates(aStates, aExtraStates);
+  var list = gAccService.getStringStates(aStates, aExtraStates);
 
   var str = "";
   for (var index = 0; index < list.length - 1; index++)
     str += list.item(index) + ", ";
 
   if (list.length != 0)
-    str += list.item(index)
+    str += list.item(index);
 
   return str;
 }
@@ -693,7 +695,7 @@ function statesToString(aStates, aExtraStates)
  */
 function eventTypeToString(aEventType)
 {
-  return gAccRetrieval.getStringEventType(aEventType);
+  return gAccService.getStringEventType(aEventType);
 }
 
 /**
@@ -701,11 +703,10 @@ function eventTypeToString(aEventType)
  */
 function relationTypeToString(aRelationType)
 {
-  return gAccRetrieval.getStringRelationType(aRelationType);
+  return gAccService.getStringRelationType(aRelationType);
 }
 
 function getLoadContext() {
-  const Ci = Components.interfaces;
   return window.QueryInterface(Ci.nsIInterfaceRequestor)
                .getInterface(Ci.nsIWebNavigation)
                .QueryInterface(Ci.nsILoadContext);
@@ -716,13 +717,13 @@ function getLoadContext() {
  */
 function getTextFromClipboard()
 {
-  var clip = Components.classes["@mozilla.org/widget/clipboard;1"].
-    getService(Components.interfaces.nsIClipboard);
+  var clip = Cc["@mozilla.org/widget/clipboard;1"].
+    getService(Ci.nsIClipboard);
   if (!clip)
     return "";
 
-  var trans = Components.classes["@mozilla.org/widget/transferable;1"].
-    createInstance(Components.interfaces.nsITransferable);
+  var trans = Cc["@mozilla.org/widget/transferable;1"].
+    createInstance(Ci.nsITransferable);
   trans.init(getLoadContext());
   if (!trans)
     return "";
@@ -735,7 +736,7 @@ function getTextFromClipboard()
   trans.getTransferData("text/unicode", str, strLength);
 
   if (str)
-    str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
+    str = str.value.QueryInterface(Ci.nsISupportsString);
   if (str)
     return str.data.substring(0, strLength.value / 2);
 
@@ -860,12 +861,12 @@ function shortenString(aString, aMaxLength)
  */
 function getMainChromeWindow(aWindow)
 {
-  return aWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                .getInterface(Components.interfaces.nsIWebNavigation)
-                .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
+  return aWindow.QueryInterface(Ci.nsIInterfaceRequestor)
+                .getInterface(Ci.nsIWebNavigation)
+                .QueryInterface(Ci.nsIDocShellTreeItem)
                 .rootTreeItem
-                .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                .getInterface(Components.interfaces.nsIDOMWindow);
+                .QueryInterface(Ci.nsIInterfaceRequestor)
+                .getInterface(Ci.nsIDOMWindow);
 }
 
 /** Sets the test plugin(s) initially expected enabled state.
