@@ -22,16 +22,16 @@ function startCallbackTimer() {
 
 
 var observer = SpecialPowers.wrapCallbackObject({
-    QueryInterface : function (iid) {
+    QueryInterface(iid) {
         const interfaces = [Ci.nsIObserver,
                             Ci.nsISupports, Ci.nsISupportsWeakReference];
 
-        if (!interfaces.some( function(v) { return iid.equals(v) } ))
+        if (!interfaces.some( function(v) { return iid.equals(v); } ))
             throw SpecialPowers.Components.results.NS_ERROR_NO_INTERFACE;
         return this;
     },
 
-    observe : function (subject, topic, data) {
+    observe(subject, topic, data) {
         var doc = getDialogDoc();
         if (doc)
             handleDialog(doc);
@@ -45,7 +45,7 @@ function getDialogDoc() {
   // through all the open windows and all the <browsers> in each.
   var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
            getService(Ci.nsIWindowMediator);
-  //var enumerator = wm.getEnumerator("navigator:browser");
+  // var enumerator = wm.getEnumerator("navigator:browser");
   var enumerator = wm.getXULWindowEnumerator(null);
 
   while (enumerator.hasMoreElements()) {
@@ -65,7 +65,7 @@ function getDialogDoc() {
                                     .contentViewer
                                     .DOMDocument;
 
-        //ok(true, "Got window: " + childDoc.location.href);
+        // ok(true, "Got window: " + childDoc.location.href);
         if (childDoc.location.href == "chrome://global/content/commonDialog.xul")
           return childDoc;
     }
@@ -80,7 +80,7 @@ function test() {
 
 
 var tests = [
-  taskify(function* test_modal_dialog_while_opening_tooltip() {
+  taskify(async function test_modal_dialog_while_opening_tooltip() {
     let panelShown;
     let popup;
 
@@ -96,9 +96,9 @@ var tests = [
     };
     startCallbackTimer();
     executeSoon(() => alert("test"));
-    yield waitForConditionPromise(() => panelShown, "Timed out waiting for panel promise to be assigned", 100);
-    yield panelShown;
+    await waitForConditionPromise(() => panelShown, "Timed out waiting for panel promise to be assigned", 100);
+    await panelShown;
 
-    yield hideInfoPromise();
+    await hideInfoPromise();
   })
 ];
