@@ -7,13 +7,6 @@
 
 requestLongerTimeout(5);
 
-/**
- * Whitelisting this test.
- * As part of bug 1077403, the leaking uncaught rejection should be fixed.
- */
-thisTestLeaksUncaughtRejectionsAndShouldBeFixed("Error: Shader Editor is " +
-  "still waiting for a WebGL context to be created.");
-
 function performChecks(target) {
   return Task.spawn(function* () {
     let toolIds = gDevTools.getToolDefinitionArray()
