@@ -47,7 +47,7 @@ EmulatePackedDepthStencil(gl::GLContext* gl)
 }
 
 WebGLRenderbuffer::WebGLRenderbuffer(WebGLContext* webgl)
-    : WebGLContextBoundObject(webgl)
+    : WebGLRefCountedObject(webgl)
     , mPrimaryRB( DoCreateRenderbuffer(webgl->gl) )
     , mEmulatePackedDepthStencil( EmulatePackedDepthStencil(webgl->gl) )
     , mSecondaryRB(0)
@@ -115,7 +115,7 @@ DoRenderbufferStorageMaybeMultisample(gl::GLContext* gl, GLsizei samples,
         break;
 
     case LOCAL_GL_DEPTH_STENCIL:
-        MOZ_CRASH("GL_DEPTH_STENCIL is not valid here.");
+        MOZ_CRASH("GFX: GL_DEPTH_STENCIL is not valid here.");
         break;
 
     default:
@@ -205,7 +205,7 @@ WebGLRenderbuffer::RenderbufferStorage(const char* funcName, uint32_t samples,
     MOZ_ASSERT(usage->maxSamplesKnown);
 
     if (samples > usage->maxSamples) {
-        mContext->ErrorInvalidValue("%s: `samples` is out of the valid range.", funcName);
+        mContext->ErrorInvalidOperation("%s: `samples` is out of the valid range.", funcName);
         return;
     }
 
@@ -215,35 +215,45 @@ WebGLRenderbuffer::RenderbufferStorage(const char* funcName, uint32_t samples,
     if (error) {
         const char* errorName = mContext->ErrorName(error);
         mContext->GenerateWarning("%s generated error %s", funcName, errorName);
+        if (error == LOCAL_GL_OUT_OF_MEMORY) {
+          // Truncate.
+          mSamples = 0;
+          mFormat = nullptr;
+          mWidth = 0;
+          mHeight = 0;
+          mImageDataStatus = WebGLImageDataStatus::NoImageData;
+
+          InvalidateStatusOfAttachedFBs(funcName);
+        }
         return;
     }
 
+    mContext->OnDataAllocCall();
+
     mSamples = samples;
     mFormat = usage;
     mWidth = width;
     mHeight = height;
     mImageDataStatus = WebGLImageDataStatus::UninitializedImageData;
 
-    InvalidateStatusOfAttachedFBs();
+    InvalidateStatusOfAttachedFBs(funcName);
 }
 
 void
-WebGLRenderbuffer::DoFramebufferRenderbuffer(GLenum attachment) const
+WebGLRenderbuffer::DoFramebufferRenderbuffer(FBTarget target, GLenum attachment) const
 {
     gl::GLContext* gl = mContext->gl;
 
     if (attachment == LOCAL_GL_DEPTH_STENCIL_ATTACHMENT) {
         const GLuint stencilRB = (mSecondaryRB ? mSecondaryRB : mPrimaryRB);
-        gl->fFramebufferRenderbuffer(LOCAL_GL_FRAMEBUFFER,
-                                     LOCAL_GL_DEPTH_ATTACHMENT,
+        gl->fFramebufferRenderbuffer(target.get(), LOCAL_GL_DEPTH_ATTACHMENT,
                                      LOCAL_GL_RENDERBUFFER, mPrimaryRB);
-        gl->fFramebufferRenderbuffer(LOCAL_GL_FRAMEBUFFER,
-                                     LOCAL_GL_STENCIL_ATTACHMENT,
+        gl->fFramebufferRenderbuffer(target.get(), LOCAL_GL_STENCIL_ATTACHMENT,
                                      LOCAL_GL_RENDERBUFFER, stencilRB);
         return;
     }
 
-    gl->fFramebufferRenderbuffer(LOCAL_GL_FRAMEBUFFER, attachment,
+    gl->fFramebufferRenderbuffer(target.get(), attachment,
                                  LOCAL_GL_RENDERBUFFER, mPrimaryRB);
 }
 
@@ -258,7 +268,7 @@ WebGLRenderbuffer::GetRenderbufferParameter(RBTarget target,
         if (!mFormat)
             return 0;
 
-        if (!mFormat->format->hasStencil)
+        if (!mFormat->format->s)
             return 0;
 
         return 8;
@@ -280,7 +290,7 @@ WebGLRenderbuffer::GetRenderbufferParameter(RBTarget target,
 
     case LOCAL_GL_RENDERBUFFER_INTERNAL_FORMAT:
         {
-            GLenum ret = 0;
+            GLenum ret = LOCAL_GL_RGBA4;
             if (mFormat) {
                 ret = mFormat->format->sizedFormat;
 
