@@ -8,10 +8,10 @@
 #include "angle/ShaderLang.h"
 #include "gfxPrefs.h"
 #include "GLContext.h"
+#include "mozilla/gfx/Logging.h"
 #include "mozilla/Preferences.h"
 #include "MurmurHash3.h"
 #include "nsPrintfCString.h"
-#include "nsTArray.h"
 #include <string>
 #include <vector>
 #include "WebGLContext.h"
@@ -29,32 +29,16 @@ IdentifierHashFunc(const char* name, size_t len)
     return hash[0];
 }
 
-static int
+static ShCompileOptions
 ChooseValidatorCompileOptions(const ShBuiltInResources& resources,
                               const mozilla::gl::GLContext* gl)
 {
-    int options = SH_VARIABLES |
-                  SH_ENFORCE_PACKING_RESTRICTIONS |
-                  SH_INIT_VARYINGS_WITHOUT_STATIC_USE |
-                  SH_OBJECT_CODE |
-                  SH_LIMIT_CALL_STACK_DEPTH |
-                  SH_INIT_GL_POSITION;
-
-    if (resources.MaxExpressionComplexity > 0) {
-        options |= SH_LIMIT_EXPRESSION_COMPLEXITY;
-    }
-
-    if (gfxPrefs::WebGLAllANGLEOptions()) {
-        return options |
-               SH_VALIDATE_LOOP_INDEXING |
-               SH_UNROLL_FOR_LOOP_WITH_INTEGER_INDEX |
-               SH_UNROLL_FOR_LOOP_WITH_SAMPLER_ARRAY_INDEX |
-               SH_EMULATE_BUILT_IN_FUNCTIONS |
-               SH_CLAMP_INDIRECT_ARRAY_BOUNDS |
-               SH_UNFOLD_SHORT_CIRCUIT |
-               SH_SCALARIZE_VEC_AND_MAT_CONSTRUCTOR_ARGS |
-               SH_REGENERATE_STRUCT_NAMES;
-    }
+    ShCompileOptions options = SH_VARIABLES |
+                               SH_ENFORCE_PACKING_RESTRICTIONS |
+                               SH_OBJECT_CODE |
+                               SH_INIT_GL_POSITION |
+                               SH_INITIALIZE_UNINITIALIZED_LOCALS |
+                               SH_INIT_OUTPUT_VARIABLES;
 
 #ifndef XP_MACOSX
     // We want to do this everywhere, but to do this on Mac, we need
@@ -63,32 +47,53 @@ ChooseValidatorCompileOptions(const ShBuiltInResources& resources,
     options |= SH_CLAMP_INDIRECT_ARRAY_BOUNDS;
 #endif
 
-#ifdef XP_MACOSX
     if (gl->WorkAroundDriverBugs()) {
+#ifdef XP_MACOSX
         // Work around https://bugs.webkit.org/show_bug.cgi?id=124684,
         // https://chromium.googlesource.com/angle/angle/+/5e70cf9d0b1bb
         options |= SH_UNFOLD_SHORT_CIRCUIT;
 
-        // Work around bug 665578 and bug 769810
-        if (gl->Vendor() == gl::GLVendor::ATI) {
-            options |= SH_EMULATE_BUILT_IN_FUNCTIONS;
-        }
-
-        // Work around bug 735560
-        if (gl->Vendor() == gl::GLVendor::Intel) {
-            options |= SH_EMULATE_BUILT_IN_FUNCTIONS;
-        }
-
-        // Work around bug 636926
-        if (gl->Vendor() == gl::GLVendor::NVIDIA) {
-            options |= SH_UNROLL_FOR_LOOP_WITH_SAMPLER_ARRAY_INDEX;
-        }
-
         // Work around that Mac drivers handle struct scopes incorrectly.
         options |= SH_REGENERATE_STRUCT_NAMES;
-    }
+        options |= SH_INIT_OUTPUT_VARIABLES;
+
+        // Work around that Intel drivers on Mac OSX handle for-loop incorrectly.
+        if (gl->Vendor() == gl::GLVendor::Intel) {
+            options |= SH_ADD_AND_TRUE_TO_LOOP_CONDITION;
+        }
 #endif
 
+        if (!gl->IsANGLE() && gl->Vendor() == gl::GLVendor::Intel) {
+            // Failures on at least Windows+Intel+OGL on:
+            // conformance/glsl/constructors/glsl-construct-mat2.html
+            options |= SH_SCALARIZE_VEC_AND_MAT_CONSTRUCTOR_ARGS;
+        }
+    }
+
+    if (gfxPrefs::WebGLAllANGLEOptions()) {
+        options = -1;
+
+        options ^= SH_INTERMEDIATE_TREE;
+        options ^= SH_LINE_DIRECTIVES;
+        options ^= SH_SOURCE_PATH;
+
+        options ^= SH_LIMIT_EXPRESSION_COMPLEXITY;
+        options ^= SH_LIMIT_CALL_STACK_DEPTH;
+
+        options ^= SH_EXPAND_SELECT_HLSL_INTEGER_POW_EXPRESSIONS;
+        options ^= SH_HLSL_GET_DIMENSIONS_IGNORES_BASE_LEVEL;
+
+        options ^= SH_DONT_REMOVE_INVARIANT_FOR_FRAGMENT_INPUT;
+        options ^= SH_REMOVE_INVARIANT_AND_CENTROID_FOR_ESSL3;
+    }
+
+    if (resources.MaxExpressionComplexity > 0) {
+        options |= SH_LIMIT_EXPRESSION_COMPLEXITY;
+    }
+    if (resources.MaxCallStackDepth > 0) {
+        options |= SH_LIMIT_CALL_STACK_DEPTH;
+    }
+
     return options;
 }
 
@@ -115,9 +120,13 @@ ShaderOutput(gl::GLContext* gl)
         case 420: return SH_GLSL_420_CORE_OUTPUT;
         case 430: return SH_GLSL_430_CORE_OUTPUT;
         case 440: return SH_GLSL_440_CORE_OUTPUT;
-        case 450: return SH_GLSL_450_CORE_OUTPUT;
         default:
-            MOZ_CRASH("Unexpected GLSL version.");
+            if (version >= 450) {
+                // "OpenGL 4.6 is also guaranteed to support all previous versions of the
+                //  OpenGL Shading Language back to version 1.10."
+                return SH_GLSL_450_CORE_OUTPUT;
+            }
+            gfxCriticalNote << "Unexpected GLSL version: " << version;
         }
     }
 
@@ -130,17 +139,12 @@ WebGLContext::CreateShaderValidator(GLenum shaderType) const
     if (mBypassShaderValidation)
         return nullptr;
 
-    ShShaderSpec spec = IsWebGL2() ? SH_WEBGL2_SPEC : SH_WEBGL_SPEC;
-    ShShaderOutput outputLanguage = gl->IsGLES() ? SH_ESSL_OUTPUT
-                                                 : SH_GLSL_COMPATIBILITY_OUTPUT;
-
-    // If we're using WebGL2 we want a more specific version of GLSL
-    if (IsWebGL2())
-        outputLanguage = ShaderOutput(gl);
+    const auto spec = (IsWebGL2() ? SH_WEBGL2_SPEC : SH_WEBGL_SPEC);
+    const auto outputLanguage = ShaderOutput(gl);
 
     ShBuiltInResources resources;
     memset(&resources, 0, sizeof(resources));
-    ShInitBuiltInResources(&resources);
+    sh::InitBuiltInResources(&resources);
 
     resources.HashFunction = webgl::IdentifierHashFunc;
 
@@ -181,8 +185,7 @@ WebGLContext::CreateShaderValidator(GLenum shaderType) const
 #endif
     }
 
-    int compileOptions = webgl::ChooseValidatorCompileOptions(resources, gl);
-
+    const auto compileOptions = webgl::ChooseValidatorCompileOptions(resources, gl);
     return webgl::ShaderValidator::Create(shaderType, spec, outputLanguage, resources,
                                           compileOptions);
 }
@@ -194,9 +197,10 @@ namespace webgl {
 /*static*/ ShaderValidator*
 ShaderValidator::Create(GLenum shaderType, ShShaderSpec spec,
                         ShShaderOutput outputLanguage,
-                        const ShBuiltInResources& resources, int compileOptions)
+                        const ShBuiltInResources& resources,
+                        ShCompileOptions compileOptions)
 {
-    ShHandle handle = ShConstructCompiler(shaderType, spec, outputLanguage, &resources);
+    ShHandle handle = sh::ConstructCompiler(shaderType, spec, outputLanguage, &resources);
     if (!handle)
         return nullptr;
 
@@ -205,7 +209,7 @@ ShaderValidator::Create(GLenum shaderType, ShShaderSpec spec,
 
 ShaderValidator::~ShaderValidator()
 {
-    ShDestruct(mHandle);
+    sh::Destruct(mHandle);
 }
 
 bool
@@ -217,7 +221,7 @@ ShaderValidator::ValidateAndTranslate(const char* source)
     const char* const parts[] = {
         source
     };
-    return ShCompile(mHandle, parts, ArrayLength(parts), mCompileOptions);
+    return sh::Compile(mHandle, parts, ArrayLength(parts), mCompileOptions);
 }
 
 void
@@ -225,7 +229,7 @@ ShaderValidator::GetInfoLog(nsACString* out) const
 {
     MOZ_ASSERT(mHasRun);
 
-    const std::string &log = ShGetInfoLog(mHandle);
+    const std::string &log = sh::GetInfoLog(mHandle);
     out->Assign(log.data(), log.length());
 }
 
@@ -234,7 +238,7 @@ ShaderValidator::GetOutput(nsACString* out) const
 {
     MOZ_ASSERT(mHasRun);
 
-    const std::string &output = ShGetObjectCode(mHandle);
+    const std::string &output = sh::GetObjectCode(mHandle);
     out->Assign(output.data(), output.length());
 }
 
@@ -254,18 +258,19 @@ ShaderValidator::CanLinkTo(const ShaderValidator* prev, nsCString* const out_log
         return false;
     }
 
-    if (ShGetShaderVersion(prev->mHandle) != ShGetShaderVersion(mHandle)) {
+    const auto shaderVersion = sh::GetShaderVersion(mHandle);
+    if (sh::GetShaderVersion(prev->mHandle) != shaderVersion) {
         nsPrintfCString error("Vertex shader version %d does not match"
                               " fragment shader version %d.",
-                              ShGetShaderVersion(prev->mHandle),
-                              ShGetShaderVersion(mHandle));
+                              sh::GetShaderVersion(prev->mHandle),
+                              sh::GetShaderVersion(mHandle));
         *out_log = error;
         return false;
     }
 
     {
-        const std::vector<sh::Uniform>* vertPtr = ShGetUniforms(prev->mHandle);
-        const std::vector<sh::Uniform>* fragPtr = ShGetUniforms(mHandle);
+        const std::vector<sh::Uniform>* vertPtr = sh::GetUniforms(prev->mHandle);
+        const std::vector<sh::Uniform>* fragPtr = sh::GetUniforms(mHandle);
         if (!vertPtr || !fragPtr) {
             nsPrintfCString error("Could not create uniform list.");
             *out_log = error;
@@ -278,7 +283,7 @@ ShaderValidator::CanLinkTo(const ShaderValidator* prev, nsCString* const out_log
                     continue;
 
                 if (!itrVert->isSameUniformAtLinkTime(*itrFrag)) {
-                    nsPrintfCString error("Uniform `%s`is not linkable between"
+                    nsPrintfCString error("Uniform `%s` is not linkable between"
                                           " attached shaders.",
                                           itrFrag->name.c_str());
                     *out_log = error;
@@ -290,63 +295,87 @@ ShaderValidator::CanLinkTo(const ShaderValidator* prev, nsCString* const out_log
         }
     }
     {
-        const std::vector<sh::Varying>* vertPtr = ShGetVaryings(prev->mHandle);
-        const std::vector<sh::Varying>* fragPtr = ShGetVaryings(mHandle);
-        if (!vertPtr || !fragPtr) {
-            nsPrintfCString error("Could not create varying list.");
+        const auto vertVars = sh::GetInterfaceBlocks(prev->mHandle);
+        const auto fragVars = sh::GetInterfaceBlocks(mHandle);
+        if (!vertVars || !fragVars) {
+            nsPrintfCString error("Could not create uniform block list.");
             *out_log = error;
             return false;
         }
 
-        nsTArray<ShVariableInfo> staticUseVaryingList;
+        for (const auto& fragVar : *fragVars) {
+            for (const auto& vertVar : *vertVars) {
+                if (vertVar.name != fragVar.name)
+                    continue;
 
-        for (auto itrFrag = fragPtr->begin(); itrFrag != fragPtr->end(); ++itrFrag) {
-            const ShVariableInfo varInfo = { itrFrag->type,
-                                             (int)itrFrag->elementCount() };
+                if (!vertVar.isSameInterfaceBlockAtLinkTime(fragVar)) {
+                    nsPrintfCString error("Interface block `%s` is not linkable between"
+                                          " attached shaders.",
+                                          fragVar.name.c_str());
+                    *out_log = error;
+                    return false;
+                }
 
+                break;
+            }
+        }
+    }
+
+    const auto& vertVaryings = sh::GetVaryings(prev->mHandle);
+    const auto& fragVaryings = sh::GetVaryings(mHandle);
+    if (!vertVaryings || !fragVaryings) {
+        nsPrintfCString error("Could not create varying list.");
+        *out_log = error;
+        return false;
+    }
+
+    {
+        std::vector<sh::ShaderVariable> staticUseVaryingList;
+
+        for (const auto& fragVarying : *fragVaryings) {
             static const char prefix[] = "gl_";
-            if (StartsWith(itrFrag->name, prefix)) {
-                if (itrFrag->staticUse)
-                    staticUseVaryingList.AppendElement(varInfo);
-
+            if (StartsWith(fragVarying.name, prefix)) {
+                if (fragVarying.staticUse) {
+                    staticUseVaryingList.push_back(fragVarying);
+                }
                 continue;
             }
 
             bool definedInVertShader = false;
             bool staticVertUse = false;
 
-            for (auto itrVert = vertPtr->begin(); itrVert != vertPtr->end(); ++itrVert) {
-                if (itrVert->name != itrFrag->name)
+            for (const auto& vertVarying : *vertVaryings) {
+                if (vertVarying.name != fragVarying.name)
                     continue;
 
-                if (!itrVert->isSameVaryingAtLinkTime(*itrFrag)) {
+                if (!vertVarying.isSameVaryingAtLinkTime(fragVarying, shaderVersion)) {
                     nsPrintfCString error("Varying `%s`is not linkable between"
                                           " attached shaders.",
-                                          itrFrag->name.c_str());
+                                          fragVarying.name.c_str());
                     *out_log = error;
                     return false;
                 }
 
                 definedInVertShader = true;
-                staticVertUse = itrVert->staticUse;
+                staticVertUse = vertVarying.staticUse;
                 break;
             }
 
-            if (!definedInVertShader && itrFrag->staticUse) {
+            if (!definedInVertShader && fragVarying.staticUse) {
                 nsPrintfCString error("Varying `%s` has static-use in the frag"
                                       " shader, but is undeclared in the vert"
-                                      " shader.", itrFrag->name.c_str());
+                                      " shader.", fragVarying.name.c_str());
                 *out_log = error;
                 return false;
             }
 
-            if (staticVertUse && itrFrag->staticUse)
-                staticUseVaryingList.AppendElement(varInfo);
+            if (staticVertUse && fragVarying.staticUse) {
+                staticUseVaryingList.push_back(fragVarying);
+            }
         }
 
-        if (!ShCheckVariablesWithinPackingLimits(mMaxVaryingVectors,
-                                                 staticUseVaryingList.Elements(),
-                                                 staticUseVaryingList.Length()))
+        if (!sh::CheckVariablesWithinPackingLimits(mMaxVaryingVectors,
+                                                 staticUseVaryingList))
         {
             *out_log = "Statically used varyings do not fit within packing limits. (see"
                        " GLSL ES Specification 1.0.17, p111)";
@@ -354,6 +383,51 @@ ShaderValidator::CanLinkTo(const ShaderValidator* prev, nsCString* const out_log
         }
     }
 
+    if (shaderVersion == 100) {
+        // Enforce ESSL1 invariant linking rules.
+        bool isInvariant_Position = false;
+        bool isInvariant_PointSize = false;
+        bool isInvariant_FragCoord = false;
+        bool isInvariant_PointCoord = false;
+
+        for (const auto& varying : *vertVaryings) {
+            if (varying.name == "gl_Position") {
+                isInvariant_Position = varying.isInvariant;
+            } else if (varying.name == "gl_PointSize") {
+                isInvariant_PointSize = varying.isInvariant;
+            }
+        }
+
+        for (const auto& varying : *fragVaryings) {
+            if (varying.name == "gl_FragCoord") {
+                isInvariant_FragCoord = varying.isInvariant;
+            } else if (varying.name == "gl_PointCoord") {
+                isInvariant_PointCoord = varying.isInvariant;
+            }
+        }
+
+        ////
+
+        const auto fnCanBuiltInsLink = [](bool vertIsInvariant, bool fragIsInvariant) {
+            if (vertIsInvariant)
+                return true;
+
+            return !fragIsInvariant;
+        };
+
+        if (!fnCanBuiltInsLink(isInvariant_Position, isInvariant_FragCoord)) {
+            *out_log = "gl_Position must be invariant if gl_FragCoord is. (see GLSL ES"
+                       " Specification 1.0.17, p39)";
+            return false;
+        }
+
+        if (!fnCanBuiltInsLink(isInvariant_PointSize, isInvariant_PointCoord)) {
+            *out_log = "gl_PointSize must be invariant if gl_PointCoord is. (see GLSL ES"
+                       " Specification 1.0.17, p39)";
+            return false;
+        }
+    }
+
     return true;
 }
 
@@ -362,7 +436,7 @@ ShaderValidator::CalcNumSamplerUniforms() const
 {
     size_t accum = 0;
 
-    const std::vector<sh::Uniform>& uniforms = *ShGetUniforms(mHandle);
+    const std::vector<sh::Uniform>& uniforms = *sh::GetUniforms(mHandle);
 
     for (auto itr = uniforms.begin(); itr != uniforms.end(); ++itr) {
         GLenum type = itr->type;
@@ -379,7 +453,7 @@ ShaderValidator::CalcNumSamplerUniforms() const
 size_t
 ShaderValidator::NumAttributes() const
 {
-  return ShGetAttributes(mHandle)->size();
+  return sh::GetAttributes(mHandle)->size();
 }
 
 // Attribs cannot be structs or arrays, and neither can vertex inputs in ES3.
@@ -388,7 +462,7 @@ bool
 ShaderValidator::FindAttribUserNameByMappedName(const std::string& mappedName,
                                                 const std::string** const out_userName) const
 {
-    const std::vector<sh::Attribute>& attribs = *ShGetAttributes(mHandle);
+    const std::vector<sh::Attribute>& attribs = *sh::GetAttributes(mHandle);
     for (auto itr = attribs.begin(); itr != attribs.end(); ++itr) {
         if (itr->mappedName == mappedName) {
             *out_userName = &(itr->name);
@@ -399,26 +473,11 @@ ShaderValidator::FindAttribUserNameByMappedName(const std::string& mappedName,
     return false;
 }
 
-bool
-ShaderValidator::FindActiveOutputMappedNameByUserName(const std::string& userName,
-                                                      const std::string** const out_mappedName) const
-{
-    const std::vector<sh::OutputVariable>& varibles = *ShGetOutputVariables(mHandle);
-    for (auto itr = varibles.begin(); itr != varibles.end(); ++itr) {
-        if (itr->name == userName) {
-            *out_mappedName = &(itr->mappedName);
-            return true;
-        }
-    }
-
-    return false;
-}
-
 bool
 ShaderValidator::FindAttribMappedNameByUserName(const std::string& userName,
                                                 const std::string** const out_mappedName) const
 {
-    const std::vector<sh::Attribute>& attribs = *ShGetAttributes(mHandle);
+    const std::vector<sh::Attribute>& attribs = *sh::GetAttributes(mHandle);
     for (auto itr = attribs.begin(); itr != attribs.end(); ++itr) {
         if (itr->name == userName) {
             *out_mappedName = &(itr->mappedName);
@@ -434,7 +493,7 @@ ShaderValidator::FindVaryingByMappedName(const std::string& mappedName,
                                          std::string* const out_userName,
                                          bool* const out_isArray) const
 {
-    const std::vector<sh::Varying>& varyings = *ShGetVaryings(mHandle);
+    const std::vector<sh::Varying>& varyings = *sh::GetVaryings(mHandle);
     for (auto itr = varyings.begin(); itr != varyings.end(); ++itr) {
         const sh::ShaderVariable* found;
         if (!itr->findInfoByMappedName(mappedName, &found, out_userName))
@@ -451,7 +510,7 @@ bool
 ShaderValidator::FindVaryingMappedNameByUserName(const std::string& userName,
                                                  const std::string** const out_mappedName) const
 {
-    const std::vector<sh::Varying>& attribs = *ShGetVaryings(mHandle);
+    const std::vector<sh::Varying>& attribs = *sh::GetVaryings(mHandle);
     for (auto itr = attribs.begin(); itr != attribs.end(); ++itr) {
         if (itr->name == userName) {
             *out_mappedName = &(itr->mappedName);
@@ -467,7 +526,7 @@ ShaderValidator::FindUniformByMappedName(const std::string& mappedName,
                                          std::string* const out_userName,
                                          bool* const out_isArray) const
 {
-    const std::vector<sh::Uniform>& uniforms = *ShGetUniforms(mHandle);
+    const std::vector<sh::Uniform>& uniforms = *sh::GetUniforms(mHandle);
     for (auto itr = uniforms.begin(); itr != uniforms.end(); ++itr) {
         const sh::ShaderVariable* found;
         if (!itr->findInfoByMappedName(mappedName, &found, out_userName))
@@ -479,7 +538,7 @@ ShaderValidator::FindUniformByMappedName(const std::string& mappedName,
 
     const size_t dotPos = mappedName.find(".");
 
-    const std::vector<sh::InterfaceBlock>& interfaces = *ShGetInterfaceBlocks(mHandle);
+    const std::vector<sh::InterfaceBlock>& interfaces = *sh::GetInterfaceBlocks(mHandle);
     for (const auto& interface : interfaces) {
 
         std::string mappedFieldName;
@@ -524,13 +583,15 @@ ShaderValidator::FindUniformByMappedName(const std::string& mappedName,
 }
 
 bool
-ShaderValidator::FindUniformBlockByMappedName(const std::string& mappedName,
-                                              std::string* const out_userName) const
+ShaderValidator::UnmapUniformBlockName(const nsACString& baseMappedName,
+                                       nsCString* const out_baseUserName) const
 {
-    const std::vector<sh::InterfaceBlock>& interfaces = *ShGetInterfaceBlocks(mHandle);
+    const std::vector<sh::InterfaceBlock>& interfaces = *sh::GetInterfaceBlocks(mHandle);
     for (const auto& interface : interfaces) {
-        if (mappedName == interface.mappedName) {
-            *out_userName = interface.name;
+        const nsDependentCString interfaceMappedName(interface.mappedName.data(),
+                                                     interface.mappedName.size());
+        if (baseMappedName == interfaceMappedName) {
+            *out_baseUserName = interface.name.data();
             return true;
         }
     }
@@ -538,5 +599,18 @@ ShaderValidator::FindUniformBlockByMappedName(const std::string& mappedName,
     return false;
 }
 
+void
+ShaderValidator::EnumerateFragOutputs(std::map<nsCString, const nsCString> &out_FragOutputs) const
+{
+    const auto* fragOutputs = sh::GetOutputVariables(mHandle);
+
+    if (fragOutputs) {
+        for (const auto& fragOutput : *fragOutputs) {
+            out_FragOutputs.insert({nsCString(fragOutput.name.c_str()),
+                                    nsCString(fragOutput.mappedName.c_str())});
+        }
+    }
+}
+
 } // namespace webgl
 } // namespace mozilla
