@@ -19,18 +19,16 @@
 
 namespace mozilla {
 namespace dom {
-
 // The global is used to extract the principal.
 already_AddRefed<InternalRequest>
 InternalRequest::GetRequestConstructorCopy(nsIGlobalObject* aGlobal, ErrorResult& aRv) const
 {
   MOZ_RELEASE_ASSERT(!mURLList.IsEmpty(), "Internal Request's urlList should not be empty when copied from constructor.");
-
-  RefPtr<InternalRequest> copy = new InternalRequest(mURLList.LastElement());
+  RefPtr<InternalRequest> copy = new InternalRequest(mURLList.LastElement(),
+                                                     mFragment);
   copy->SetMethod(mMethod);
   copy->mHeaders = new InternalHeaders(*mHeaders);
   copy->SetUnsafeRequest();
-
   copy->mBodyStream = mBodyStream;
   copy->mForceOriginHeader = true;
   // The "client" is not stored in our implementation. Fetch API users should
@@ -40,6 +38,8 @@ InternalRequest::GetRequestConstructorCopy(nsIGlobalObject* aGlobal, ErrorResult
   copy->mPreserveContentCodings = true;
   copy->mReferrer = mReferrer;
   copy->mReferrerPolicy = mReferrerPolicy;
+  copy->mEnvironmentReferrerPolicy = mEnvironmentReferrerPolicy;
+  copy->mIntegrity = mIntegrity;
 
   copy->mContentPolicyType = mContentPolicyTypeOverridden ?
                              mContentPolicyType :
@@ -48,7 +48,6 @@ InternalRequest::GetRequestConstructorCopy(nsIGlobalObject* aGlobal, ErrorResult
   copy->mCredentialsMode = mCredentialsMode;
   copy->mCacheMode = mCacheMode;
   copy->mRedirectMode = mRedirectMode;
-  copy->mCreatedByFetchEvent = mCreatedByFetchEvent;
   copy->mContentPolicyTypeOverridden = mContentPolicyTypeOverridden;
   return copy.forget();
 }
@@ -73,10 +72,74 @@ InternalRequest::Clone()
   if (replacementBody) {
     mBodyStream.swap(replacementBody);
   }
-
   return clone.forget();
 }
-
+InternalRequest::InternalRequest(const nsACString& aURL,
+                                 const nsACString& aFragment)
+  : mMethod("GET")
+  , mHeaders(new InternalHeaders(HeadersGuardEnum::None))
+  , mContentPolicyType(nsIContentPolicy::TYPE_FETCH)
+  , mReferrer(NS_LITERAL_STRING(kFETCH_CLIENT_REFERRER_STR))
+  , mReferrerPolicy(ReferrerPolicy::_empty)
+  , mEnvironmentReferrerPolicy(net::RP_Unset)
+  , mMode(RequestMode::No_cors)
+  , mCredentialsMode(RequestCredentials::Omit)
+  , mResponseTainting(LoadTainting::Basic)
+  , mCacheMode(RequestCache::Default)
+  , mRedirectMode(RequestRedirect::Follow)
+  , mAuthenticationFlag(false)
+  , mForceOriginHeader(false)
+  , mPreserveContentCodings(false)
+    // FIXME(nsm): This should be false by default, but will lead to the
+    // algorithm never loading data: URLs right now. See Bug 1018872 about
+    // how certain contexts will override it to set it to true. Fetch
+    // specification does not handle this yet.
+  , mSameOriginDataURL(true)
+  , mSkipServiceWorker(false)
+  , mSynchronous(false)
+  , mUnsafeRequest(false)
+  , mUseURLCredentials(false)
+{
+  MOZ_ASSERT(!aURL.IsEmpty());
+  AddURL(aURL, aFragment);
+}
+InternalRequest::InternalRequest(const nsACString& aURL,
+                                 const nsACString& aFragment,
+                                 const nsACString& aMethod,
+                                 already_AddRefed<InternalHeaders> aHeaders,
+                                 RequestCache aCacheMode,
+                                 RequestMode aMode,
+                                 RequestRedirect aRequestRedirect,
+                                 RequestCredentials aRequestCredentials,
+                                 const nsAString& aReferrer,
+                                 ReferrerPolicy aReferrerPolicy,
+                                 nsContentPolicyType aContentPolicyType,
+                                 const nsAString& aIntegrity)
+  : mMethod(aMethod)
+  , mHeaders(aHeaders)
+  , mContentPolicyType(aContentPolicyType)
+  , mReferrer(aReferrer)
+  , mReferrerPolicy(aReferrerPolicy)
+  , mEnvironmentReferrerPolicy(net::RP_Unset)
+  , mMode(aMode)
+  , mCredentialsMode(aRequestCredentials)
+  , mResponseTainting(LoadTainting::Basic)
+  , mCacheMode(aCacheMode)
+  , mRedirectMode(aRequestRedirect)
+  , mIntegrity(aIntegrity)
+  , mAuthenticationFlag(false)
+  , mForceOriginHeader(false)
+  , mPreserveContentCodings(false)
+    // FIXME See the above comment in the default constructor.
+  , mSameOriginDataURL(true)
+  , mSkipServiceWorker(false)
+  , mSynchronous(false)
+  , mUnsafeRequest(false)
+  , mUseURLCredentials(false)
+{
+  MOZ_ASSERT(!aURL.IsEmpty());
+  AddURL(aURL, aFragment);
+}
 InternalRequest::InternalRequest(const InternalRequest& aOther)
   : mMethod(aOther.mMethod)
   , mURLList(aOther.mURLList)
@@ -84,11 +147,14 @@ InternalRequest::InternalRequest(const InternalRequest& aOther)
   , mContentPolicyType(aOther.mContentPolicyType)
   , mReferrer(aOther.mReferrer)
   , mReferrerPolicy(aOther.mReferrerPolicy)
+  , mEnvironmentReferrerPolicy(aOther.mEnvironmentReferrerPolicy)
   , mMode(aOther.mMode)
   , mCredentialsMode(aOther.mCredentialsMode)
   , mResponseTainting(aOther.mResponseTainting)
   , mCacheMode(aOther.mCacheMode)
   , mRedirectMode(aOther.mRedirectMode)
+  , mIntegrity(aOther.mIntegrity)
+  , mFragment(aOther.mFragment)
   , mAuthenticationFlag(aOther.mAuthenticationFlag)
   , mForceOriginHeader(aOther.mForceOriginHeader)
   , mPreserveContentCodings(aOther.mPreserveContentCodings)
@@ -97,7 +163,6 @@ InternalRequest::InternalRequest(const InternalRequest& aOther)
   , mSynchronous(aOther.mSynchronous)
   , mUnsafeRequest(aOther.mUnsafeRequest)
   , mUseURLCredentials(aOther.mUseURLCredentials)
-  , mCreatedByFetchEvent(aOther.mCreatedByFetchEvent)
   , mContentPolicyTypeOverridden(aOther.mContentPolicyTypeOverridden)
 {
   // NOTE: does not copy body stream... use the fallible Clone() for that
@@ -167,6 +232,7 @@ InternalRequest::MapContentPolicyTypeToRequestContext(nsContentPolicyType aConte
   case nsIContentPolicy::TYPE_INTERNAL_SCRIPT:
   case nsIContentPolicy::TYPE_INTERNAL_SCRIPT_PRELOAD:
   case nsIContentPolicy::TYPE_INTERNAL_SERVICE_WORKER:
+  case nsIContentPolicy::TYPE_INTERNAL_WORKER_IMPORT_SCRIPTS:
     context = RequestContext::Script;
     break;
   case nsIContentPolicy::TYPE_INTERNAL_WORKER:
@@ -177,6 +243,7 @@ InternalRequest::MapContentPolicyTypeToRequestContext(nsContentPolicyType aConte
     break;
   case nsIContentPolicy::TYPE_INTERNAL_IMAGE:
   case nsIContentPolicy::TYPE_INTERNAL_IMAGE_PRELOAD:
+  case nsIContentPolicy::TYPE_INTERNAL_IMAGE_FAVICON:
     context = RequestContext::Image;
     break;
   case nsIContentPolicy::TYPE_INTERNAL_STYLESHEET:
@@ -335,8 +402,7 @@ InternalRequest::MapChannelToRequestMode(nsIChannel* aChannel)
     return RequestMode::Same_origin;
   }
 
-  uint32_t securityMode;
-  MOZ_ALWAYS_SUCCEEDS(loadInfo->GetSecurityMode(&securityMode));
+  uint32_t securityMode = loadInfo->GetSecurityMode();
 
   switch(securityMode) {
     case nsILoadInfo::SEC_REQUIRE_SAME_ORIGIN_DATA_INHERITS:
@@ -375,27 +441,7 @@ InternalRequest::MapChannelToRequestCredentials(nsIChannel* aChannel)
   nsCOMPtr<nsILoadInfo> loadInfo;
   MOZ_ALWAYS_SUCCEEDS(aChannel->GetLoadInfo(getter_AddRefs(loadInfo)));
 
-  uint32_t securityMode;
-  MOZ_ALWAYS_SUCCEEDS(loadInfo->GetSecurityMode(&securityMode));
-
-  // TODO: Remove following code after stylesheet and image support cookie policy
-  if (securityMode == nsILoadInfo::SEC_NORMAL) {
-    uint32_t loadFlags;
-    aChannel->GetLoadFlags(&loadFlags);
-
-    if (loadFlags & nsIRequest::LOAD_ANONYMOUS) {
-      return RequestCredentials::Omit;
-    } else {
-      bool includeCrossOrigin;
-      nsCOMPtr<nsIHttpChannelInternal> internalChannel = do_QueryInterface(aChannel);
-
-      internalChannel->GetCorsIncludeCredentials(&includeCrossOrigin);
-      if (includeCrossOrigin) {
-        return RequestCredentials::Include;
-      }
-    }
-    return RequestCredentials::Same_origin;
-  }
+  MOZ_DIAGNOSTIC_ASSERT(loadInfo->GetSecurityMode() != nsILoadInfo::SEC_NORMAL);
 
   uint32_t cookiePolicy = loadInfo->GetCookiePolicy();
 
@@ -420,5 +466,11 @@ InternalRequest::MaybeSkipCacheIfPerformingRevalidation()
   }
 }
 
+void
+InternalRequest::SetPrincipalInfo(UniquePtr<mozilla::ipc::PrincipalInfo> aPrincipalInfo)
+{
+  mPrincipalInfo = Move(aPrincipalInfo);
+}
+
 } // namespace dom
 } // namespace mozilla
